@@ -41,6 +41,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "gui/specific/GUIXMoto.h"
 #include "xmscene/Camera.h"
 #include "xmscene/Entity.h"
+#include "UserConfig.h"
 
 #include <curl/curl.h>
 #include <iomanip>
@@ -52,6 +53,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "states/StateMessageBox.h"
 #include "XMotoLoadReplaysInterface.h"
 #include "Replay.h"
+#include <sstream>
 
   bool GameApp::haveMouseMoved() {
     int nX,nY;
@@ -73,10 +75,10 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
     time(&T);
     pTime = localtime(&T);
     if(pTime != NULL) {
-      sprintf(cBuf,"%d-%02d-%02d %02d:%02d:%02d",
-              pTime->tm_year+1900, pTime->tm_mon+1, pTime->tm_mday,
-	      pTime->tm_hour, pTime->tm_min, pTime->tm_sec);                    
-    }    
+      snprintf(cBuf,256, "%d-%02d-%02d %02d:%02d:%02d",
+	       pTime->tm_year+1900, pTime->tm_mon+1, pTime->tm_mday,
+	       pTime->tm_hour, pTime->tm_min, pTime->tm_sec);                    
+    }
     return cBuf;
   }
 
@@ -123,7 +125,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
     /* Set window title */
     SDL_WM_SetCaption(XMBuild::getVersionString(true).c_str(), XMBuild::getVersionString(true).c_str());
 
-#if !defined(WIN32) && !defined(__APPLE__) 
+#if !defined(WIN32) && !defined(__APPLE__) && !defined(__amigaos4__) 
     SDL_Surface *v_icon = SDL_LoadBMP(GAMEDATADIR "/xmoto_icone_x.ico");
     if(v_icon != NULL) {
       SDL_SetColorKey(v_icon, SDL_SRCCOLORKEY,
@@ -148,6 +150,7 @@ GameApp::~GameApp() {
   XMSession::instance()->destroy();
 
   StateManager::cleanStates();
+  delete m_userConfig;
 }
 
 GameApp::GameApp() {
@@ -184,6 +187,8 @@ GameApp::GameApp() {
   m_hasMouseFocus    = true;
   m_hasKeyboardFocus = true;
   m_isIconified      = false;
+
+  m_userConfig = new UserConfig();
 }
    
   /*===========================================================================
@@ -204,14 +209,14 @@ GameApp::GameApp() {
     
     /* User preference for format? must be either jpeg or png */
     if(v_ShotExtension != "jpeg" && v_ShotExtension != "jpg" && v_ShotExtension != "png") {
-      Logger::Log("** Warning ** : unsupported screenshot format '%s', using png instead!", v_ShotExtension.c_str());
+      LogWarning("unsupported screenshot format '%s', using png instead!", v_ShotExtension.c_str());
       v_ShotExtension = "png";
     }    
 
     do {
       nShot++;
       if(nShot > 9999) {
-	Logger::Log("Too many screenshots !");
+	LogWarning("Too many screenshots !");
 	delete pShot;
 	return;
       }
@@ -222,7 +227,7 @@ GameApp::GameApp() {
     try {
       pShot->saveFile(v_destFile.c_str());
     } catch(Exception &e) {
-      Logger::Log(std::string("Unable to save the screenshot: " + e.getMsg()).c_str());
+      LogError(std::string("Unable to save the screenshot: " + e.getMsg()).c_str());
     }
 
     delete pShot;
@@ -246,43 +251,6 @@ GameApp::GameApp() {
     }
   }
 
-  /*===========================================================================
-  Key down event
-  ===========================================================================*/
-void GameApp::keyDown(int nKey, SDLMod mod, int nChar) {
-  StateManager::instance()->keyDown(nKey, mod, nChar);
-}
-
-/*===========================================================================
-  Key up event
-  ===========================================================================*/
-  void GameApp::keyUp(int nKey, SDLMod mod) {
-    StateManager::instance()->keyUp(nKey, mod);
-  }
-
-void GameApp::changeFocus(bool i_hasFocus) {
-  StateManager::instance()->changeFocus(i_hasFocus);
-}
-
-void GameApp::changeVisibility(bool i_visible) {
-  StateManager::instance()->changeVisibility(i_visible);
-}
-
-  /*===========================================================================
-  Mouse events
-  ===========================================================================*/
-  void GameApp::mouseDoubleClick(int nButton) {
-    StateManager::instance()->mouseDoubleClick(nButton);
-  }
-
-  void GameApp::mouseDown(int nButton) {
-    StateManager::instance()->mouseDown(nButton);
-  }
-
-  void GameApp::mouseUp(int nButton) {
-    StateManager::instance()->mouseUp(nButton);
-  }
-
   std::string GameApp::determineNextLevel(const std::string& i_id_level) {
     if(m_currentPlayingList == NULL) {
       return "";
@@ -418,7 +386,7 @@ std::string GameApp::_getGhostReplayPath_bestOfTheRoom(unsigned int i_number, st
 			    "ON a.id_profile = b.id_profile "
 			    "WHERE a.id_level=\""   + xmDatabase::protectString(p_levelId) + "\" "
 			    "AND   a.isFinished=1 "
-			    "ORDER BY a.finishTime LIMIT 1;",
+			    "ORDER BY a.finishTime+0 LIMIT 1;",
 			    nrow);    
     if(nrow == 0) {
       v_pDb->read_DB_free(v_result);
@@ -488,13 +456,19 @@ std::string GameApp::_getGhostReplayPath_bestOfTheRoom(unsigned int i_number, st
 
   void GameApp::switchUglyMode(bool bUgly) {
     XMSession::instance()->setUgly(bUgly);
-    if(bUgly == false) {
-      SDL_ShowCursor(SDL_DISABLE);        
-    } else {
-      SDL_ShowCursor(SDL_ENABLE);
-    }
+    
+    displayCursor(bUgly);
   }
 
+void GameApp::displayCursor(bool display)
+{
+  if(display == false) {
+    SDL_ShowCursor(SDL_DISABLE);        
+  } else {
+    SDL_ShowCursor(SDL_ENABLE);
+  }
+}
+
   void GameApp::switchTestThemeMode(bool mode) {
     XMSession::instance()->setTestTheme(mode);
   }
@@ -508,7 +482,7 @@ std::string GameApp::_getGhostReplayPath_bestOfTheRoom(unsigned int i_number, st
       Theme::instance()->load(xmDatabase::instance("main")->themes_getFileName(XMSession::instance()->theme()));
     } catch(Exception &e) {
       /* unable to load the theme, load the default one */
-      Theme::instance()->load(xmDatabase::instance("main")->themes_getFileName(THEME_DEFAULT_THEMENAME));
+      Theme::instance()->load(xmDatabase::instance("main")->themes_getFileName(DEFAULT_THEME)); // no XMDefault::DefaultTheme, the DEFAULT_THEME one is included into xmoto files
     }
   }
 
@@ -531,7 +505,7 @@ std::string GameApp::_getGhostReplayPath_bestOfTheRoom(unsigned int i_number, st
 	if(FS::getFileBaseName(ReplayFiles[i]) == "Latest") {
 	  continue;
 	}
-	addReplay(ReplayFiles[i], pDb);
+	addReplay(ReplayFiles[i], pDb, false);
 	if(pLoadReplaysInterface != NULL){
 	  pLoadReplaysInterface->loadReplayHook(ReplayFiles[i], (int)((i*100)/((float)ReplayFiles.size())));
 	}
@@ -543,15 +517,8 @@ std::string GameApp::_getGhostReplayPath_bestOfTheRoom(unsigned int i_number, st
     pDb->replays_add_end();
   }
 
-  void GameApp::addReplay(const std::string& i_file, xmDatabase* threadDb) {
+  void GameApp::addReplay(const std::string& i_file, xmDatabase* pDb, bool sendMessage) {
     ReplayInfo* rplInfos;
-    xmDatabase* pDb = NULL;
-
-    if(threadDb == NULL){
-      pDb = xmDatabase::instance("main");
-    } else {
-      pDb = threadDb;
-    }
     
     rplInfos = Replay::getReplayInfos(FS::getFileBaseName(i_file));
     if(rplInfos == NULL) {
@@ -564,7 +531,8 @@ std::string GameApp::_getGhostReplayPath_bestOfTheRoom(unsigned int i_number, st
 		       rplInfos->Player,
 		       rplInfos->IsFinished,
 		       rplInfos->finishTime);
-      StateManager::instance()->sendAsynchronousMessage("REPLAYS_UPDATED");
+      if(sendMessage == true)
+	StateManager::instance()->sendAsynchronousMessage("REPLAYS_UPDATED");
 
     } catch(Exception &e2) {
       delete rplInfos;
@@ -584,7 +552,7 @@ std::string GameApp::_getGhostReplayPath_bestOfTheRoom(unsigned int i_number, st
     m_PlaySpecificLevelFile = i_leveFile;
   }
 
-void GameApp::addGhosts(MotoGame* i_motogame, Theme* i_theme) {
+void GameApp::addGhosts(Scene* i_motogame, Theme* i_theme) {
   std::string v_replay_MYBEST;
   std::string v_replay_THEBEST;
   std::string v_replay_BESTOFROOM[ROOMS_NB_MAX];
@@ -593,30 +561,39 @@ void GameApp::addGhosts(MotoGame* i_motogame, Theme* i_theme) {
   int v_player_finishTime;
   bool v_exists;
 
+  LogDebug("addGhosts stategy:");
+
   v_replay_MYBEST_tmp = _getGhostReplayPath_bestOfThePlayer(i_motogame->getLevelSrc()->Id(), v_player_finishTime);
 
   /* first, add the best of the room -- because if mybest or thebest = bestofroom, i prefer to see writen bestofroom */
   for(unsigned int i=0; i<XMSession::instance()->nbRoomsEnabled(); i++) {
     if( (XMSession::instance()->ghostStrategy_BESTOFREFROOM()    && i==0) ||
 	(XMSession::instance()->ghostStrategy_BESTOFOTHERROOMS() && i!=0) ){
+
+      LogDebug("Choosing ghost for room %i", i);
+
       v_replay_BESTOFROOM[i] = _getGhostReplayPath_bestOfTheRoom(i, i_motogame->getLevelSrc()->Id(), v_finishTime);
-      
+      LogDebug("the room ghost: %s", v_replay_BESTOFROOM[i].c_str());
+
       /* add MYBEST if MYBEST if better the BESTOF the other ROOM */
       if(v_player_finishTime > 0 && (v_finishTime < 0 || v_player_finishTime < v_finishTime)) {
 	v_replay_BESTOFROOM[i] = v_replay_MYBEST_tmp;
+	LogDebug("my best time is %i ; room one is %i", v_player_finishTime, v_finishTime);
+	LogDebug("my best is better than the one of the room => choose it");
       }
       
       v_exists = false;
       for(unsigned int j=0; j<i; j++) {
       	if(v_replay_BESTOFROOM[i] == v_replay_BESTOFROOM[j]) {
       	  v_exists = true;
+	  LogDebug("the ghost is already set by room %i", j);
       	}
       }
 
       if(v_replay_BESTOFROOM[i] != "" && v_exists == false) {
-	Logger::Log("add ghost %s\n", v_replay_BESTOFROOM[i].c_str());
+	LogInfo("add ghost %s", v_replay_BESTOFROOM[i].c_str());
 	i_motogame->addGhostFromFile(v_replay_BESTOFROOM[i],
-				     xmDatabase::instance("main")->webrooms_getName(XMSession::instance()->idRoom(i)),
+				     xmDatabase::instance("main")->webrooms_getName(XMSession::instance()->idRoom(i)), i==0,
 				     Theme::instance(), Theme::instance()->getGhostTheme(),
 				     TColor(255,255,255,0),
 				     TColor(GET_RED(i_theme->getGhostTheme()->getUglyRiderColor()),
@@ -630,17 +607,21 @@ void GameApp::addGhosts(MotoGame* i_motogame, Theme* i_theme) {
 
   /* second, add your best */
   if(XMSession::instance()->ghostStrategy_MYBEST()) {
+    LogDebug("Choosing ghost MYBEST");
     v_replay_MYBEST = _getGhostReplayPath_bestOfThePlayer(i_motogame->getLevelSrc()->Id(), v_finishTime);
+    LogDebug("MYBEST ghost is %s", v_replay_MYBEST.c_str());
     if(v_replay_MYBEST != "") {
       v_exists = false;
       for(unsigned int i=0; i<XMSession::instance()->nbRoomsEnabled(); i++) {
 	if(v_replay_MYBEST == v_replay_BESTOFROOM[i]) {
+	  LogDebug("the ghost is already set by room %i", i);
 	  v_exists = true;
 	}
       }
 
       if(v_exists == false) {
-	i_motogame->addGhostFromFile(v_replay_MYBEST, GAMETEXT_GHOST_BEST,
+	LogDebug("add ghost %s", v_replay_MYBEST.c_str());
+	i_motogame->addGhostFromFile(v_replay_MYBEST, GAMETEXT_GHOST_BEST, true,
 				     i_theme, i_theme->getGhostTheme(),
 				     TColor(85,255,255,0),
 				     TColor(GET_RED(i_theme->getGhostTheme()->getUglyRiderColor()),
@@ -654,18 +635,21 @@ void GameApp::addGhosts(MotoGame* i_motogame, Theme* i_theme) {
 
   /* third, the best locally */
   if(XMSession::instance()->ghostStrategy_THEBEST()) {
+    LogDebug("Choosing ghost LOCAL BEST");
     v_replay_THEBEST = _getGhostReplayPath_bestOfLocal(i_motogame->getLevelSrc()->Id(), v_finishTime);
     if(v_replay_THEBEST != "") {
 
       v_exists = false;
       for(unsigned int i=0; i<XMSession::instance()->nbRoomsEnabled(); i++) {
 	if(v_replay_THEBEST == v_replay_BESTOFROOM[i]) {
+	  LogDebug("the ghost is already set by room %i", i);
 	  v_exists = true;
 	}
       }
 
       if(v_replay_THEBEST != v_replay_MYBEST && v_exists == false) { /* don't add two times the same ghost */
-	i_motogame->addGhostFromFile(v_replay_THEBEST, GAMETEXT_GHOST_LOCAL,
+	LogDebug("add ghost %s", v_replay_THEBEST.c_str());
+	i_motogame->addGhostFromFile(v_replay_THEBEST, GAMETEXT_GHOST_LOCAL, false,
 				     i_theme, i_theme->getGhostTheme(),
 				     TColor(255,200,140,0),
 				     TColor(GET_RED(i_theme->getGhostTheme()->getUglyRiderColor()),
@@ -677,6 +661,7 @@ void GameApp::addGhosts(MotoGame* i_motogame, Theme* i_theme) {
     }
   }
 
+  LogDebug("addGhosts stategy finished");
 }
 
 void GameApp::addLevelToFavorite(const std::string& i_levelId) {
@@ -719,34 +704,59 @@ bool GameApp::isRequestingEnd() {
   return m_bQuit;
 }
 
+void GameApp::playMenuMusic(const std::string& i_music) {
+  if(XMSession::instance()->enableAudio() && XMSession::instance()->enableMenuMusic()) {
+    playMusic(i_music);
+  } else {
+    playMusic("");
+  }
+}
+
+void GameApp::playGameMusic(const std::string& i_music) {
+  if(XMSession::instance()->enableAudio() && XMSession::instance()->enableGameMusic()) {
+    playMusic(i_music);
+  } else {
+    playMusic("");
+  }
+}
+
 void GameApp::playMusic(const std::string& i_music) {
-  if( (XMSession::instance()->enableAudio() && XMSession::instance()->enableMenuMusic()) || i_music == "") {
-    if(i_music != m_playingMusic) {
-      try {
-	if(i_music == "") {
-	  m_playingMusic = "";
-	  Sound::stopMusic();
-	} else {
-	  m_playingMusic = i_music;
-	  Sound::playMusic(Theme::instance()->getMusic(i_music)->FilePath());
-	}
-      } catch(Exception &e) {
-	Logger::Log("** Warning ** : PlayMusic(%s) failed", i_music.c_str());
+  LogDebug("Playing '%s'\n", i_music.c_str());
+
+  if(i_music != m_playingMusic) {
+    try {
+      if(i_music == "") {
+	m_playingMusic = "";
 	Sound::stopMusic();
+      } else {
+	m_playingMusic = i_music;
+	Sound::playMusic(Theme::instance()->getMusic(i_music)->FilePath());
       }
+    } catch(Exception &e) {
+      LogWarning("PlayMusic(%s) failed", i_music.c_str());
+      Sound::stopMusic();
     }
   }
 }
 
+void GameApp::toogleEnableMusic() {
+  m_playingMusic = ""; // tell the game the last music played is "" otherwise, it doesn't know when music is disable then enable back
+  XMSession::instance()->setEnableAudio(! XMSession::instance()->enableAudio());
+  Sound::setActiv(XMSession::instance()->enableAudio());
+
+  if(XMSession::instance()->enableAudio()) {
+    SysMessage::instance()->displayText(SYS_MSG_AUDIO_ENABLED);
+  } else {
+    SysMessage::instance()->displayText(SYS_MSG_AUDIO_DISABLED);
+  }
+  StateManager::instance()->sendAsynchronousMessage("ENABLEAUDIO_CHANGED");
+}
+
 std::string GameApp::getWebRoomURL(unsigned int i_number, xmDatabase* pDb) {
   char **v_result;
   unsigned int nrow;
   std::string v_url;
 
-  if(pDb == NULL){
-    pDb = xmDatabase::instance("main");
-  }
-
   v_result = pDb->readDB("SELECT highscoresUrl FROM webrooms WHERE id_room=" + XMSession::instance()->idRoom(i_number) + ";", nrow);
   if(nrow != 1) {
     pDb->read_DB_free(v_result);
@@ -763,10 +773,6 @@ std::string GameApp::getWebRoomName(unsigned int i_number, xmDatabase* pDb) {
   unsigned int nrow;
   std::string v_name;
 
-  if(pDb == NULL){
-    pDb = xmDatabase::instance("main");
-  }
-
   /* set the room name ; set to WR if it cannot be determined */
   v_name = "WR";
 
@@ -811,14 +817,14 @@ void GameApp::loadLevelHook(std::string i_level, int i_percentage)
   std::ostringstream v_percentage;
   v_percentage << i_percentage;
   v_percentage << "%";
-  _UpdateLoadingScreen(0, std::string(GAMETEXT_LOAD_LEVEL_HOOK) + std::string("\n") + v_percentage.str() + std::string(" ") + i_level);
+  _UpdateLoadingScreen(std::string(GAMETEXT_LOAD_LEVEL_HOOK) + std::string("\n") + v_percentage.str() + std::string(" ") + i_level);
 
   /* pump events to so that windows don't think the appli is crashed */
   SDL_PumpEvents();
 }
 
 void GameApp::updatingDatabase(std::string i_message) {
-  _UpdateLoadingScreen(0, i_message);
+  _UpdateLoadingScreen(i_message);
 
   /* pump events to so that windows don't think the appli is crashed */
   SDL_PumpEvents();
