@@ -32,6 +32,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "Input.h"
 #include "db/xmDatabase.h"
 #include "helpers/Log.h"
+#include "helpers/Random.h"
 
 #include <curl/curl.h>
 #include "XMSession.h"
@@ -39,6 +40,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "drawlib/DrawLib.h"
 #include "Packager.h"
 #include "helpers/SwapEndian.h"
+#include "helpers/Text.h"
 #include "SysMessage.h"
 #include "gui/specific/GUIXMoto.h"
 #include "Credits.h"
@@ -52,11 +54,50 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "states/StateMainMenu.h"
 #include "states/StateMessageBox.h"
 
+#include "UserConfig.h"
+#include "Renderer.h"
+#include "net/NetServer.h"
+#include "net/NetClient.h"
+#include "net/NetActions.h"
+#include "include/xm_SDL_net.h"
+
+#define MOUSE_DBCLICK_TIME 0.250f
+
+#ifdef __amigaos4__
+#include <dos/dos.h>
+#include <proto/exec.h>
+BOOL LibraryOK( STRPTR libname, ULONG version, ULONG revision )
+ {
+     struct Library *libBase;
+     BOOL ok = FALSE;
+     if( libBase = (struct Library *)IExec->OpenLibrary( libname, version ))
+     {
+     if (libBase->lib_Version > version || (libBase->lib_Version == version && libBase->lib_Revision >=revision))
+       {
+             ok =  TRUE;
+         }
+         IExec->CloseLibrary( libBase );
+     }
+     return ok;
+ }
+#endif 
+
 #if defined(WIN32)
 int SDL_main(int nNumArgs, char **ppcArgs) {
 #else
 int main(int nNumArgs, char **ppcArgs) {
 #endif
+
+#ifdef __amigaos4__
+     /* checking for MiniGL 2.0 */
+
+     if(!LibraryOK( "minigl.library", 2, 0 ))
+        {
+        printf("I need MiniGL 2.0 Beta or newer. Please upgrade!\n");
+         return RETURN_FAIL;
+        }
+#endif
+
   /* Start application */
   try {     
     /* Setup basic info */
@@ -64,7 +105,7 @@ int main(int nNumArgs, char **ppcArgs) {
   }
   catch (Exception &e) {
     if(Logger::isInitialized()) {
-      Logger::Log((std::string("Exception: ") + e.getMsg()).c_str());
+      LogError((std::string("Exception: ") + e.getMsg()).c_str());
     }    
 
     printf("fatal exception : %s\n", e.getMsg().c_str());        
@@ -72,7 +113,7 @@ int main(int nNumArgs, char **ppcArgs) {
     
 #if defined(WIN32)
     char cBuf[1024];
-    sprintf(cBuf,"Fatal exception occured: %s\n"
+    snprintf(cBuf, 1024, "Fatal exception occured: %s\n"
 	    "Consult the file xmoto.log for more information about what\n"
 	    "might has occured.\n",e.getMsg().c_str());                    
     MessageBox(NULL,cBuf,"X-Moto Error",MB_OK|MB_ICONERROR);
@@ -82,13 +123,19 @@ int main(int nNumArgs, char **ppcArgs) {
 }
 
 void GameApp::run(int nNumArgs, char** ppcArgs) {
-  run_load(nNumArgs, ppcArgs);
-  run_loop();
+  try {
+    run_load(nNumArgs, ppcArgs);
+    run_loop();
+  } catch(Exception &e) {
+    run_unload();
+    throw e;
+  }
   run_unload();
 }
 
 void GameApp::run_load(int nNumArgs, char** ppcArgs) {
   XMArguments v_xmArgs;
+  m_isODEInitialized = false;
   bool v_useGraphics = true;
 
   /* check args */
@@ -108,6 +155,12 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
     return;
   }
 
+  /* overwrite default by command line */
+  if(v_xmArgs.isOptDefaultTheme()) {
+    XMDefault::DefaultTheme = v_xmArgs.getOpt_defaultTheme_theme();
+  }
+  /**/
+
   /* init sub-systems */
   SwapEndian::Swap_Init();
   srand(time(NULL));
@@ -136,35 +189,39 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
   Logger::init(FS::getUserDir() + "/xmoto.log");
 
   /* load config file, the session */
-  XMSession::createDefaultConfig(&m_Config);
-  m_Config.loadFile();
-  XMSession::instance()->load(&m_Config); /* overload default session by userConfig */
+  XMSession::createDefaultConfig(m_userConfig);
+  m_userConfig->loadFile();
+  
+
+  XMSession::instance("file")->load(m_userConfig); /* overload default session by userConfig */
+  (* XMSession::instance("live")) = (* XMSession::instance("file")) ; /* copying the resulting session in "live" instance */
+  (* XMSession::instance("temp")) = (* XMSession::instance("file")) ; /* copying the resulting session in a needed temporary instance */
+  XMSession::setDefaultInstance("live");
+
   XMSession::instance()->load(&v_xmArgs); /* overload default session by xmargs     */
+
   Logger::setVerbose(XMSession::instance()->isVerbose()); /* apply verbose mode */
 
-#ifdef USE_GETTEXT
-    std::string v_locale = Locales::init(XMSession::instance()->language());
-#endif
-
-  Logger::Log("compiled at "__DATE__" "__TIME__);
+  LogInfo(std::string("X-Moto " + XMBuild::getVersionString(true)).c_str());
+  LogInfo("compiled at "__DATE__" "__TIME__);
   if(SwapEndian::bigendien) {
-    Logger::Log("Systeme is bigendien");
+    LogInfo("Systeme is bigendien");
   } else {
-    Logger::Log("Systeme is littleendien");
+    LogInfo("Systeme is littleendien");
   }
-  Logger::Log("User directory: %s", FS::getUserDir().c_str());
-  Logger::Log("Data directory: %s", FS::getDataDir().c_str());
-
-#ifdef USE_GETTEXT
-  Logger::Log("Locales set to '%s' (directory '%s')", v_locale.c_str(), LOCALESDIR);
-#endif
+  LogInfo("User directory: %s", FS::getUserDir().c_str());
+  LogInfo("Data directory: %s", FS::getDataDir().c_str());
 
   if(v_xmArgs.isOptListLevels() || v_xmArgs.isOptListReplays() || v_xmArgs.isOptReplayInfos()) {
     v_useGraphics = false;
   }
 
+  // init not so random numbers
+  NotSoRandom::init();
+
   _InitWin(v_useGraphics);
 
+  /* Init the window */
   if(v_useGraphics) {
     /* init drawLib */
     drawLib = DrawLib::DrawLibFromName(XMSession::instance()->drawlib());
@@ -178,14 +235,13 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
     drawLib->setNoGraphics(v_useGraphics == false);
     drawLib->setDontUseGLExtensions(XMSession::instance()->glExts() == false);
 
-    /* Init! */
     drawLib->init(XMSession::instance()->resolutionWidth(), XMSession::instance()->resolutionHeight(), XMSession::instance()->bpp(), XMSession::instance()->windowed());
     /* drawlib can change the final resolution if it fails, then, reinit session one's */
-    XMSession::instance()->setResolutionWidth(drawLib->getDispWidth());
-    XMSession::instance()->setResolutionHeight(drawLib->getDispHeight());
-    XMSession::instance()->setBpp(drawLib->getDispBPP());
-    XMSession::instance()->setWindowed(drawLib->getWindowed());
-    Logger::Log("Resolution: %ix%i (%i bpp)", XMSession::instance()->resolutionWidth(), XMSession::instance()->resolutionHeight(), XMSession::instance()->bpp());
+    XMSession::instance("temp")->setResolutionWidth(drawLib->getDispWidth());
+    XMSession::instance("temp")->setResolutionHeight(drawLib->getDispHeight());
+    XMSession::instance("temp")->setBpp(drawLib->getDispBPP());
+    XMSession::instance("temp")->setWindowed(drawLib->getWindowed());
+    LogInfo("Resolution: %ix%i (%i bpp)", XMSession::instance("temp")->resolutionWidth(), XMSession::instance("temp")->resolutionHeight(), XMSession::instance("temp")->bpp());
     /* */
     
     if(!drawLib->isNoGraphics()) {        
@@ -194,11 +250,50 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
     }
   }
 
+  /* database -- require drawlib initialized */
+  /* thus, resolution/bpp/windowed parameters cannot be stored in the db (or some minor modifications are required) */
+  xmDatabase* pDb = xmDatabase::instance("main");
+  pDb->init(DATABASE_FILE,
+	    XMSession::instance()->profile() == "" ? std::string("") : XMSession::instance()->profile(),
+	    FS::getDataDir(), FS::getUserDir(), FS::binCheckSum(),
+	    v_xmArgs.isOptNoDBDirsCheck() == false,
+	    v_useGraphics ? this : NULL);
+  if(XMSession::instance()->sqlTrace()) {
+    pDb->setTrace(XMSession::instance()->sqlTrace());
+  }
+  
+  XMSession::instance("file")->loadProfile(XMSession::instance("file")->profile(), pDb);
+  (* XMSession::instance("live")) = (* XMSession::instance("file")) ; /* copying the resulting session in "live" instance */
+  
+  /* Live instance should be altered by drawLib changes. Apply this alterations */
+  XMSession::instance()->setResolutionWidth(XMSession::instance("temp")->resolutionWidth());
+  XMSession::instance()->setResolutionHeight(XMSession::instance("temp")->resolutionHeight());
+  XMSession::instance()->setBpp(XMSession::instance("temp")->bpp());
+  XMSession::instance()->setWindowed(XMSession::instance("temp")->windowed());
+  XMSession::destroy("temp");
+  
+  XMSession::instance()->load(&v_xmArgs); /* overload default session by xmargs     */
+  LogInfo("SiteKey: %s", XMSession::instance()->sitekey().c_str());
+  XMSession::enablePropagation("file");
+
+#ifdef USE_GETTEXT
+  std::string v_locale = Locales::init(XMSession::instance()->language());
+  LogInfo("Locales set to '%s' (directory '%s')", v_locale.c_str(), LOCALESDIR);
+#endif
+
+  // allocate the statemanager instance so that if it fails, it's not in a thread (serverThread for example)
+  StateManager::instance();
+
   /* Init sound system */
   if(v_useGraphics) {
     Sound::init(XMSession::instance());
   }
 
+  // no command line need the network for the moment
+  if(v_useGraphics) {
+    initNetwork();
+  }
+
   /* Init renderer */
   if(v_useGraphics) {
     switchUglyMode(XMSession::instance()->ugly());
@@ -210,16 +305,6 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
       GameRenderer::instance()->loadDebugInfo(XMSession::instance()->gDebugFile());
   }
 
-  /* database */
-  xmDatabase* pDb = xmDatabase::instance("main");
-  pDb->init(DATABASE_FILE,
-	    XMSession::instance()->profile() == "" ? std::string("") : XMSession::instance()->profile(),
-	    FS::getDataDir(), FS::getUserDir(), FS::binCheckSum(),
-	    v_useGraphics ? this : NULL);
-  if(XMSession::instance()->sqlTrace()) {
-    pDb->setTrace(XMSession::instance()->sqlTrace());
-  }
-
   /* load theme */
   if(pDb->themes_isIndexUptodate() == false) {
     ThemeChoicer::initThemesFromDir(pDb);
@@ -229,11 +314,11 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
   } catch(Exception &e) {
     /* if the theme cannot be loaded, try to reload from files */
     /* perhaps that the xm.db comes from an other computer */
-    Logger::Log("** warning ** : Theme cannot be reload, try to update themes into the database");
+    LogWarning("Theme cannot be reload, try to update themes into the database");
     ThemeChoicer::initThemesFromDir(pDb);
     reloadTheme();
   }
-  
+
   /* load levels */
   if(pDb->levels_isIndexUptodate() == false) {
       LevelsManager::instance()->reloadLevelsFromLvl(pDb, v_useGraphics ? this : NULL);
@@ -242,39 +327,12 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
   
   /* Update replays */
   if(pDb->replays_isIndexUptodate() == false) {
-    initReplaysFromDir();
+    initReplaysFromDir(pDb);
   }
   
   /* List replays? */  
   if(v_xmArgs.isOptListReplays()) {
-    char **v_result;
-    unsigned int nrow;
-    
-    printf("\nReplay                    Level                     Player\n");
-    printf("-----------------------------------------------------------------------\n");
-    
-    v_result = pDb->readDB("SELECT a.name, a.id_profile, b.name "
-			    "FROM replays AS a LEFT OUTER JOIN levels AS b "
-			    "ON a.id_level = b.id_level;", nrow);
-    if(nrow == 0) {
-      printf("(none)\n");
-    } else {
-      std::string v_levelName;
-
-      for(unsigned int i=0; i<nrow; i++) {
-	if(pDb->getResult(v_result, 3, i, 2) == NULL) {
-	  v_levelName = GAMETEXT_UNKNOWN;
-	} else {
-	  v_levelName = pDb->getResult(v_result, 3, i, 2);
-	}
-	printf("%-25s %-25s %-25s\n",
-	       pDb->getResult(v_result, 3, i, 0),
-	       v_levelName.c_str(),
-	       pDb->getResult(v_result, 3, i, 1)
-	       );
-      }
-    }
-    pDb->read_DB_free(v_result);
+    pDb->replays_print();
     quit();
     return;
   }
@@ -326,36 +384,35 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
     return;
   }
   
-  _UpdateLoadingScreen(0, GAMETEXT_INITMENUS);
+  _UpdateLoadingScreen();
 
-  /* Load sounds */
-  try {
-    for(unsigned int i=0; i<Theme::instance()->getSoundsList().size(); i++) {
-      Sound::loadSample(Theme::instance()->getSoundsList()[i]->FilePath());
-    }
-    Sound::loadSample("Textures/Sounds/Squeek.ogg");
-  } catch(Exception &e) {
-    Logger::Log("*** Warning *** : %s\n", e.getMsg().c_str());
-    /* hum, not cool */
-  }
-    
-  Logger::Log(" %d sound%s loaded",Sound::getNumSamples(),Sound::getNumSamples()==1?"":"s");
-    
   /* Find all files in the textures dir and load them */     
   UITexture::setApp(this);
   UIWindow::setDrawLib(getDrawLib());
 
+  // init physics
+  // if(dInitODE2(0) == 0) { /* erreur */} ; // ode 0.10
+  dInitODE();
+  m_isODEInitialized = true;
+
   /* Initialize renderer */
   GameRenderer::instance()->init(drawLib);
   
   /* build handler */
-  InputHandler::instance()->init(&m_Config);
+  InputHandler::instance()->init(m_userConfig, pDb, XMSession::instance()->profile(), XMSession::instance()->enableJoysticks());
   Replay::enableCompression(XMSession::instance()->compressReplays());
   
   /* load packs */
   LevelsManager::checkPrerequires();
   LevelsManager::instance()->makePacks(XMSession::instance()->profile(), XMSession::instance()->idRoom(0), XMSession::instance()->debug(), xmDatabase::instance("main"));
-  
+
+  /* Update stats */
+  if(XMSession::instance()->profile() != "") {
+    pDb->stats_xmotoStarted(XMSession::instance()->sitekey(), XMSession::instance()->profile());
+  }
+
+  /* try to not run sql at the same time you enter in the main menu (a thread to compute packs is run (concurrency)) */
+
   /* What to do? */
   if(m_PlaySpecificLevelFile != "") {
     try {
@@ -368,7 +425,7 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
   if((m_PlaySpecificLevelId != "")) {
     /* ======= PLAY SPECIFIC LEVEL ======= */
     StateManager::instance()->pushState(new StatePreplayingGame(m_PlaySpecificLevelId, false));
-    Logger::Log("Playing as '%s'...", XMSession::instance()->profile().c_str());
+    LogInfo("Playing as '%s'...", XMSession::instance()->profile().c_str());
   }
   else if(m_PlaySpecificReplay != "") {
     /* ======= PLAY SPECIFIC REPLAY ======= */
@@ -376,70 +433,62 @@ void GameApp::run_load(int nNumArgs, char** ppcArgs) {
     }
   else {
     /* display what must be displayed */
-    StateMainMenu* pMainMenu = new StateMainMenu();
-    StateManager::instance()->pushState(pMainMenu);
-    
-    /* Do we have a player profile? */
-    if(XMSession::instance()->profile() == "") {
-      StateManager::instance()->pushState(new StateEditProfile(pMainMenu));
-    } 
-    
-    /* Should we show a notification box? (with important one-time info) */
-    if(XMSession::instance()->notifyAtInit()) {
-      StateManager::instance()->pushState(new StateMessageBox(NULL, GAMETEXT_NOTIFYATINIT, UI_MSGBOX_OK));
-      XMSession::instance()->setNotifyAtInit(false);
-    }
+    StateManager::instance()->pushState(new StateMainMenu());
   }
-  
-  if (XMSession::instance()->ugly()){
-    drawLib->clearGraphics();
-  }
-  drawFrame();
-  drawLib->flushGraphics();
-  
-  /* Update stats */
-  if(XMSession::instance()->profile() != "") {
-    pDb->stats_xmotoStarted(XMSession::instance()->profile());
-  }
-  
-  Logger::Log("UserInit ended at %.3f", GameApp::getXMTime());
+
+  LogInfo("UserInit ended at %.3f", GameApp::getXMTime());
 }
 
 void GameApp::manageEvent(SDL_Event* Event) {
-  int ch=0;
   static int nLastMouseClickX = -100,nLastMouseClickY = -100;
   static int nLastMouseClickButton = -100;
   static float fLastMouseClickTime = 0.0f;
   int nX,nY;
-  
+  std::string utf8Char;  
+
+  if(Event->type == SDL_KEYDOWN || Event->type == SDL_KEYUP) {
+    /* don't allow simple modifier key */
+    if(Event->key.keysym.sym == SDLK_RSHIFT ||
+       Event->key.keysym.sym == SDLK_LSHIFT ||
+       Event->key.keysym.sym == SDLK_RCTRL  ||
+       Event->key.keysym.sym == SDLK_LCTRL  ||
+       Event->key.keysym.sym == SDLK_RALT   ||
+       Event->key.keysym.sym == SDLK_LALT   ||
+       Event->key.keysym.sym == SDLK_RMETA  ||
+       Event->key.keysym.sym == SDLK_LMETA
+       ) {
+      return;
+    }
+  }
+
   /* What event? */
   switch(Event->type) {
-  case SDL_KEYDOWN: 
-    if((Event->key.keysym.unicode&0xff80)==0) {
-      ch = Event->key.keysym.unicode & 0x7F;
-    }
-    keyDown(Event->key.keysym.sym, Event->key.keysym.mod, ch);            
+  case SDL_KEYDOWN:
+    //printf("%i\n", Event->key.keysym.sym);
+    utf8Char = unicode2utf8(Event->key.keysym.unicode);
+    StateManager::instance()->xmKey(INPUT_DOWN, XMKey(Event->key.keysym.sym, Event->key.keysym.mod, utf8Char));
     break;
-  case SDL_KEYUP: 
-    keyUp(Event->key.keysym.sym, Event->key.keysym.mod);            
+  case SDL_KEYUP:
+    utf8Char = unicode2utf8(Event->key.keysym.unicode);
+    StateManager::instance()->xmKey(INPUT_UP, XMKey(Event->key.keysym.sym, Event->key.keysym.mod, utf8Char));
     break;
   case SDL_QUIT:  
     /* Force quit */
     quit();
     break;
-  case SDL_MOUSEBUTTONDOWN:
-    /* Pass ordinary click */
-    mouseDown(Event->button.button);
-    
+  case SDL_MOUSEBUTTONDOWN:    
     /* Is this a double click? */
     getMousePos(&nX,&nY);
     if(nX == nLastMouseClickX &&
        nY == nLastMouseClickY &&
        nLastMouseClickButton == Event->button.button &&
-       (getXMTime() - fLastMouseClickTime) < 0.250f) {                
+       (getXMTime() - fLastMouseClickTime) < MOUSE_DBCLICK_TIME) {
       
       /* Pass double click */
-      mouseDoubleClick(Event->button.button);                
+      StateManager::instance()->xmKey(INPUT_DOWN, XMKey(Event->button.button, 2));
+    } else {
+      /* Pass ordinary click */
+      StateManager::instance()->xmKey(INPUT_DOWN, XMKey(Event->button.button));
     }
     fLastMouseClickTime = getXMTime();
     nLastMouseClickX = nX;
@@ -448,37 +497,50 @@ void GameApp::manageEvent(SDL_Event* Event) {
     
     break;
   case SDL_MOUSEBUTTONUP:
-    mouseUp(Event->button.button);
+    StateManager::instance()->xmKey(INPUT_UP, XMKey(Event->button.button));
     break;
-    
+
+  case SDL_JOYAXISMOTION:
+  StateManager::instance()->xmKey(InputHandler::instance()->joystickAxisSens(Event->jaxis.value),
+				  XMKey(InputHandler::instance()->getJoyId(Event->jbutton.which),
+					Event->jaxis.axis, Event->jaxis.value));
+  break;
+  
+  case SDL_JOYBUTTONDOWN:
+  StateManager::instance()->xmKey(INPUT_DOWN, XMKey(InputHandler::instance()->getJoyId(Event->jbutton.which), Event->jbutton.button));
+  break;
+  
+  case SDL_JOYBUTTONUP:
+  StateManager::instance()->xmKey(INPUT_UP, XMKey(InputHandler::instance()->getJoyId(Event->jbutton.which), Event->jbutton.button));
+  break;
+  
   case SDL_ACTIVEEVENT:
     
     if((Event->active.state & SDL_APPMOUSEFOCUS) != 0) { // mouse focus
       if(m_hasKeyboardFocus == false) {
-	changeFocus(Event->active.gain == 1);
+	StateManager::instance()->changeFocus(Event->active.gain == 1);
       }
       m_hasMouseFocus = (Event->active.gain == 1);
     }
     
     if((Event->active.state & SDL_APPINPUTFOCUS) != 0) { // keyboard focus
       if(m_hasMouseFocus == false) {
-	changeFocus(Event->active.gain == 1);
+	StateManager::instance()->changeFocus(Event->active.gain == 1);
       }
       m_hasKeyboardFocus = (Event->active.gain == 1);
     }
     
     if((Event->active.state & SDL_APPACTIVE) != 0) {
-      changeVisibility(Event->active.gain == 1);
+      StateManager::instance()->changeVisibility(Event->active.gain == 1);
       m_isIconified = (Event->active.gain == 0);
     }
-    
   }
 }
 
 void GameApp::run_loop() {
   SDL_Event Event;
 
-  while(!m_bQuit) {
+  while(m_bQuit == false) {
     /* Handle SDL events */            
     SDL_PumpEvents();
 
@@ -506,6 +568,10 @@ void GameApp::run_loop() {
 }
 
 void GameApp::run_unload() {
+  if(Logger::isInitialized()) {
+    LogInfo("UserUnload started at %.3f", GameApp::getXMTime());
+  }
+
   if(m_pWebHighscores != NULL) {
     delete m_pWebHighscores;
   }        
@@ -517,39 +583,58 @@ void GameApp::run_unload() {
   if(GameRenderer::instance() != NULL) {
     GameRenderer::instance()->unprepareForNewLevel(); /* just to be sure, shutdown can happen quite hard */
     GameRenderer::instance()->shutdown();
-    InputHandler::instance()->uninit();
+    InputHandler::instance()->uninit(); // uinit the input, but you can still save the config
   }
 
+  uninitNetwork();
+
   StateManager::destroy();
   
   if(Sound::isInitialized()) {
     Sound::uninit();
   }
 
+  if(m_isODEInitialized ==  true){
+    dCloseODE(); // uninit ODE
+  }
+
   GameRenderer::destroy();
   SysMessage::destroy();  
 
+  if(Logger::isInitialized()) {
+    LogDebug("UserUnload saveConfig at %.3f", GameApp::getXMTime());
+  }
   if(drawLib != NULL) { /* save config only if drawLib was initialized */
-    XMSession::instance()->save(&m_Config);
-    InputHandler::instance()->saveConfig(&m_Config);
-    m_Config.saveFile();
+    XMSession::instance("file")->save(m_userConfig, xmDatabase::instance("main"));
+    InputHandler::instance()->saveConfig(m_userConfig, xmDatabase::instance("main"), XMSession::instance("file")->profile());
+    m_userConfig->saveFile();
+  }
+
+  if(Logger::isInitialized()) {
+    LogDebug("UserUnload saveConfig ended at %.3f", GameApp::getXMTime());
   }
 
   if(drawLib != NULL) {
     drawLib->unInit();
   }
+
+  InputHandler::destroy();
+  LevelsManager::destroy();
+  Theme::destroy();
+  XMSession::destroy("live");
+  XMSession::destroy("file");
+
+  if(Logger::isInitialized()) {
+    LogInfo("UserUnload ended at %.3f", GameApp::getXMTime());
+  }
+
+  /* Shutdown SDL */
+  SDL_Quit();
     
   if(Logger::isInitialized()) {
     Logger::uninit();
   }
 
-  InputHandler::destroy();
-  LevelsManager::destroy();
-  Theme::destroy();
-  XMSession::destroy();
-
-  /* Shutdown SDL */
-  SDL_Quit();
 }
 
 
@@ -600,10 +685,9 @@ void GameApp::_Wait()
   /*===========================================================================
   Update loading screen
   ===========================================================================*/
-  void GameApp::_UpdateLoadingScreen(float fDone, const std::string &NextTask) {
+  void GameApp::_UpdateLoadingScreen(const std::string &NextTask) {
     FontManager* v_fm;
     FontGlyph* v_fg;
-    int v_border = 3;
     int v_fh;
 
     getDrawLib()->clearGraphics();
@@ -616,24 +700,47 @@ void GameApp::_Wait()
 		      getDrawLib()->getDispWidth()/2 - 256,
 		      getDrawLib()->getDispHeight()/2 - 30,
 		      MAKE_COLOR(255,255,255, 255));
-
-    getDrawLib()->drawBox(Vector2f(getDrawLib()->getDispWidth()/2 - 256,
-				   getDrawLib()->getDispHeight()/2 - 30),              
-			  Vector2f(getDrawLib()->getDispWidth()/2 - 256 + (512.0f*fDone),
-				   getDrawLib()->getDispHeight()/2 - 30 + v_border),
-			  0,MAKE_COLOR(255,255,255,255));
-
-    getDrawLib()->drawBox(Vector2f(getDrawLib()->getDispWidth()/2 - 256,
-				   getDrawLib()->getDispHeight()/2 -30 + v_fh),              
-			  Vector2f(getDrawLib()->getDispWidth()/2 - 256 + (512.0f*fDone),
-				   getDrawLib()->getDispHeight()/2 - 30 + v_fh + v_border),
-			  0,MAKE_COLOR(255,255,255,255));
     
-    v_fm = getDrawLib()->getFontSmall();
-    v_fg = v_fm->getGlyph(NextTask);
-    v_fm->printString(v_fg,
-		      getDrawLib()->getDispWidth()/2 - 256,
-		      getDrawLib()->getDispHeight()/2 -30 + v_fh + 2,
-		      MAKE_COLOR(255,255,255,255));      
+    if(NextTask != "") {
+      v_fm = getDrawLib()->getFontSmall();
+      v_fg = v_fm->getGlyph(NextTask);
+      v_fm->printString(v_fg,
+			getDrawLib()->getDispWidth()/2 - 256,
+			getDrawLib()->getDispHeight()/2 -30 + v_fh + 2,
+			MAKE_COLOR(255,255,255,255));      
+    }
     getDrawLib()->flushGraphics();
   }
+
+
+void GameApp::initNetwork() {
+  if(SDLNet_Init()==-1) {
+    throw Exception(SDLNet_GetError());
+  }
+
+  // start server
+  if(XMSession::instance()->serverStartAtStartup()) {
+    NetServer::instance()->start();
+  }
+
+}
+
+void GameApp::uninitNetwork() {
+  // stop the client
+  if(NetClient::instance()->isConnected()) {
+    NetClient::instance()->disconnect();
+  }
+  NetClient::destroy();
+
+  if(NetServer::instance()->isStarted()) {
+    NetServer::instance()->stop();
+  }
+
+  NetServer::destroy();
+
+  if(Logger::isInitialized()) {
+    NetAction::logStats();
+  }
+
+  SDLNet_Quit();
+}
