@@ -27,11 +27,12 @@
 #include "GameEvents.h"
 #include "DBuffer.h"
 #include "Game.h"
-  
+#include "xmscene/PhysicsSettings.h"
+
   /*===========================================================================
     Decoding of event stream
     ===========================================================================*/
-  void MotoGame::unserializeGameEvents(DBuffer *Buffer, std::vector<RecordedGameEvent *> *v_ReplayEvents, bool bDisplayInformation) {
+  void Scene::unserializeGameEvents(DBuffer *Buffer, std::vector<RecordedGameEvent *> *v_ReplayEvents, bool bDisplayInformation) {
     RecordedGameEvent *p;    
 
     try {
@@ -39,11 +40,11 @@
       while((*Buffer).numRemainingBytes() > 0) {
       p = new RecordedGameEvent;
       p->bPassed = false;
-      p->Event   = MotoGameEvent::getUnserialized(*Buffer, bDisplayInformation);
+      p->Event   = SceneEvent::getUnserialized(*Buffer, bDisplayInformation);
       v_ReplayEvents->push_back(p);
       }
     } catch(Exception &e) {
-      Logger::Log("** Warning ** : unable to unserialize game events !");
+      LogWarning("unable to unserialize game events !");
       throw e;
     }
   }
@@ -51,14 +52,14 @@
   /*===========================================================================
     Encoding of event buffer 
     ===========================================================================*/
-  void MotoGame::_SerializeGameEventQueue(DBuffer* Buffer, MotoGameEvent *pEvent) {
+  void Scene::_SerializeGameEventQueue(DBuffer* Buffer, SceneEvent *pEvent) {
     pEvent->serialize(*Buffer);
   }
 
   /*===========================================================================
     Serializer
     ===========================================================================*/
-  void MotoGame::getSerializedBikeState(BikeState *i_bikeState, int i_time, SerializedBikeState *pState) {
+void Scene::getSerializedBikeState(BikeState *i_bikeState, int i_time, SerializedBikeState *pState, PhysicsSettings* i_physicsSettings) {
     /* Get. */
     pState->fGameTime = GameApp::timeToFloat(i_time);
     
@@ -102,7 +103,7 @@
     pState->fMaxYDiff = fMaxY;
     
     /* Update engine stuff */    
-    int n = (int)(((i_bikeState->fBikeEngineRPM-ENGINE_MIN_RPM)/ENGINE_MAX_RPM)*255.0f);
+    int n = (int)(((i_bikeState->fBikeEngineRPM-i_physicsSettings->EngineRpmMin())/i_physicsSettings->EngineRpmMax())*255.0f);
     if(n<0) n=0;
     if(n>255) n=255;
     pState->cBikeEngineRPM = n;
