@@ -40,7 +40,8 @@ Central place to keep many in-game text strings
 #define GAMETEXT_44KHZ                _("44 kHz")
 #define GAMETEXT_8BIT                 _("8-bit")
 #define GAMETEXT_ABORT                _("Quit level")
-#define GAMETEXT_ADDTOFAVORITE       _("Add to favorite")
+#define GAMETEXT_ACCOUNT_PASSWORD     _("Web account password for profile '%s':")
+#define GAMETEXT_ADDTOFAVORITE        _("Add to favorite")
 #define GAMETEXT_ADDTOBLACKLIST       _("Add to blacklist")
 #define GAMETEXT_ACTION               _("Action")
 #define GAMETEXT_ALL                  _("All (on this computer)")
@@ -54,6 +55,9 @@ Central place to keep many in-game text strings
 #define GAMETEXT_AUTOMATIC            _("Automatic")
 #define GAMETEXT_AUTOSAVEREPLAYS      _("Automatic saving of replays")
 #define GAMETEXT_AUTOZOOM             _("Auto zoom")
+#define GAMETEXT_DB                   _("Database")
+#define GAMETEXT_CAMERAACTIVEZOOM     _("Camera Active Zoom")
+#define GAMETEXT_BEATINGMODE          _("Beating mode")
 #define GAMETEXT_BESTTIMES            _("Best Times")
 #define GAMETEXT_BESTPLAYER           _("Best player")
 #define GAMETEXT_BRAKE                _("Brake")
@@ -76,14 +80,40 @@ Central place to keep many in-game text strings
 #define GAMETEXT_CONFIGUREJOYSTICK    _("Configure Joystick...")
 #define GAMETEXT_CONTROLS             _("Controls")
 
+#define GAMETEXT_KEY_LEFTCONTROL       	  _("left control")
+#define GAMETEXT_KEY_RIGHTCONTROL      	  _("right control")
+#define GAMETEXT_KEY_LEFTSHIFT         	  _("left shift")
+#define GAMETEXT_KEY_RIGHTSHIFT        	  _("right shift")
+#define GAMETEXT_KEY_LEFTALT           	  _("left alt")
+#define GAMETEXT_KEY_RIGHTALT          	  _("right alt")
+#define GAMETEXT_KEY_LEFTMETA          	  _("left meta")
+#define GAMETEXT_KEY_RIGHTMETA         	  _("right meta")
+#define GAMETEXT_MOUSE_BUTTON          	  _("button %i")
+#define GAMETEXT_MOUSE_LEFTBUTTON      	  _("left button")
+#define GAMETEXT_MOUSE_MIDDLEBUTTON    	  _("middle button")
+#define GAMETEXT_MOUSE_RIGHTBUTTON     	  _("right button")
+#define GAMETEXT_MOUSE_WHEELUPBUTTON   	  _("wheel up")
+#define GAMETEXT_MOUSE_WHEELDOWNBUTTON 	  _("wheel down")
+#define GAMETEXT_NOJOYSTICKFOUND          _("No joystick found")
+#define GAMETEXT_JOYSTICKSFOUND(A)        ngettext("%i joystick found", "%i joysticks found", A)
+#define GAMETEXT_JOYSTICK_AXIS         	  _("axis %i")
+#define GAMETEXT_JOYSTICK_DIRECTION_LEFT  _("left")
+#define GAMETEXT_JOYSTICK_DIRECTION_RIGHT _("right")
+#define GAMETEXT_JOYSTICK_DIRECTION_UP    _("up")
+#define GAMETEXT_JOYSTICK_DIRECTION_DOWN  _("down")
+#define GAMETEXT_JOYSTICK_DIRECTION_NONE  _("none")
+#define GAMETEXT_JOYSTICK_BUTTON          _("button %i")
+#define GAMETEXT_SCRIPTACTION             _("Script action")
+
 #define GAMETEXT_CREDITS                                                                     \
   std::string(_("Lead Programming")) +                                                       \
     ":Rasmus Neckelmann;"                                                                    \
     ":Nicolas Adenis-Lamarre;"                                                               \
     ":Emmanuel Gorse;"                                                                       \
-    ":Kees Jongenburger;"                                                                    \
     ":;"                                                                                     \
     + std::string(_("Additional Programming")) +                                             \
+    ":Richard Franks;"                                                                       \
+    ":Kees Jongenburger;"                                                                    \
     ":Eric Piel;"                                                                            \
     ":Christoph Sommer;"                                                                     \
     ":Frederic Huard;"                                                                       \
@@ -119,6 +149,7 @@ Central place to keep many in-game text strings
     ":Lucas Hermann Negri (" + std::string(_("Brazilian Portuguese")) + ");"                 \
     ":Jānis Rūcis (" + std::string(_("Latvian")) + ");"                                      \
     ":Kristian Jagd (" + std::string(_("Danish")) + ");"                                     \
+    ":Tadas Masiulionis (" + std::string(_("Lithuanian")) + ");"                                     \
     ":;"                                                                                     \
     + std::string(_("Main testers")) +                                                       \
     ":Jose Alberto Coelho;"                                                                  \
@@ -141,7 +172,13 @@ Central place to keep many in-game text strings
 
 #define GAMETEXT_CREDITSBUTTON        _("Credits")
 #define GAMETEXT_PLAYER               _("Player")
+#define GAMETEXT_MUSICONALLLEVELS      _("Play music on all levels")
 #define GAMETEXT_DATE                 _("Date")
+#define GAMETEXT_DIFFICULTY           _("Difficulty")
+#define GAMETEXT_DBSYNCHRONIZE_EXPLANATION _("X-Moto uses a database on your computer to save all your scores and statistics. You can send these data using your web account on the X-Moto server. It is useful to save your data or if you play X-Moto from different places.")
+#define GAMETEXT_DBSYNCHRONIZE_WARNING _("If you manually copied the X-Moto user files (.xmoto) on some other computers, you should not use this functionality otherwise your statistics will be duplicated/not synchronized. If you did that, you should remove the copies you did.")
+#define GAMETEXT_DBSYNCHRONIZE _("Synchronise")
+#define GAMETEXT_DBSYNCHRONIZE_ONQUIT _("Synchronize when X-Moto ends")
 #define GAMETEXT_DB_UPGRADING               _("Upgrading database")
 #define GAMETEXT_DB_UPGRADING_STATS_PROFILE _("Upgrading database stats for the profile")
 #define GAMETEXT_DB_UPGRADING_PROFILE _("Upgrading database profile")
@@ -156,6 +193,7 @@ Central place to keep many in-game text strings
 #define GAMETEXT_DESCRIPTION          _("Description")
 #define GAMETEXT_DISPLAYGHOSTTIMEDIFF _("Display Ghost time difference")
 #define GAMETEXT_DISPLAYGHOSTINFO     _("Display Ghost Information")
+#define GAMETEXT_DISPLAYBIKERARROW    _("Display bike arrow indication")
 #define GAMETEXT_DIRECTCONN           _("Direct Connection")
 #define GAMETEXT_DLGHOST              _("Downloading the ghost...")
 #define GAMETEXT_DLHIGHSCORE          _("Downloading the highscore...")
@@ -175,7 +213,8 @@ Central place to keep many in-game text strings
 #define GAMETEXT_ENABLEENGINESOUND    _("Enable Engine Sound")
 #define GAMETEXT_ENABLEGHOST          _("Enable Ghost")
 #define GAMETEXT_ENABLEINGAMEWORLDRECORD _("Show World Record in-game")
-#define GAMETEXT_ENABLEMUSIC          _("Enable Music")
+#define GAMETEXT_ENABLEMENUMUSIC          _("Enable menu music")
+#define GAMETEXT_ENABLEGAMEMUSIC          _("Enable game music")
 #define GAMETEXT_ENABLEWEBHIGHSCORES  _("Enable WWW Access")
 #define GAMETEXT_ENTERPLAYERNAME      _("Enter player name")
 #define GAMETEXT_ENTERREPLAYNAME      _("Enter name of replay")
@@ -209,6 +248,7 @@ Central place to keep many in-game text strings
 #define GAMETEXT_GHOST_STRATEGY_MYBEST  _("The best of my replays (on this computer)")
 #define GAMETEXT_GHOST_STRATEGY_THEBEST _("The best of the replays (on this computer)")
 #define GAMETEXT_GHOSTOF              _("Ghost of %s")
+#define GAMETEXT_WEBGHOSTOF           _("Web ghost of %s")
 #define GAMETEXT_GHOSTTAB             _("Ghost")
 #define GAMETEXT_HELP                 _("Help")
 
@@ -223,11 +263,8 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 "  Esc - " + std::string(_("Pause")) + "\n" \
 "  Return - " + std::string(_("Restart the level")) + "\n" \
 "  PageDown/PageUp - " + std::string(_("Previous/Next level")) + "\n" \
-"  F3 - " + std::string(_("Add/remove to favorite levels")) + "\n"\
-"  ctrl+B - " + std::string(_("Add/remove to blacklisted levels")) + "\n"\
 "  ctrl+M - " + std::string(_("change mirror mode")) + "\n"\
 "  F8 - " + std::string(_("Enable/disable web")) + "\n" \
-"  F9 - " + std::string(_("Enable/disable ugly mode")) + "\n" \
 "  F5 - " + std::string(_("Reload levels, themes and replays from directories")) + "\n" \
 "  F12 - " + std::string(_("Take a screenshot")) + "\n" \
 "\n" \
@@ -236,14 +273,12 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 
 #define GAMETEXT_HIDEGHOSTS           _("Hide ghosts")
 #define GAMETEXT_HIGH                 _("High")
+#define GAMETEXT_HIGHSCOREOFTHEROOM   _("Highscore of the room")
 #define GAMETEXT_LOAD_LEVEL_HOOK      _("Levels are being added into the database. Please wait.")
 #define GAMETEXT_INETCONF             _("Internet Configuration")
-#define GAMETEXT_INITINPUT            _("Initializing input system...")
-#define GAMETEXT_INITMENUS            _("Initializing menus...")
-#define GAMETEXT_INITRENDERER         _("Initializing renderer...")
-#define GAMETEXT_INITTEXT             _("Initializing text renderer...")
 #define GAMETEXT_INITZOOM             _("View Level Initially")
 #define GAMETEXT_JOYSTICK             _("Joystick")
+#define GAMETEXT_ENABLEJOYSTICKS      _("Enable joysticks")
 #define GAMETEXT_JUSTDEAD             _("Oops!")
 #define GAMETEXT_JUSTDEAD_RESTART     _("Press ENTER to try again")
 #define GAMETEXT_JUSTDEAD_DISPLAYMENU _("Press ESC to display the menu")
@@ -273,6 +308,9 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define GAMETEXT_LOADINGTEXTURES      _("Loading textures...")
 #define GAMETEXT_LOGIN                _("Login")
 #define GAMETEXT_LOW                  _("Low")
+#define GAMETEXT_MAIN                 _("Main")
+#define GAMETEXT_MOUSE                _("Mouse")
+#define GAMETEXT_THEME                _("Theme")
 #define GAMETEXT_MEDIUM               _("Medium")
 #define GAMETEXT_MENUGFX              _("Menu Graphics")
 #define CONTEXTHELP_MINISTATTIMEFORPACKLEVEL _("Total time you scored / total highscore time for levels you finished")
@@ -334,8 +372,10 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define GAMETEXT_PLAYNEXT             _("Play Next Level")
 #define GAMETEXT_PORT                 _("Port")
 #define GAMETEXT_PRESSANYKEYTO        _("Press key you want to '%s' or ESC to cancel...")
+#define GAMETEXT_PRESSCTRLCTOCHAT     _("Press %s to chat")
 #define GAMETEXT_PROXYSERVER          _("Proxy Server")
 #define GAMETEXT_PROXYCONFIG          _("Configure Proxy...")
+#define GAMETEXT_QUALITY              _("Quality")
 #define GAMETEXT_QUICKSTART           _("Quick start")
 #define GAMETEXT_QUIT                 _("Quit Game")
 #define GAMETEXT_QUITMESSAGE          _("Do you really want to quit?")
@@ -350,6 +390,7 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define GAMETEXT_REPLAYNOTFOUND       _("The replay can't be played!")
 #define GAMETEXT_REPLAYOF             _("Replay of %s")
 #define GAMETEXT_REPLAYS              _("Replays")
+#define GAMETEXT_RESETTODEFAULTS      _("Are you sure you want to reset config to defaults ?")
 #define GAMETEXT_RESTART              _("Restart This Level")
 #define GAMETEXT_RESUME               _("Resume Playing")
 #define GAMETEXT_ROOM                 _("Room")
@@ -366,6 +407,13 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define GAMETEXT_STATISTICS           _("Statistics")
 #define GAMETEXT_STATS                _("STATS")
 #define GAMETEXT_STEREO               _("Stereo")
+#define GAMETEXT_SYNC_DOWN            _("Synchronisation down")
+#define GAMETEXT_SYNC_UP              _("Synchronisation up")
+#define GAMETEXT_SYNC_DONE            _("Synchronisation done successfully")
+#define GAMETEXT_SWITCHKEY            _("Key '%s' has been switched to '%s'")
+#define GAMETEXT_SWITCHUGLYMODE       _("Enable/disable ugly mode")
+#define GAMETEXT_SWITCHFAVORITE       _("Add/remove to favorite levels")
+#define GAMETEXT_SWITCHBLACKLIST      _("Add/remove to blacklisted levels")
 #define GAMETEXT_THEMES               _("Theme")
 #define GAMETEXT_THEMEHOSTED          _("Available")
 #define GAMETEXT_THEMENOTHOSTED       _("To download")
@@ -374,6 +422,7 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define GAMETEXT_TIME                 _("Time")
 #define GAMETEXT_TRYAGAIN             _("Try This Level Again")
 #define GAMETEXT_TUTORIAL             _("Tutorial")
+#define GAMETEXT_UNDEFINED            _("Undefined")
 #define GAMETEXT_UNKNOWN              _("Unknown")
 #define GAMETEXT_UNPACKED_LEVELS_PACK _("Unpacked levels")
 #define GAMETEXT_UNUPDATABLETHEMEONWEB _("Can't update this theme !\nThe theme is not available on the web\nor your theme list is not up to date")
@@ -385,12 +434,13 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define GAMETEXT_UPDATINGLEVELS       _("Updating level lists...")
 #define GAMETEXT_UPLOAD_HIGHSCORE     _("Upload")
 #define GAMETEXT_UPLOAD_ALL_HIGHSCORES _("Upload all highscores")
-#define GAMETEXT_UPLOAD_HIGHSCORE_ERROR _("An unexpected error has occurred. There may be some problems with the web site.")
+#define GAMETEXT_UPLOAD_ERROR _("An unexpected error has occurred. There may be some problems with the web site.")
 #define GAMETEXT_UPLOAD_HIGHSCORE_WEB_WARNING_BEFORE _("Oh no !")
 #define GAMETEXT_UPLOADING_HIGHSCORE  _("Uploading the highscore...")
 #define GAMETEXT_USECRAPPYINFORMATION _("Use crappy information")
 #define GAMETEXT_USEENVVARS           _("Use Environment Vars")
 #define GAMETEXT_USEPROFILE           _("Use Profile")
+#define GAMETEXT_ALLOWWEBFORMS        _("Allow web forms")
 #define GAMETEXT_USINGHTTPPROXY       _("Using HTTP Proxy")
 #define GAMETEXT_USINGSOCKS4PROXY     _("Using SOCKS4 Proxy")
 #define GAMETEXT_USINGSOCKS5PROXY     _("Using SOCKS5 Proxy")
@@ -400,7 +450,6 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define GAMETEXT_WANTTOUPDATELEVEL    _("Do you want to update level \"%s\"?")
 #define GAMETEXT_WARNING              _("Warning")
 #define GAMETEXT_WORLDRECORDNA        "--:--:--"
-#define GAMETEXT_WWWMAINTAB           _("Main")
 #define GAMETEXT_WWWTAB               _("WWW")
 #define GAMETEXT_WWWROOMSTAB_REFERENCE _("Reference room")
 #define GAMETEXT_WWWROOMSTAB_OTHER     _("Room %i")
@@ -420,10 +469,39 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define GAMETEXT_XSECONDS             _("%d seconds")
 #define GAMETEXT_YES                  _("Yes")
 #define GAMETEXT_YES_FOR_ALL          _("Yes to all")
+#define GAMETEXT_YOURBESTTIME  	      _("Your best time")
 #define GAMETEXT_ZOOMIN  	      _("Zoom in")
 #define GAMETEXT_ZOOMINIT 	      _("Reinitialize zoom")
 #define GAMETEXT_ZOOMOUT 	      _("Zoom out")
 
+#define GAMETEXT_SERVER _("Server")
+#define GAMETEXT_PORT _("Port")
+#define GAMETEXT_STARTATSTARTUP _("Launch at startup")
+#define GAMETEXT_SERVERSTART _("Start the server")
+#define GAMETEXT_SERVERSTOP _("Stop the server")
+#define GAMETEXT_SERVERSTATUSON _("Server is currently started")
+#define GAMETEXT_SERVERSTATUSOFF _("Server is currently stopped")
+#define GAMETEXT_DEFAULT_PORT _("Default port")
+#define GAMETEXT_CUSTOM_PORT _("Custom port")
+#define GAMETEXT_NETWORK _("Network")
+#define GAMETEXT_CLIENTSTATUSON _("You're currently connected")
+#define GAMETEXT_CLIENTSTATUSOFF _("You're currently disconnected")
+#define GAMETEXT_CLIENTCONNECT _("Connect")
+#define GAMETEXT_CLIENTDISCONNECT _("Disconnect")
+#define GAMETEXT_UNABLETOCONNECTONTHESERVER _("Unable to connect on the server")
+#define GAMETEXT_CLIENTNETWORKERROR _("Network error: you're disconnected")
+#define GAMETEXT_CONNECTED_PLAYERS _("Connected players:")
+#define GAMETEXT_CHATMESSAGE _("Chat message")
+#define GAMETEXT_CLIENTCONNECTSERVER _("%s joined the game")
+#define GAMETEXT_CLIENTDISCONNECTSERVER _("%s left the game")
+#define GAMETEXT_CLIENTPLAYING _("%s is playing level %s")
+#define UNTRANSLATED_GAMETEXT_TOO_MUCH_CLIENTS "Sorry, there are already too much clients connected"
+#define UNTRANSLATED_GAMETEXT_SERVER_PROTOCOL_VERSION_INCOMPATIBLE "Server protocol version incompatible"
+
+			  // to get them into the .po
+#define GAMETEXT_TOO_MUCH_CLIENTS _(UNTRANSLATED_GAMETEXT_TOO_MUCH_CLIENTS)
+#define GAMETEXT_SERVER_PROTOCOL_VERSION_INCOMPATIBLE _(UNTRANSLATED_GAMETEXT_SERVER_PROTOCOL_VERSION_INCOMPATIBLE)
+
 /* Context help strings */
 #define CONTEXTHELP_UPDATEHIGHSCORES _("Download the latest X-Moto world records and check for new levels")
 #define CONTEXTHELP_PROXYCONFIG _("Configure how you are connected to the Internet")
@@ -459,6 +537,7 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define CONTEXTHELP_DOWNLOAD_BEST_TIMES _("Automatically download best times off the net when the game starts")
 #define CONTEXTHELP_INGAME_WORLD_RECORD _("Show the World Record for a given level when playing")
 #define CONTEXTHELP_USECRAPPYINFORMATION _("Use crappy information from the website to update the crappy pack")
+#define CONTEXTHELP_ALLOWWEBFORMS _("Allow X-Moto to ask you your mind about levels")
 #define CONTEXTHELP_HIGHCOLOR _("Enable high-color graphics")
 #define CONTEXTHELP_TRUECOLOR _("Enable true-color graphics")
 #define CONTEXTHELP_RESOLUTION _("Select graphics resolution")
@@ -509,6 +588,7 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define CONTEXTHELP_GHOST_STRATEGIES _("Choose which ghost to display")
 #define CONTEXTHELP_MOTIONBLURGHOST _("Make motion blur effect for the ghost (if supported by your graphics card)")
 #define CONTEXTHELP_DISPLAY_GHOST_INFO _("When starting a level with a ghost, display who the ghost is of")
+#define CONTEXTHELP_DISPLAY_BIKER_ARROW _("Display an arrow to display where are located the bikes on the map")
 #define CONTEXTHELP_DISPLAY_GHOST_TIMEDIFF _("Display the time difference between the ghost and you")
 #define CONTEXTHELP_AUTOSAVEREPLAYS _("If you make a highscore it will automatically be saved as a replay")
 #define CONTEXTHELP_VIEWTHEHIGHSCORE _("View the replay of the room's highscore")
@@ -518,15 +598,17 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 
 #define CONTEXTHELP_STATS _("Show various statistics about X-Moto")
 #define CONTEXTHELP_UPDATESTATS _("Update statistics (also happens each time X-Moto is started)")
-#define CONTEXTHELP_MUSIC _("Enables background music in the main menu")
+#define CONTEXTHELP_MENUMUSIC _("Enables background music in the main menu")
+#define CONTEXTHELP_GAMEMUSIC _("Enables background music while playing")
 
 #define CONTEXTHELP_THEMES _("Choose the X-Moto graphics theme")
 #define CONTEXTHELP_UPDATETHEMESLIST _("Check for new themes on the web")
 #define CONTEXTHELP_GETSELECTEDTHEME _("Download or update the selected theme")
 #define CONTEXTHELP_UPLOAD_HIGHSCORE _("Upload the replay on the website of highscores")
 #define CONTEXTHELP_UPLOAD_HIGHSCORE_ALL _("Upload your best replays to your room (useful when it is just created)")
-#define CONTEXTHELP_WWW_MAIN_TAB _("Configure the main www options")
-#define CONTEXTHELP_WWW_ROOMS_TAB _("Choose where to upload/download replays, highscore lists (for more information, check the website)")
+#define CONTEXTHELP_WWW_MAIN_OPTIONS _("Configure the main www options")
+#define CONTEXTHELP_WWW_OPTIONS _("Configure the www options")
+#define CONTEXTHELP_WWW_ROOMS_OPTIONS _("Choose where to upload/download replays, highscore lists (for more information, check the website)")
 #define CONTEXTHELP_WWW_ROOMS_LIST _("Choose your room")
 #define CONTEXTHELP_UPDATEROOMSLIST _("Get the rooms list from the web")
 #define CONTEXTHELP_ROOM_LOGIN _("Upload of highscore requires to log in (Except for WR)")
@@ -534,6 +616,7 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define CONTEXTHELP_ENGINE_COUNTER _("Show the speedometer when playing")
 #define CONTEXTHELP_DEATHANIM _("Enable animation of bike falling apart when dead")
 #define CONTEXTHELP_INITZOOM _("Automatically scroll over the level before starting playing it")
+#define CONTEXTHELP_CAMERAACTIVEZOOM _("Run a dynamic zoom while playing")
 #define CONTEXTHELP_CREDITS _("View the X-Moto credits")
 #define CONTEXTHELP_REPLAYCOL _("Name of replay")
 #define CONTEXTHELP_REPLAYLEVELCOL _("Level played in replay")
@@ -565,6 +648,50 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define CONTEXTHELP_REPLAYS_CLEAN _("Delete replays that X-Moto estimates to be not interesting to keep")
 #define CONTEXTHELP_CHILDREN_COMPLIANT _("Remove levels not suitable for children")
 #define CONTEXTHELP_ROOM_ENABLE _("Enable this room")
+#define CONTEXTHELP_WWW_PASSWORD _("Password associated to your profile for web access")
+#define CONTEXTHELP_GENERAL_MAIN_OPTIONS _("Main options")
+#define CONTEXTHELP_THEME_OPTIONS        _("Configure the X-Moto theme")
+#define CONTEXTHELP_GHOSTS_OPTIONS       _("Configure ghosts displayed while playing the game")
+#define CONTEXTHELP_DB_OPTIONS           _("Configure options linked to your X-Moto database")
+#define CONTEXTHELP_DBSYNCHRONIZE _("Synchronise your database on the web")
+#define CONTEXTHELP_DBSYNCHRONIZE_ONQUIT _("Synchronise your database when your quit X-Moto")
+#define CONTEXTHELP_ENABLEJOYSTICKS _("Enable joysticks events")
+#define CONTEXTHELP_BEATINGMODE _("Preselect the retry button when dying or finishing a level")
+
+#define GAMETEXT_GIVEYOURMING _("Give your mind about this level")
+#define GAMETEXT_NOIDEA       _("No idea")
+#define GAMETEXT_BEGINNER     _("Beginner")
+#define GAMETEXT_EXPERT       _("Expert")
+#define GAMETEXT_MASTER       _("Master")
+#define GAMETEXT_GOD          _("God")
+#define GAMETEXT_NOTNICEATALL _("Not nice at all")
+#define GAMETEXT_NOTNICE      _("Not nice")
+#define GAMETEXT_NICE         _("Nice")
+#define GAMETEXT_REALLYNICE   _("Really nice")
+#define GAMETEXT_SENDTHISREPORT _("Send this report")
+#define GAMETEXT_SKIPTHISREPORT _("Skip this report")
+
+#define CONTEXTHELP_NOIDEA    	  _("You don't want to vote")
+#define CONTEXTHELP_DBEGINNER 	  _("Really easy")
+#define CONTEXTHELP_DMEDIUM   	  _("No technical difficulty")
+#define CONTEXTHELP_DEXPERT   	  _("At least one difficulty")
+#define CONTEXTHELP_DMASTER   	  _("Several difficulties")
+#define CONTEXTHELP_DGOD      	  _("Impossible")
+#define CONTEXTHELP_QNOTNICEATALL _("Basic blocks, no decoration")
+#define CONTEXTHELP_QNOTNICE      _("Basic decoration, not smooth blocks")
+#define CONTEXTHELP_QMEDIUM       _("Not original")
+#define CONTEXTHELP_QNICE         _("Nice, could be better")
+#define CONTEXTHELP_QREALLYNICE   _("Several textures, sprites, ...")
+#define CONTEXTHELP_SENDTHISREPORT  _("Send this report on the webserver")
+#define CONTEXTHELP_SKIPTHISREPORT  _("Don't send this report on the webserver")
+
+#define CONTEXTHELP_SERVER_OPTIONS _("Server configuration to play via the network")
+#define CONTEXTHELP_STARTSERVERATSTARTUP _("Start the server at X-Moto startup")
+#define CONTEXTHELP_SERVERSTARTSTOP _("Start/stop the server")
+#define CONTEXTHELP_SERVERPORT _("Port on which the server must listen to")
+#define CONTEXTHELP_SERVER _("Server on which you want to connect")
+#define CONTEXTHELP_CLIENTCONNECTDISCONNECT _("Connect/disconnect the client to the server")
+#define CONTEXTHELP_MUSICONALLLEVELS _("Enables background music for levels that do not have their own music track")
 
 #define VPACKAGENAME_LEVELS_WITH_NO_HIGHSCORE   _("Levels with no highscore")
 #define VPACKAGENAME_INCOMPLETED_LEVELS         _("Levels you have not completed")
@@ -578,7 +705,7 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define VPACKAGENAME_EASIEST_LEVELS             _("Easiest levels")
 #define VPACKAGENAME_CRAPPIEST_LEVELS            _("Crappiest levels")
 #define VPACKAGENAME_SCRIPTED                   _("Scripted levels")
-#define VPACKAGENAME_MUSICAL                    _("Musical levels")
+#define VPACKAGENAME_PHYSICS                    _("Physics levels")
 #define VPACKAGENAME_BEST_DRIVER                _("By best driver")
 #define VPACKAGENAME_LAST_PLAYED                _("Last played levels")
 #define VPACKAGENAME_NEVER_PLAYED               _("Never played levels")
@@ -611,7 +738,7 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define VPACKAGENAME_DESC_EASIEST_LEVELS         _("Easiest X-Moto levels according to the web votes")
 #define VPACKAGENAME_DESC_CRAPPIEST_LEVELS       _("Crappiest X-Moto levels according to the web votes")
 #define VPACKAGENAME_DESC_SCRIPTED                   _("X-Moto levels which are dynamic")
-#define VPACKAGENAME_DESC_MUSICAL                    _("X-Moto levels with a background music")
+#define VPACKAGENAME_DESC_PHYSICS                    _("X-Moto levels using strongly physics")
 #define VPACKAGENAME_DESC_LAST_PLAYED                _("Last levels you have played")
 #define VPACKAGENAME_DESC_NEVER_PLAYED               _("X-Moto levels you never played")
 #define VPACKAGENAME_DESC_MOST_PLAYED                _("X-Moto levels you have played the most")
@@ -652,6 +779,8 @@ std::string(_("You control your bike using the keyboard")) + ":\n" \
 #define SYS_MSG_TRAIN_RESTORING         _("Training position %i/%i restored")
 #define SYS_MSG_TRAIN_STORED            _("Stored as training position %i")
 
+#define SYS_MSG_NO_NEXT_HIGHSCORE(A)    ngettext("Can't find a highscore in the next %i level", "Can't find a highscore in the next %i levels", A)
+
 // font to use to choice the GROUP : currently, two groups, GENERAL and ASIAN
 #define FONT_GROUP _("FontGroup:GENERAL")
 
