@@ -27,34 +27,53 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "helpers/Log.h"
 #include "VFileIO.h"
 #include "WWWAppInterface.h"
+#include <sstream>
 
 LevelsPack::LevelsPack(std::string i_name, const std::string& i_sql, bool i_ascSort) {
-  m_name         = i_name;
-  m_showTimes    = true;
-  m_showWebTimes = true;
-  //setHintsFromFile();
-  m_sql_levels   = i_sql;
-  m_ascSort      = i_ascSort;
+  m_name             = i_name;
+  m_showTimes        = true;
+  m_showWebTimes     = true;
+  m_sql_levels       = i_sql;
+  m_ascSort          = i_ascSort;
+  m_nbLevels         = -1;
+  m_nbFinishedLevels = -1;
 }
 
 LevelsPack::~LevelsPack() {
 }
 
-int LevelsPack::getNumberOfLevels(xmDatabase *i_db) {
+void LevelsPack::updateCount(xmDatabase *i_db, const std::string& i_profile) {
   char **v_result;
   unsigned int nrow;
-  int n;
 
-  v_result = i_db->readDB("SELECT count(id_level) FROM (" + m_sql_levels + ");",
-			  nrow);
+  /* number of levels*/
+  v_result = i_db->readDB("SELECT count(id_level) FROM (" + m_sql_levels + ");", nrow);
+
   if(i_db->getResult(v_result, 1, 0, 0) == NULL) {
     i_db->read_DB_free(v_result);
-    return 0;
+    throw Exception("Unable to update level pack count");
   }
 
-  n = atoi(i_db->getResult(v_result, 1, 0, 0));
+  m_nbLevels = atoi(i_db->getResult(v_result, 1, 0, 0));
   i_db->read_DB_free(v_result);
-  return n;
+
+  /* finished levels */
+  v_result = i_db->readDB("SELECT count(1) FROM (SELECT a.id_level FROM (" + m_sql_levels +
+			  ") AS a INNER JOIN stats_profiles_levels AS b ON a.id_level=b.id_level "
+			  "WHERE b.id_profile=\"" + xmDatabase::protectString(i_profile) + "\" AND b.nbCompleted+0 > 0 "
+			  "GROUP BY a.id_level);",
+			  nrow);
+
+  if(nrow != 1) {
+    i_db->read_DB_free(v_result);
+    throw Exception("Unable to update level pack count");
+  }
+  m_nbFinishedLevels = atoi(i_db->getResult(v_result, 1, 0, 0));
+  i_db->read_DB_free(v_result);
+}
+
+int LevelsPack::getNumberOfLevels() {
+  return m_nbLevels;
 }
 
 std::string LevelsPack::getLevelsQuery() const {
@@ -70,27 +89,11 @@ std::string LevelsPack::getLevelsWithHighscoresQuery(const std::string& i_profil
     "ON (a.id_level = b.id_level AND b.id_room=" + i_id_room + ") "
     "LEFT OUTER JOIN profile_completedLevels AS c "
     "ON (a.id_level=c.id_level AND c.id_profile=\"" + xmDatabase::protectString(i_profile) + "\") "
-    "GROUP BY a.id_level ORDER BY min(a.sort_field) " + std::string(m_ascSort ? "ASC":"DESC") + ";";
+    "GROUP BY a.id_level ORDER BY MIN(a.sort_field) " + std::string(m_ascSort ? "ASC":"DESC") + ";";
 }
 
-int LevelsPack::getNumberOfFinishedLevels(xmDatabase *i_db, const std::string& i_profile) {
-  char **v_result;
-  unsigned int nrow;
-  int n;
-
-  v_result = i_db->readDB("SELECT count(a.id_level) FROM (" +
-			  m_sql_levels +
-			  ") AS a INNER JOIN stats_profiles_levels AS b "
-			  "ON a.id_level=b.id_level WHERE b.id_profile=\"" +
-			  xmDatabase::protectString(i_profile) + "\" AND b.nbCompleted > 0;",
-			  nrow);
-  if(i_db->getResult(v_result, 1, 0, 0) == NULL) {
-    i_db->read_DB_free(v_result);
-    return 0;
-  }
-  n = atoi(i_db->getResult(v_result, 1, 0, 0));
-  i_db->read_DB_free(v_result);
-  return n;
+int LevelsPack::getNumberOfFinishedLevels() {
+  return m_nbFinishedLevels;
 }
 
 void LevelsPack::setHintsFromFile() {
@@ -170,10 +173,12 @@ void LevelsPack::setShowWebTimes(bool i_showWebTimes) {
 }
 
 LevelsManager::LevelsManager() {
+  m_levelsPackMutex = SDL_CreateMutex();
 }
 
 LevelsManager::~LevelsManager() {
   clean();
+  SDL_DestroyMutex(m_levelsPackMutex);
 }
  
 void LevelsManager::clean() {
@@ -214,6 +219,7 @@ void LevelsManager::makePacks(const std::string& i_playerName,
   char **v_result;
   unsigned int nrow;
 
+  lockLevelsPacks();
   cleanPacks();
 
   /* all levels */
@@ -267,31 +273,32 @@ void LevelsManager::makePacks(const std::string& i_playerName,
   v_pack->setDescription(VPACKAGENAME_DESC_SCRIPTED);
   m_levelsPacks.push_back(v_pack);
 
-  /* musical */
-  v_pack = new LevelsPack(std::string(VPACKAGENAME_MUSICAL),
+  /* physics */
+  v_pack = new LevelsPack(std::string(VPACKAGENAME_PHYSICS),
 			  "SELECT a.id_level AS id_level, a.name AS name, UPPER(a.name) AS sort_field "
 			  "FROM levels AS a "
 			  "LEFT OUTER JOIN weblevels AS b ON a.id_level=b.id_level "
                           "LEFT OUTER JOIN levels_blacklist AS c ON (a.id_level = c.id_level AND c.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
-			  "WHERE a.music<>\"\" AND (b.crappy IS NULL OR xm_userCrappy(b.crappy)=0) "
+			  "WHERE a.isPhysics=1 AND (b.crappy IS NULL OR xm_userCrappy(b.crappy)=0) "
 			  "AND (b.children_compliant IS NULL OR xm_userChildrenCompliant(b.children_compliant)=1) "
 			  "AND c.id_level IS NULL");
   v_pack->setGroup(GAMETEXT_PACK_SPECIAL);
-  v_pack->setDescription(VPACKAGENAME_DESC_MUSICAL);
+  v_pack->setDescription(VPACKAGENAME_DESC_PHYSICS);
   m_levelsPacks.push_back(v_pack);
 
   /* levels i've not finished */
   v_pack = new LevelsPack(std::string(VPACKAGENAME_INCOMPLETED_LEVELS),
-			  "SELECT a.id_level AS id_level, a.name AS name, UPPER(a.name) AS sort_field "
+			  "SELECT a.id_level AS id_level, MIN(a.name) AS name, MIN(UPPER(a.name)) AS sort_field "
 			  "FROM levels AS a LEFT OUTER JOIN stats_profiles_levels AS b "
 			  "ON (a.id_level=b.id_level AND "
 			  "b.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
 			  "LEFT OUTER JOIN weblevels AS c ON a.id_level=c.id_level "
 			  "LEFT OUTER JOIN levels_blacklist AS d ON (a.id_level = d.id_level AND d.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
-			  "WHERE b.nbCompleted = 0 OR b.id_profile IS NULL "
-			  "AND d.id_level IS NULL "
+			  "WHERE d.id_level IS NULL "
 			  "AND (c.crappy IS NULL OR xm_userCrappy(c.crappy)=0) "
-			  "AND (c.children_compliant IS NULL OR xm_userChildrenCompliant(c.children_compliant)=1)");
+			  "AND (c.children_compliant IS NULL OR xm_userChildrenCompliant(c.children_compliant)=1) "
+			  "GROUP BY a.id_level, b.id_profile "
+			  "HAVING MAX(b.nbCompleted+0) = 0 OR MAX(b.nbCompleted+0) IS NULL");
   v_pack->setGroup(GAMETEXT_PACK_SPECIAL);
   v_pack->setDescription(VPACKAGENAME_DESC_INCOMPLETED_LEVELS);
   m_levelsPacks.push_back(v_pack);
@@ -340,11 +347,11 @@ void LevelsManager::makePacks(const std::string& i_playerName,
   v_pack = new LevelsPack(std::string(VPACKAGENAME_YOU_HAVE_NOT_THE_HIGHSCORE),
 			  "SELECT a.id_level AS id_level, a.name AS name, UPPER(a.name) AS sort_field "
 			  "FROM levels AS a INNER JOIN "
-			  "webhighscores AS b ON a.id_level = b.id_level "
+			  "webhighscores AS b ON (a.id_level = b.id_level "
+			  "AND b.id_room=" + i_id_room + ") "
 			  "LEFT OUTER JOIN weblevels AS c ON a.id_level=c.id_level "
 			  "LEFT OUTER JOIN levels_blacklist AS d ON (a.id_level = d.id_level AND d.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
-			  "WHERE b.id_room=" + i_id_room + " "
-			  "AND b.id_profile<>\"" + xmDatabase::protectString(i_playerName) + "\" "
+			  "WHERE b.id_profile<>\"" + xmDatabase::protectString(i_playerName) + "\" "
 			  "AND (c.crappy IS NULL OR xm_userCrappy(c.crappy)=0) "
 			  "AND (c.children_compliant IS NULL OR xm_userChildrenCompliant(c.children_compliant)=1) "
 			  "AND d.id_level IS NULL");
@@ -356,11 +363,11 @@ void LevelsManager::makePacks(const std::string& i_playerName,
   v_pack = new LevelsPack(std::string(VPACKAGENAME_LAST_HIGHSCORES),
 			  "SELECT a.id_level AS id_level, a.name AS name, b.date AS sort_field "
 			  "FROM levels AS a INNER JOIN "
-			  "webhighscores AS b ON a.id_level = b.id_level "
+			  "webhighscores AS b ON (a.id_level = b.id_level "
+			  "AND b.id_room=" + i_id_room + ") "
 			  "LEFT OUTER JOIN weblevels AS c ON a.id_level=c.id_level "
 			  "LEFT OUTER JOIN levels_blacklist AS d ON (a.id_level = d.id_level AND d.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
-			  "WHERE b.id_room=" + i_id_room + " "
-			  "AND (c.crappy IS NULL OR xm_userCrappy(c.crappy)=0) "
+			  "WHERE (c.crappy IS NULL OR xm_userCrappy(c.crappy)=0) "
 			  "AND (c.children_compliant IS NULL OR xm_userChildrenCompliant(c.children_compliant)=1) "
 			  "AND d.id_level IS NULL "
 			  "ORDER by b.date DESC LIMIT 100", false);
@@ -386,11 +393,11 @@ void LevelsManager::makePacks(const std::string& i_playerName,
   v_pack = new LevelsPack(std::string(VPACKAGENAME_OLDEST_HIGHSCORES),
 			  "SELECT a.id_level AS id_level, a.name AS name, b.date AS sort_field "
 			  "FROM levels AS a INNER JOIN "
-			  "webhighscores AS b ON a.id_level = b.id_level "
+			  "webhighscores AS b ON (a.id_level = b.id_level "
+			  "AND b.id_room=" + i_id_room + ") "
 			  "LEFT OUTER JOIN weblevels AS c ON a.id_level=c.id_level "
 			  "LEFT OUTER JOIN levels_blacklist AS d ON (a.id_level = d.id_level AND d.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
-			  "WHERE b.id_room=" + i_id_room + " "
-			  "AND (c.crappy IS NULL OR xm_userCrappy(c.crappy)=0)"
+			  "WHERE (c.crappy IS NULL OR xm_userCrappy(c.crappy)=0)"
 			  "AND (c.children_compliant IS NULL OR xm_userChildrenCompliant(c.children_compliant)=1) "
 			  " AND d.id_level IS NULL "
 			  "ORDER by b.date ASC LIMIT 100");
@@ -402,7 +409,8 @@ void LevelsManager::makePacks(const std::string& i_playerName,
   v_pack = new LevelsPack(std::string(VPACKAGENAME_MEDAL_PLATINIUM),
 			  "SELECT b.id_level AS id_level, b.name AS name, UPPER(b.name) AS sort_field "
 			  "FROM webhighscores AS a INNER JOIN levels AS b "
-			  "ON a.id_level = b.id_level "
+			  "ON (a.id_level = b.id_level "
+			  "AND a.id_room=" + i_id_room + ") "
 			  "LEFT OUTER JOIN weblevels AS c ON a.id_level=c.id_level "
 			  "LEFT OUTER JOIN levels_blacklist AS d ON (a.id_level = d.id_level AND d.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
 			  "WHERE a.id_profile = \"" + xmDatabase::protectString(i_playerName) + "\" "
@@ -431,11 +439,12 @@ void LevelsManager::makePacks(const std::string& i_playerName,
 			  "AND e.id_level IS NULL "
 			  "GROUP BY a.id_level " /* get the best highscores */
 			  ") "
-			  "WHERE (webFinishTime IS NULL OR webFinishTime >= userFinishTime*0.95) "
+			  "WHERE (webFinishTime IS NULL OR webFinishTime+0 >= userFinishTime*0.95) "
 			  "EXCEPT " /* remove the webhighscores where the player's name is the same */
 			  "SELECT b.id_level AS id_level, b.name AS name, UPPER(b.name) AS sort_field "
 			  "FROM webhighscores AS a INNER JOIN levels AS b "
-			  "ON a.id_level = b.id_level "
+			  "ON (a.id_level = b.id_level "
+			  "AND a.id_room=" + i_id_room + ") "
 			  "WHERE a.id_profile = \"" + xmDatabase::protectString(i_playerName) + "\""
 			  );
   v_pack->setGroup(GAMETEXT_PACK_MEDALS);
@@ -460,12 +469,13 @@ void LevelsManager::makePacks(const std::string& i_playerName,
 			  "GROUP BY a.id_level " /* get the best highscores */
 			  ") "
 			  "WHERE (webFinishTime IS NOT NULL "
-			  "AND webFinishTime >= userFinishTime*0.90 "
-			  "AND webFinishTime < userFinishTime*0.95) "
+			  "AND webFinishTime+0 >= userFinishTime*0.90 "
+			  "AND webFinishTime+0 < userFinishTime*0.95) "
 			  "EXCEPT " /* remove the webhighscores where the player's name is the same */
 			  "SELECT b.id_level AS id_level, b.name AS name, UPPER(b.name) AS sort_field "
 			  "FROM webhighscores AS a INNER JOIN levels AS b "
-			  "ON a.id_level = b.id_level "
+			  "ON (a.id_level = b.id_level "
+			  "AND a.id_room=" + i_id_room + ") "
 			  "WHERE a.id_profile = \"" + xmDatabase::protectString(i_playerName) + "\""
 			  );
   v_pack->setGroup(GAMETEXT_PACK_MEDALS);
@@ -490,12 +500,13 @@ void LevelsManager::makePacks(const std::string& i_playerName,
 			  "GROUP BY a.id_level " /* get the best highscores */
 			  ") "
 			  "WHERE (webFinishTime IS NOT NULL "
-			  "AND webFinishTime >= userFinishTime*0.80 "
-			  "AND webFinishTime < userFinishTime*0.90) "
+			  "AND webFinishTime+0 >= userFinishTime*0.80 "
+			  "AND webFinishTime+0 < userFinishTime*0.90) "
 			  "EXCEPT " /* remove the webhighscores where the player's name is the same */
 			  "SELECT b.id_level AS id_level, b.name AS name, UPPER(b.name) AS sort_field "
 			  "FROM webhighscores AS a INNER JOIN levels AS b "
-			  "ON a.id_level = b.id_level "
+			  "ON (a.id_level = b.id_level "
+			  "AND a.id_room=" + i_id_room + ") "
 			  "WHERE a.id_profile = \"" + xmDatabase::protectString(i_playerName) + "\""
 			  );
   v_pack->setGroup(GAMETEXT_PACK_MEDALS);
@@ -520,12 +531,13 @@ void LevelsManager::makePacks(const std::string& i_playerName,
 			  "GROUP BY a.id_level "
 			  ") "
 			  "WHERE (webFinishTime IS NOT NULL "
-			  "AND webFinishTime < userFinishTime*0.80) "
+			  "AND webFinishTime+0 < userFinishTime*0.80) "
 			  "OR userFinishTime IS NULL "
 			  "EXCEPT " /* remove the webhighscores where the player's name is the same */
 			  "SELECT b.id_level AS id_level, b.name AS name, UPPER(b.name) AS sort_field "
 			  "FROM webhighscores AS a INNER JOIN levels AS b "
-			  "ON a.id_level = b.id_level "
+			  "ON (a.id_level = b.id_level "
+			  "AND a.id_room=" + i_id_room + ") "
 			  "WHERE a.id_profile = \"" + xmDatabase::protectString(i_playerName) + "\""
 			  );
   v_pack->setGroup(GAMETEXT_PACK_MEDALS);
@@ -535,7 +547,7 @@ void LevelsManager::makePacks(const std::string& i_playerName,
   /* stats */
   /* last played levels */
   v_pack = new LevelsPack(std::string(VPACKAGENAME_LAST_PLAYED),
-			  "SELECT a.id_level AS id_level, a.name AS name, b.last_play_date AS sort_field "
+			  "SELECT a.id_level AS id_level, MIN(a.name) AS name, MAX(b.last_play_date) AS sort_field "
 			  "FROM levels AS a INNER JOIN stats_profiles_levels AS b "
 			  "ON (a.id_level=b.id_level AND "
 			  "b.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
@@ -545,7 +557,8 @@ void LevelsManager::makePacks(const std::string& i_playerName,
 			  "AND (c.crappy IS NULL OR xm_userCrappy(c.crappy)=0) "
 			  "AND (c.children_compliant IS NULL OR xm_userChildrenCompliant(c.children_compliant)=1) "
 			  "AND d.id_level IS NULL "
-			  "ORDER BY b.last_play_date DESC "
+			  "GROUP BY a.id_level, b.id_profile "
+			  "ORDER BY MAX(b.last_play_date) DESC "
 			  "LIMIT 100", false);
   v_pack->setGroup(GAMETEXT_PACK_STATS);
   v_pack->setDescription(VPACKAGENAME_DESC_LAST_PLAYED);
@@ -569,7 +582,7 @@ void LevelsManager::makePacks(const std::string& i_playerName,
   
   /* most played levels */
   v_pack = new LevelsPack(std::string(VPACKAGENAME_MOST_PLAYED),
-			  "SELECT a.id_level AS id_level, a.name AS name, b.nbPlayed AS sort_field "
+			  "SELECT a.id_level AS id_level, MIN(a.name) AS name, SUM(b.nbPlayed) AS sort_field "
 			  "FROM levels AS a INNER JOIN stats_profiles_levels AS b "
 			  "ON (a.id_level=b.id_level AND "
 			  "b.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
@@ -578,38 +591,40 @@ void LevelsManager::makePacks(const std::string& i_playerName,
 			  "WHERE (c.crappy IS NULL OR xm_userCrappy(c.crappy)=0) "
 			  "AND (c.children_compliant IS NULL OR xm_userChildrenCompliant(c.children_compliant)=1) "
 			  "AND d.id_level IS NULL "
-			  "ORDER BY b.nbPlayed DESC LIMIT 100", false);
+			  "GROUP BY a.id_level, b.id_profile "
+			  "ORDER BY MAX(b.nbPlayed+0) DESC LIMIT 100", false);
   v_pack->setGroup(GAMETEXT_PACK_STATS);
   v_pack->setDescription(VPACKAGENAME_DESC_MOST_PLAYED);
   m_levelsPacks.push_back(v_pack);
 
   /* less played levels */
   v_pack = new LevelsPack(std::string(VPACKAGENAME_LESS_PLAYED),
-			  "SELECT a.id_level AS id_level, a.name AS name, b.nbPlayed AS sort_field "
+			  "SELECT a.id_level AS id_level, MIN(a.name) AS name, SUM(b.nbPlayed) AS sort_field "
 			  "FROM levels AS a INNER JOIN stats_profiles_levels AS b "
 			  "ON (a.id_level=b.id_level AND "
 			  "b.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
 			  "LEFT OUTER JOIN weblevels AS c ON a.id_level=c.id_level "
 			  "LEFT OUTER JOIN levels_blacklist AS d ON (a.id_level = d.id_level AND d.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
-			  "WHERE b.nbPlayed > 0 "
+			  "WHERE b.nbPlayed+0 > 0 "
 			  "AND (c.crappy IS NULL OR xm_userCrappy(c.crappy)=0) "
 			  "AND (c.children_compliant IS NULL OR xm_userChildrenCompliant(c.children_compliant)=1) "
 			  "AND d.id_level IS NULL "
-			  "ORDER BY b.nbPlayed ASC LIMIT 100");
+			  "GROUP BY a.id_level, b.id_profile "
+			  "ORDER BY MIN(b.nbPlayed+0) ASC LIMIT 100");
   v_pack->setGroup(GAMETEXT_PACK_STATS);
   v_pack->setDescription(VPACKAGENAME_DESC_LESS_PLAYED);
   m_levelsPacks.push_back(v_pack);
 
   /* nicest levels */
   v_pack = new LevelsPack(std::string(VPACKAGENAME_NICEST_LEVELS),
-			  "SELECT a.id_level AS id_level, a.name AS name, b.quality AS sort_field "
+			  "SELECT a.id_level AS id_level, a.name AS name, b.quality+0 AS sort_field "
 			  "FROM levels AS a INNER JOIN weblevels AS b ON a.id_level=b.id_level "
 			  "LEFT OUTER JOIN levels_blacklist AS c ON (a.id_level = c.id_level AND c.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
-			  "WHERE b.quality >= 4.5 "
+			  "WHERE b.quality+0 >= 4.5 "
 			  "AND (b.crappy IS NULL OR xm_userCrappy(b.crappy)=0) "
 			  "AND (b.children_compliant IS NULL OR xm_userChildrenCompliant(b.children_compliant)=1) "
 			  "AND c.id_level IS NULL "
-			  "ORDER BY b.quality DESC LIMIT 100", false);
+			  "ORDER BY b.quality+0 DESC LIMIT 100", false);
   v_pack->setGroup(GAMETEXT_PACK_WEBVOTES);
   v_pack->setDescription(VPACKAGENAME_DESC_NICEST_LEVELS);
   m_levelsPacks.push_back(v_pack);
@@ -617,14 +632,14 @@ void LevelsManager::makePacks(const std::string& i_playerName,
   /* crappiest levels */
   if(i_bDebugMode) {
     v_pack = new LevelsPack(std::string(VPACKAGENAME_CRAPPIEST_LEVELS),
-			    "SELECT a.id_level AS id_level, a.name AS name, b.quality AS sort_field "
+			    "SELECT a.id_level AS id_level, a.name AS name, b.quality+0 AS sort_field "
 			    "FROM levels AS a INNER JOIN weblevels AS b ON a.id_level=b.id_level "
 			    "LEFT OUTER JOIN levels_blacklist AS c ON (a.id_level = c.id_level AND c.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
-			    "WHERE b.quality <= 1.5 "
+			    "WHERE b.quality+0 <= 1.5 "
 			    "AND (b.crappy IS NULL OR xm_userCrappy(b.crappy)=0) "
 			    "AND (b.children_compliant IS NULL OR xm_userChildrenCompliant(b.children_compliant)=1) "
 			    "AND c.id_level IS NULL "
-			    "ORDER BY b.quality ASC LIMIT 100");
+			    "ORDER BY b.quality+0 ASC LIMIT 100");
     v_pack->setGroup(GAMETEXT_PACK_WEBVOTES);
     v_pack->setDescription(VPACKAGENAME_DESC_CRAPPIEST_LEVELS);
     m_levelsPacks.push_back(v_pack);
@@ -632,39 +647,40 @@ void LevelsManager::makePacks(const std::string& i_playerName,
 
   /* easiest levels */
   v_pack = new LevelsPack(std::string(VPACKAGENAME_EASIEST_LEVELS),
-			  "SELECT a.id_level AS id_level, a.name AS name, b.difficulty AS sort_field "
+			  "SELECT a.id_level AS id_level, a.name AS name, b.difficulty+0 AS sort_field "
 			  "FROM levels AS a INNER JOIN weblevels AS b ON a.id_level=b.id_level "
 			  "LEFT OUTER JOIN levels_blacklist AS c ON (a.id_level = c.id_level AND c.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
-			  "WHERE b.difficulty <= 1.5 "
+			  "WHERE b.difficulty+0 <= 1.5 "
 			  "AND (b.crappy IS NULL OR xm_userCrappy(b.crappy)=0) "
 			  "AND (b.children_compliant IS NULL OR xm_userChildrenCompliant(b.children_compliant)=1) "
 			  "AND c.id_level IS NULL "
-			  "ORDER BY b.difficulty ASC LIMIT 100");
+			  "ORDER BY b.difficulty+0 ASC LIMIT 100");
   v_pack->setGroup(GAMETEXT_PACK_WEBVOTES);
   v_pack->setDescription(VPACKAGENAME_DESC_EASIEST_LEVELS);
   m_levelsPacks.push_back(v_pack);
 
   /* hardest levels */
   v_pack = new LevelsPack(std::string(VPACKAGENAME_HARDEST_LEVELS),
-			  "SELECT a.id_level AS id_level, a.name AS name, b.difficulty AS sort_field "
+			  "SELECT a.id_level AS id_level, a.name AS name, b.difficulty+0 AS sort_field "
 			  "FROM levels AS a INNER JOIN weblevels AS b ON a.id_level=b.id_level "
 			  "LEFT OUTER JOIN levels_blacklist AS c ON (a.id_level = c.id_level AND c.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
-			  "WHERE b.difficulty >= 4.5 "
+			  "WHERE b.difficulty+0 >= 4.5 "
 			  "AND (b.crappy IS NULL OR xm_userCrappy(b.crappy)=0) "
 			  "AND (b.children_compliant IS NULL OR xm_userChildrenCompliant(b.children_compliant)=1) "
 			  "AND c.id_level IS NULL "
-			  "ORDER BY b.difficulty DESC LIMIT 100", false);
+			  "ORDER BY b.difficulty+0 DESC LIMIT 100", false);
   v_pack->setGroup(GAMETEXT_PACK_WEBVOTES);
   v_pack->setDescription(VPACKAGENAME_DESC_HARDEST_LEVELS);
   m_levelsPacks.push_back(v_pack);
   
   /* short levels */
   v_pack = new LevelsPack(std::string(VPACKAGENAME_SHORT_LEVELS),
-                          "SELECT a.id_level AS id_level, a.name AS name, b.finishTime AS sort_field "
-                          "FROM levels AS a INNER JOIN webhighscores AS b ON a.id_level=b.id_level "
+                          "SELECT a.id_level AS id_level, a.name AS name, b.finishTime+0 AS sort_field "
+                          "FROM levels AS a INNER JOIN webhighscores AS b ON (a.id_level=b.id_level "
+			  "AND b.id_room=" + i_id_room + ") "
 			  "LEFT OUTER JOIN levels_blacklist AS c ON (a.id_level = c.id_level AND c.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
 			  "LEFT OUTER JOIN weblevels AS d ON a.id_level=d.id_level "
-                          "WHERE b.finishTime < 2500 AND c.id_level IS NULL "
+                          "WHERE b.finishTime+0 < 2500 AND c.id_level IS NULL "
 			  "AND (d.crappy IS NULL OR xm_userCrappy(d.crappy)=0) "
 			  "AND (d.children_compliant IS NULL OR xm_userChildrenCompliant(d.children_compliant)=1)");
   v_pack->setGroup(GAMETEXT_PACK_BY_LENGTH);
@@ -673,11 +689,12 @@ void LevelsManager::makePacks(const std::string& i_playerName,
   
   /* Medium levels */
   v_pack = new LevelsPack(std::string(VPACKAGENAME_MEDIUM_LEVELS),
-                          "SELECT a.id_level AS id_level, a.name AS name, b.finishTime AS sort_field "
-                          "FROM levels AS a INNER JOIN webhighscores AS b ON a.id_level=b.id_level "
+                          "SELECT a.id_level AS id_level, a.name AS name, b.finishTime+0 AS sort_field "
+                          "FROM levels AS a INNER JOIN webhighscores AS b ON (a.id_level=b.id_level "
+			  "AND b.id_room=" + i_id_room + ") "
 			  "LEFT OUTER JOIN levels_blacklist AS c ON (a.id_level = c.id_level AND c.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
 			  "LEFT OUTER JOIN weblevels AS d ON a.id_level=d.id_level "
-                          "WHERE b.finishTime >= 2500 AND b.finishTime < 6000 AND c.id_level IS NULL "
+                          "WHERE b.finishTime+0 >= 2500 AND b.finishTime+0 < 6000 AND c.id_level IS NULL "
 			  "AND (d.crappy IS NULL OR xm_userCrappy(d.crappy)=0) "
 			  "AND (d.children_compliant IS NULL OR xm_userChildrenCompliant(d.children_compliant)=1)");
   v_pack->setGroup(GAMETEXT_PACK_BY_LENGTH);
@@ -686,11 +703,12 @@ void LevelsManager::makePacks(const std::string& i_playerName,
   
   /* Long levels*/
   v_pack = new LevelsPack(std::string(VPACKAGENAME_LONG_LEVELS),
-                          "SELECT a.id_level AS id_level, a.name AS name, b.finishTime AS sort_field "
-                          "FROM levels AS a INNER JOIN webhighscores AS b ON a.id_level=b.id_level "
+                          "SELECT a.id_level AS id_level, a.name AS name, b.finishTime+0 AS sort_field "
+                          "FROM levels AS a INNER JOIN webhighscores AS b ON (a.id_level=b.id_level "
+			  "AND b.id_room=" + i_id_room + ") "
 			  "LEFT OUTER JOIN levels_blacklist AS c ON (a.id_level = c.id_level AND c.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
 			  "LEFT OUTER JOIN weblevels AS d ON a.id_level=d.id_level "
-                          "WHERE b.finishTime >= 6000 AND b.finishTime < 12000 AND c.id_level IS NULL "
+                          "WHERE b.finishTime+0 >= 6000 AND b.finishTime+0 < 12000 AND c.id_level IS NULL "
 			  "AND (d.crappy IS NULL OR xm_userCrappy(d.crappy)=0) "
 			  "AND (d.children_compliant IS NULL OR xm_userChildrenCompliant(d.children_compliant)=1)");
   v_pack->setGroup(GAMETEXT_PACK_BY_LENGTH);
@@ -699,11 +717,12 @@ void LevelsManager::makePacks(const std::string& i_playerName,
   
   /* Very long levels*/
   v_pack = new LevelsPack(std::string(VPACKAGENAME_VERY_LONG_LEVELS),
-                          "SELECT a.id_level AS id_level, a.name AS name, b.finishTime AS sort_field "
-                          "FROM levels AS a INNER JOIN webhighscores AS b ON a.id_level=b.id_level "
+                          "SELECT a.id_level AS id_level, a.name AS name, b.finishTime+0 AS sort_field "
+                          "FROM levels AS a INNER JOIN webhighscores AS b ON (a.id_level=b.id_level "
+			  "AND b.id_room=" + i_id_room + ") "
 			  "LEFT OUTER JOIN levels_blacklist AS c ON (a.id_level = c.id_level AND c.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
 			  "LEFT OUTER JOIN weblevels AS d ON a.id_level=d.id_level "
-                          "WHERE b.finishTime >= 12000 AND c.id_level IS NULL "
+                          "WHERE b.finishTime+0 >= 12000 AND c.id_level IS NULL "
 			  "AND (d.crappy IS NULL OR xm_userCrappy(d.crappy)=0) "
 			  "AND (d.children_compliant IS NULL OR xm_userChildrenCompliant(d.children_compliant)=1)");
   v_pack->setGroup(GAMETEXT_PACK_BY_LENGTH);
@@ -757,11 +776,11 @@ void LevelsManager::makePacks(const std::string& i_playerName,
     v_pack = new LevelsPack(v_n.str() + v_id_profile,
 			    "SELECT a.id_level AS id_level, a.name AS name, UPPER(a.name) AS sort_field "
 			    "FROM levels AS a INNER JOIN "
-			    "webhighscores AS b ON a.id_level = b.id_level "
+			    "webhighscores AS b ON (a.id_level = b.id_level "
+			    "AND b.id_room=" + i_id_room + ") "
 			    "LEFT OUTER JOIN weblevels AS c ON a.id_level=c.id_level "
 			    "LEFT OUTER JOIN levels_blacklist AS d ON (a.id_level = d.id_level AND d.id_profile=\"" + xmDatabase::protectString(i_playerName) + "\") "
-			    "WHERE b.id_room=" + i_id_room + " "
-			    "AND b.id_profile=\"" + xmDatabase::protectString(v_id_profile) + "\" "
+			    "WHERE b.id_profile=\"" + xmDatabase::protectString(v_id_profile) + "\" "
 			    "AND (c.crappy IS NULL OR xm_userCrappy(c.crappy)=0) "
 			    "AND (c.children_compliant IS NULL OR xm_userChildrenCompliant(c.children_compliant)=1) "
 			    "AND d.id_level IS NULL");
@@ -770,8 +789,9 @@ void LevelsManager::makePacks(const std::string& i_playerName,
     v_pack->setDescription(v_levelPackStr);
     m_levelsPacks.push_back(v_pack);
   }
-  i_db->read_DB_free(v_result);  
- 
+  i_db->read_DB_free(v_result);
+
+  unlockLevelsPacks();
 }
 
 const std::vector<LevelsPack *>& LevelsManager::LevelsPacks() {
@@ -812,6 +832,7 @@ void LevelsManager::reloadExternalLevels(xmDatabase* i_db, XMotoLoadLevelsInterf
 		       v_level->PackNum(),
 		       v_level->Music(),
 		       v_level->isScripted(),
+		       v_level->isPhysics(),
 		       true);
     } catch(Exception &e) {
     }
@@ -844,9 +865,10 @@ void LevelsManager::addExternalLevel(std::string i_levelFile, xmDatabase *i_db)
 		     v_level->PackNum(),
 		     v_level->Music(),
 		     v_level->isScripted(),
+		     v_level->isPhysics(),
 		     true);
   } catch(Exception &e) {
-    Logger::Log("** Warning ** : Unable to add external level (%s)", e.getMsg().c_str());
+    LogWarning("Unable to add external level (%s)", e.getMsg().c_str());
   }
   delete v_level;
 }
@@ -892,9 +914,10 @@ void LevelsManager::reloadInternalLevels(xmDatabase* i_db, XMotoLoadLevelsInterf
 		       v_level->PackNum(),
 		       v_level->Music(),
 		       v_level->isScripted(),
+		       v_level->isPhysics(),
 		       false);
     } catch(Exception &e) {
-      Logger::Log("** Warning (just mean that the level has been updated if the level is in xmoto.bin) ** : %s (%s - %s)",
+      LogWarning("(just mean that the level has been updated if the level is in xmoto.bin) ** : %s (%s - %s)",
 		e.getMsg().c_str(),
 		v_level->Name().c_str(),
 		v_level->FileName().c_str());
@@ -1015,10 +1038,11 @@ void LevelsManager::updateLevelsFromLvl(const std::vector<std::string> &NewLvl,
 		       v_level->PackNum(),
 		       v_level->Music(),
 		       v_level->isScripted(),
+		       v_level->isPhysics(),
 		       false);
       i_db->levels_addToNew(v_level->Id(), false);
     } catch(Exception &e) {
-      Logger::Log("** Warning ** : %s", e.getMsg().c_str() );
+      LogWarning("%s", e.getMsg().c_str() );
     }
     delete v_level;
   }
@@ -1047,11 +1071,12 @@ void LevelsManager::updateLevelsFromLvl(const std::vector<std::string> &NewLvl,
 			  v_level->PackNum(),
 			  v_level->Music(),
 			  v_level->isScripted(),
+			  v_level->isPhysics(),
 			  false);
       i_db->levels_addToNew(v_level->Id(), true);
 
     } catch(Exception &e) {
-      Logger::Log("** Warning ** : %s", e.getMsg().c_str() );
+      LogWarning("%s", e.getMsg().c_str() );
     }
     delete v_level;
   }
@@ -1129,9 +1154,10 @@ std::string LevelsManager::getQuickStartPackQuery(unsigned int i_qualityMIN, uns
 
   /* if xmoto run only 1 time, run the tutorial pack */
   bool v_tutorials = false;
-  v_result = i_db->readDB("SELECT nbStarts "
+  v_result = i_db->readDB("SELECT SUM(nbStarts) "
 			  "FROM stats_profiles "
-			  "WHERE id_profile=\"" + xmDatabase::protectString(i_profile) + "\";",
+			  "WHERE id_profile=\"" + xmDatabase::protectString(i_profile) + "\" "
+			  "GROUP BY id_profile;",
 			  nrow);
   if(nrow == 0) {
     v_tutorials = true;
@@ -1171,10 +1197,10 @@ std::string LevelsManager::getQuickStartPackQuery(unsigned int i_qualityMIN, uns
 			  "FROM levels AS a "
 			  "INNER JOIN weblevels AS b ON a.id_level = b.id_level "
 			  "LEFT OUTER JOIN levels_blacklist AS c ON (a.id_level = c.id_level AND c.id_profile = \"" + xmDatabase::protectString(i_profile) + "\") "
-			  "WHERE b.quality    <= " + v_qualityMAXStr.str()    + " "
-			  "AND   b.quality    >= " + v_qualityMINStr.str()    + " "
-			  "AND   b.difficulty <= " + v_difficultyMAXStr.str() + " "
-			  "AND   b.difficulty >= " + v_difficultyMINStr.str() + " "
+			  "WHERE b.quality+0    <= " + v_qualityMAXStr.str()    + " "
+			  "AND   b.quality+0    >= " + v_qualityMINStr.str()    + " "
+			  "AND   b.difficulty+0 <= " + v_difficultyMAXStr.str() + " "
+			  "AND   b.difficulty+0 >= " + v_difficultyMINStr.str() + " "
 			  "AND   xm_userCrappy(b.crappy)=0 "
 			  "AND   xm_userChildrenCompliant(b.children_compliant)=1 "
 			  "AND   c.id_level IS NULL "
@@ -1193,10 +1219,10 @@ std::string LevelsManager::getQuickStartPackQuery(unsigned int i_qualityMIN, uns
       "LEFT OUTER JOIN profile_completedLevels AS d "
       "ON (a.id_level=d.id_level AND d.id_profile=\"" + xmDatabase::protectString(i_profile) + "\") "
       "LEFT OUTER JOIN levels_blacklist AS e ON (a.id_level = e.id_level AND e.id_profile=\"" + xmDatabase::protectString(i_profile) + "\") "
-      "WHERE b.quality    <= " + v_qualityMAXStr.str()    + " "
-      "AND   b.quality    >= " + v_qualityMINStr.str()    + " "
-      "AND   b.difficulty <= " + v_difficultyMAXStr.str() + " "
-      "AND   b.difficulty >= " + v_difficultyMINStr.str() + " "
+      "WHERE b.quality+0    <= " + v_qualityMAXStr.str()    + " "
+      "AND   b.quality+0    >= " + v_qualityMINStr.str()    + " "
+      "AND   b.difficulty+0 <= " + v_difficultyMAXStr.str() + " "
+      "AND   b.difficulty+0 >= " + v_difficultyMINStr.str() + " "
       "AND   xm_userCrappy(b.crappy)=0 "
       "AND   xm_userChildrenCompliant(b.children_compliant)=1 "
       "AND   e.id_level IS NULL "
@@ -1217,3 +1243,11 @@ std::string LevelsManager::getQuickStartPackQuery(unsigned int i_qualityMIN, uns
       "GROUP BY a.id_level ORDER BY RANDOM();";
   }
 }
+
+void LevelsManager::lockLevelsPacks() {
+  SDL_LockMutex(m_levelsPackMutex);
+}
+
+void LevelsManager::unlockLevelsPacks() {
+  SDL_UnlockMutex(m_levelsPackMutex);
+}
