@@ -27,6 +27,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "helpers/Log.h"
 #include "xmscene/Level.h"
 #include "xmscene/Block.h"
+#include <sstream>
 
 luaL_reg LuaLibGame::m_gameFuncs[] = {
   {"GetTime",                      LuaLibGame::L_Game_GetTime},
@@ -35,6 +36,7 @@ luaL_reg LuaLibGame::m_gameFuncs[] = {
   {"MoveBlock",                    LuaLibGame::L_Game_MoveBlock},
   {"GetBlockPos",                  LuaLibGame::L_Game_GetBlockPos},
   {"SetBlockPos",                  LuaLibGame::L_Game_SetBlockPos},
+  {"SetPhysicsBlockPos",           LuaLibGame::L_Game_SetPhysicsBlockPos},
   {"PlaceInGameArrow",             LuaLibGame::L_Game_PlaceInGameArrow},
   {"PlaceScreenArrow",             LuaLibGame::L_Game_PlaceScreenArrow},
   {"HideArrow",                    LuaLibGame::L_Game_HideArrow},
@@ -57,6 +59,8 @@ luaL_reg LuaLibGame::m_gameFuncs[] = {
   {"SetDynamicEntityNone",         LuaLibGame::L_Game_SetDynamicEntityNone},
   {"SetDynamicBlockRotation",      LuaLibGame::L_Game_SetDynamicBlockRotation},
   {"SetDynamicBlockSelfRotation",  LuaLibGame::L_Game_SetDynamicBlockSelfRotation},
+  {"SetPhysicsBlockSelfRotation",  LuaLibGame::L_Game_SetPhysicsBlockSelfRotation},
+  {"SetPhysicsBlockTranslation",   LuaLibGame::L_Game_SetPhysicsBlockTranslation},
   {"SetDynamicBlockTranslation",   LuaLibGame::L_Game_SetDynamicBlockTranslation},
   {"SetDynamicBlockNone",          LuaLibGame::L_Game_SetDynamicBlockNone},
   {"CameraZoom",        	   LuaLibGame::L_Game_CameraZoom},
@@ -80,9 +84,12 @@ luaL_reg LuaLibGame::m_gameFuncs[] = {
   {"PlaySound",                    LuaLibGame::L_Game_PlaySound},
   {"PlayMusic",                    LuaLibGame::L_Game_PlayMusic},
   {"StopMusic",                    LuaLibGame::L_Game_StopMusic},
+  {"GetPlayerVelocity",            LuaLibGame::L_Game_GetPlayerVelocity},
+  {"GetPlayerSpeed",               LuaLibGame::L_Game_GetPlayerSpeed},
+  {"GetPlayerAngle",               LuaLibGame::L_Game_GetPlayerAngle},   
   {NULL, NULL}
 };
-MotoGame*     LuaLibGame::m_exec_world              = NULL;
+Scene*     LuaLibGame::m_exec_world              = NULL;
 InputHandler* LuaLibGame::m_exec_activeInputHandler = NULL;
 
 /*===========================================================================
@@ -99,14 +106,14 @@ lua_Number X_luaL_check_number(lua_State *L,int narg) {
   return d;    
 }
 
-LuaLibGame::LuaLibGame(MotoGame *i_pMotoGame) {
+LuaLibGame::LuaLibGame(Scene *i_pScene) {
   m_pL = lua_open();
   luaopen_base(m_pL);   
   luaopen_math(m_pL);
   luaopen_table(m_pL);
   luaL_openlib(m_pL, "Game", m_gameFuncs, 0);
 
-  m_pMotoGame           = i_pMotoGame;
+  m_pScene           = i_pScene;
   m_pActiveInputHandler = InputHandler::instance();
 }
 
@@ -115,7 +122,7 @@ LuaLibGame::~LuaLibGame() {
 }
 
 void LuaLibGame::setWorld() {
-  m_exec_world              = m_pMotoGame;
+  m_exec_world              = m_pScene;
   m_exec_activeInputHandler = m_pActiveInputHandler;
 }
 
@@ -285,18 +292,20 @@ std::string LuaLibGame::getErrorMsg() {
   ===========================================================================*/
 int LuaLibGame::L_Game_Log(lua_State *pL) {
   /* no event for this */
-  
+
   std::string Out;
 
-  for(int i=0; i<lua_gettop(pL); i++)
+  for(int i=0; i<args_numberOfArguments(pL); i++)
     Out.append(luaL_checkstring(pL, i+1));
 
-  Logger::Log((char *)Out.c_str());
+  LogInfo((char *)Out.c_str());
 
   return 0;    
 }
   
 int LuaLibGame::L_Game_ClearMessages(lua_State *pL) {
+  args_CheckNumberOfArguments(pL, 0);
+
   /* event for this */
   m_exec_world->createGameEvent(new MGE_ClearMessages(m_exec_world->getTime()));
   return 0;
@@ -327,6 +336,8 @@ int LuaLibGame::L_Game_HideArrow(lua_State *pL) {
 }
   
 int LuaLibGame::L_Game_GetTime(lua_State *pL) {
+  args_CheckNumberOfArguments(pL, 0);
+
   /* no event for this */
 
   /* Get current game time */
@@ -369,6 +380,16 @@ int LuaLibGame::L_Game_MoveBlock(lua_State *pL) {
 						 X_luaL_check_number(pL,3)));
   return 0;
 }
+
+int LuaLibGame::L_Game_SetPhysicsBlockPos(lua_State *pL) {
+  /* event for this */
+
+  m_exec_world->createGameEvent(new MGE_SetPhysicsBlockPos(m_exec_world->getTime(),
+						 luaL_checkstring(pL,1),
+						 X_luaL_check_number(pL,2),
+						 X_luaL_check_number(pL,3)));
+  return 0;
+}
   
 int LuaLibGame::L_Game_GetBlockPos(lua_State *pL) {
   /* no event for this */
@@ -485,7 +506,11 @@ int LuaLibGame::L_Game_SetKeyHook(lua_State *pL) {
   /* no event for this */
 
   if(m_exec_activeInputHandler != NULL) {
-    m_exec_activeInputHandler->addScriptKeyHook(m_exec_world,luaL_checkstring(pL,1),luaL_checkstring(pL,2));
+    try {
+      m_exec_activeInputHandler->addScriptKeyHook(m_exec_world, luaL_checkstring(pL,1), luaL_checkstring(pL,2));
+    } catch(Exception &e) {
+      m_exec_world->gameMessage("** Warning ** : addScriptKeyHook failed", true);      
+    }
   }
   return 0;
 }
@@ -494,7 +519,7 @@ int LuaLibGame::L_Game_GetKeyByAction(lua_State *pL) {
   /* no event for this */
 
   if(m_exec_activeInputHandler != NULL) {
-    lua_pushstring(pL,m_exec_activeInputHandler->getKeyByAction(luaL_checkstring(pL,1)).c_str());
+    lua_pushstring(pL,m_exec_activeInputHandler->getFancyKeyByAction(luaL_checkstring(pL,1)).c_str());
     return 1;
   }
   return 0;
@@ -540,7 +565,7 @@ int LuaLibGame::L_Game_SetDynamicEntitySelfRotation(lua_State *pL) {
 }
 
 int LuaLibGame::L_Game_SetDynamicEntityTranslation(lua_State *pL) {
-  /* event for this */    
+  /* event for this */
   m_exec_world->createGameEvent(new MGE_SetDynamicEntityTranslation(m_exec_world->getTime(),
 								    luaL_checkstring(pL,1),
 								    X_luaL_check_number(pL,2),
@@ -580,6 +605,29 @@ int LuaLibGame::L_Game_SetDynamicBlockSelfRotation(lua_State *pL) {
   return 0;
 }
 
+int LuaLibGame::L_Game_SetPhysicsBlockSelfRotation(lua_State *pL) {
+  /* event for this */
+  m_exec_world->createGameEvent(new MGE_SetPhysicsBlockSelfRotation(m_exec_world->getTime(),
+								    luaL_checkstring(pL,1),
+								    (int)(X_luaL_check_number(pL,2)),
+								    (int)X_luaL_check_number(pL,3),
+								    (int)X_luaL_check_number(pL,4))); 
+  return 0;
+}
+
+int LuaLibGame::L_Game_SetPhysicsBlockTranslation(lua_State* pL)
+{
+  m_exec_world->createGameEvent(new MGE_SetPhysicsBlockTranslation(m_exec_world->getTime(),
+								   luaL_checkstring(pL, 1),
+								   X_luaL_check_number(pL, 2),
+								   X_luaL_check_number(pL, 3),
+								   (int)X_luaL_check_number(pL, 4),
+								   (int)X_luaL_check_number(pL, 5),
+								   (int)X_luaL_check_number(pL, 6)));
+
+  return 0;
+}
+
 int LuaLibGame::L_Game_SetDynamicBlockTranslation(lua_State *pL) {
   /* event for this */    
   m_exec_world->createGameEvent(new MGE_SetDynamicBlockTranslation(m_exec_world->getTime(),
@@ -781,3 +829,57 @@ int LuaLibGame::L_Game_StopMusic(lua_State *pL) {
   m_exec_world->createGameEvent(new MGE_StopMusic(m_exec_world->getTime()));
   return 0;
 }
+
+int LuaLibGame::L_Game_GetPlayerVelocity(lua_State *pL) {
+  /* no event for this */
+  int v_player = (int)X_luaL_check_number(pL,1);
+
+  if(v_player < 0 || (unsigned int)v_player >= m_exec_world->Players().size()) {
+    throw Exception("Invalid player");
+  }
+  lua_pushnumber(pL, m_exec_world->Players()[v_player]->getBikeLinearVel());
+
+  return 1;
+}
+
+int LuaLibGame::L_Game_GetPlayerSpeed(lua_State *pL) {
+  /* no event for this */
+  int v_player = (int)X_luaL_check_number(pL,1);
+  
+  if(v_player < 0 || (unsigned int)v_player >= m_exec_world->Players().size()) {
+    throw Exception("Invalid player");
+  }
+  lua_pushnumber(pL, m_exec_world->Players()[v_player]->getBikeEngineSpeed());
+
+  return 1;
+}
+
+int LuaLibGame::L_Game_GetPlayerAngle(lua_State *pL) {
+  /* no event for this */
+  int v_player = (int)X_luaL_check_number(pL,1);
+  
+  if(v_player < 0 || (unsigned int)v_player >= m_exec_world->Players().size()) {
+    throw Exception("Invalid player");
+  }
+  lua_pushnumber(pL, m_exec_world->Players()[v_player]->getAngle());
+
+  return 1; //return 1 value
+}
+
+int LuaLibGame::args_numberOfArguments(lua_State *pL) {
+  return lua_gettop(pL);
+}
+
+void LuaLibGame::args_CheckNumberOfArguments(lua_State *pL, int i_from, int i_to) {
+  int n = args_numberOfArguments(pL);
+
+  if(i_to == -1) {
+    if(n != i_from) {
+      throw Exception("Invalid number of arguments");
+    }
+  } else {
+    if(n < i_from || n > i_to) {
+      throw Exception("Invalid number of arguments");
+    }
+  }
+}
