@@ -22,45 +22,14 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #define __LUALIBGAME_H__
 
 #include <string>
+#include "include/xm_lua.h"
 
-extern "C" {
-  #if defined(WIN32) 
-    #include "lua.h"
-    #include "lauxlib.h"
-    #include "lualib.h"
-  #else
-    #if defined(HAVE_LUA5_1_LUA_H)
-      #include <lua5.1/lua.h>
-      #include <lua5.1/lauxlib.h>
-      #include <lua5.1/lualib.h>
-    #elif defined(HAVE_LUA51_LUA_H)
-      #include <lua51/lua.h>
-      #include <lua51/lauxlib.h>
-      #include <lua51/lualib.h>      
-    #elif defined(HAVE_LUA50_LUA_H)
-      #include <lua50/lua.h>
-      #include <lua50/lauxlib.h>
-      #include <lua50/lualib.h>
-    #elif defined(HAVE_LUA_LUA_H)
-      #include <lua/lua.h>
-      #include <lua/lauxlib.h>
-      #include <lua/lualib.h>
-    #elif defined(HAVE_LUA_H)
-      #include <lua.h>
-      #include <lualib.h>
-      #include <lauxlib.h>
-    #else
-      #error Missing lua
-    #endif
-  #endif
-}
-
-class MotoGame;
+class Scene;
 class InputHandler;
 
 class LuaLibGame {
 public:
-  LuaLibGame(MotoGame *i_pMotoGame);
+  LuaLibGame(Scene *i_pScene);
   ~LuaLibGame();
 
   void loadScript(const std::string& i_scriptCode, const std::string& i_scriptFilename);
@@ -75,12 +44,12 @@ public:
 
 private:
   lua_State *m_pL;
-  MotoGame* m_pMotoGame;
+  Scene* m_pScene;
   InputHandler *m_pActiveInputHandler;
 
   static InputHandler* m_exec_activeInputHandler;
-  static MotoGame*     m_exec_world;
-  static luaL_reg            m_gameFuncs[];
+  static Scene*     m_exec_world;
+  static luaL_reg      m_gameFuncs[];
 
   /*
     static lua lib calls are share between the lulibgame instances
@@ -95,6 +64,7 @@ private:
   static int L_Game_MoveBlock(lua_State *pL);
   static int L_Game_GetBlockPos(lua_State *pL);
   static int L_Game_SetBlockPos(lua_State *pL);
+  static int L_Game_SetPhysicsBlockPos(lua_State *pL);
   static int L_Game_PlaceInGameArrow(lua_State *pL);
   static int L_Game_PlaceScreenArrow(lua_State *pL);
   static int L_Game_HideArrow(lua_State *pL);
@@ -117,6 +87,8 @@ private:
   static int L_Game_SetDynamicEntityNone(lua_State *pL);
   static int L_Game_SetDynamicBlockRotation(lua_State *pL);
   static int L_Game_SetDynamicBlockSelfRotation(lua_State *pL);
+  static int L_Game_SetPhysicsBlockSelfRotation(lua_State *pL);
+  static int L_Game_SetPhysicsBlockTranslation(lua_State* pL);
   static int L_Game_SetDynamicBlockTranslation(lua_State *pL);
   static int L_Game_SetDynamicBlockNone(lua_State *pL);
   static int L_Game_CameraZoom(lua_State *pL);
@@ -140,6 +112,16 @@ private:
   static int L_Game_PlaySound(lua_State *pL);
   static int L_Game_PlayMusic(lua_State *pL);
   static int L_Game_StopMusic(lua_State *pL);
+  static int L_Game_GetPlayerVelocity(lua_State *pL);
+  static int L_Game_GetPlayerSpeed(lua_State *pL);
+  static int L_Game_GetPlayerAngle(lua_State *pL);
+
+  /* get the number of arguments */
+  static int args_numberOfArguments(lua_State *pL);
+
+  /* arguments checks ; throw exception on failure */
+  static void args_CheckNumberOfArguments(lua_State *pL, int i_from, int i_to = -1);
+
 };
 
 
