@@ -25,6 +25,10 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "xmscene/Scene.h"
 #include "gui/basic/GUI.h"
 
+#ifdef ENABLE_OPENGL
+#include "include/xm_OpenGL.h"
+#endif
+
 class ParticlesSource;
 class Universe;
 class BlockVertex;
@@ -126,7 +130,7 @@ class GameRenderer : public Singleton<GameRenderer> {
 public:
   void init(DrawLib* i_drawLib); /* only called at start-up, and not per-level */
   void shutdown(void);
-  void render(MotoGame* i_scene);
+  void render(Scene* i_scene);
 
   void prepareForNewLevel(Universe* i_universe);
   void unprepareForNewLevel(void);
@@ -148,11 +152,18 @@ public:
   void setShowEngineCounter(bool i_value);
   bool showMinimap() const;
   void showReplayHelp(float p_speed, bool bAllowRewind);
-  void switchFollow(MotoGame* i_scene);
+  void switchFollow(Scene* i_scene);
+
+  void setShowGhostsText(bool i_value);
+  bool showGhostsText() const;
+
+  unsigned int currentRegistrationStage() const;
 
 private:
-  void renderMiniMap(MotoGame* i_scene, int x,int y,int nWidth,int nHeight);
+  void renderMiniMap(Scene* i_scene, int x,int y,int nWidth,int nHeight);
   void renderEngineCounter(int x,int y,int nWidth,int nHeight, float pSpeed, float pLinVel = -1);
+  
+  Vector2f calculateChangeDirPosition(Biker* i_biker, const Vector2f i_p);
       
   std::string getBestTime(void) {return m_bestTime;}
 
@@ -172,15 +183,13 @@ private:
 
   bool m_allowGhostEffect; // ask to not do the ghost effect
 
-  float m_fNextGhostInfoUpdate;
-  int m_nGhostInfoTrans;
-
   float m_previousEngineSpeed;
   float m_previousEngineLinVel;
 
   bool m_showMinimap;
   bool m_showEngineCounter;
   bool m_showTimePanel;
+  bool m_showGhostsText;
 
   /* FBO overlay */
   SFXOverlay m_Overlay;
@@ -200,6 +209,7 @@ private:
   float m_xScaleDefault;
   float m_yScaleDefault;
   float m_rotationAngleForTheFrame;
+  MiscSprite* m_arrowSprite;
 
   /* Subroutines */
 	
@@ -208,12 +218,13 @@ private:
   void calculateCameraScaleAndScreenAABB(Camera* pCamera, AABB& bbox);
 
   void renderEngineCounterNeedle(int nWidth, int nHeight, Vector2f center, float value);
+
+  void displayArrowIndication(Biker* i_biker, AABB *i_screenBBox);
+  bool getBikerDirection(Biker* i_biker, AABB* i_screenBBox, Vector2f* o_arrowPoint, float* o_arrowAngle, AABBSide* o_side);
 	
-	
-  void _RenderSprites(MotoGame* i_scene, bool bForeground,bool bBackground);
-  void _RenderSprite(MotoGame* i_scene, Entity *pSprite, float i_sizeMult = 1.0);
-  void _RenderBike(BikeState *pBike, BikeParameters *pBikeParms, BikerTheme *p_theme,
-		   bool i_renderBikeFront = true,
+  void _RenderSprites(Scene* i_scene, bool bForeground,bool bBackground);
+  void _RenderSprite(Scene* i_scene, Entity *pSprite, float i_sizeMult = 1.0);
+  void _RenderBike(Biker* i_biker, bool i_renderBikeFront = true,
 		   const TColor&  i_filterColor = TColor(255, 255, 255, 0),
 		   const TColor&  i_filterUglyColor = TColor(255, 255, 255, 0));
   void renderBodyPart(const Vector2f& i_from, const Vector2f& i_to,
@@ -223,29 +234,28 @@ private:
 		      float i_c41, float i_c42,
 		      Sprite *i_sprite,
 		      const TColor& i_filterColor,
-		      DriveDir i_direction,
+		      Biker* i_biker,
 		      int i_90_rotation = 0
 		      );
-  void _RenderBlocks(MotoGame* i_scene);
+  void _RenderBlocks(Scene* i_scene);
   void _RenderBlock(Block* block);
   void _RenderBlockEdges(Block* block);
-  void _RenderDynamicBlocks(MotoGame* i_scene, bool bBackground=false);
-  void _RenderBackground(MotoGame* i_scene);
-  void _RenderLayers(MotoGame* i_scene, bool renderFront);
-  void _RenderLayer(MotoGame* i_scene, int layer);
-  void _RenderSky(MotoGame* i_scene, float i_zoom, float i_offset, const TColor& i_color,
+  void _RenderDynamicBlocks(Scene* i_scene, bool bBackground=false);
+  void _RenderBackground(Scene* i_scene);
+  void _RenderLayers(Scene* i_scene, bool renderFront);
+  void _RenderLayer(Scene* i_scene, int layer);
+  void _RenderSky(Scene* i_scene, float i_zoom, float i_offset, const TColor& i_color,
 		  float i_driftZoom, const TColor &i_driftColor, bool i_drifted);
-  void _RenderGameMessages(MotoGame* i_scene); 
-  void _RenderGameStatus(MotoGame* i_scene);
-  void _RenderParticles(MotoGame* i_scene, bool bFront=true);
+  void _RenderGameMessages(Scene* i_scene); 
+  void _RenderGameStatus(Scene* i_scene);
+  void _RenderParticles(Scene* i_scene, bool bFront=true);
   void _RenderParticleDraw(Vector2f P,Texture *pTexture,float fSize,float fAngle, TColor c);
-  void _RenderParticle(MotoGame* i_scene, ParticlesSource *i_source);
-  void _RenderInGameText(Vector2f P,const std::string &Text,Color c = 0xffffffff);
+  void _RenderParticle(Scene* i_scene, ParticlesSource *i_source, unsigned int sprite=0);
+  void _RenderInGameText(Vector2f P,const std::string &Text,Color c = 0xffffffff, float i_xcentering = 0.0, float i_ycentering = 0.0);
   void _RenderZone(Zone *i_zone);
 
-  void _RenderGhost(MotoGame* i_scene, Biker* i_ghost, int i);
+  void _RenderGhost(Scene* i_scene, Biker* i_ghost, int i, float i_textOffset);
 
-  void _DrawRotatedMarker(Vector2f Pos,dReal *pfRot);     
   void _RenderDebugInfo(void);
 
   void _RenderAlphaBlendedSection(Texture *pTexture,const Vector2f &p0,const Vector2f &p1,const Vector2f &p2,const Vector2f &p3,
@@ -256,16 +266,16 @@ private:
   void _RenderCircle(unsigned int nSteps,Color CircleColor,const Vector2f &C,float fRadius);
   void _deleteGeoms(std::vector<Geom *>& geom, bool useFree=false);
 
-  void renderTimePanel(MotoGame* i_scene);
-  void renderReplayHelpMessage(MotoGame* i_scene);
+  void renderTimePanel(Scene* i_scene);
+  void renderReplayHelpMessage(Scene* i_scene);
 
   Texture* loadTexture(std::string textureName);
   Texture* loadTextureEdge(std::string textureName);
   int  edgeGeomExists(Block* pBlock, std::string texture);
   void initCameras(Universe* i_universe);
   int  loadBlock(Block* pBlock, Universe* i_universe, unsigned int currentScene, int sameSceneAs, int blockIndex);
-  int  loadBlockGeom(Block* pBlock, std::vector<Geom *>* pGeoms, Texture* pTexture, Vector2f Center, MotoGame* pScene);
-  int  loadBlockEdge(Block* pBlock, Vector2f Center, MotoGame* pScene);
+  int  loadBlockGeom(Block* pBlock, std::vector<Geom *>* pGeoms, Texture* pTexture, Vector2f Center, Scene* pScene);
+  int  loadBlockEdge(Block* pBlock, Vector2f Center, Scene* pScene);
   void calculateEdgePosition(Block* pBlock,
 			     BlockVertex* vertexA1,
 			     BlockVertex* vertexB1,
@@ -281,6 +291,10 @@ private:
 			    Vector2f B2,   Vector2f A2,
 			    Vector2f& ua1, Vector2f& ub1,
 			    Vector2f& ub2, Vector2f& ua2);
+
+  void beginTexturesRegistration();
+  void endTexturesRegistration();
+  unsigned int m_curRegistrationStage;
 };
 
 #endif
