@@ -31,13 +31,13 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include <time.h> 
 #include <ctime>
 #include <iostream>
-//#include <locale> /* can this safely be removed? */
 
 #include "helpers/Log.h"
 #include "helpers/SwapEndian.h"
 #include "GameEvents.h"
 #include "Game.h"
 #include "xmscene/Bike.h"
+#include "xmscene/Block.h"
 #include "db/xmDatabase.h"
 
   bool Replay::m_bEnableCompression = true;
@@ -99,7 +99,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
     /* Save it */
     FileHandle *pfh = FS::openOFile(std::string("Replays/") + m_FileName);
     if(pfh == NULL) {
-      Logger::Log("** Warning ** : Failed to open replay file for output: %s",(std::string("Replays/") + m_FileName).c_str());
+      LogWarning("Failed to open replay file for output: %s",(std::string("Replays/") + m_FileName).c_str());
       return;
     }        
     
@@ -189,7 +189,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
         /* Try adding a .rpl extension */
         pfh = FS::openIFile(std::string("Replays/") + FileName + std::string(".rpl"));
         if(pfh == NULL) {    
-          Logger::Log("** Warning ** : Failed to open replay file for input: %s",(std::string("Replays/") + FileName).c_str());
+          LogWarning("Failed to open replay file for input: %s",(std::string("Replays/") + FileName).c_str());
           throw Exception("Unable to open the replay");
         }
       }
@@ -198,32 +198,32 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
     /* Read header */
     int nVersion = FS::readByte(pfh); 
     if(bDisplayInformation) {
-      printf("%-30s: %i\n", "Replay file version", nVersion);
+      LogInfo("%-30s: %i\n", "Replay file version", nVersion);
     }
        
     /* Supported version? */
     if(nVersion != 0 && nVersion != 1) {
       FS::closeFile(pfh);
-      Logger::Log("** Warning ** : Unsupported replay file version (%d): %s",nVersion,(std::string("Replays/") + FileName).c_str());
-      throw Exception("Unable to open the replay");
+      LogWarning("Unsupported replay file version (%d): %s",nVersion,(std::string("Replays/") + FileName).c_str());
+      throw Exception("Unable to open the replay (unsupported version)");
     } else {
       /* Little/big endian safety check */
       if(FS::readInt_LE(pfh) != 0x12345678) {
         FS::closeFile(pfh);
-        Logger::Log("** Warning ** : Sorry, the replay you're trying to open are not endian-compatible with your computer!");
+        LogWarning("Sorry, the replay you're trying to open are not endian-compatible with your computer!");
         throw Exception("Unable to open the replay");        
       }
   
       /* Read level ID */
       m_LevelID = FS::readString(pfh);
       if(bDisplayInformation) {
-	printf("%-30s: %s\n", "Level Id", m_LevelID.c_str());
+	LogInfo("%-30s: %s\n", "Level Id", m_LevelID.c_str());
       }
 
       /* Read player name */
       Player = m_PlayerName = FS::readString(pfh);
       if(bDisplayInformation) {
-	printf("%-30s: %s\n", "Player", Player.c_str());
+	LogInfo("%-30s: %s\n", "Player", Player.c_str());
       }      
 
       /* Read replay frame rate */
@@ -232,7 +232,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
       /* Read state size */
       m_nStateSize = FS::readInt_LE(pfh);
       if(bDisplayInformation) {
-	printf("%-30s: %i\n", "State size", m_nStateSize);
+	LogInfo("%-30s: %i\n", "State size", m_nStateSize);
       } 
       
       /* Read finish time if any */
@@ -240,9 +240,9 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
       m_finishTime = GameApp::floatToTime(FS::readFloat_LE(pfh));
       if(bDisplayInformation) {
 	if(m_bFinished) {
-	  printf("%-30s: %.2f (%f)\n", "Finish time", m_finishTime / 100.0, m_finishTime / 100.0);
+	  LogInfo("%-30s: %.2f (%f)\n", "Finish time", m_finishTime / 100.0, m_finishTime / 100.0);
 	} else {
-	  printf("%-30s: %s\n", "Finish time", "unfinished");
+	  LogInfo("%-30s: %s\n", "Finish time", "unfinished");
 	}
       }
 
@@ -251,7 +251,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
         /* Read uncompressed size */
         m_nInputEventsDataSize = FS::readInt_LE(pfh);
 	if(bDisplayInformation) {
-	  printf("%-30s: %i\n", "Events data size", m_nInputEventsDataSize);
+	  LogInfo("%-30s: %i\n", "Events data size", m_nInputEventsDataSize);
 	} 
         m_pcInputEventsData = new char [m_nInputEventsDataSize];
         
@@ -260,7 +260,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
           /* Compressed */          
           int nCompressedEventsSize = FS::readInt_LE(pfh);
 	  if(bDisplayInformation) {
-	    printf("%-30s: %i\n", "Compressed events data size", nCompressedEventsSize);
+	    LogInfo("%-30s: %i\n", "Compressed events data size", nCompressedEventsSize);
 	  } 
           
           char *pcCompressedEvents = new char [nCompressedEventsSize];
@@ -275,7 +275,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
             delete [] m_pcInputEventsData; m_pcInputEventsData = NULL;
             FS::closeFile(pfh);
             _FreeReplay();
-            Logger::Log("** Warning ** : Failed to uncompress events in replay: %s",FileName.c_str());
+            LogWarning("Failed to uncompress events in replay: %s",FileName.c_str());
             throw Exception("Unable to open the replay");
           }
           
@@ -294,25 +294,25 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
       /* Read chunks */
       unsigned int nNumChunks = FS::readInt_LE(pfh);
       if(bDisplayInformation) {
-	printf("%-30s: %i\n", "Number of chunks", nNumChunks);
+	LogInfo("%-30s: %i\n", "Number of chunks", nNumChunks);
       }
       if(nNumChunks == 0) {
 	FS::closeFile(pfh);
 	_FreeReplay();
-	Logger::Log("** Warning ** : try to open a replay with no chunk");
+	LogWarning("try to open a replay with no chunk");
 	throw Exception("Replay with no chunk !");
       }
 
       for(unsigned int i=0;i<nNumChunks;i++) {
 	if(bDisplayInformation) {
-	  printf("Chunk %02i\n", i);
+	  LogInfo("Chunk %02i\n", i);
 	}  
   
         ReplayStateChunk* Chunk = new ReplayStateChunk();
         Chunk->nNumStates = FS::readInt_LE(pfh);
 
 	if(bDisplayInformation) {
-	  printf("   %-27s: %i\n", "Number of states", Chunk->nNumStates);
+	  LogInfo("   %-27s: %i\n", "Number of states", Chunk->nNumStates);
 	} 
 
         Chunk->pcChunkData = new char [Chunk->nNumStates * m_nStateSize];
@@ -320,13 +320,13 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
         /* Compressed or not compressed? */
         if(FS::readBool(pfh)) {
 	  if(bDisplayInformation) {
-	    printf("   %-27s: %s\n", "Compressed data", "true");
+	    LogInfo("   %-27s: %s\n", "Compressed data", "true");
 	  } 
 
           /* Compressed! - read compressed size */
           int nCompressedSize = FS::readInt_LE(pfh);
 	  if(bDisplayInformation) {
-	    printf("   %-27s: %i\n", "Compressed states size", nCompressedSize);
+	    LogInfo("   %-27s: %i\n", "Compressed states size", nCompressedSize);
 	  }
 
           /* Read compressed data */
@@ -338,7 +338,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
           uLongf nSrcLen = nCompressedSize;
           int nZRet = uncompress((Bytef *)Chunk->pcChunkData,&nDestLen,(Bytef *)pcCompressed,nSrcLen);
           if(nZRet != Z_OK || nDestLen != Chunk->nNumStates * m_nStateSize) {
-            Logger::Log("** Warning ** : Failed to uncompress chunk %d in replay: %s",i,FileName.c_str());
+            LogWarning("Failed to uncompress chunk %d in replay: %s",i,FileName.c_str());
             delete [] pcCompressed;
             Chunk->pcChunkData = NULL;
             FS::closeFile(pfh);
@@ -351,7 +351,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
         }
         else {
 	  if(bDisplayInformation) {
-	    printf("   %-27s: %s\n", "Compressed data", "false");
+	    LogInfo("   %-27s: %s\n", "Compressed data", "false");
 	  } 
 
           /* Not compressed! */
@@ -370,11 +370,11 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
     
     /* Reconstruct game events that are going to happen during the replay */
     if(bDisplayInformation) {
-      printf("%-30s:\n", "Game Events");
+      LogInfo("%-30s:\n", "Game Events");
     }
 
     /* unserialize events */
-    MotoGame::unserializeGameEvents(this, &m_ReplayEvents, bDisplayInformation);
+    Scene::unserializeGameEvents(this, &m_ReplayEvents, bDisplayInformation);
     initOutput(1024);
     for(unsigned int i=0; i<m_ReplayEvents.size(); i++) {
       m_ReplayEvents[i]->Event->serialize(*this);
@@ -382,7 +382,19 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     return m_LevelID;
   }
-      
+
+  void Replay::storeBlocks(const std::vector<Block *>& i_blocks) {
+     for(unsigned int i=0; i<i_blocks.size(); i++) {
+       if(i_blocks[i]->isPhysics() == true) {
+	 /* store this block in the .rpl if the position changed */
+	 // 
+	 // i_blocks[i]->Id();
+	 // i_blocks[i]->DynamicPosition();
+	 // i_blocks[i]->DynamicRotation();
+       }
+     }
+  }
+
   void Replay::storeState(const SerializedBikeState& state) {
     char *addr;
 
@@ -467,10 +479,10 @@ bool Replay::nextState(int p_frames) {
   return true;
 }
 
-  void Replay::loadState(BikeState* state) {
+  void Replay::loadState(BikeState* state, PhysicsSettings* i_physicsSettings) {
     /* (11th july, 2006) rasmus: i've swapped the two following lines, it apparently fixes
        interpolation in replays, but i don't know if it break something else */  
-    peekState(state);
+    peekState(state, i_physicsSettings);
 
     m_bEndOfFile = (m_nCurChunk       == m_Chunks.size()-1 && 
         (int)m_nCurState  == m_Chunks[m_nCurChunk]->nNumStates-1);
@@ -480,14 +492,14 @@ bool Replay::nextState(int p_frames) {
     }
   }
   
-  void Replay::peekState(BikeState* state) {
+  void Replay::peekState(BikeState* state, PhysicsSettings* i_physicsSettings) {
     SerializedBikeState v_bs;
 
     /* Like loadState() but this one does not advance the cursor... it just takes a peek */
     memcpy((char *)&v_bs, &m_Chunks[m_nCurChunk]->pcChunkData[((int)m_nCurState)*m_nStateSize], m_nStateSize);
     SwapEndian::LittleSerializedBikeState(v_bs);
 
-    BikeState::convertStateFromReplay(&v_bs, state);
+    BikeState::convertStateFromReplay(&v_bs, state, i_physicsSettings);
   }
   
   std::string Replay::giveAutomaticName() {
@@ -527,7 +539,7 @@ bool Replay::nextState(int p_frames) {
       /* Try opening it */
       FileHandle *pfh = FS::openIFile("Replays/" + p_ReplayName + ".rpl");
       if(pfh == NULL) {
-	pfh = FS::openIFile(p_ReplayName);
+	pfh = FS::openIFile(p_ReplayName, true);
 	if(pfh == NULL) {
 	  return NULL;
 	}
