@@ -22,7 +22,9 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #define __SYSMESSAGE_H__
 
 #include <string>
+#include <vector>
 #include "helpers/Singleton.h"
+#include "helpers/Color.h"
 
 class DrawLib;
 
@@ -35,12 +37,31 @@ private:
 
 public:
   void setDrawLib(DrawLib* i_drawLib);
-  void displayText(std::string i_msg);
+  void displayText(const std::string& i_msg);
+  void displayError(const std::string& i_msg);
+  void displayInformation(const std::string& i_msg);
+  void addConsoleLine(const std::string& i_line);
+  void showConsole();
   void render();
 
 private:
+  /* information message */
   std::string m_txt;
   float m_startDisplay;
+
+  /* error msg */
+  std::vector<std::string> m_errorTxt;
+  float m_startDisplayError;
+  void drawBoxMsg(float& io_nextStartTime, std::vector<std::string>& i_msg, Color i_color);
+
+  /* information msg */
+  std::vector<std::string> m_informationTxt;
+  float m_startDisplayInformation;
+
+  /* console */
+  std::vector<std::string> m_console;
+  float m_consoleLastShowTime;
+
   DrawLib* m_drawLib;
 };
 
