@@ -22,16 +22,17 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "helpers/VExcept.h"
 #include "VXml.h"
 #include "Game.h"
+#include "Renderer.h"
 #include "md5sum/md5file.h"
 #include "db/xmDatabase.h"
 #include "helpers/Log.h"
 #include "VFileIO.h"
 
-std::vector<Sprite*> Theme::getSpritesList() {
+std::vector<Sprite*>& Theme::getSpritesList() {
   return m_sprites;
 }
 
-std::vector<ThemeSound*> Theme::getSoundsList() {
+std::vector<ThemeSound*>& Theme::getSoundsList() {
   return m_sounds;
 }
 
@@ -81,8 +82,8 @@ std::string Theme::Name() const {
   return m_name;
 }
 
-Texture* Theme::loadTexture(std::string p_fileName, bool bSmall, bool bClamp, FilterMode eFilterMode) {
-  return m_texMan.loadTexture(p_fileName.c_str(), bSmall, bClamp, eFilterMode);
+Texture* Theme::loadTexture(std::string p_fileName, bool bSmall, bool bClamp, FilterMode eFilterMode, bool persistent, Sprite* associateSprite) {
+  return m_texMan.loadTexture(p_fileName.c_str(), bSmall, bClamp, eFilterMode, persistent, associateSprite);
 }
 
 std::vector<ThemeFile>* Theme::getRequiredFiles() {
@@ -90,11 +91,12 @@ std::vector<ThemeFile>* Theme::getRequiredFiles() {
 }
 
 void Theme::load(std::string p_themeFile) {
-  Logger::Log(std::string("Loading theme from file " + p_themeFile).c_str());
+  LogInfo(std::string("Loading theme from file " + p_themeFile).c_str());
 
   m_requiredFiles.clear();
 
-  cleanSprites(); /* removing existing sprites */
+  m_texMan.removeAssociatedSpritesFromTextures();
+  cleanSprites();
   cleanMusics();
   cleanSounds();
 
@@ -167,28 +169,52 @@ void Theme::loadSpritesFromXML(TiXmlElement *p_ThemeXmlDataElement) {
        so that all sprite type can be animated
     */
     pc = pVarElem->Attribute("fileBase");
-    v_isAnimation = pc != NULL;
+    v_isAnimation = (pc != NULL);
 
-    if(v_spriteType == "Entity" && v_isAnimation) {
+    if(v_spriteType == "BikerPart") {
+      newSpriteFromXML<BikerPartSprite>(pVarElem,
+					THEME_BIKERPART_SPRITE_FILE_DIR,
+					"BikerPart");
+    }
+    else if(v_spriteType == "Effect") {
+      newSpriteFromXML<EffectSprite>(pVarElem,
+				     THEME_EFFECT_SPRITE_FILE_DIR,
+				     "Effect");
+    }
+    else if(v_spriteType == "Font") {
+      newSpriteFromXML<FontSprite>(pVarElem,
+				   THEME_FONT_SPRITE_FILE_DIR,
+				   "Font");
+    }
+    else if(v_spriteType == "Misc") {
+      newSpriteFromXML<MiscSprite>(pVarElem,
+				   THEME_MISC_SPRITE_FILE_DIR,
+				   "Misc");
+    }
+    else if(v_spriteType == "Texture") {
+      newSpriteFromXML<TextureSprite>(pVarElem,
+				      THEME_TEXTURE_SPRITE_FILE_DIR,
+				      "Texture");
+    }
+    else if(v_spriteType == "UI") {
+      newSpriteFromXML<UISprite>(pVarElem,
+				 THEME_UI_SPRITE_FILE_DIR,
+				 "UI");
+    }
+
+    else if(v_spriteType == "Entity" && v_isAnimation == true) {
       newAnimationSpriteFromXML(pVarElem);
-    } else if(v_spriteType == "BikerPart") {
-      newBikerPartSpriteFromXML(pVarElem);
-    } else if(v_spriteType == "Entity" && v_isAnimation == false) {
+    }
+    else if(v_spriteType == "Entity" && v_isAnimation == false) {
       newDecorationSpriteFromXML(pVarElem);
-    } else if(v_spriteType == "Effect") {
-      newEffectSpriteFromXML(pVarElem);
-    } else if(v_spriteType == "Font") {
-      newFontSpriteFromXML(pVarElem);
-    } else if(v_spriteType == "Misc") {
-      newMiscSpriteFromXML(pVarElem);
-    } else if(v_spriteType == "Texture") {
-      newTextureSpriteFromXML(pVarElem);
-    } else if(v_spriteType == "UI") {
-      newUISpriteFromXML(pVarElem);
-    } else if(v_spriteType == "EdgeEffect") {
+    }
+
+    else if(v_spriteType == "EdgeEffect") {
       newEdgeEffectSpriteFromXML(pVarElem);
-    } else {
-      Logger::Log("Warning: unknown type '%s' in theme file !", v_spriteType.c_str());
+    }
+
+    else {
+      LogWarning("unknown type '%s' in theme file !", v_spriteType.c_str());
     }
   }
 
@@ -263,6 +289,15 @@ Sprite* Theme::getSprite(enum SpriteType pSpriteType, std::string pName) {
   return NULL;
 }
 
+std::string  Theme::getHashMusic(const std::string& i_key) {
+    unsigned int hash=0;
+    for(unsigned int i=0; i<i_key.size(); i++) {
+      hash += (unsigned int)(i_key[i]);
+    }
+    return m_musics[hash % m_musics.size()]->Name();
+}
+
+
 ThemeMusic* Theme::getMusic(std::string i_name) {
   for(unsigned int i=0; i<m_musics.size(); i++) {
     if(m_musics[i]->Name() == i_name) {
@@ -302,6 +337,47 @@ void Theme::cleanSounds() {
   m_sounds.clear(); 
 }
 
+
+template <typename SpriteType>
+void Theme::newSpriteFromXML(TiXmlElement *pVarElem,
+			     const char*   fileDir,
+			     const char*   spriteTypeName)
+{
+  std::string v_name;
+  std::string v_fileName;
+  const char *pc;
+  std::string v_sum;
+
+  pc = pVarElem->Attribute("name");
+  if(pc == NULL) {
+    LogWarning("%s with no name", spriteTypeName);
+    return;
+  }
+  v_name = pc;
+
+  pc = pVarElem->Attribute("file");
+  if(pc == NULL) {
+    LogWarning("%s with no file", spriteTypeName);
+    return;
+  }
+  v_fileName = pc;
+
+  pc = pVarElem->Attribute("sum");
+  if(pc != NULL)
+    v_sum = pc;
+
+  if(isAFileOutOfDate(fileDir + std::string("/") + v_fileName) == false) {
+    Sprite* pSprite = new SpriteType(this, v_name, v_fileName);
+    pSprite->setOrder(m_sprites.size());
+
+    m_sprites.push_back(pSprite);
+    
+    ThemeFile v_file = {fileDir + std::string("/") + v_fileName, v_sum};
+    m_requiredFiles.push_back(v_file);
+  }
+}
+
+
 void Theme::newAnimationSpriteFromXML(TiXmlElement *pVarElem) {
   std::string v_name;
   std::string v_fileBase;
@@ -316,15 +392,24 @@ void Theme::newAnimationSpriteFromXML(TiXmlElement *pVarElem) {
   float global_delay   = 0.1;
 
   pc = pVarElem->Attribute("name");
-  if(pc == NULL) {Logger::Log("** Animation with no name"); return;}
+  if(pc == NULL) {
+    LogWarning("Animation with no name");
+    return;
+  }
   v_name = pc;
   
   pc = pVarElem->Attribute("fileBase");
-  if(pc == NULL) {Logger::Log("** Animation with no fileBase"); return;}
+  if(pc == NULL) {
+    LogWarning("Animation with no fileBase");
+    return;
+  }
   v_fileBase = pc;
 
   pc = pVarElem->Attribute("fileExtension");
-  if(pc == NULL) {Logger::Log("** Animation with no fileExtension"); return;}
+  if(pc == NULL) {
+    LogWarning("Animation with no fileExtension");
+    return;
+  }
   v_fileExtension = pc;
 
   pc = pVarElem->Attribute("centerX");
@@ -343,6 +428,7 @@ void Theme::newAnimationSpriteFromXML(TiXmlElement *pVarElem) {
   if(pc != NULL) {global_delay = atof(pc);}
 
   v_anim = new AnimationSprite(this, v_name, v_fileBase, v_fileExtension);
+  v_anim->setOrder(m_sprites.size());
   m_sprites.push_back(v_anim);
 
   int n = 0;
@@ -377,12 +463,15 @@ void Theme::newAnimationSpriteFromXML(TiXmlElement *pVarElem) {
     if(pc != NULL) { v_sum = pc; };
 
     if(n < 100) {
-      sprintf(buf, "%02i", n);
+      snprintf(buf, 3, "%02i", n);
 
       if(isAFileOutOfDate(THEME_ANIMATION_SPRITE_FILE_DIR + std::string("/") +
 			  v_fileBase + std::string(buf) + std::string(".") + v_fileExtension) == false) {
 	v_anim->addFrame(v_centerX, v_centerY, v_width, v_height, v_delay);
 
+	std::string fileName = THEME_ANIMATION_SPRITE_FILE_DIR + std::string("/") +
+	  v_fileBase + std::string(buf) + std::string(".") + v_fileExtension;
+
 	ThemeFile v_file = {THEME_ANIMATION_SPRITE_FILE_DIR + std::string("/") +
 			    v_fileBase + std::string(buf) + std::string(".") + v_fileExtension,
 			    v_sum};
@@ -393,105 +482,96 @@ void Theme::newAnimationSpriteFromXML(TiXmlElement *pVarElem) {
   }
 }
 
-void Theme::newBikerPartSpriteFromXML(TiXmlElement *pVarElem) {
+void Theme::newDecorationSpriteFromXML(TiXmlElement *pVarElem)
+{
+  // create decoration sprites as an animation sprite with one frame
   std::string v_name;
   std::string v_fileName;
+  std::string v_fileBase;
+  std::string v_fileExtension;
+  std::string v_blendmode;
   const char *pc;
-  std::string v_sum;
-
-  pc = pVarElem->Attribute("name");
-  if(pc == NULL) {Logger::Log("** BikerPart with no name"); return;}
-  v_name = pc;
-
-  pc = pVarElem->Attribute("file");
-  if(pc == NULL) {Logger::Log("** BikerPart with no file"); return;}
-  v_fileName = pc;
-
-  pc = pVarElem->Attribute("sum");
-  if(pc != NULL) { v_sum = pc; };
-
-  if(isAFileOutOfDate(THEME_BIKERPART_SPRITE_FILE_DIR + std::string("/") + v_fileName) == false) {
-    m_sprites.push_back(new BikerPartSprite(this, v_name, v_fileName));
-    ThemeFile v_file = {THEME_BIKERPART_SPRITE_FILE_DIR + std::string("/") + v_fileName, v_sum};
-    m_requiredFiles.push_back(v_file);
-  }
-}
-
-void Theme::newDecorationSpriteFromXML(TiXmlElement *pVarElem) {
-  std::string v_name;
-  std::string v_fileName;
+  std::string v_sum = "";
+  AnimationSprite *v_anim;
   float v_width;
   float v_height;
   float v_centerX;
   float v_centerY;
-  std::string v_blendmode = "default";
-  const char *pc;
-  std::string v_sum;
+  float v_delay;
 
   float global_centerX = 0.5;
   float global_centerY = 0.5;
   float global_width   = 1.0;
   float global_height  = 1.0;
+  float global_delay   = 0.1;
 
   pc = pVarElem->Attribute("name");
-  if(pc == NULL) {Logger::Log("** Sprite with no name"); return;}
+  if(pc == NULL) {
+    LogWarning("Sprite with no name");
+    return;
+  }
   v_name = pc;
-
+  
   pc = pVarElem->Attribute("file");
-  if(pc == NULL) {Logger::Log("** Sprite with no name"); return;}
+  if(pc == NULL) {
+    LogWarning("Sprite with no name");
+    return;
+  }
   v_fileName = pc;
 
+  // split fileName in fileBase and fileExtension to mimic an animation sprite
+  unsigned int dotPos = v_fileName.find_last_of('.');
+  v_fileBase = v_fileName.substr(0, dotPos);
+  v_fileExtension = v_fileName.substr(dotPos+1);
+
   pc = pVarElem->Attribute("width");
-  if(pc != NULL) {v_width = atof(pc);} else {v_width = global_width;}
+  if(pc != NULL)
+    v_width = atof(pc);
+  else
+    v_width = global_width;
 
   pc = pVarElem->Attribute("height");
-  if(pc != NULL) {v_height = atof(pc);} else {v_height = global_height;}
+  if(pc != NULL)
+    v_height = atof(pc);
+  else
+    v_height = global_height;
 
   pc = pVarElem->Attribute("centerX");
-  if(pc != NULL) {v_centerX = atof(pc);} else {v_centerX = global_centerX;}
+  if(pc != NULL)
+    v_centerX = atof(pc);
+  else
+    v_centerX = global_centerX;
 
   pc = pVarElem->Attribute("centerY");
-  if(pc != NULL) {v_centerY = atof(pc);} else {v_centerY = global_centerY;}
-  
+  if(pc != NULL)
+    v_centerY = atof(pc);
+  else
+    v_centerY = global_centerY;
+
   pc = pVarElem->Attribute("blendmode");
-  if(pc != NULL) v_blendmode = pc;
+  if(pc != NULL)
+    v_blendmode = pc;
+  else
+    v_blendmode = "default";
 
   pc = pVarElem->Attribute("sum");
-  if(pc != NULL) { v_sum = pc; };
+  if(pc != NULL)
+    v_sum = pc;
 
-  if(isAFileOutOfDate(THEME_DECORATION_SPRITE_FILE_DIR + std::string("/") + v_fileName) == false) {
-    m_sprites.push_back(new DecorationSprite(this, v_name, v_fileName,
-					     v_width,
-					     v_height,
-					     v_centerX,
-					     v_centerY,
-					     strToBlendMode(v_blendmode)
-					     ));
-    ThemeFile v_file = {THEME_DECORATION_SPRITE_FILE_DIR + std::string("/") + v_fileName, v_sum};
-    m_requiredFiles.push_back(v_file);
-  }
-}
+  v_delay = global_delay;
 
-void Theme::newEffectSpriteFromXML(TiXmlElement *pVarElem) {
-  std::string v_name;
-  std::string v_fileName;
-  const char *pc;
-  std::string v_sum;
+  v_anim = new AnimationSprite(this, v_name, v_fileBase, v_fileExtension);
+  v_anim->setBlendMode(strToBlendMode(v_blendmode));
+  v_anim->setOrder(m_sprites.size());
 
-  pc = pVarElem->Attribute("name");
-  if(pc == NULL) {Logger::Log("** Effect with no name"); return;}
-  v_name = pc;
+  m_sprites.push_back(v_anim);
 
-  pc = pVarElem->Attribute("file");
-  if(pc == NULL) {Logger::Log("** Effect with no file"); return;}
-  v_fileName = pc;
+  if(isAFileOutOfDate(THEME_ANIMATION_SPRITE_FILE_DIR + std::string("/") +
+		      v_fileBase + std::string(".") + v_fileExtension) == false) {
+    v_anim->addFrame(v_centerX, v_centerY, v_width, v_height, v_delay);
 
-  pc = pVarElem->Attribute("sum");
-  if(pc != NULL) { v_sum = pc; };
-
-  if(isAFileOutOfDate(THEME_EFFECT_SPRITE_FILE_DIR + std::string("/") + v_fileName) == false) {
-    m_sprites.push_back(new EffectSprite(this, v_name, v_fileName));
-    ThemeFile v_file = {THEME_EFFECT_SPRITE_FILE_DIR + std::string("/") + v_fileName, v_sum};
+    ThemeFile v_file = {THEME_ANIMATION_SPRITE_FILE_DIR + std::string("/") +
+			v_fileBase + std::string(".") + v_fileExtension, v_sum};
     m_requiredFiles.push_back(v_file);
   }
 }
@@ -505,128 +585,50 @@ void Theme::newEdgeEffectSpriteFromXML(TiXmlElement *pVarElem) {
   std::string v_sum;
 
   pc = pVarElem->Attribute("name");
-  if(pc == NULL) {Logger::Log("** Edge with no name"); return;}
+  if(pc == NULL) {
+    LogWarning("Edge with no name");
+    return;
+  }
   v_name = pc;
 
   pc = pVarElem->Attribute("file");
-  if(pc == NULL) {Logger::Log("** Edge with no file"); return;}
+  if(pc == NULL) {
+    LogWarning("Edge with no file");
+    return;
+  }
   v_fileName = pc;
 
   pc = pVarElem->Attribute("scale");
-  if(pc == NULL) {Logger::Log("** Edge with no scale"); return;}
+  if(pc == NULL) {
+    LogWarning("Edge with no scale");
+    return;
+  }
   v_scale = pc;
 
   pc = pVarElem->Attribute("depth");
-  if(pc == NULL) {Logger::Log("Edge with no depth"); return;}
+  if(pc == NULL) {
+    LogWarning("Edge with no depth");
+    return;
+  }
   v_depth = pc;
 
   pc = pVarElem->Attribute("sum");
   if(pc != NULL) { v_sum = pc; };
 
   if(isAFileOutOfDate(THEME_EDGEEFFECT_SPRITE_FILE_DIR + std::string("/") + v_fileName) == false) {
-    m_sprites.push_back(new EdgeEffectSprite(this, v_name, v_fileName,
-					     atof(v_scale.c_str()),
-					     atof(v_depth.c_str())));
+    Sprite* pSprite = new EdgeEffectSprite(this, v_name, v_fileName,
+					   atof(v_scale.c_str()),
+					   atof(v_depth.c_str()));
+    pSprite->setOrder(m_sprites.size());
+
+    m_sprites.push_back(pSprite);
+
     ThemeFile v_file = {THEME_EDGEEFFECT_SPRITE_FILE_DIR + std::string("/") + v_fileName, v_sum};
     m_requiredFiles.push_back(v_file);
   }
 }
 
-void Theme::newFontSpriteFromXML(TiXmlElement *pVarElem) {
-  std::string v_name;
-  std::string v_fileName;
-  const char *pc;
-  std::string v_sum;
 
-  pc = pVarElem->Attribute("name");
-  if(pc == NULL) {Logger::Log("** Font with no name"); return;}
-  v_name = pc;
-
-  pc = pVarElem->Attribute("file");
-  if(pc == NULL) {Logger::Log("Font with no file"); return;}
-  v_fileName = pc;
-
-  pc = pVarElem->Attribute("sum");
-  if(pc != NULL) { v_sum = pc; };
-
-  if(isAFileOutOfDate(THEME_FONT_SPRITE_FILE_DIR + std::string("/") + v_fileName) == false) {
-    m_sprites.push_back(new FontSprite(this, v_name, v_fileName));
-    ThemeFile v_file = {THEME_FONT_SPRITE_FILE_DIR + std::string("/") + v_fileName, v_sum};
-    m_requiredFiles.push_back(v_file);
-  }
-}
-
-void Theme::newMiscSpriteFromXML(TiXmlElement *pVarElem) {
-  std::string v_name;
-  std::string v_fileName;
-  const char *pc;
-  std::string v_sum;
-
-  pc = pVarElem->Attribute("name");
-  if(pc == NULL) {Logger::Log("** Misc with no name"); return;}
-  v_name = pc;
-
-  pc = pVarElem->Attribute("file");
-  if(pc == NULL) {Logger::Log("** Misc with no file"); return;}
-  v_fileName = pc;
-
-  pc = pVarElem->Attribute("sum");
-  if(pc != NULL) { v_sum = pc; };
-
-  if(isAFileOutOfDate(THEME_MISC_SPRITE_FILE_DIR + std::string("/") + v_fileName) == false) {
-    m_sprites.push_back(new MiscSprite(this, v_name, v_fileName));
-    ThemeFile v_file = {THEME_MISC_SPRITE_FILE_DIR + std::string("/") + v_fileName, v_sum};
-    m_requiredFiles.push_back(v_file);
-  }
-}
-
-void Theme::newUISpriteFromXML(TiXmlElement *pVarElem) {
-  std::string v_name;
-  std::string v_fileName;
-  const char *pc;
-  std::string v_sum;
-
-  pc = pVarElem->Attribute("name");
-  if(pc == NULL) {Logger::Log("** UI with no name"); return;}
-  v_name = pc;
-
-  pc = pVarElem->Attribute("file");
-  if(pc == NULL) {Logger::Log("** UI with no file"); return;}
-  v_fileName = pc;
-
-  pc = pVarElem->Attribute("sum");
-  if(pc != NULL) { v_sum = pc; };
-
-  if(isAFileOutOfDate(THEME_UI_SPRITE_FILE_DIR + std::string("/") + v_fileName) == false) {
-    m_sprites.push_back(new UISprite(this, v_name, v_fileName));
-    ThemeFile v_file = {THEME_UI_SPRITE_FILE_DIR + std::string("/") + v_fileName, v_sum};
-    m_requiredFiles.push_back(v_file);
-  }
-}
-
-void Theme::newTextureSpriteFromXML(TiXmlElement *pVarElem) {
-  std::string v_name;
-  std::string v_fileName;
-  const char *pc;
-  std::string v_sum;
-
-  pc = pVarElem->Attribute("name");
-  if(pc == NULL) {Logger::Log("** Texture with no name"); return;}
-  v_name = pc;
-
-  pc = pVarElem->Attribute("file");
-  if(pc == NULL) {Logger::Log("** Texture with no file"); return;}
-  v_fileName = pc;
-
-  pc = pVarElem->Attribute("sum");
-  if(pc != NULL) { v_sum = pc; };
-
-  if(isAFileOutOfDate(THEME_TEXTURE_SPRITE_FILE_DIR + std::string("/") + v_fileName) == false) {
-    m_sprites.push_back(new TextureSprite(this, v_name, v_fileName));
-    ThemeFile v_file = {THEME_TEXTURE_SPRITE_FILE_DIR + std::string("/") + v_fileName, v_sum};
-    m_requiredFiles.push_back(v_file);
-  }
-}
 
 BikerTheme* Theme::getPlayerTheme() {
   return m_player;
@@ -640,6 +642,7 @@ Sprite::Sprite(Theme* p_associated_theme, std::string v_name) {
   m_associated_theme = p_associated_theme;
   m_name = v_name;
   m_blendmode = SPRITE_BLENDMODE_DEFAULT;
+  m_persistent = false;
 }
 
 Sprite::~Sprite() {
@@ -653,13 +656,23 @@ Texture* Sprite::getTexture(bool bSmall, bool bClamp, FilterMode eFilterMode) {
     v_currentTexture = m_associated_theme->loadTexture(getCurrentTextureFileName(),
 						       bSmall,
 						       bClamp,
-						       eFilterMode);
+						       eFilterMode,
+						       m_persistent,
+						       this);
     setCurrentTexture(v_currentTexture);
   }
-  
+
+  if(m_persistent == false)
+    v_currentTexture->curRegistrationStage = GameRenderer::instance()->currentRegistrationStage();
+
   return v_currentTexture;
 }
 
+void Sprite::setOrder(unsigned int order)
+{
+  m_order = order;
+}
+
 SpriteBlendMode Sprite::getBlendMode() {
   return m_blendmode;
 }
@@ -672,12 +685,15 @@ std::string Sprite::getFileDir() {
   return THEME_SPRITE_FILE_DIR;
 }
 
+
+
 AnimationSprite::AnimationSprite(Theme* p_associated_theme, std::string p_name, std::string p_fileBase, std::string p_fileExtention) : Sprite(p_associated_theme, p_name) {
   m_current_frame = 0;
   m_fileBase      = p_fileBase;
   m_fileExtension = p_fileExtention;
   m_fFrameTime    = 0.0;
   m_type          = SPRITE_TYPE_ANIMATION;
+  m_animation     = false;
 }
 
 AnimationSprite::~AnimationSprite() {
@@ -689,15 +705,22 @@ std::string AnimationSprite::getFileDir() {
 }
 
 Texture* AnimationSprite::getCurrentTexture() {
-  if(m_frames.size() == 0) {return NULL;}
+  if(m_frames.size() == 0) {
+    return NULL;
+  }
+  
   return m_frames[getCurrentFrame()]->getTexture();
 }
 
 std::string AnimationSprite::getCurrentTextureFileName() {
-  char v_num[3];
-  snprintf(v_num, 3, "%02i", getCurrentFrame() % 100); // max 100 frames by animation
+  if(m_animation == false){
+    return getFileDir() + "/" + m_fileBase + std::string(".") + m_fileExtension;
+  }else{
+    char v_num[3];
+    snprintf(v_num, 3, "%02i", getCurrentFrame() % 100); // max 100 frames by animation
 
-  return getFileDir() + "/" + m_fileBase + std::string(v_num) + std::string(".") + m_fileExtension;
+    return getFileDir() + "/" + m_fileBase + std::string(v_num) + std::string(".") + m_fileExtension;
+  }
 }
 
 void AnimationSprite::setCurrentTexture(Texture *p_texture) {
@@ -705,16 +728,21 @@ void AnimationSprite::setCurrentTexture(Texture *p_texture) {
 }
 
 int AnimationSprite::getCurrentFrame() {
-  /* Next frame? */
-  float v_realTime = GameApp::getXMTime();
+  if(m_animation == false){
+    return 0;
+  }else{
+    /* Next frame? */
+    float v_realTime = GameApp::getXMTime();
 
-  while(v_realTime > m_fFrameTime + m_frames[m_current_frame]->getDelay()) {
-    m_fFrameTime = v_realTime;
-    m_current_frame++;
-    if(m_current_frame == m_frames.size()) m_current_frame = 0;      
+    while(v_realTime > m_fFrameTime + m_frames[m_current_frame]->getDelay()) {
+      m_fFrameTime = v_realTime;
+      m_current_frame++;
+      if(m_current_frame == m_frames.size())
+	m_current_frame = 0;      
+    }
+
+    return m_current_frame;
   }
-
-  return m_current_frame;
 }
 
 float AnimationSprite::getCenterX() {
@@ -735,6 +763,8 @@ float AnimationSprite::getHeight() {
 
 void AnimationSprite::addFrame(float p_centerX, float p_centerY, float p_width, float p_height, float p_delay) {
   m_frames.push_back(new AnimationSpriteFrame(this, p_centerX, p_centerY, p_width, p_height, p_delay));
+  if(m_frames.size() > 1)
+    m_animation = true;
 }
 
 void AnimationSprite::cleanFrames() {
@@ -744,6 +774,36 @@ void AnimationSprite::cleanFrames() {
   m_frames.clear();
 }
 
+void AnimationSprite::loadTextures()
+{
+  unsigned int saveCurFrame = m_current_frame;
+
+  // reset frameTime so that getCurrentFrame does not increment it
+  m_fFrameTime = GameApp::getXMTime();
+
+  for(unsigned int i=0; i<m_frames.size(); i++) {
+    m_current_frame = i;
+    getTexture();
+  }
+
+  m_current_frame = saveCurFrame;
+}
+
+void AnimationSprite::invalidateTextures()
+{
+  unsigned int saveCurFrame = m_current_frame;
+
+  // reset frameTime so that getCurrentFrame does not increment it
+  m_fFrameTime = GameApp::getXMTime();
+
+  for(unsigned int i=0; i<m_frames.size(); i++) {
+    m_current_frame = i;
+    setCurrentTexture(NULL);
+  }
+
+  m_current_frame = saveCurFrame;
+}
+
 AnimationSpriteFrame::AnimationSpriteFrame(AnimationSprite *p_associatedAnimationSprite,
              float p_centerX,
              float p_centerY,
@@ -793,6 +853,7 @@ float AnimationSpriteFrame::getDelay() const {
 
 BikerPartSprite::BikerPartSprite(Theme* p_associated_theme, std::string p_name, std::string p_fileName) : SimpleFrameSprite(p_associated_theme, p_name, p_fileName) {
   m_type = SPRITE_TYPE_BIKERPART;
+  m_persistent = true;
 }
 
 BikerPartSprite::~BikerPartSprite() {
@@ -802,39 +863,6 @@ std::string BikerPartSprite::getFileDir() {
   return THEME_BIKERPART_SPRITE_FILE_DIR;
 }
 
-DecorationSprite::DecorationSprite(Theme* p_associated_theme, std::string p_name, std::string p_fileName, float p_width, float p_height, float p_centerX, float p_centerY, SpriteBlendMode BlendMode) : SimpleFrameSprite(p_associated_theme, p_name, p_fileName) {
-  m_width   = p_width;
-  m_height  = p_height;
-  m_centerX = p_centerX;
-  m_centerY = p_centerY;
-  m_type    = SPRITE_TYPE_DECORATION;
-  
-  setBlendMode(BlendMode);
-}
-
-DecorationSprite::~DecorationSprite() {
-}
-
-std::string DecorationSprite::getFileDir() {
-  return THEME_DECORATION_SPRITE_FILE_DIR;
-}
-
-float DecorationSprite::getWidth() {
-  return m_width;
-}
-
-float DecorationSprite::getHeight() {
-  return m_height;
-}
-
-float DecorationSprite::getCenterX() {
-  return m_centerX;
-}
-
-float DecorationSprite::getCenterY() {
-  return m_centerY;
-}
-
 EffectSprite::EffectSprite(Theme* p_associated_theme, std::string p_name, std::string p_fileName) : SimpleFrameSprite(p_associated_theme, p_name, p_fileName) {
   m_type = SPRITE_TYPE_EFFECT;
 }
@@ -873,6 +901,7 @@ Texture* EdgeEffectSprite::getTexture(bool bSmall, bool bClamp, FilterMode eFilt
 
 FontSprite::FontSprite(Theme* p_associated_theme, std::string p_name, std::string p_fileName) : SimpleFrameSprite(p_associated_theme, p_name, p_fileName) {
   m_type = SPRITE_TYPE_FONT;
+  m_persistent = true;
 }
 
 FontSprite::~FontSprite() {
@@ -884,6 +913,7 @@ std::string FontSprite::getFileDir() {
 
 MiscSprite::MiscSprite(Theme* p_associated_theme, std::string p_name, std::string p_fileName) : SimpleFrameSprite(p_associated_theme, p_name, p_fileName) {
   m_type = SPRITE_TYPE_MISC;
+  m_persistent = true;
 }
 
 MiscSprite::~MiscSprite() {
@@ -895,6 +925,7 @@ std::string MiscSprite::getFileDir() {
 
 UISprite::UISprite(Theme* p_associated_theme, std::string p_name, std::string p_fileName) : SimpleFrameSprite(p_associated_theme, p_name, p_fileName) {
   m_type = SPRITE_TYPE_UI;
+  m_persistent = true;
 }
 
 UISprite::~UISprite() {
@@ -923,7 +954,18 @@ SimpleFrameSprite::SimpleFrameSprite(Theme* p_associated_theme, std::string p_na
 SimpleFrameSprite::~SimpleFrameSprite() {
 }
 
-Texture* SimpleFrameSprite::getCurrentTexture() {
+void SimpleFrameSprite::loadTextures()
+{
+  getTexture();
+}
+
+void SimpleFrameSprite::invalidateTextures()
+{
+  setCurrentTexture(NULL);
+}
+
+Texture* SimpleFrameSprite::getCurrentTexture()
+{
   return m_texture;
 }
 
@@ -1022,7 +1064,7 @@ void ThemeChoicer::initThemesFromDir(xmDatabase *i_db) {
       if(i_db->themes_exists(v_name) == false) {
 	i_db->themes_add(v_name, v_themesFiles[i]);
       } else {
-	Logger::Log(std::string("Theme " + v_name + " is present several times").c_str());
+	LogWarning(std::string("Theme " + v_name + " is present several times").c_str());
       }
     } catch(Exception &e) {
       /* anyway, give up this theme */
