@@ -30,6 +30,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   #include <dirent.h>
 #endif
 #include <sys/stat.h>
+#include <stdarg.h>
 
 #include "helpers/VExcept.h"
 #include "helpers/Log.h"
@@ -73,7 +74,7 @@ std::string win32_getUserDir(bool i_asUtf8 = false) {
 }
 #endif
 
-#ifndef WIN32
+#if !defined(WIN32) && !defined(__MORPHOS__) && !defined(__amigaos4__)
 void strlwr(char *pc) {
   for(unsigned int i=0; i<strlen(pc); i++) pc[i] = tolower(pc[i]);
 }
@@ -146,7 +147,7 @@ bool str_match_wildcard(char *pcMWildcard,char *pcMString, bool CaseSensitive) {
 
 void FS::_ThrowFileError(FileHandle *pfh,std::string Description) {
   char cBuf[512];
-  sprintf(cBuf,"%s (%s%s): %s",pfh->Name.c_str(),pfh->bRead?"I":"",pfh->bWrite?"O":"",Description.c_str());
+  snprintf(cBuf, 512, "%s (%s%s): %s", pfh->Name.c_str(),pfh->bRead?"I":"", pfh->bWrite?"O":"", Description.c_str());
   throw Exception(cBuf);
 }
 
@@ -458,7 +459,7 @@ FileHandle *FS::openOFile(std::string Path) {
   
 FileHandle *FS::openIFile(std::string Path, bool i_includeCurrentDir) {
   FileHandle *pfh = new FileHandle;
-    
+
   /* Okay. Absolute path? */
   if(isPathAbsolute(Path)) {      
     /* Yes ma'am */
@@ -468,13 +469,15 @@ FileHandle *FS::openIFile(std::string Path, bool i_includeCurrentDir) {
     /* user dir ? */
     pfh->fp = fopen((m_UserDir + std::string("/") + Path).c_str(),"rb");        
 
+    /* data dir ? */
     if(pfh->fp == NULL) {
-      if(m_bGotDataDir) { /* data dir ? */
+      if(m_bGotDataDir) {
 	pfh->fp = fopen((m_DataDir + std::string("/") + Path).c_str(),"rb");        
       }
 
+      /* current working dir ? */
       if(pfh->fp == NULL) {
-	if(i_includeCurrentDir) { /* current working dir ? */
+	if(i_includeCurrentDir) {
 	  pfh->fp = fopen(Path.c_str(),"rb");
 	}
       }
@@ -519,6 +522,7 @@ FileHandle *FS::openIFile(std::string Path, bool i_includeCurrentDir) {
     pfh->nSize = ftell(pfh->fp);
     fseek(pfh->fp,0,SEEK_SET);        
   }
+
   return pfh;
 }
     
@@ -737,7 +741,7 @@ void FS::writeBool(FileHandle *pfh,bool v) {
   
 void FS::writeLine(FileHandle *pfh,std::string Line) {
   char cBuf[512];
-  sprintf(cBuf,"%s\r\n",Line.c_str());
+  snprintf(cBuf, 512, "%s\r\n", Line.c_str());
   if(!writeBuf(pfh,cBuf,strlen(cBuf))) _ThrowFileError(pfh,"writeLine -> failed");
 }
   
@@ -745,9 +749,9 @@ void FS::writeLineF(FileHandle *pfh,char *pcFmt,...) {
   char cBuf[512],cBuf2[512];
   va_list List;
   va_start(List,pcFmt);
-  vsprintf(cBuf,pcFmt,List);
+  vsnprintf(cBuf, 512, pcFmt, List);
   va_end(List);
-  sprintf(cBuf2,"%s\r\n",cBuf);
+  snprintf(cBuf2, 512, "%s\r\n", cBuf);
   if(!writeBuf(pfh,cBuf2,strlen(cBuf2))) _ThrowFileError(pfh,"writeLineF -> failed");
 }
     
@@ -919,7 +923,7 @@ void FS::deleteFile(const std::string &File) {
 bool FS::copyFile(const std::string &From,const std::string &To, std::string &To_really_done) {
   /* All file copying must happen inside the user directory... */
   if(m_UserDir == "") {
-    Logger::Log("** Warning ** : No user directory, can't copy file '%s' to '%s'",From.c_str(),To.c_str());
+    LogWarning("No user directory, can't copy file '%s' to '%s'",From.c_str(),To.c_str());
     return false;
   }
 
@@ -946,7 +950,7 @@ bool FS::copyFile(const std::string &From,const std::string &To, std::string &To
     int i = 1;
     while(1) {
       char cTemp[1024];
-      sprintf(cTemp,"%s (%d)%s",FullToBaseName.c_str(),i,FullToExt.c_str());
+      snprintf(cTemp, 1024, "%s (%d)%s", FullToBaseName.c_str(), i, FullToExt.c_str());
         
       /* What about this file then? */
       fp = fopen(cTemp,"rb");
@@ -995,7 +999,7 @@ bool FS::copyFile(const std::string &From,const std::string &To, std::string &To
 	fclose(in);
 	fclose(out);
 	remove(FullTo.c_str());
-	Logger::Log("** Warning ** : Failed to copy all of '%s' to '%s'",FullFrom.c_str(),FullTo.c_str());
+	LogWarning("Failed to copy all of '%s' to '%s'",FullFrom.c_str(),FullTo.c_str());
 	return false;
       }
         
@@ -1003,14 +1007,14 @@ bool FS::copyFile(const std::string &From,const std::string &To, std::string &To
     }
     else {
       fclose(in);
-      Logger::Log("** Warning ** : Failed to open file for output: %s",FullTo.c_str());
+      LogWarning("Failed to open file for output: %s",FullTo.c_str());
       return false;      
     }
       
     fclose(in);
   }
   else {
-    Logger::Log("** Warning ** : Failed to open file for input: %s",FullFrom.c_str());
+    LogWarning("Failed to open file for input: %s",FullFrom.c_str());
     return false;
   }
     
@@ -1063,7 +1067,24 @@ void FS::init(const std::string& AppDir, const std::string& i_binFile, const std
     m_bGotDataDir = true;
   } 
   else throw Exception("invalid process directory");
-           
+#elif defined(__MORPHOS__) || defined(__amigaos4__)         
+  if(i_userDirPath != "") {
+    m_UserDir = m_UserDirUTF8 = i_userDirPath;
+  } else {
+    m_UserDir = "PROGDIR:userdata"; /*getenv("HOME");*/
+    if(!isDir(m_UserDir))
+      throw Exception("invalid user home directory");
+
+    m_UserDir = m_UserDir /*+ std::string("/.") + AppDir*/;
+    m_UserDirUTF8 = m_UserDir;
+  }
+
+  /* And the data dir? */
+  m_DataDir = std::string("PROGDIR:data");
+  if(isDir(m_DataDir)) {
+    /* Got a system-wide installation to fall back to! */
+    m_bGotDataDir = true;
+  }
 #else /* Assume unix-like */
       /* Determine users home dir, so we can find out where to get/save user 
 	 files */
@@ -1071,10 +1092,10 @@ void FS::init(const std::string& AppDir, const std::string& i_binFile, const std
   if(i_userDirPath != "") {
     m_UserDir = m_UserDirUTF8 = i_userDirPath;
   } else {
-    m_UserDir = getenv("HOME");            
+    m_UserDir = getenv("HOME");
     if(!isDir(m_UserDir)) 
       throw Exception("invalid user home directory");
-      
+
     m_UserDir = m_UserDir + std::string("/.") + AppDir;
     m_UserDirUTF8 = m_UserDir;
   }
