@@ -8,7 +8,6 @@
 #include "helpers/VExcept.h"
 #include "helpers/Log.h"
 #include "VXml.h"
-#include <curl/curl.h>
 #include "GameText.h"
 
 #ifdef WIN32
@@ -24,6 +23,8 @@
 #include "db/xmDatabase.h"
 #include "md5sum/md5file.h"
 
+#define DEFAULT_WWW_MSGFILE "wwwMsg.xml"
+
 void WebRoom::downloadReplay(const std::string& i_url) {
   std::string i_rplFilename = FS::getReplaysDir()
     + "/" 
@@ -70,30 +71,26 @@ void WebRoom::setWebsiteInfos(const std::string& i_id_room,
   m_proxy_settings    = pProxySettings;
 }
 
-void WebRoom::update(unsigned int i_number)
+/* use the id of the room so that if several people share the same room, it doesn't need to recheck if md5 is ok */
+void WebRoom::update(const std::string& i_id_room)
 {
   /* download xml file */
   f_curl_download_data v_data;
-  std::ostringstream v_strRoom;
-  v_strRoom << i_number;
 
   v_data.v_WebApp               = m_WebRoomApp;
   v_data.v_nb_files_performed   = 0;
   v_data.v_nb_files_to_download = 1;
 
-  FSWeb::downloadFileBz2UsingMd5(m_userFilename_prefix + v_strRoom.str() + ".xml",
+  FSWeb::downloadFileBz2UsingMd5(m_userFilename_prefix + i_id_room + ".xml",
 				 m_webhighscores_url,
 				 FSWeb::f_curl_progress_callback_download,
 				 &v_data,
 				 m_proxy_settings);
 }
 
-void WebRoom::upgrade(unsigned int i_number, xmDatabase *i_db)
+void WebRoom::upgrade(const std::string& i_id_room, xmDatabase *i_db)
 {
-  std::ostringstream v_strRoom;
-  v_strRoom << i_number;
-
-  i_db->webhighscores_updateDB(m_userFilename_prefix + v_strRoom.str() + ".xml", m_webhighscores_url);
+  i_db->webhighscores_updateDB(m_userFilename_prefix + i_id_room + ".xml", m_webhighscores_url);
 }
 
 size_t FSWeb::writeData(void *ptr, size_t size, size_t nmemb, FILE *stream)
@@ -187,7 +184,7 @@ void FSWeb::downloadFile(const std::string &p_local_file,
 			 const ProxySettings *p_proxy_settings
 			 ) {
   
-  Logger::Log(std::string("downloading " + 
+  LogInfo(std::string("downloading " + 
 			  p_web_file     +
 			  " to "         +
 			  p_local_file
@@ -274,9 +271,13 @@ void FSWeb::downloadFile(const std::string &p_local_file,
   }
 
   if(res != CURLE_OK) {
+    char v_err[256];
+
     remove(v_local_file_tmp.c_str());
-    throw Exception("error : unable to download file "
-        + p_web_file);  
+
+    snprintf(v_err, 256, "error : unable to download file %s (curl[%i]: %s)",
+	     p_web_file.c_str(), res, curl_easy_strerror(res));
+    throw Exception(v_err);
   }
 
   /* replace file */
@@ -290,35 +291,33 @@ void FSWeb::downloadFile(const std::string &p_local_file,
   }
 }
 
-void FSWeb::uploadReplay(std::string p_replayFilename,
-       std::string p_id_room,
-       std::string p_login,
-       std::string p_password,
-       std::string p_url_to_transfert,
-       WWWAppInterface *p_WebApp,
-       const ProxySettings *p_proxy_settings,
-       bool &p_msg_status,
-       std::string &p_msg) {
+void FSWeb::uploadReplay(const std::string& p_replayFilename,
+			 const std::string& p_id_room,
+			 const std::string& p_login,
+			 const std::string& p_password,
+			 const std::string& p_url_to_transfert,
+			 WWWAppInterface *p_WebApp,
+			 const ProxySettings *p_proxy_settings,
+			 bool &p_msg_status,
+			 std::string &p_msg) {
   CURL *v_curl;
   CURLcode v_res;
   std::string v_proxy_server;
   std::string v_proxy_auth_str;
   std::string v_www_agent = WWW_AGENT;
-  f_curl_upload_data v_data;
-  struct curl_slist *v_headers;
   std::string v_accept_language;
 
   FILE *v_destinationFile;
   std::string v_local_file;
-  v_local_file = FS::getUserDir() + "/" + DEFAULT_REPLAYUPLOAD_MSGFILE;
+  v_local_file = FS::getUserDir() + "/" + DEFAULT_WWW_MSGFILE;
 
   struct curl_httppost *v_post, *v_last;
 
-  Logger::Log(std::string("Uploading replay " + p_replayFilename).c_str());
+  LogInfo(std::string("Uploading replay " + p_replayFilename).c_str());
 
   /* open the file */
   if( (v_destinationFile = fopen(v_local_file.c_str(), "wb")) == false) {
-    throw Exception("error : unable to open output file " DEFAULT_REPLAYUPLOAD_MSGFILE);
+    throw Exception("error : unable to open output file " DEFAULT_WWW_MSGFILE);
   }
       
   v_curl = curl_easy_init();
@@ -328,8 +327,6 @@ void FSWeb::uploadReplay(std::string p_replayFilename,
     throw Exception("error : unable to init curl"); 
   }
 
-  curl_easy_setopt(v_curl, CURLOPT_URL, p_url_to_transfert.c_str());
-
   v_post = NULL;
   v_last = NULL;
   
@@ -345,75 +342,23 @@ void FSWeb::uploadReplay(std::string p_replayFilename,
   curl_formadd(&v_post, &v_last, CURLFORM_COPYNAME, "replay",
          CURLFORM_FILE, p_replayFilename.c_str(), CURLFORM_END);
 
-  curl_easy_setopt(v_curl, CURLOPT_HTTPPOST, v_post);
-
-  curl_easy_setopt(v_curl, CURLOPT_TIMEOUT, DEFAULT_TRANSFERT_TIMEOUT);
-  curl_easy_setopt(v_curl, CURLOPT_CONNECTTIMEOUT, DEFAULT_TRANSFERT_CONNECT_TIMEOUT);
-  curl_easy_setopt(v_curl, CURLOPT_WRITEDATA, v_destinationFile);
-  curl_easy_setopt(v_curl, CURLOPT_WRITEFUNCTION, FSWeb::writeData);
-  curl_easy_setopt(v_curl, CURLOPT_USERAGENT,  v_www_agent.c_str());
-  curl_easy_setopt(v_curl, CURLOPT_NOSIGNAL, 1);
-  curl_easy_setopt(v_curl, CURLOPT_FAILONERROR, 1);
-  curl_easy_setopt(v_curl, CURLOPT_FOLLOWLOCATION, 1);
-  curl_easy_setopt(v_curl, CURLOPT_ENCODING, "gzip,deflate");
-
-  v_headers = NULL;
-  v_accept_language = "Accept-Language: " + std::string(WEB_LANGUAGE);
-  v_headers = curl_slist_append(v_headers, v_accept_language.c_str()); 
-  curl_easy_setopt(v_curl, CURLOPT_HTTPHEADER, v_headers);
-
-  /* set proxy settings */
-  if(p_proxy_settings != NULL && p_proxy_settings->getTypeStr() != "") {
-    /* v_proxy_server because 
-       after call to 
-       curl_easy_setopt(v_curl, CURLOPT_PROXY, p_proxy_settings->getServer().c_str());
-       result is destroyed on call to curl_easy_perform
-    */
-    v_proxy_server = p_proxy_settings->getServer();
-    v_proxy_auth_str =
-      p_proxy_settings->getAuthentificationUser()
-      + ":"
-      + p_proxy_settings->getAuthentificationPassword();
-
-    if(p_proxy_settings->useDefaultServer() == false) {
-      curl_easy_setopt(v_curl, CURLOPT_PROXY, v_proxy_server.c_str());
-    }
-    
-    if(p_proxy_settings->useDefaultPort() == false) {
-      curl_easy_setopt(v_curl, CURLOPT_PROXYPORT, p_proxy_settings->getPort());
-    }
-    
-    curl_easy_setopt(v_curl, CURLOPT_PROXYTYPE, p_proxy_settings->getType());
-   
-    if(p_proxy_settings->useDefaultAuthentification() == false) {
-      curl_easy_setopt(v_curl, CURLOPT_PROXYUSERPWD,
-           v_proxy_auth_str.c_str());
-    }
-  }
-  /* ***** */
-
-  if(p_WebApp != NULL) {
-    v_data.v_WebApp = p_WebApp;
-
-    curl_easy_setopt(v_curl, CURLOPT_NOPROGRESS, false);
-    curl_easy_setopt(v_curl, CURLOPT_PROGRESSFUNCTION, FSWeb::f_curl_progress_callback_upload);
-    curl_easy_setopt(v_curl, CURLOPT_PROGRESSDATA, &v_data);
-  }
-
-  v_res = curl_easy_perform(v_curl);
+  v_res = performPostCurl(v_curl, v_post, p_url_to_transfert, v_destinationFile, p_WebApp, p_proxy_settings);
 
   fclose(v_destinationFile);
 
-  /* free the headers */
-  curl_slist_free_all(v_headers);
-
-
   /* CURLE_ABORTED_BY_CALLBACK is not considered as an error */
   if(v_res != CURLE_ABORTED_BY_CALLBACK) {
+
     if(v_res != CURLE_OK) {
+      char v_err[256];
+      
       curl_easy_cleanup(v_curl);
       remove(v_local_file.c_str());
-      throw Exception("error : unable to perform curl");  
+      
+      snprintf(v_err, 256, "error : unable to perform curl (curl[%i]: %s)",
+	       v_res, curl_easy_strerror(v_res));
+      
+      throw Exception(v_err);
     }
   }
   curl_easy_cleanup(v_curl);
@@ -423,15 +368,171 @@ void FSWeb::uploadReplay(std::string p_replayFilename,
     p_msg_status = 0;
     p_msg = "Upload aborted\nYou will be able to validate it later ;-)";
   } else {
-    uploadReplayAnalyseMsg(v_local_file, p_msg_status, p_msg);
+    uploadAnalyseMsg("xmoto_uploadReplayResult", v_local_file, p_msg_status, p_msg);
   }
 
-  remove(v_local_file.c_str());
+  if(XMSession::instance()->debug() == false) {
+    remove(v_local_file.c_str());
+  }
 }
 
-void FSWeb::uploadReplayAnalyseMsg(std::string p_filename,
-           bool &p_msg_status_ok,
-           std::string &p_msg) {
+CURLcode FSWeb::performPostCurl(CURL *p_curl,
+				struct curl_httppost *p_post,
+				const std::string& p_url_to_transfert,
+				FILE* p_destinationFile,
+				WWWAppInterface *p_WebApp,
+				const ProxySettings *p_proxy_settings) {
+  CURLcode v_res;
+  std::string v_www_agent = WWW_AGENT;
+  struct curl_slist *v_headers;
+  f_curl_upload_data v_data;
+  std::string v_accept_language;
+  std::string v_proxy_server;
+  std::string v_proxy_auth_str;
+
+  curl_easy_setopt(p_curl, CURLOPT_URL,            p_url_to_transfert.c_str());
+  curl_easy_setopt(p_curl, CURLOPT_HTTPPOST,       p_post);
+  curl_easy_setopt(p_curl, CURLOPT_TIMEOUT,        DEFAULT_TRANSFERT_TIMEOUT);
+  curl_easy_setopt(p_curl, CURLOPT_CONNECTTIMEOUT, DEFAULT_TRANSFERT_CONNECT_TIMEOUT);
+  curl_easy_setopt(p_curl, CURLOPT_WRITEDATA,      p_destinationFile);
+  curl_easy_setopt(p_curl, CURLOPT_WRITEFUNCTION,  FSWeb::writeData);
+  curl_easy_setopt(p_curl, CURLOPT_USERAGENT,      v_www_agent.c_str());
+  curl_easy_setopt(p_curl, CURLOPT_NOSIGNAL,       1);
+  curl_easy_setopt(p_curl, CURLOPT_FAILONERROR,    1);
+  curl_easy_setopt(p_curl, CURLOPT_FOLLOWLOCATION, 1);
+  curl_easy_setopt(p_curl, CURLOPT_ENCODING,       "gzip,deflate");
+  v_headers = NULL;
+  v_accept_language = "Accept-Language: " + std::string(WEB_LANGUAGE);
+  v_headers = curl_slist_append(v_headers, v_accept_language.c_str()); 
+  curl_easy_setopt(p_curl, CURLOPT_HTTPHEADER, v_headers);
+
+  /* set proxy settings */
+  if(p_proxy_settings != NULL && p_proxy_settings->getTypeStr() != "") {
+    /* v_proxy_server because 
+       after call to 
+       curl_easy_setopt(p_curl, CURLOPT_PROXY, p_proxy_settings->getServer().c_str());
+       result is destroyed on call to curl_easy_perform
+    */
+    v_proxy_server = p_proxy_settings->getServer();
+    v_proxy_auth_str =
+      p_proxy_settings->getAuthentificationUser()
+      + ":"
+      + p_proxy_settings->getAuthentificationPassword();
+
+    if(p_proxy_settings->useDefaultServer() == false) {
+      curl_easy_setopt(p_curl, CURLOPT_PROXY, v_proxy_server.c_str());
+    }
+    
+    if(p_proxy_settings->useDefaultPort() == false) {
+      curl_easy_setopt(p_curl, CURLOPT_PROXYPORT, p_proxy_settings->getPort());
+    }
+    
+    curl_easy_setopt(p_curl, CURLOPT_PROXYTYPE, p_proxy_settings->getType());
+   
+    if(p_proxy_settings->useDefaultAuthentification() == false) {
+      curl_easy_setopt(p_curl, CURLOPT_PROXYUSERPWD,v_proxy_auth_str.c_str());
+    }
+  }
+  /* ***** */
+
+  if(p_WebApp != NULL) {
+    v_data.v_WebApp = p_WebApp;
+
+    curl_easy_setopt(p_curl, CURLOPT_NOPROGRESS, false);
+    curl_easy_setopt(p_curl, CURLOPT_PROGRESSFUNCTION, FSWeb::f_curl_progress_callback_upload);
+    curl_easy_setopt(p_curl, CURLOPT_PROGRESSDATA, &v_data);
+  }
+
+  v_res = curl_easy_perform(p_curl);
+
+  /* free the headers */
+  curl_slist_free_all(v_headers);
+
+  return v_res;
+}
+
+void FSWeb::sendVote(const std::string& p_id_level,
+		     const std::string& p_difficulty_value,
+		     const std::string& p_quality_value,
+		     const std::string& p_url_to_transfert,
+		     WWWAppInterface *p_WebApp,
+		     const ProxySettings *p_proxy_settings,
+		     bool &p_msg_status,
+		     std::string &p_msg) {
+  CURL *v_curl;
+  CURLcode v_res;
+
+  FILE *v_destinationFile;
+  std::string v_local_file;
+  v_local_file = FS::getUserDir() + "/" + DEFAULT_WWW_MSGFILE;
+
+  struct curl_httppost *v_post, *v_last;
+
+  LogInfo(std::string("Sending vote").c_str());
+
+  /* open the file */
+  if( (v_destinationFile = fopen(v_local_file.c_str(), "wb")) == false) {
+    throw Exception("error : unable to open output file " DEFAULT_WWW_MSGFILE);
+  }
+      
+  v_curl = curl_easy_init();
+  if(v_curl == NULL) {
+    fclose(v_destinationFile);
+    remove(v_local_file.c_str());
+    throw Exception("error : unable to init curl"); 
+  }
+
+  v_post = NULL;
+  v_last = NULL;
+  
+  curl_formadd(&v_post, &v_last, CURLFORM_COPYNAME, "game_id",
+         CURLFORM_PTRCONTENTS, p_id_level.c_str(), CURLFORM_END);
+
+  curl_formadd(&v_post, &v_last, CURLFORM_COPYNAME, "difficulty",
+         CURLFORM_PTRCONTENTS, p_difficulty_value.c_str(), CURLFORM_END);
+
+  curl_formadd(&v_post, &v_last, CURLFORM_COPYNAME, "quality",
+         CURLFORM_PTRCONTENTS, p_quality_value.c_str(), CURLFORM_END);
+
+  v_res = performPostCurl(v_curl, v_post, p_url_to_transfert, v_destinationFile, p_WebApp, p_proxy_settings);
+
+  fclose(v_destinationFile);
+
+  /* CURLE_ABORTED_BY_CALLBACK is not considered as an error */
+  if(v_res != CURLE_ABORTED_BY_CALLBACK) {
+
+    if(v_res != CURLE_OK) {
+      char v_err[256];
+      
+      curl_easy_cleanup(v_curl);
+      remove(v_local_file.c_str());
+      
+      snprintf(v_err, 256, "error : unable to perform curl (curl[%i]: %s)",
+	       v_res, curl_easy_strerror(v_res));
+      
+      throw Exception(v_err);
+    }
+  }
+  curl_easy_cleanup(v_curl);
+
+  /* analyse de la réponse */
+  if(v_res == CURLE_ABORTED_BY_CALLBACK) {
+    p_msg_status = 0;
+    p_msg = "Aborted";
+  } else {
+    uploadAnalyseMsg("xmoto_sendVoteResult", v_local_file, p_msg_status, p_msg);
+  }
+
+  if(XMSession::instance()->debug() == false) {
+    remove(v_local_file.c_str());
+  }
+}
+
+
+void FSWeb::uploadAnalyseMsg(const std::string& p_key,
+			     const std::string& p_filename,
+			     bool &p_msg_status_ok,
+			     std::string &p_msg) {
   XMLDocument v_Xml;
   TiXmlDocument *v_XmlData;
   TiXmlElement *v_XmlDataElement;
@@ -439,7 +540,7 @@ void FSWeb::uploadReplayAnalyseMsg(std::string p_filename,
   std::string v_success;
   const char *pc;
   TiXmlNode * pChild;
-
+  
   /* open the file */
   v_Xml.readFromFile(p_filename);   
   v_XmlData = v_Xml.getLowLevelAccess();
@@ -449,7 +550,7 @@ void FSWeb::uploadReplayAnalyseMsg(std::string p_filename,
   }
   
   /* read the res and msg */
-  v_XmlDataElement = v_XmlData->FirstChildElement("xmoto_uploadReplayResult");
+  v_XmlDataElement = v_XmlData->FirstChildElement(p_key.c_str());
   if(v_XmlDataElement == NULL) {
     throw Exception("unable to analyze xml file result");
   }
@@ -482,6 +583,100 @@ void FSWeb::uploadReplayAnalyseMsg(std::string p_filename,
   p_msg = pc;
 }
 
+/* could be factorized with uploadReplay */
+void FSWeb::uploadDbSync(const std::string& p_dbSyncFilename,
+			 const std::string& p_login,
+			 const std::string& p_password,
+			 const std::string& p_siteKey,
+			 int p_dbSyncServer,
+			 const std::string& p_url_to_transfert,
+			 WWWAppInterface *p_WebApp,
+			 const ProxySettings *p_proxy_settings,
+			 bool &p_msg_status,
+			 std::string& p_msg, 
+			 const std::string& p_answerFile) {
+  CURL *v_curl;
+  CURLcode v_res;
+  std::string v_proxy_server;
+  std::string v_proxy_auth_str;
+  std::string v_www_agent = WWW_AGENT;
+  std::string v_accept_language;
+
+  FILE *v_destinationFile;
+
+  struct curl_httppost *v_post, *v_last;
+
+  // std::ostringstream v_dbSyncServerStr;
+  // curl seems not to like .str().c_str()
+  char v_syncStr[256];
+
+  LogInfo(std::string("Uploading dbsync " + p_dbSyncFilename + " to " + p_url_to_transfert).c_str());
+
+  /* open the file */
+  if( (v_destinationFile = fopen(p_answerFile.c_str(), "wb")) == false) {
+    throw Exception("error : unable to open output file " + p_answerFile);
+  }
+      
+  v_curl = curl_easy_init();
+  if(v_curl == NULL) {
+    fclose(v_destinationFile);
+    remove(p_answerFile.c_str());
+    throw Exception("error : unable to init curl"); 
+  }
+
+  v_post = NULL;
+  v_last = NULL;
+
+  snprintf(v_syncStr, 256, "%i", p_dbSyncServer);
+
+  curl_formadd(&v_post, &v_last, CURLFORM_COPYNAME, "login",
+	       CURLFORM_PTRCONTENTS, p_login.c_str(), CURLFORM_END);
+
+  curl_formadd(&v_post, &v_last, CURLFORM_COPYNAME, "password",
+	       CURLFORM_PTRCONTENTS, p_password.c_str(), CURLFORM_END);
+
+  curl_formadd(&v_post, &v_last, CURLFORM_COPYNAME, "downSiteKey",
+	       CURLFORM_PTRCONTENTS, p_siteKey.c_str(), CURLFORM_END);
+
+  curl_formadd(&v_post, &v_last, CURLFORM_COPYNAME, "downDbSync",
+	       CURLFORM_PTRCONTENTS, v_syncStr, CURLFORM_END);
+
+  curl_formadd(&v_post, &v_last, CURLFORM_COPYNAME, "dbSync",
+	       CURLFORM_FILE, p_dbSyncFilename.c_str(), CURLFORM_END);
+
+  v_res = performPostCurl(v_curl, v_post, p_url_to_transfert, v_destinationFile, p_WebApp, p_proxy_settings);
+
+  fclose(v_destinationFile);
+
+  /* CURLE_ABORTED_BY_CALLBACK is not considered as an error */
+  if(v_res != CURLE_ABORTED_BY_CALLBACK) {
+
+    if(v_res != CURLE_OK) {
+      char v_err[256];
+      
+      curl_easy_cleanup(v_curl);
+
+      if(XMSession::instance()->debug() == false) {
+	remove(p_answerFile.c_str());
+      }      
+
+      snprintf(v_err, 256, "error : unable to perform curl (curl[%i]: %s)",
+	       v_res, curl_easy_strerror(v_res));
+      
+      throw Exception(v_err);
+    }
+  }
+  curl_easy_cleanup(v_curl);
+
+  /* analyse de la réponse */
+  if(v_res == CURLE_ABORTED_BY_CALLBACK) {
+    p_msg_status = 0;
+    p_msg = "DbSync aborted";
+  } else {
+    uploadAnalyseMsg("xmoto_uploadDbSyncResult", p_answerFile, p_msg_status, p_msg);
+  }
+}
+
 WebLevels::WebLevels(WWWAppInterface *p_WebLevelApp) {
   m_WebLevelApp    = p_WebLevelApp;
   m_proxy_settings = NULL;
@@ -724,7 +919,7 @@ void WebThemes::updateTheme(xmDatabase* i_pDb, const std::string& i_id_theme, WW
   }
 
   try {
-    Logger::Log("WWW: Downloading a theme...");
+    LogInfo("WWW: Downloading a theme...");
     
     std::string v_destinationFile, v_destinationFileXML, v_destinationFileXML_tmp;
     std::string v_sourceFile;
@@ -897,7 +1092,7 @@ void WebThemes::updateTheme(xmDatabase* i_pDb, const std::string& i_id_theme, WW
       }
     }
     
-    Logger::Log("** Warning ** : Failed to update theme %s (%s)", i_id_theme.c_str(), e.getMsg().c_str());
+    LogWarning("Failed to update theme %s (%s)", i_id_theme.c_str(), e.getMsg().c_str());
     throw Exception("Failed to update the theme");
     return;
   }
@@ -914,7 +1109,7 @@ void WebThemes::updateThemeList(xmDatabase* i_pDb, WWWAppInterface* i_WebLevelAp
     }
   }
 
-  Logger::Log("WWW: Checking for new or updated themes...");
+  LogInfo("WWW: Checking for new or updated themes...");
   v_data.v_WebApp = i_WebLevelApp;
   v_data.v_nb_files_performed   = 0;
   v_data.v_nb_files_to_download = 1;
@@ -922,3 +1117,25 @@ void WebThemes::updateThemeList(xmDatabase* i_pDb, WWWAppInterface* i_WebLevelAp
 				 FSWeb::f_curl_progress_callback_download, &v_data, XMSession::instance()->proxySettings());
   i_pDb->webthemes_updateDB(v_destinationFile);
 }
+
+bool WebThemes::isUpdatable(xmDatabase* i_pDb, const std::string& i_id_theme) {
+  char **v_result;
+  unsigned int nrow;
+
+  v_result = i_pDb->readDB("SELECT id_theme FROM webthemes WHERE id_theme=\"" + xmDatabase::protectString(i_id_theme) + "\";",
+			   nrow);
+  if(nrow == 1) {
+    i_pDb->read_DB_free(v_result);
+    return true;
+  }
+  i_pDb->read_DB_free(v_result);
+  return false;
+}
+
+void XMSync::syncUp(xmDatabase* i_pDb, const ProxySettings* pProxySettings,
+		    const std::string& i_sitekey, const std::string& i_profile) {
+  // sync_buildServerFile(const std::string& i_outFile, const std::string& i_sitekey, const std::string& i_profile)
+}
+
+void XMSync::syncDown() {
+}
