@@ -28,6 +28,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include <string>
 #include <vector>
 #include <stdio.h>
+#include <curl/curl.h>
 
 #if !defined(WIN32) && !defined(__APPLE__) && !defined(__MACH__)
   //#define USE_HASH_MAP // removed because it seems to segfault i don't know why when i refresh levels using F5 and quit
@@ -44,13 +45,14 @@ class ThemeChoice;
 #define DEFAULT_TRANSFERT_TIMEOUT         240
 #define DEFAULT_TRANSFERT_CONNECT_TIMEOUT 15
 #define DEFAULT_WEBLEVELS_URL             "http://xmoto.tuxfamily.org/levels.xml"
+#define DEFAULT_UPLOADDBSYNC_URL          "http://xmoto.tuxfamily.org/tools/UploadDbSync.php"
 #define DEFAULT_WEBLEVELS_FILENAME        "weblevels.xml"
 #define DEFAULT_WEBLEVELS_DIR             "downloaded"
 #define DEFAULT_WEBTHEMES_URL             "http://xmoto.tuxfamily.org/themes.xml"
 #define DEFAULT_WEBTHEMES_FILENAME        "webthemes.xml"
 #define DEFAULT_WEBTHEMES_SPRITESURLBASE  "http://xmoto.tuxfamily.org/sprites"
 #define DEFAULT_UPLOADREPLAY_URL          "http://xmoto.tuxfamily.org/tools/UploadReplay.php"
-#define DEFAULT_REPLAYUPLOAD_MSGFILE      "UploadReplayMsg.xml"
+#define DEFAULT_SENDVOTE_URL              "http://xmoto.tuxfamily.org/tools/SendVote.php"
 #define DEFAULT_WEBROOMS_URL              "http://xmoto.tuxfamily.org/rooms.xml"
 #define DEFAULT_WEBROOMS_FILENAME         "webrooms.xml"
 #define DEFAULT_WEBROOM_NAME              "WR"
@@ -124,16 +126,37 @@ class FSWeb {
 				      void *p_data,
 				      const ProxySettings *p_proxy_settings);
 
-  static void uploadReplay(std::string p_replayFilename,
-			   std::string p_id_room,
-			   std::string p_login,
-			   std::string p_password,
-			   std::string p_url_to_transfert,
+  static void uploadReplay(const std::string& p_replayFilename,
+			   const std::string& p_id_room,
+			   const std::string& p_login,
+			   const std::string& p_password,
+			   const std::string& p_url_to_transfert,
 			   WWWAppInterface *p_WebApp,
 			   const ProxySettings *p_proxy_settings,
 			   bool &p_msg_status,
 			   std::string &p_msg);
 
+  static void uploadDbSync(const std::string& p_dbSyncFilename,
+			   const std::string& p_login,
+			   const std::string& p_password,
+			   const std::string& p_siteKey,
+			   int p_dbSyncServer,
+			   const std::string& p_url_to_transfert,
+			   WWWAppInterface *p_WebApp,
+			   const ProxySettings *p_proxy_settings,
+			   bool &p_msg_status,
+			   std::string& p_msg,
+			   const std::string& p_answerFile);
+
+  static void sendVote(const std::string& p_id_level,
+		       const std::string& p_difficulty_value,
+		       const std::string& p_quality_value,
+		       const std::string& p_url_to_transfert,
+		       WWWAppInterface *p_WebApp,
+		       const ProxySettings *p_proxy_settings,
+		       bool &p_msg_status,
+		       std::string &p_msg);
+
   static int f_curl_progress_callback_upload(void *clientp,
 					     double dltotal,
 					     double dlnow,
@@ -148,9 +171,17 @@ class FSWeb {
 
  private:
   static size_t writeData(void *ptr, size_t size, size_t nmemb, FILE *stream);
-  static void   uploadReplayAnalyseMsg(std::string p_filename,
-				       bool &p_msg_status_ok,
-				       std::string &p_msg);
+  static void   uploadAnalyseMsg(const std::string& p_key,
+				 const std::string& p_filename,
+				 bool &p_msg_status_ok,
+				 std::string &p_msg);
+
+  static CURLcode performPostCurl(CURL *p_curl,
+				  struct curl_httppost *p_post,
+				  const std::string& p_url_to_transfert,
+				  FILE* p_destinationFile,
+				  WWWAppInterface *p_WebApp,
+				  const ProxySettings *p_proxy_settings);
 };
 
 class WebRoom {
@@ -158,8 +189,8 @@ class WebRoom {
   WebRoom(WWWAppInterface* p_WebRoomApp);
   ~WebRoom();
 
-  void update(unsigned int i_number); /* throws exceptions */
-  void upgrade(unsigned int i_number, xmDatabase* i_db); /* throws exceptions */
+  void update(const std::string& i_id_room); /* throws exceptions */
+  void upgrade(const std::string& i_id_room, xmDatabase* i_db); /* throws exceptions */
 
   /* return NULL if no data found */
   void setWebsiteInfos(const std::string& i_id_room,
@@ -223,7 +254,14 @@ class WebThemes {
  public:
   static void updateTheme(xmDatabase* i_pDb, const std::string& i_id_theme, WWWAppInterface* i_WebLevelApp);
   static void updateThemeList(xmDatabase* i_pDb, WWWAppInterface* i_WebLevelApp);
-  
+  static bool isUpdatable(xmDatabase* i_pDb, const std::string& i_id_theme);
+};
+
+class XMSync {
+ public:
+  static void syncUp(xmDatabase* i_pDb, const ProxySettings* pProxySettings,
+		     const std::string& i_sitekey, const std::string& i_profile);
+  static void syncDown();
 };
 
 #endif /* WEBSTUFFS */
