@@ -19,19 +19,24 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 =============================================================================*/
 
 #include "xmDatabase.h"
-#include "helpers/VExcept.h"
-#include "GameText.h"
-#include "WWW.h"
-#include "helpers/Log.h"
-#include "XMSession.h"
-#include "VFileIO.h"
+#include "../helpers/VExcept.h"
+#include "../GameText.h"
+#include "../WWW.h"
+#include "../Game.h"
+#include "../helpers/Log.h"
+#include "../XMSession.h"
+#include "../VFileIO.h"
+#include <sstream>
 
-#define XMDB_VERSION 20
+#define XMDB_VERSION         28
+#define DB_MAX_SQL_RUNTIME 0.25
+#define DB_BUSY_TIMEOUT   60000 // 60 seconds
 
 bool xmDatabase::Trace = false;
 
 xmDatabase::xmDatabase()
 {
+  m_db = NULL;
 }
 
 void xmDatabase::init(const std::string& i_dbFile,
@@ -39,6 +44,7 @@ void xmDatabase::init(const std::string& i_dbFile,
 		      const std::string& i_gameDir,
 		      const std::string& i_userDir,
 		      const std::string& i_binPackCheckSum,
+		      bool i_dbDirsCheck,
 		      XmDatabaseUpdateInterface *i_interface) {
   int v_version;
 
@@ -46,29 +52,31 @@ void xmDatabase::init(const std::string& i_dbFile,
   m_requiredReplaysUpdateAfterInit = false;
   m_requiredThemesUpdateAfterInit  = false;
 
+  //LogDebug("openDB(%X)", this);
   if(sqlite3_open(i_dbFile.c_str(), &m_db) != 0) {
     throw Exception("Unable to open the database (" + i_dbFile
 		    + ") : " + sqlite3_errmsg(m_db));
   }
 
+  sqlite3_busy_timeout(m_db, DB_BUSY_TIMEOUT);
   sqlite3_trace(m_db, sqlTrace, NULL);
   createUserFunctions();
 
 //  if(sqlite3_threadsafe() == 0) {
-//    Logger::Log("** Warning ** Sqlite is not threadSafe !!!");
+//    LogWarning("Sqlite is not threadSafe !!!");
 //  } else {
-//    Logger::Log("Sqlite is threadSafe");
+//    LogInfo("Sqlite is threadSafe");
 //  }
 
   v_version = getXmDbVersion();
-  Logger::Log("XmDb version is %i", v_version);
+  LogInfo("XmDb version is %i", v_version);
 
   if(v_version > XMDB_VERSION) {
     throw Exception("Your XM database required a newer version of xmoto");
   }
 
   if(v_version < XMDB_VERSION) {
-    Logger::Log("Update XmDb version from %i to %i", v_version, XMDB_VERSION);
+    LogInfo("Update XmDb version from %i to %i", v_version, XMDB_VERSION);
 
     if(i_interface != NULL) {
       i_interface->updatingDatabase(GAMETEXT_DB_UPGRADING);
@@ -76,11 +84,12 @@ void xmDatabase::init(const std::string& i_dbFile,
     upgradeXmDbToVersion(v_version, i_profile, i_interface); 
   }
 
-  /* check if gameDir and userDir are the same - otherwise, the computer probably changed */
-  if(FS::areSamePath(i_gameDir, getXmDbGameDir()) == false ||
-     FS::areSamePath(i_userDir, getXmDbUserDir()) == false ||
-     i_binPackCheckSum != getXmDbBinPackCheckSum()
-     ) {
+  /* check if gameDir and userDir are the same - otherwise, the computer probably changed */  
+  std::string v_oldGameDir = getXmDbGameDir();
+  std::string v_oldUserDir = getXmDbUserDir();
+  bool v_areDirectoryOK = FS::areSamePath(i_gameDir, v_oldGameDir) && FS::areSamePath(i_userDir, v_oldUserDir);
+
+  if((v_areDirectoryOK == false && i_dbDirsCheck) || i_binPackCheckSum != getXmDbBinPackCheckSum()) {
     m_requiredLevelsUpdateAfterInit  = true;
     m_requiredReplaysUpdateAfterInit = true;
     m_requiredThemesUpdateAfterInit  = true;
@@ -90,18 +99,82 @@ void xmDatabase::init(const std::string& i_dbFile,
 
     /* -- first initialisation or xmoto.bin/userdir update -- */
     webLoadDataFirstTime();
+  } else {
+    // directory are not ok, but check directory is disabled => update the directories anyway
+    if(v_areDirectoryOK == false) {
+      setXmDbGameDir(i_gameDir);
+      setXmDbUserDir(i_userDir);
+
+      try {
+	updateXMDirectories(v_oldGameDir, i_gameDir, v_oldUserDir, i_userDir);
+      } catch(Exception &e) {
+	/* forcing update */
+	m_requiredLevelsUpdateAfterInit  = true;
+	m_requiredReplaysUpdateAfterInit = true;
+	m_requiredThemesUpdateAfterInit  = true;
+
+	/* -- first initialisation or xmoto.bin/userdir update -- */
+	webLoadDataFirstTime();
+      }
+    }
   }
 }
 
-void xmDatabase::init(const std::string& i_dbFile)
-{
-  if(sqlite3_open(i_dbFile.c_str(), &m_db) != 0){
-    throw Exception("Unable to open the database ("
-		    + i_dbFile
-		    + ") : "
-		    + sqlite3_errmsg(m_db));
-  }
+void xmDatabase::updateXMDirectories(const std::string& i_oldGameDir, const std::string& i_newGameDir,
+				     const std::string& i_oldUserDir, const std::string& i_newUserDir) {
+  LogInfo("Updating XM directories from %s to %s", i_oldGameDir.c_str(), i_newGameDir.c_str());
+  LogInfo("Updating XM directories from %s to %s", i_oldUserDir.c_str(), i_newUserDir.c_str());
 
+  try {
+    simpleSql("BEGIN TRANSACTION;");
+
+    simpleSql("UPDATE levels SET filepath ="
+	      " xm_replaceStart(filepath, \"" + protectString(i_oldGameDir) + "\", \"" + protectString(i_newGameDir) + "\")"
+	      " WHERE filepath LIKE \""  + protectString(i_oldGameDir) + "%\";");
+
+    simpleSql("UPDATE themes SET filepath ="
+	      " xm_replaceStart(filepath, \"" + protectString(i_oldGameDir) + "\", \"" + protectString(i_newGameDir) + "\")"
+	      " WHERE filepath LIKE \""  + protectString(i_oldGameDir) + "%\";");
+
+    simpleSql("UPDATE levels SET filepath ="
+	      " xm_replaceStart(filepath, \"" + protectString(i_oldUserDir) + "\", \"" + protectString(i_newUserDir) + "\")"
+	      " WHERE filepath LIKE \""  + protectString(i_oldUserDir) + "%\";");
+
+    simpleSql("UPDATE themes SET filepath ="
+	      " xm_replaceStart(filepath, \"" + protectString(i_oldUserDir) + "\", \"" + protectString(i_newUserDir) + "\")"
+	      " WHERE filepath LIKE \""  + protectString(i_oldUserDir) + "%\";");
+
+    simpleSql("COMMIT;");
+  } catch(Exception &e) {
+    simpleSql("ROLLBACK;");
+    throw e;
+  }
+}
+
+void xmDatabase::init(const std::string& i_dbFile, bool i_readOnly)
+{
+  /* my sqlite version doesn't allow readonly ; wait to allow this */
+//  if(i_readOnly) {
+//    LogDebug("Database opened in read-only mode");
+//    if(sqlite3_open_v2(i_dbFile.c_str(), &m_db, SQLITE_OPEN_READONLY, NULL) != 0){
+//      if(m_db != NULL) {
+//	sqlite3_close(m_db); // close even if it fails as requested in the documentation
+//	m_db = NULL;
+//      }
+//      throw Exception("Unable to open the database (" + i_dbFile + ") : " + sqlite3_errmsg(m_db));
+//    }
+//  } else {
+//
+    if(sqlite3_open(i_dbFile.c_str(), &m_db) != 0){
+      if(m_db != NULL) {
+	sqlite3_close(m_db); // close even if it fails as requested in the documentation
+	m_db = NULL;
+      }
+      throw Exception("Unable to open the database (" + i_dbFile + ") : " + sqlite3_errmsg(m_db));
+    }
+//  }
+
+  sqlite3_busy_timeout(m_db, DB_BUSY_TIMEOUT);
   sqlite3_trace(m_db, sqlTrace, NULL);
   createUserFunctions();
 }
@@ -120,103 +193,76 @@ void xmDatabase::sqlTrace(void* arg1, const char* sql) {
   }
 }
 
-std::string xmDatabase::getXmDbGameDir() {
+/* return false if the parameter doesn't exist */
+bool xmDatabase::getXmParameterKey(const std::string& i_key, std::string& o_value) {
   char **v_result;
   unsigned int nrow;
-  std::string v_dir;
 
-  v_result = readDB("SELECT value FROM xm_parameters WHERE param=\"gameDir\";", nrow);
+  v_result = readDB("SELECT value FROM xm_parameters WHERE param=\"" + protectString(i_key) + "\";", nrow);
   if(nrow != 1) {
     read_DB_free(v_result);
-    return "";
+    return false;
   }
 
-  v_dir = getResult(v_result, 1, 0, 0);
+  o_value = getResult(v_result, 1, 0, 0);
   read_DB_free(v_result);
 
-  return v_dir;
+  return true;
+}
+
+std::string xmDatabase::getXmDbGameDir() {
+  std::string v_dir;
+
+  if(getXmParameterKey("gameDir", v_dir)) {
+    return v_dir;
+  }
+  return "";
 }
 
 std::string xmDatabase::getXmDbUserDir() {
-  char **v_result;
-  unsigned int nrow;
   std::string v_dir;
 
-  v_result = readDB("SELECT value FROM xm_parameters WHERE param=\"userDir\";", nrow);
-  if(nrow != 1) {
-    read_DB_free(v_result);
-    return "";
+  if(getXmParameterKey("userDir", v_dir)) {
+    return v_dir;
   }
+  return "";
+}
 
-  v_dir = getResult(v_result, 1, 0, 0);
+void xmDatabase::setXmParameterKey(const std::string& i_key, const std::string& i_value) {
+  char **v_result;
+  unsigned int nrow;
+  
+  v_result = readDB("SELECT value FROM xm_parameters WHERE param=\"" + protectString(i_key) + "\";", nrow);
   read_DB_free(v_result);
 
-  return v_dir;
+  if(nrow == 0) {
+    simpleSql("INSERT INTO xm_parameters(param, value) VALUES(\"" + protectString(i_key) + "\", \""
+	      + protectString(i_value) + "\");");
+  } else {
+    simpleSql("UPDATE xm_parameters SET value=\""
+	      + protectString(i_value) + "\" WHERE param=\"" + protectString(i_key) + "\"");
+  }
 }
 
 void xmDatabase::setXmDbGameDir(const std::string& i_gameDir) {
-  char **v_result;
-  unsigned int nrow;
-
-  v_result = readDB("SELECT value FROM xm_parameters WHERE param=\"gameDir\";", nrow);
-  read_DB_free(v_result);
-
-  if(nrow == 0) {
-    simpleSql("INSERT INTO xm_parameters(param, value) VALUES(\"gameDir\", \""
-	      + protectString(i_gameDir) + "\");");
-  } else {
-    simpleSql("UPDATE xm_parameters SET value=\""
-	      + protectString(i_gameDir) + "\" WHERE param=\"gameDir\"");
-  }
+  setXmParameterKey("gameDir", i_gameDir);
 }
 
 void xmDatabase::setXmDbUserDir(const std::string& i_userDir) {
-  char **v_result;
-  unsigned int nrow;
-
-  v_result = readDB("SELECT value FROM xm_parameters WHERE param=\"userDir\";", nrow);
-  read_DB_free(v_result);
-
-  if(nrow == 0) {
-    simpleSql("INSERT INTO xm_parameters(param, value) VALUES(\"userDir\", \""
-	      + protectString(i_userDir) + "\");");
-  } else {
-    simpleSql("UPDATE xm_parameters SET value=\""
-	      + protectString(i_userDir) + "\" WHERE param=\"userDir\"");
-  }
+  setXmParameterKey("userDir", i_userDir);
 }
 
 std::string xmDatabase::getXmDbBinPackCheckSum() {
-  char **v_result;
-  unsigned int nrow;
-  std::string v_dir;
+  std::string v_sum;
 
-  v_result = readDB("SELECT value FROM xm_parameters WHERE param=\"binPackCkSum\";", nrow);
-  if(nrow != 1) {
-    read_DB_free(v_result);
-    return "";
+  if(getXmParameterKey("binPackCkSum", v_sum)) {
+    return v_sum;
   }
-
-  v_dir = getResult(v_result, 1, 0, 0);
-  read_DB_free(v_result);
-
-  return v_dir;
+  return "";
 }
 
 void xmDatabase::setXmDbBinPackCheckSum(const std::string& i_binPackChecksum) {
-  char **v_result;
-  unsigned int nrow;
-
-  v_result = readDB("SELECT value FROM xm_parameters WHERE param=\"binPackCkSum\";", nrow);
-  read_DB_free(v_result);
-
-  if(nrow == 0) {
-    simpleSql("INSERT INTO xm_parameters(param, value) VALUES(\"binPackCkSum\", \""
-	      + protectString(i_binPackChecksum) + "\");");
-  } else {
-    simpleSql("UPDATE xm_parameters SET value=\""
-	      + protectString(i_binPackChecksum) + "\" WHERE param=\"binPackCkSum\"");
-  }
+  setXmParameterKey("binPackCkSum", i_binPackChecksum);
 }
 
 int xmDatabase::getXmDbVersion() {
@@ -263,10 +309,56 @@ void xmDatabase::updateXmDbVersion(int i_newVersion) {
 	    " WHERE param=\"xmdb_version\";");
 }
 
+std::string xmDatabase::getXmDbSiteKey() {
+  std::string v_sitekey;
+
+  if(getXmParameterKey("siteKey", v_sitekey)) {
+    return v_sitekey;
+  }
+
+  // site key is empty, generate it
+  return setXmDbSiteKey();
+}
+
+std::string xmDatabase::setXmDbSiteKey() {
+  std::string v_sitekey = generateSiteKey();
+  setXmParameterKey("siteKey", v_sitekey);
+  return v_sitekey;
+}
+
+std::string xmDatabase::generateSiteKey() {
+  std::string v_sitekey = "";
+  struct tm *pTime;
+  time_t T;
+  char cBuf[256] = "";
+  int n;
+  std::ostringstream v_rd;
+
+  time(&T);
+  pTime = localtime(&T);
+  if(pTime != NULL) {
+    snprintf(cBuf, 256, "%d%02d%02d%02d%02d%02d",
+	     pTime->tm_year+1900, pTime->tm_mon+1, pTime->tm_mday, pTime->tm_hour, pTime->tm_min, pTime->tm_sec);                    
+    v_sitekey += cBuf;
+  }
+  // add random key
+  n = (int) (10000000.0 * (rand() / (RAND_MAX + 1.0)));
+  v_rd << n;
+  v_sitekey += v_rd.str();
+
+  //printf("siteKey: %s\n", v_sitekey.c_str());
+  return v_sitekey;
+}
+
 void xmDatabase::upgradeXmDbToVersion(int i_fromVersion,
 				      const std::string& i_profile,
 				      XmDatabaseUpdateInterface *i_interface) {
   std::string v_errMsg;
+  std::string v_sitekey;
+
+  if(i_fromVersion != 0) { /* cannot create site key if xm_parameters doesn't exist */
+    v_sitekey = getXmDbSiteKey();
+  }
 
   /* no break in this swicth ! */
   switch(i_fromVersion) {
@@ -275,6 +367,7 @@ void xmDatabase::upgradeXmDbToVersion(int i_fromVersion,
     try {
       simpleSql("CREATE TABLE xm_parameters(param PRIMARY KEY, value);"
 		"INSERT INTO xm_parameters(param, value) VALUES(\"xmdb_version\", 1);");
+      v_sitekey = getXmDbSiteKey();
     } catch(Exception &e) {
       throw Exception("Unable to update xmDb from 0: " + e.getMsg());
     }
@@ -293,7 +386,7 @@ void xmDatabase::upgradeXmDbToVersion(int i_fromVersion,
     try {
       updateDB_stats(i_interface);
     } catch(Exception &e) {
-      Logger::Log(std::string("Oups, updateDB_stats() failed: " + e.getMsg()).c_str());
+      LogError(std::string("Oups, updateDB_stats() failed: " + e.getMsg()).c_str());
     }
 
   case 2:
@@ -314,7 +407,7 @@ void xmDatabase::upgradeXmDbToVersion(int i_fromVersion,
       try {
 	updateDB_favorite(i_profile, i_interface);
       } catch(Exception &e) {
-	Logger::Log(std::string("Oups, updateDB_favorite() failed: " + e.getMsg()).c_str());
+	LogError(std::string("Oups, updateDB_favorite() failed: " + e.getMsg()).c_str());
       }
       updateXmDbVersion(4);
     } catch(Exception &e) {
@@ -330,7 +423,7 @@ void xmDatabase::upgradeXmDbToVersion(int i_fromVersion,
       try {
 	updateDB_profiles(i_interface);
       } catch(Exception &e) {
-	Logger::Log(std::string("Oups, updateDB_profiles() failed: " + e.getMsg()).c_str());
+	LogError(std::string("Oups, updateDB_profiles() failed: " + e.getMsg()).c_str());
       }
       updateXmDbVersion(5);
     } catch(Exception &e) {
@@ -478,6 +571,100 @@ void xmDatabase::upgradeXmDbToVersion(int i_fromVersion,
       throw Exception("Unable to update xmDb from 19: " + e.getMsg());
     }
 
+  case 20:
+    try {
+      simpleSql("CREATE TABLE profiles_configs(id_profile, key, value, PRIMARY KEY(id_profile, key));");
+      updateXmDbVersion(21);
+    } catch(Exception &e) {
+      throw Exception("Unable to update xmDb from 20: " + e.getMsg());
+    }
+
+  case 21:
+    try {
+      updateDB_config("" /* site key still does exist */);
+      updateXmDbVersion(22);
+    } catch(Exception &e) {
+      throw Exception("Unable to update xmDb from 21: " + e.getMsg());
+    }
+
+  case 22:
+    try {
+      // add column sitekey as primary key - sqlite doesn't support that without recreating the table
+      simpleSql("ALTER  TABLE stats_profiles RENAME TO stats_profiles_old;");
+      simpleSql("CREATE TABLE stats_profiles(sitekey, id_profile, nbStarts, since, PRIMARY KEY(sitekey, id_profile));");
+      simpleSql("INSERT INTO  stats_profiles SELECT \"" + protectString(v_sitekey) + "\", id_profile, nbStarts, since FROM stats_profiles_old;");
+      simpleSql("DROP   TABLE stats_profiles_old;");
+
+      updateXmDbVersion(23);
+    } catch(Exception &e) {
+      throw Exception("Unable to update xmDb from 22: " + e.getMsg());
+    }
+
+  case 23:
+    try {
+      // add column sitekey as primary key - sqlite doesn't support that without recreating the table
+      simpleSql("ALTER  TABLE stats_profiles_levels RENAME TO stats_profiles_levels_old;");
+      simpleSql("CREATE TABLE stats_profiles_levels(sitekey, id_profile, id_level, nbPlayed, nbDied, nbCompleted, nbRestarted, playedTime, last_play_date DEFAULT NULL, synchronized, PRIMARY KEY(sitekey, id_profile, id_level));");
+      simpleSql("INSERT INTO  stats_profiles_levels SELECT \"" + protectString(v_sitekey) + "\", id_profile, id_level, nbPlayed, nbDied, nbCompleted, nbRestarted, playedTime, last_play_date, 0 FROM stats_profiles_levels_old;");
+      simpleSql("DROP   TABLE stats_profiles_levels_old;");
+      /* recreate the index */
+      simpleSql("CREATE INDEX stats_profiles_levels_last_play_date_idx1 ON stats_profiles_levels(last_play_date);");
+      // index not redondante with the key while it is choiced when sitekey is not set ; make xmoto really faster
+      simpleSql("CREATE INDEX stats_profiles_levels_id_profile_id_level_idx1 ON stats_profiles_levels(id_profile, id_level);");
+
+      updateXmDbVersion(24);
+    } catch(Exception &e) {
+      throw Exception("Unable to update xmDb from 23: " + e.getMsg());
+    }
+
+  case 24:
+    try {
+      simpleSql("ALTER TABLE profile_completedLevels ADD COLUMN sitekey;");
+      simpleSql("ALTER TABLE profile_completedLevels ADD COLUMN synchronized;");
+      simpleSql("UPDATE profile_completedLevels SET sitekey = \"" + protectString(v_sitekey) + "\";");
+      simpleSql("UPDATE profile_completedLevels SET synchronized = 0;");
+      simpleSql("DROP INDEX profile_completedLevels_idx1;");
+      /* recreate the index */
+      simpleSql("CREATE INDEX profile_completedLevels_idx1 "
+		"ON profile_completedLevels(sitekey, id_profile, id_level);");
+      // index not redondante with the idx1 while it is choiced when sitekey is not set ; make xmoto really faster
+      simpleSql("CREATE INDEX profile_completedLevels_idx2 "
+		"ON profile_completedLevels(id_profile, id_level);");
+
+      updateXmDbVersion(25);
+    } catch(Exception &e) {
+      throw Exception("Unable to update xmDb from 24: " + e.getMsg());
+    }
+
+  case 25:
+    try {
+      simpleSql("ALTER TABLE levels ADD COLUMN isPhysics DEFAULT 0;");
+      simpleSql("CREATE INDEX levels_isPhysics_idx1 ON levels(isPhysics);");
+      updateXmDbVersion(26);
+      m_requiredLevelsUpdateAfterInit = true;
+    } catch(Exception &e) {
+      throw Exception("Unable to update xmDb from 25: " + e.getMsg());
+    }
+
+  case 26:
+    try {
+      simpleSql("ALTER TABLE profile_completedLevels ADD COLUMN dbSync DEFAULT NULL;");
+      simpleSql("CREATE INDEX profile_completedLevels_dbSync_idx1 ON profile_completedLevels(dbSync);");
+      simpleSql("ALTER TABLE stats_profiles_levels ADD COLUMN dbSync DEFAULT NULL;");
+      simpleSql("CREATE INDEX stats_profiles_levels_dbSync_idx1 ON stats_profiles_levels(dbSync);");
+      updateXmDbVersion(27);
+    } catch(Exception &e) {
+      throw Exception("Unable to update xmDb from 26: " + e.getMsg());
+    }
+
+  case 27:
+    try {
+      simpleSql("CREATE TABLE profiles_votes(id_profile, id_level, PRIMARY KEY(id_profile, id_level));");
+      updateXmDbVersion(28);
+    } catch(Exception &e) {
+      throw Exception("Unable to update xmDb from 27: " + e.getMsg());
+    }
+
     // next
   }
 }
@@ -508,8 +695,7 @@ void xmDatabase::simpleSql(const std::string& i_sql) {
   char *errMsg;
   std::string v_errMsg;
 
-  //Logger::Log(i_sql.c_str());
-  //printf("%s\n", i_sql.c_str());
+  //LogDebug("simpleSql(%X): %s", this, i_sql.c_str());
 
   if(sqlite3_exec(m_db,
 		  i_sql.c_str(),
@@ -517,6 +703,7 @@ void xmDatabase::simpleSql(const std::string& i_sql) {
 		  NULL, &errMsg) != SQLITE_OK) {
     v_errMsg = errMsg;
     sqlite3_free(errMsg);
+    LogInfo(std::string("simpleSql failed on running : " + i_sql).c_str());
     throw Exception(v_errMsg);
   }
 }
@@ -537,17 +724,30 @@ char** xmDatabase::readDB(const std::string& i_sql, unsigned int &i_nrow) {
   char *errMsg;
   std::string v_errMsg;
   int v_nrow;
+	double v_startTime, v_endTime;
 
-  //printf("%s\n", i_sql.c_str());
 
+	//LogDebug("readDB(%X): %s", this, i_sql.c_str());
+
+	v_startTime = GameApp::getXMTime();
   if(sqlite3_get_table(m_db,
 		       i_sql.c_str(),
 		       &v_result, &v_nrow, &ncolumn, &errMsg)
      != SQLITE_OK) {
     v_errMsg = errMsg;
     sqlite3_free(errMsg);
+    LogError("xmDb failed while running :");
+    LogInfo("%s", i_sql.c_str());
+    LogError("%s", v_errMsg.c_str())
     throw Exception("xmDb: " + v_errMsg);
+
   }
+  v_endTime = GameApp::getXMTime();
+
+  if(v_endTime - v_startTime > DB_MAX_SQL_RUNTIME) {
+    LogWarning("long query time detected (%.3f'') for query '%s'", v_endTime - v_startTime, i_sql.c_str());
+  }
+
   i_nrow = (unsigned int) v_nrow;
 
   return v_result;
@@ -593,6 +793,11 @@ void xmDatabase::createUserFunctions() {
 			     user_xm_userChildrenCompliant, NULL, NULL) != SQLITE_OK) {
     throw Exception("xmDatabase::createUserFunctions() failed !");
   }
+
+  if(sqlite3_create_function(m_db, "xm_replaceStart", 3, SQLITE_ANY, NULL,
+			     user_xm_replaceStart, NULL, NULL) != SQLITE_OK) {
+    throw Exception("xmDatabase::createUserFunctions() failed !");
+  }
 }
 
 void xmDatabase::user_xm_floord(sqlite3_context* i_context, int i_nArgs, sqlite3_value** i_values) {
@@ -638,3 +843,28 @@ void xmDatabase::user_xm_userChildrenCompliant(sqlite3_context* i_context, int i
   v_value = sqlite3_value_int(i_values[0]);
   sqlite3_result_int(i_context, XMSession::instance()->useChildrenCompliant() ? v_value : 1);
 }
+
+void xmDatabase::user_xm_replaceStart(sqlite3_context* i_context, int i_nArgs, sqlite3_value** i_values) {
+  std::string v_value, v_old, v_new, v_result, v_begin, v_end;
+
+  if(i_nArgs != 3) {
+    throw Exception("user_xm_replaceStart failed !");
+  }
+
+  v_value = (char*) sqlite3_value_text(i_values[0]);
+  v_old   = (char*) sqlite3_value_text(i_values[1]);
+  v_new   = (char*) sqlite3_value_text(i_values[2]);
+
+  v_begin = v_value.substr(0, v_old.length());
+
+  if(v_begin != v_old) {
+    /* change nothing */
+    sqlite3_result_text(i_context, v_value.c_str(), -1, SQLITE_TRANSIENT);
+    return;
+  }
+
+  v_end = v_value.substr(v_old.length(), v_value.length() - v_old.length());
+  v_result = v_new + v_end;
+
+  sqlite3_result_text(i_context, v_result.c_str(), -1, SQLITE_TRANSIENT);
+}
