@@ -25,7 +25,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include <string>
 #include <vector>
 #include "xmDatabaseUpdateInterface.h"
-#include "helpers/MultiSingleton.h"
+#include "../helpers/MultiSingleton.h"
 
 class Level;
 
@@ -42,8 +42,9 @@ public:
 	    const std::string& i_gameDir,
 	    const std::string& i_userDir,
 	    const std::string& i_binPackCheckSum,
+	    bool i_dbDirsCheck,
 	    XmDatabaseUpdateInterface *i_interface = NULL);
-  void init(const std::string& i_dbFileUTF8);
+  void init(const std::string& i_dbFileUTF8, bool i_readOnly = false);
 
   /* RULE:
      all write access must be done from class xmDatabase
@@ -57,21 +58,26 @@ public:
   static void setTrace(bool i_value);
 
   /* stats */
-  void stats_createProfile(const std::string& i_profile);
+  void stats_createProfile(const std::string& i_sitekey, const std::string& i_profile);
   void stats_destroyProfile(const std::string& i_profile);
-  void stats_levelCompleted(const std::string &PlayerName,
+  bool stats_checkKeyExists_stats_profiles(const std::string& i_sitekey, const std::string& i_profile);
+  void stats_levelCompleted(const std::string& i_sitekey,
+			    const std::string &PlayerName,
 			    const std::string &LevelID,
 			    int i_playTime);
-  void stats_died(const std::string &PlayerName,
+  void stats_died(const std::string& i_sitekey,
+		  const std::string &PlayerName,
 		  const std::string &LevelID,
 		  int i_playTime);
-  void stats_abortedLevel(const std::string &PlayerName,
+  void stats_abortedLevel(const std::string& i_sitekey,
+			  const std::string &PlayerName,
 			  const std::string &LevelID,
 			  int i_playTime);
-  void stats_levelRestarted(const std::string &PlayerName,
+  void stats_levelRestarted(const std::string& i_sitekey,
+			    const std::string &PlayerName,
 			    const std::string &LevelID,
 			    int i_playTime);
-  void stats_xmotoStarted(const std::string &PlayerName);
+  void stats_xmotoStarted(const std::string& i_sitekey, const std::string &PlayerName);
 
   /* levels */
   void levels_updateDB(const std::vector<Level *>& i_levels,
@@ -92,14 +98,14 @@ public:
 		  const std::string& i_checkSum,
 		  const std::string& i_author, const std::string& i_description,
 		  const std::string& i_date, const std::string& i_packName, const std::string& i_packNum,
-		  const std::string& i_music, bool i_isScripted,
+		  const std::string& i_music, bool i_isScripted, bool i_isPhysics,
 		  bool i_isToReload);
   void levels_update(const std::string& i_id_level,
 		     const std::string& i_filepath, const std::string& i_name,
 		     const std::string& i_checkSum,
 		     const std::string& i_author, const std::string& i_description,
 		     const std::string& i_date, const std::string& i_packName, const std::string& i_packNum,
-		     const std::string& i_music, bool i_isScripted,
+		     const std::string& i_music, bool i_isScripted, bool i_isPhysics,
 		     bool i_isToReload);
   void levels_add_end();
   void levels_cleanNoWWWLevels();
@@ -115,6 +121,7 @@ public:
   void replays_add_end();
   void replays_delete(const std::string& i_replay);
   bool replays_exists(const std::string& i_name);
+  void replays_print();
 
   /* themes */
   bool themes_isIndexUptodate() const;
@@ -129,7 +136,10 @@ public:
   std::string themes_getFileName(const std::string& i_id_theme);
 
   /* profiles */
-  void profiles_addFinishTime(const std::string& i_profile, const std::string& i_id_level,
+  void profiles_addFinishTime(const std::string& i_sitekey, const std::string& i_profile, const std::string& i_id_level,
+			      const std::string& i_timeStamp, int i_finishTime);
+  /* for old migration, when sitekey didn't exist */
+  void profiles_addFinishTime_nositekey(const std::string& i_profile, const std::string& i_id_level,
 			      const std::string& i_timeStamp, int i_finishTime);
 
   /* web */
@@ -142,6 +152,34 @@ public:
   int webrooms_getHighscoreTime(const std::string& i_id_room,
 				const std::string& i_id_level); /* or a negativ value if not */
   void webLoadDataFirstTime();
+  bool isOnTheWeb(const std::string& i_id_level);
+
+  /* configuration */
+	/* if you save several values, use begin and end to make it faster */
+	void config_setValue_begin();
+	void config_setValue_end();
+  std::string config_getString(const std::string& i_id_profile, const std::string& i_key,  const std::string& i_default);
+  bool        config_getBool(const std::string& i_id_profile, const std::string& i_key,    bool i_default);
+  int         config_getInteger(const std::string& i_id_profile, const std::string& i_key, int i_default);
+  float       config_getFloat(const std::string& i_id_profile, const std::string& i_key,   float i_default);
+  void        config_setString(const std::string& i_id_profile, const std::string& i_key,  const std::string& i_value);
+  void        config_setBool(const std::string& i_id_profile, const std::string& i_key,    bool i_value);
+  void        config_setInteger(const std::string& i_id_profile, const std::string& i_key, int i_value);
+  void        config_setFloat(const std::string& i_id_profile, const std::string& i_key,   float i_value);
+
+  // sync
+  std::string getXmDbSiteKey();
+  void sync_buildServerFile(const std::string& i_outFile, const std::string& i_sitekey, const std::string& i_profile);
+  void sync_updateDB(const std::string& i_profile, const std::string& i_sitekey, const std::string& i_file, int i_newDbSyncServer);
+
+  /**
+     @brief Mark all lines that should be synchronized as synchronized
+   */
+  void setSynchronized();
+
+  /* votes */
+  void markAsVoted(const std::string& i_profile, const std::string& i_id_level);
+  bool isVoted(const std::string& i_profile, const std::string& i_id_level);
 
   /* data fixes */
   void fixStatsProfilesLevelsNbCompleted();
@@ -159,6 +197,7 @@ public:
   static void user_xm_lvlUpdatedToTxt(sqlite3_context* i_context, int i_nArgs, sqlite3_value** i_values);
   static void user_xm_userCrappy(sqlite3_context* i_context, int i_nArgs, sqlite3_value** i_values);
   static void user_xm_userChildrenCompliant(sqlite3_context* i_context, int i_nArgs, sqlite3_value** i_values);
+  static void user_xm_replaceStart(sqlite3_context* i_context, int i_nArgs, sqlite3_value** i_values);
 
   /* function used to synchronise with the last xmoto version */
   int  getXmDbVersion();
@@ -173,6 +212,24 @@ public:
   void setXmDbUserDir(const std::string& i_userDir);
   void setXmDbBinPackCheckSum(const std::string& i_binPackChecksum);
 
+  /* update db directories without reanalysing all the files on the disk */
+  void updateXMDirectories(const std::string& i_oldGameDir, const std::string& i_newGameDir,
+			   const std::string& i_oldUserDir, const std::string& i_newUserDir);
+
+  std::string setXmDbSiteKey();
+  
+  /* multiple sites */
+  static std::string generateSiteKey();
+
+  /* config factorize sql*/
+  bool getXmParameterKey(const std::string& i_key, std::string& o_value); // get a parameter value as string
+  void setXmParameterKey(const std::string& i_key, const std::string& i_value); // set a parameter value as string
+  std::string config_getValue(const std::string& i_id_profile, const std::string& i_key, unsigned int& o_nrow);
+  void config_setValue(const std::string& i_id_profile, const std::string& i_key, const std::string& i_value);
+
+  /* if i_sitekey is empty, don't care of it -- require for old db without sitekey */
+  void updateDB_config(const std::string& i_sitekey); /* conversion of the old file values */
+
   /* trace */
   static void sqlTrace(void*, const char* sql);
 
@@ -184,8 +241,8 @@ public:
   void updateDB_stats(XmDatabaseUpdateInterface *i_interface = NULL);    /* statistics */
   void updateDB_favorite(const std::string& i_profile,
 			 XmDatabaseUpdateInterface *i_interface = NULL); /* favorite */
-  bool stats_checkKeyExists_stats_profiles(const std::string& i_profile);
-  bool stats_checkKeyExists_stats_profiles_levels(const std::string& i_profile,
+  bool stats_checkKeyExists_stats_profiles_levels(const std::string& i_sitekey,
+						  const std::string& i_profile,
 						  const std::string& i_level);
   void updateDB_profiles(XmDatabaseUpdateInterface *i_interface = NULL); /* profiles */
 
