@@ -19,10 +19,10 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 =============================================================================*/
 
 #include "xmDatabase.h"
-#include "xmscene/Level.h"
-#include "VXml.h"
-#include "helpers/Log.h"
-#include "VFileIO.h"
+#include "../xmscene/Level.h"
+#include "../VXml.h"
+#include "../helpers/Log.h"
+#include "../VFileIO.h"
 
 void xmDatabase::levels_add_begin(bool i_isToReload) {
   simpleSql("BEGIN TRANSACTION;");
@@ -49,7 +49,7 @@ void xmDatabase::levels_updateDB(const std::vector<Level *>&    i_levels,
     levels_add(i_levels[i]->Id()      , i_levels[i]->FileName(), i_levels[i]->Name(),
 	       i_levels[i]->Checksum(), i_levels[i]->Author()  , i_levels[i]->Description(),
 	       i_levels[i]->Date()    , i_levels[i]->Pack()    , i_levels[i]->PackNum(),
-	       i_levels[i]->Music()   , i_levels[i]->isScripted(),
+	       i_levels[i]->Music()   , i_levels[i]->isScripted(), i_levels[i]->isPhysics(),
 	       i_isToReload
 	       );
   }
@@ -162,11 +162,11 @@ void xmDatabase::levels_add(const std::string& i_id_level,
 			    const std::string& i_packName,
 			    const std::string& i_packNum,
 			    const std::string& i_music,
-			    bool i_isScripted,
+			    bool i_isScripted, bool i_isPhysics,
 			    bool i_isToReload) {
   simpleSql("INSERT INTO levels(id_level,"
 	    "filepath, name, checkSum, author, description, "
-	    "date_str, packName, packNum, music, isScripted, isToReload) "
+	    "date_str, packName, packNum, music, isScripted, isPhysics, isToReload) "
 	    "VALUES(\"" + protectString(i_id_level) + "\", " +
 	    "\"" + protectString(i_filepath)        + "\", " +
 	    "\"" + protectString(i_name)            + "\", " +
@@ -178,6 +178,7 @@ void xmDatabase::levels_add(const std::string& i_id_level,
 	    "\"" + protectString(i_packNum)         + "\", " +
 	    "\"" + protectString(i_music)           + "\", " +
 	    std::string(i_isScripted  ? "1" : "0")  + ", "   +
+	    std::string(i_isPhysics   ? "1" : "0")  + ", "   +
 	    std::string(i_isToReload  ? "1" : "0")  +
 	    std::string(");"));
 }
@@ -187,7 +188,7 @@ void xmDatabase::levels_update(const std::string& i_id_level,
 			       const std::string& i_checkSum,
 			       const std::string& i_author, const std::string& i_description,
 			       const std::string& i_date, const std::string& i_packName, const std::string& i_packNum,
-			       const std::string& i_music, bool i_isScripted,
+			       const std::string& i_music, bool i_isScripted, bool i_isPhysics,
 			       bool i_isToReload) {
   simpleSql("UPDATE levels SET name=\""  + 
 	    protectString(i_name)     	 + "\", filepath=\"" +
@@ -199,7 +200,8 @@ void xmDatabase::levels_update(const std::string& i_id_level,
 	    protectString(i_packName)    + "\", packNum=\"" +
 	    protectString(i_packNum)     + "\", music=\"" +
 	    protectString(i_music)       + "\", isScripted=" +
-	    std::string(i_isScripted  ? "1" : "0") + ", isToReload=" +
+	    std::string(i_isScripted  ? "1" : "0") + ", isPhysics=" +
+	    std::string(i_isPhysics   ? "1" : "0") + ", isToReload=" +
 	    std::string(i_isToReload  ? "1" : "0") + " WHERE id_level=\"" +
 	    protectString(i_id_level)    + "\";");
 }
@@ -224,18 +226,20 @@ void xmDatabase::levels_cleanNoWWWLevels() {
     v_filepath = getResult(v_result, 3, i, 2);
 
     if(FS::isInUserDir(v_filepath)) { // remove only files of the user dir
-      Logger::Log("Removing level %s (%s)", v_name.c_str(), v_filepath.c_str());
+      LogInfo("Removing level %s (%s)", v_name.c_str(), v_filepath.c_str());
       try {
 	simpleSql("DELETE FROM levels WHERE id_level=\"" + protectString(v_id_level) + "\";");
 
 	v_basename  = FS::getFileBaseName(v_filepath);
 	if(FS::renameUserFile(v_filepath, v_savePath + "/" + v_basename + ".lvl") == false) {
-	  Logger::Log("Unable to move the file into the trash");
+	  LogWarning("Unable to move the file into the trash");
 	}
       } catch(Exception &e) {
-	Logger::Log("Removing the level failed !");
+	LogWarning("Removing the level failed !");
       }
 
+    } else {
+      LogWarning("NOT Removing level %s (%s) : the level is not is the user space", v_name.c_str(), v_filepath.c_str());
     }
   }
   read_DB_free(v_result);
