@@ -19,10 +19,11 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 =============================================================================*/
 
 #include "xmDatabase.h"
-#include "VXml.h"
-#include "helpers/VExcept.h"
-#include "helpers/Log.h"
-#include "WWW.h"
+#include "../VXml.h"
+#include "../helpers/VExcept.h"
+#include "../helpers/Log.h"
+#include "../WWW.h"
+#include <sstream>
 
 bool xmDatabase::webrooms_checkKeyExists_id_room(const std::string& i_id_room) {
   return checkKey("SELECT count(1) FROM webrooms "
@@ -194,6 +195,10 @@ int xmDatabase::webrooms_getHighscoreTime(const std::string& i_id_room,
   return atoi(v_res.c_str());
 }
 
+bool xmDatabase::isOnTheWeb(const std::string& i_id_level) {
+  return checkKey("SELECT count(1) FROM weblevels WHERE id_level=\"" + protectString(i_id_level) + "\";");
+}
+
 void xmDatabase::weblevels_updateDB(const std::string& i_weblevelsFile) {
   XMLDocument v_webLXml;
   TiXmlDocument *v_webLXmlData;
@@ -331,7 +336,7 @@ void xmDatabase::webrooms_updateDB(const std::string& i_webroomsFile) {
     while(pVarElem != NULL) {
       
       pc = pVarElem->Attribute("name");
-      if(pc == NULL) continue;
+      if(pc == NULL) continue; // hehe it feels the unlimited loop, too tired to fix that tonight
       v_RoomName = pc;
   
       pc = pVarElem->Attribute("highscores_url");
@@ -428,12 +433,12 @@ void xmDatabase::webLoadDataFirstTime() {
     read_DB_free(v_result);
 
     if(v_update) {
-      Logger::Log("Loading weblevels with delivered weblevels.xml");
+      LogInfo("Loading weblevels with delivered weblevels.xml");
       weblevels_updateDB("default/weblevels.xml");
     }
   } catch(Exception &e) {
     /* ok, no pb */
-    Logger::Log("** Warning ** : Loading delivered weblevels.xml failed");
+    LogWarning("Loading delivered weblevels.xml failed");
   }
 
   /* updating webhighscores table with defaults */
@@ -446,11 +451,11 @@ void xmDatabase::webLoadDataFirstTime() {
     read_DB_free(v_result);
 
     if(v_update) {
-      Logger::Log("Loading webhighscores with delivered webhighscores.xml");
+      LogInfo("Loading webhighscores with delivered webhighscores.xml");
       webhighscores_updateDB("default/webhighscores.xml", DEFAULT_WEBHIGHSCORES_URL);
     }
   } catch(Exception &e) {
     /* ok, no pb */
-    Logger::Log("** Warning ** : Loading delivered webhighscores.xml failed");
+    LogWarning("Loading delivered webhighscores.xml failed");
   }
 }
