@@ -22,13 +22,14 @@
  *  Simple 2D drawing library, built closely on top of OpenGL.
  */
 #include "DrawLibOpenGL.h"
-#include "helpers/utf8.h"
-#include "xmscene/Camera.h"
-#include "helpers/Log.h"
-#include "Image.h"
-#include "VFileIO.h"
-#include "VTexture.h"
-#include "helpers/Singleton.h"
+#include "../helpers/utf8.h"
+#include "../xmscene/Camera.h"
+#include "../helpers/Log.h"
+#include "../Image.h"
+#include "../VFileIO.h"
+#include "../VTexture.h"
+#include "../include/xm_hashmap.h"
+#include "../helpers/Singleton.h"
 
 #define UTF8_INTERLINE_SPACE 2
 #define UTF8_INTERCHAR_SPACE 0
@@ -113,11 +114,11 @@ public:
   virtual ~GLFontManager();
 
   FontGlyph* getGlyph(const std::string& i_string);
-  void printString(FontGlyph* i_glyph, int i_x, int i_y, Color i_color, bool i_shadowEffect = false);
+  void printString(FontGlyph* i_glyph, int i_x, int i_y, Color i_color, float i_perCentered = -1.0, bool i_shadowEffect = false);
   void printStringGrad(FontGlyph* i_glyph, int i_x, int i_y,
-		       Color c1, Color c2, Color c3, Color c4, bool i_shadowEffect = false);
+		       Color c1, Color c2, Color c3, Color c4, float i_perCentered = -1.0, bool i_shadowEffect = false);
   void printStringGradOne(FontGlyph* i_glyph, int i_x, int i_y,
-			  Color c1, Color c2, Color c3, Color c4);
+			  Color c1, Color c2, Color c3, Color c4, float i_perCentered = -1.0);
 
   virtual unsigned int nbGlyphsInMemory();
 
@@ -127,6 +128,8 @@ private:
 
   std::vector<GLFontGlyphLetter*> m_glyphsLettersList;
   HashNamespace::hash_map<const char*, GLFontGlyphLetter*, HashNamespace::hash<const char*>, hashcmp_str> m_glyphsLetters;
+
+  unsigned int getLonguestLineSize(const std::string& i_value, unsigned int i_start = 0, unsigned int i_nbLinesToRead = -1);
 };
 
 
@@ -155,24 +158,24 @@ DrawLibOpenGL::DrawLibOpenGL() : DrawLib(){
 /*===========================================================================
   Transform an OpenGL vertex to pure 2D 
   ===========================================================================*/
-void DrawLibOpenGL::glVertexSP(float x,float y) {
-  glVertex2f(x, m_nActualHeight - y);
+void DrawLibOpenGL::glVertexSP(float x,float y) { 	 
+  glVertex2f(x, m_nActualHeight - y); 	 
+} 	 
+
+void DrawLibOpenGL::glVertex(float x,float y) { 	 
+  glVertex2f(x, y); 	 
+} 	 
+
+void DrawLibOpenGL::glTexCoord(float x,float y){ 	 
+  glTexCoord2f(x, y); 	 
 }
 
-void DrawLibOpenGL::glVertex(float x,float y) {
-  glVertex2f(x,y);
-}
-
-void DrawLibOpenGL::glTexCoord(float x,float y){
-  glTexCoord2f(x,y);
-} 
-
-void DrawLibOpenGL::screenProjVertex(float *x,float *y) {
+void DrawLibOpenGL::screenProjVertex(float *x, float *y) {
   *y = m_nActualHeight - (*y);
 }
 
-void DrawLibOpenGL::setClipRect(int x , int y , unsigned int w , unsigned int h){
-  glScissor(x,m_nDispHeight - (y+h),w,h);
+void DrawLibOpenGL::setClipRect(int x, int y, unsigned int w, unsigned int h){
+  glScissor(x, m_nDispHeight - (y+h), w, h);
     
   m_nLScissorX = x;
   m_nLScissorY = y;
@@ -181,23 +184,23 @@ void DrawLibOpenGL::setClipRect(int x , int y , unsigned int w , unsigned int h)
 }
 
 void DrawLibOpenGL::setClipRect(SDL_Rect * clip_rect){
-  if (clip_rect != NULL){
-    setClipRect(clip_rect->x,clip_rect->y,clip_rect->w,clip_rect->h);
+  if(clip_rect != NULL){
+    setClipRect(clip_rect->x, clip_rect->y, clip_rect->w, clip_rect->h);
   }
 }
 
-void DrawLibOpenGL::getClipRect(int *px,int *py,int *pnWidth,int *pnHeight) {
+void DrawLibOpenGL::getClipRect(int *px, int *py, int *pnWidth, int *pnHeight) {
   *px = m_nLScissorX;
   *py = m_nLScissorY;
-  *pnWidth = m_nLScissorW;
+  *pnWidth  = m_nLScissorW;
   *pnHeight = m_nLScissorH;
 }  
   
-void DrawLibOpenGL::setScale(float x,float y){
+void DrawLibOpenGL::setScale(float x, float y){
   glScalef(x,y,1);
 }
-void DrawLibOpenGL::setTranslate(float x,float y){
-  glTranslatef(x,y, 0);
+void DrawLibOpenGL::setTranslate(float x, float y){
+  glTranslatef(x, y, 0);
 }
 
 void DrawLibOpenGL::setMirrorY() {
@@ -214,121 +217,123 @@ void DrawLibOpenGL::setLineWidth(float width){
   glLineWidth(width);
 }
  
-void DrawLibOpenGL::init(unsigned int nDispWidth,unsigned int nDispHeight,unsigned int nDispBPP,bool bWindowed){
-
-    
+void DrawLibOpenGL::init(unsigned int nDispWidth, unsigned int nDispHeight, unsigned int nDispBPP, bool bWindowed){
   /* Set suggestions */
-  m_nDispWidth = nDispWidth;
+  m_nDispWidth  = nDispWidth;
   m_nDispHeight = nDispHeight;
-  m_nDispBPP = nDispBPP;
-  m_bWindowed = bWindowed;
+  m_nDispBPP    = nDispBPP;
+  m_bWindowed   = bWindowed;
 
   /* Get some video info */
-  const SDL_VideoInfo *pVidInfo=SDL_GetVideoInfo();
-  if(pVidInfo==NULL)
+  const SDL_VideoInfo* pVidInfo = SDL_GetVideoInfo();
+  if(pVidInfo == NULL)
     throw Exception("(1) SDL_GetVideoInfo : " + std::string(SDL_GetError()));
   
   /* Determine target bit depth */
-  if(m_bWindowed) 
+  if(m_bWindowed ==  true) 
     /* In windowed mode we can't tinker with the bit-depth */
     m_nDispBPP=pVidInfo->vfmt->BitsPerPixel;      
 
   /* Setup GL stuff */
   /* 2005-10-05 ... note that we no longer ask for explicit settings... it's
      better to do it per auto */
-  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1); 
+  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1); 
   
   /* Create video flags */
   int nFlags = SDL_OPENGL;
-  if(!m_bWindowed) nFlags|=SDL_FULLSCREEN;
+  if(m_bWindowed == false)
+    nFlags |= SDL_FULLSCREEN;
   
   /* At last, try to "set the video mode" */
-  if((m_screen=SDL_SetVideoMode(m_nDispWidth,m_nDispHeight,m_nDispBPP,nFlags))==NULL) {
-    Logger::Log("** Warning ** : Tried to set video mode %ix%i @ %i-bit, but SDL failed (%s)\n"
-		"                Now SDL will try determining a proper mode itself.", m_nDispWidth, m_nDispHeight, m_nDispBPP, SDL_GetError());
+  if((m_screen=SDL_SetVideoMode(m_nDispWidth, m_nDispHeight, m_nDispBPP, nFlags)) == NULL) {
+    LogWarning("Tried to set video mode %ix%i @ %i-bit, but SDL failed (%s)\n"
+	       "                Now SDL will try determining a proper mode itself.", m_nDispWidth, m_nDispHeight, m_nDispBPP, SDL_GetError());
     m_nDispBPP = 0;
 
     /* Hmm, try letting it decide the BPP automatically */
-    if((m_screen=SDL_SetVideoMode(m_nDispWidth,m_nDispHeight,0,nFlags))==NULL) {       
+    if((m_screen = SDL_SetVideoMode(m_nDispWidth, m_nDispHeight, 0, nFlags)) == NULL) {       
       /* Still no luck */
-      Logger::Log("** Warning ** : Still no luck, now we'll try 800x600 in a window.");
-      m_nDispWidth = 800; m_nDispHeight = 600; m_nDispBPP = 0;       
-      m_bWindowed = true;
-      if((m_screen=SDL_SetVideoMode(m_nDispWidth,m_nDispHeight,0,SDL_OPENGL))==NULL) {       
+      LogWarning("Still no luck, now we'll try 800x600 in a window.");
+      m_nDispWidth  = 800;
+      m_nDispHeight = 600;
+      m_nDispBPP    = 0;       
+      m_bWindowed   = true;
+      m_screen = SDL_SetVideoMode(m_nDispWidth, m_nDispHeight, 0, SDL_OPENGL);
+      if(m_screen == NULL) {       
 	throw Exception("SDL_SetVideoMode : " + std::string(SDL_GetError()));
-      }       
+      }
     }
   }
-    
+
   /* Retrieve actual configuration */
-  pVidInfo=SDL_GetVideoInfo();
-  if(pVidInfo==NULL)
+  pVidInfo = SDL_GetVideoInfo();
+  if(pVidInfo == NULL)
     throw Exception("(2) SDL_GetVideoInfo : " + std::string(SDL_GetError()));
                     
-  m_nDispBPP=pVidInfo->vfmt->BitsPerPixel;
+  m_nDispBPP = pVidInfo->vfmt->BitsPerPixel;
 
   /* Did we get a z-buffer? */        
   int nDepthBits;
-  SDL_GL_GetAttribute(SDL_GL_DEPTH_SIZE,&nDepthBits);
+  SDL_GL_GetAttribute(SDL_GL_DEPTH_SIZE, &nDepthBits);
   if(nDepthBits == 0)
     throw Exception("no depth buffer");  
   
-  m_menuCamera = new Camera(Vector2i(0,0),
-			    Vector2i(m_nDispWidth,m_nDispHeight));
+  m_menuCamera = new Camera(Vector2i(0, 0),
+			    Vector2i(m_nDispWidth, m_nDispHeight));
   m_menuCamera->setCamera2d();
-    
+
   glClearDepth(1);
   glDepthFunc(GL_LEQUAL);
-     
+
   /* Output some general info */
-  Logger:: Log("GL: %s (%s)",glGetString(GL_RENDERER),glGetString(GL_VENDOR));
-  if(glGetString(GL_RENDERER) == NULL || 
-     glGetString(GL_VENDOR) == NULL) {
-    Logger::Log("** Warning ** : GL strings NULL!");
+  LogInfo("GL: %s (%s)", glGetString(GL_RENDERER), glGetString(GL_VENDOR));
+  if(glGetString(GL_RENDERER) == NULL
+     || glGetString(GL_VENDOR) == NULL) {
+    LogWarning("GL strings NULL!");
     throw Exception("GL strings are NULL!");
   }
-    
+
   /* Windows: check whether we are using the standard GDI OpenGL software driver... If
      so make sure the user is warned */
 #if defined(WIN32) 
   if(!strcmp(reinterpret_cast<const char *>(glGetString(GL_RENDERER)),"GDI Generic") &&
      !strcmp(reinterpret_cast<const char *>(glGetString(GL_VENDOR)),"Microsoft Corporation")) {
-    Logger::Log("** Warning ** : No GL hardware acceleration!");
+    LogWarning("No GL hardware acceleration!");
     //m_UserNotify = "It seems that no OpenGL hardware acceleration is available!\n"
     //               "Please make sure OpenGL is configured properly.";
   }
 #endif
     
   /* Init OpenGL extensions */
-  if(m_bDontUseGLExtensions) {
-    m_bVBOSupported = false;
-    m_bFBOSupported = false;
+  if(m_bDontUseGLExtensions == true) {
+    m_bVBOSupported     = false;
+    m_bFBOSupported     = false;
     m_bShadersSupported = false;
   }
   else {
     m_bVBOSupported = isExtensionSupported("GL_ARB_vertex_buffer_object");
     m_bFBOSupported = isExtensionSupported("GL_EXT_framebuffer_object");
       
-    m_bShadersSupported = isExtensionSupported("GL_ARB_fragment_shader") &&
-      isExtensionSupported("GL_ARB_vertex_shader") &&
-      isExtensionSupported("GL_ARB_shader_objects");
+    m_bShadersSupported = isExtensionSupported("GL_ARB_fragment_shader")
+      && isExtensionSupported("GL_ARB_vertex_shader")
+      && isExtensionSupported("GL_ARB_shader_objects");
   }
     
-  if(m_bVBOSupported) {
+  if(m_bVBOSupported == true) {
     glGenBuffersARB=(PFNGLGENBUFFERSARBPROC)SDL_GL_GetProcAddress("glGenBuffersARB");
     glBindBufferARB=(PFNGLBINDBUFFERARBPROC)SDL_GL_GetProcAddress("glBindBufferARB");
     glBufferDataARB=(PFNGLBUFFERDATAARBPROC)SDL_GL_GetProcAddress("glBufferDataARB");
     glDeleteBuffersARB=(PFNGLDELETEBUFFERSARBPROC)SDL_GL_GetProcAddress("glDeleteBuffersARB");      
 
-    glEnableClientState( GL_VERTEX_ARRAY );   
-    glEnableClientState( GL_TEXTURE_COORD_ARRAY );
+    glEnableClientState(GL_VERTEX_ARRAY);
+    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
           
-    Logger::Log("GL: using ARB_vertex_buffer_object");    
+    LogInfo("GL: using ARB_vertex_buffer_object");
   }
   else
-    Logger::Log("GL: not using ARB_vertex_buffer_object");    
+    LogInfo("GL: not using ARB_vertex_buffer_object");
       
-  if(m_bFBOSupported) {
+  if(m_bFBOSupported == true) {
     glIsRenderbufferEXT = (PFNGLISRENDERBUFFEREXTPROC)SDL_GL_GetProcAddress("glIsRenderbufferEXT");
     glBindRenderbufferEXT = (PFNGLBINDRENDERBUFFEREXTPROC)SDL_GL_GetProcAddress("glBindRenderbufferEXT");
     glDeleteRenderbuffersEXT = (PFNGLDELETERENDERBUFFERSEXTPROC)SDL_GL_GetProcAddress("glDeleteRenderbuffersEXT");
@@ -347,12 +352,12 @@ void DrawLibOpenGL::init(unsigned int nDispWidth,unsigned int nDispHeight,unsign
     glGetFramebufferAttachmentParameterivEXT = (PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVEXTPROC)SDL_GL_GetProcAddress("glGetFramebufferAttachmentParameterivEXT");
     glGenerateMipmapEXT = (PFNGLGENERATEMIPMAPEXTPROC)SDL_GL_GetProcAddress("glGenerateMipmapEXT");
           
-    Logger::Log("GL: using EXT_framebuffer_object");
+    LogInfo("GL: using EXT_framebuffer_object");
   }
   else
-    Logger::Log("GL: not using EXT_framebuffer_object");
+    LogInfo("GL: not using EXT_framebuffer_object");
       
-  if(m_bShadersSupported) {
+  if(m_bShadersSupported == true) {
     glBindAttribLocationARB = (PFNGLBINDATTRIBLOCATIONARBPROC)SDL_GL_GetProcAddress("glBindAttribLocationARB");
     glGetActiveAttribARB = (PFNGLGETACTIVEATTRIBARBPROC)SDL_GL_GetProcAddress("glGetActiveAttribARB");
     glGetAttribLocationARB = (PFNGLGETATTRIBLOCATIONARBPROC)SDL_GL_GetProcAddress("glGetAttribLocationARB");
@@ -396,21 +401,21 @@ void DrawLibOpenGL::init(unsigned int nDispWidth,unsigned int nDispHeight,unsign
     glGetUniformivARB = (PFNGLGETUNIFORMIVARBPROC)SDL_GL_GetProcAddress("glGetUniformivARB");
     glGetShaderSourceARB = (PFNGLGETSHADERSOURCEARBPROC)SDL_GL_GetProcAddress("glGetShaderSourceARB");    
         
-    Logger::Log("GL: using ARB_fragment_shader/ARB_vertex_shader/ARB_shader_objects");
+    LogInfo("GL: using ARB_fragment_shader/ARB_vertex_shader/ARB_shader_objects");
   }
   else
-    Logger::Log("GL: not using ARB_fragment_shader/ARB_vertex_shader/ARB_shader_objects");
+    LogInfo("GL: not using ARB_fragment_shader/ARB_vertex_shader/ARB_shader_objects");
     
   /* Set background color to black */
-  glClearColor(0.0f,0.0f,0.0f,0.0f);
+  glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
   glClear(GL_COLOR_BUFFER_BIT);
   SDL_GL_SwapBuffers();  
 }
 
-void DrawLibOpenGL::unInit(){
+void DrawLibOpenGL::unInit()
+{
 }
-        
-  
+
 /*===========================================================================
   Check for OpenGL extension
   ===========================================================================*/
@@ -418,20 +423,21 @@ bool DrawLibOpenGL::isExtensionSupported(std::string Ext) {
   const unsigned char *pcExtensions = NULL;
   const unsigned char *pcStart;
   unsigned char *pcWhere,*pcTerminator;
-    
+
   pcExtensions = glGetString(GL_EXTENSIONS);
   if(pcExtensions == NULL) {
-    Logger::Log("Failed to determine OpenGL extensions. Try stopping all other\n"
-		"applications that might use your OpenGL hardware.\n"
-		"If it still doesn't work, please create a detailed bug report.\n"
-		);
+    LogError("Failed to determine OpenGL extensions. Try stopping all other\n"
+	     "applications that might use your OpenGL hardware.\n"
+	     "If it still doesn't work, please create a detailed bug report.\n"
+	     );
     throw Exception("glGetString() : NULL");
   }
-    
+
   pcStart = pcExtensions;
   while(1) {
-    pcWhere = (unsigned char *)strstr((const char*)pcExtensions,Ext.c_str());
-    if(pcWhere == NULL) break;
+    pcWhere = (unsigned char *)strstr((const char*)pcExtensions, Ext.c_str());
+    if(pcWhere == NULL)
+      break;
     pcTerminator = pcWhere + Ext.length();
     if(pcWhere == pcStart || *(pcWhere-1) == ' ')
       if(*pcTerminator == ' ' || *pcTerminator == '\0')
@@ -440,11 +446,11 @@ bool DrawLibOpenGL::isExtensionSupported(std::string Ext) {
   }
   return false;
 }
-  
+
 /*===========================================================================
   Grab screen contents
   ===========================================================================*/
-Img *DrawLibOpenGL::grabScreen(int i_reduce) {
+Img* DrawLibOpenGL::grabScreen(int i_reduce) {
   unsigned int v_imgH = m_nDispHeight / i_reduce;
   unsigned int v_imgW = m_nDispWidth  / i_reduce;
 
@@ -486,10 +492,6 @@ void DrawLibOpenGL::startDraw(DrawMode mode){
   
 void DrawLibOpenGL::endDraw(){
   glEnd();
-  if (m_texture != NULL){
-    glDisable(GL_TEXTURE_2D);
-    m_texture = NULL;
-  }
   if (m_blendMode != BLEND_MODE_NONE){
     glDisable(GL_BLEND);
   }
@@ -509,29 +511,29 @@ void DrawLibOpenGL::removePropertiesAfterEnd() {
   }
 }
 
-void DrawLibOpenGL::setColor(Color color){
-  glColor4ub(GET_RED(color),GET_GREEN(color),GET_BLUE(color),GET_ALPHA(color));
+void DrawLibOpenGL::setColor(Color color){ 	 
+  glColor4ub(GET_RED(color), GET_GREEN(color), GET_BLUE(color), GET_ALPHA(color)); 	 
 }
 
-void DrawLibOpenGL::setTexture(Texture *texture,BlendMode blendMode){
+void DrawLibOpenGL::setTexture(Texture *texture, BlendMode blendMode){
   setBlendMode(blendMode);
-  if (texture != NULL){
+  if(texture != NULL){
     /* bind texture only if different than the current one */
     if(m_texture == NULL || texture->Name != m_texture->Name){
-      glBindTexture(GL_TEXTURE_2D,texture->nID);
+      glBindTexture(GL_TEXTURE_2D, texture->nID);
     }
     glEnable(GL_TEXTURE_2D);
   } else {
     //so the texture is set to null
     //if the texture was not null we need
     //to disable the current texture
-    if (m_texture != NULL){
+    if(m_texture != NULL){
       glDisable(GL_TEXTURE_2D);
     }
   }
   m_texture = texture;
-} 
-  
+}
+
 void DrawLibOpenGL::setBlendMode(BlendMode blendMode){
   if(blendMode != BLEND_MODE_NONE){
     glEnable(GL_BLEND);
@@ -664,7 +666,7 @@ int ScrapTextures::allocateAndLoadTexture(unsigned int width,
   } else {
     // there's four scraps, approximatly less than 100 characters, this
     // should not happen
-    Logger::Log("Scrap is full.");
+    LogError("Scrap is full.");
     throw Exception("Scrap is full !");
   }
 }
@@ -709,7 +711,13 @@ GLFontGlyphLetter::GLFontGlyphLetter(const std::string& i_value, TTF_Font* i_ttf
 
   v_surf = TTF_RenderUTF8_Blended(i_ttf, i_value.c_str(), v_color);
   if (v_surf == NULL) {
-    throw Exception("GLFontGlyphLetter: " + std::string(TTF_GetError()));
+    //throw Exception("GLFontGlyphLetter: " + std::string(TTF_GetError()));
+    //sometimes, it happends with some special char, give a 2nd chance
+
+    v_surf = TTF_RenderUTF8_Blended(i_ttf, " ", v_color);
+    if (v_surf == NULL) {
+      throw Exception("GLFontGlyphLetter: " + std::string(TTF_GetError()));
+    }
   }
   SDL_SetAlpha(v_surf, 0, 0);
 
@@ -928,24 +936,27 @@ FontGlyph* GLFontManager::getGlyph(const std::string& i_string) {
 }
 
 void GLFontManager::printStringGrad(FontGlyph* i_glyph, int i_x, int i_y,
-				    Color c1, Color c2, Color c3, Color c4, bool i_shadowEffect) {
+				    Color c1, Color c2, Color c3, Color c4, float i_perCentered, bool i_shadowEffect) {
 
   if(i_shadowEffect) {
-    printStringGradOne(i_glyph, i_x,   i_y,   INVERT_COLOR(c1), INVERT_COLOR(c2), INVERT_COLOR(c3), INVERT_COLOR(c4));
-    printStringGradOne(i_glyph, i_x+1, i_y+1, c1, c2, c3, c4);
+    printStringGradOne(i_glyph, i_x,   i_y,   INVERT_COLOR(c1), INVERT_COLOR(c2), INVERT_COLOR(c3), INVERT_COLOR(c4), i_perCentered);
+    printStringGradOne(i_glyph, i_x+1, i_y+1, c1, c2, c3, c4, i_perCentered);
   } else {
-    printStringGradOne(i_glyph, i_x, i_y, c1, c2, c3, c4);
+    printStringGradOne(i_glyph, i_x, i_y, c1, c2, c3, c4, i_perCentered);
   }
 }
 
 void GLFontManager::printStringGradOne(FontGlyph* i_glyph, int i_x, int i_y,
-				       Color c1, Color c2, Color c3, Color c4) {
+				       Color c1, Color c2, Color c3, Color c4, float i_perCentered) {
   GLFontGlyph* v_glyph = (GLFontGlyph*) i_glyph;
   GLFontGlyphLetter* v_glyphLetter;
   int v_x, v_y;
-  unsigned int n = 0;
+  unsigned int n;
   std::string v_value, v_char;
   unsigned int v_lineHeight;
+  unsigned int v_longuest_linesize;
+  unsigned int v_current_linesize;
+  unsigned int v_size;
 
   int oldTextureId = -1;
   int newTextureId;
@@ -981,21 +992,29 @@ void GLFontManager::printStringGradOne(FontGlyph* i_glyph, int i_x, int i_y,
   if(v_value == "")
     return;
 
-  v_y = -i_y + m_drawLib->getDispHeight() - v_glyph->firstLineDrawHeight();/* la taille de la 1ere ligne */
-  v_x = i_x;
-  v_lineHeight = 0;
-
   try {
     /* draw the glyph */
     glEnable(GL_TEXTURE_2D);
     glEnable(GL_BLEND);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    // the first glBegin/glEnd will draw nothing
+    glBegin(GL_QUADS);
 
-    unsigned int size = v_value.size();
-    while(n < size) {
+    v_longuest_linesize = getLonguestLineSize(v_value);
+    v_size = v_value.size();
+    v_current_linesize = getLonguestLineSize(v_value, 0, 1);
+    v_x = i_x + (v_longuest_linesize - v_current_linesize)/2 * (i_perCentered+1.0);
+
+    v_y = -i_y + m_drawLib->getDispHeight() - v_glyph->firstLineDrawHeight();/* la taille de la 1ere ligne */
+    v_lineHeight = 0;
+
+    n = 0;
+
+    while(n < v_size) {
       utf8::getNextChar(v_value, n, v_char);
       if(v_char == "\n") {
-	v_x  = i_x;
+	v_current_linesize = getLonguestLineSize(v_value, n, 1);
+	v_x  = i_x + (v_longuest_linesize - v_current_linesize)/2 * (i_perCentered+1.0);
 	v_y -= v_lineHeight + UTF8_INTERLINE_SPACE;
 	v_lineHeight = 0;
       } else {
@@ -1042,8 +1061,46 @@ void GLFontManager::printStringGradOne(FontGlyph* i_glyph, int i_x, int i_y,
   }
 }
 
-void GLFontManager::printString(FontGlyph* i_glyph, int i_x, int i_y, Color i_color, bool i_shadowEffect) {
-  printStringGrad(i_glyph, i_x, i_y, i_color, i_color, i_color, i_color, i_shadowEffect);
+unsigned int GLFontManager::getLonguestLineSize(const std::string& i_value, unsigned int i_start, unsigned int i_nbLinesToRead) {
+  unsigned int v_longuest_linesize = 0;
+  unsigned int v_current_linesize;
+  unsigned int n = i_start;
+  unsigned int v_size = i_value.size();
+  std::string v_char;
+  unsigned int v_line_n = 0;
+  GLFontGlyphLetter* v_glyphLetter;
+
+  v_current_linesize = -UTF8_INTERCHAR_SPACE;
+  while(n < v_size) {
+    utf8::getNextChar(i_value, n, v_char);
+    if(v_char == "\n") {
+      if(v_current_linesize > v_longuest_linesize) {
+	v_longuest_linesize = v_current_linesize;
+      }
+      v_current_linesize = 0;
+      v_line_n++;
+
+      // enough lines read
+      if(i_nbLinesToRead > 0 && v_line_n == i_nbLinesToRead) {
+	return v_longuest_linesize;
+      }
+    } else {
+      v_glyphLetter = m_glyphsLetters[v_char.c_str()];
+      if(v_glyphLetter != NULL) {
+	v_current_linesize += v_glyphLetter->realWidth() + UTF8_INTERCHAR_SPACE;
+      }
+    }
+  }
+
+  if(v_current_linesize > v_longuest_linesize) {
+    v_longuest_linesize = v_current_linesize;
+  }
+
+  return v_longuest_linesize;
+}
+
+void GLFontManager::printString(FontGlyph* i_glyph, int i_x, int i_y, Color i_color, float i_perCentered, bool i_shadowEffect) {
+  printStringGrad(i_glyph, i_x, i_y, i_color, i_color, i_color, i_color, i_perCentered, i_shadowEffect);
 }
 
 #endif
