@@ -21,12 +21,17 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 /* 
  *  In-game graphical user interface
  */
-#include "VXml.h"
 #include "GUI.h"
-#include "GameText.h"
-#include "drawlib/DrawLib.h"
-#include "Game.h"
-#include "helpers/Log.h"
+#include "../../VXml.h"
+#include "../../GameText.h"
+#include "../../drawlib/DrawLib.h"
+#include "../../Game.h"
+#include "../../helpers/Log.h"
+#include "../../helpers/utf8.h"
+
+#ifdef ENABLE_OPENGL
+#include "../../include/xm_OpenGL.h"
+#endif
 
   DrawLib* UIWindow::m_drawLib = NULL;
 
@@ -206,7 +211,7 @@ bool UIWindow::isUglyMode() {
   /*===========================================================================
   Special message box for querying keypresses
   ===========================================================================*/    
-  bool UIQueryKeyBox::keyDown(int nKey, SDLMod mod,int nChar) {
+  bool UIQueryKeyBox::keyDown(int nKey, SDLMod mod, const std::string& i_utf8Char) {
     //    MessageBox(NULL,"HELLO",NULL,MB_OK);
     return false;
   }
@@ -218,19 +223,24 @@ bool UIWindow::isUglyMode() {
     /* Go through buttons... anything clicked? */
     for(unsigned int i=0;i<m_nNumButtons;i++) {
       if(m_pButtons[i]->isClicked()) {  
-        if(m_pButtons[i]->getCaption() == GAMETEXT_OK) return UI_MSGBOX_OK;
-        if(m_pButtons[i]->getCaption() == GAMETEXT_CANCEL) return UI_MSGBOX_CANCEL;
-        if(m_pButtons[i]->getCaption() == GAMETEXT_YES) return UI_MSGBOX_YES;
-        if(m_pButtons[i]->getCaption() == GAMETEXT_NO) return UI_MSGBOX_NO;
-        if(m_pButtons[i]->getCaption() == GAMETEXT_YES_FOR_ALL) return UI_MSGBOX_YES_FOR_ALL;
+        if(m_pButtons[i]->getCaption() == GAMETEXT_OK)
+	  return UI_MSGBOX_OK;
+        if(m_pButtons[i]->getCaption() == GAMETEXT_CANCEL)
+	  return UI_MSGBOX_CANCEL;
+        if(m_pButtons[i]->getCaption() == GAMETEXT_YES)
+	  return UI_MSGBOX_YES;
+        if(m_pButtons[i]->getCaption() == GAMETEXT_NO)
+	  return UI_MSGBOX_NO;
+        if(m_pButtons[i]->getCaption() == GAMETEXT_YES_FOR_ALL)
+	  return UI_MSGBOX_YES_FOR_ALL;
       }
     }
     return UI_MSGBOX_NOTHING;
   }
   
   bool UIMsgBox::setClicked(std::string Text) {
-    bool b=false;
-    for(unsigned int i=0;i<m_nNumButtons;i++) {
+    bool b = false;
+    for(unsigned int i=0; i<m_nNumButtons; i++) {
       if(m_pButtons[i]->getCaption() == Text) {
         m_pButtons[i]->setClicked(true);
         b = true;
@@ -290,13 +300,15 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
   void UIMsgBox::paint(void) {
     /* Should the OK button be disabled? (if any) */
     if(m_bTextInput && m_TextInput.empty()) {
-      for(unsigned int i=0;i<m_nNumButtons;i++) {
-        if(m_pButtons[i]->getCaption() == GAMETEXT_OK) m_pButtons[i]->enableWindow(false);
+      for(unsigned int i=0; i<m_nNumButtons; i++) {
+        if(m_pButtons[i]->getCaption() == GAMETEXT_OK)
+	  m_pButtons[i]->enableWindow(false);
       }
     }
     else {
-      for(unsigned int i=0;i<m_nNumButtons;i++) {
-        if(m_pButtons[i]->getCaption() == GAMETEXT_OK) m_pButtons[i]->enableWindow(true);
+      for(unsigned int i=0; i<m_nNumButtons; i++) {
+        if(m_pButtons[i]->getCaption() == GAMETEXT_OK)
+	  m_pButtons[i]->enableWindow(true);
       }
     }   
   
@@ -310,7 +322,7 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
     }
   }
   
-  bool UIMsgBox::keyDown(int nKey, SDLMod mod,int nChar) {
+  bool UIMsgBox::keyDown(int nKey, SDLMod mod, const std::string& i_utf8Char) {
     switch(nKey) {
       case SDLK_ESCAPE:
         if(!setClicked(GAMETEXT_CANCEL))
@@ -324,21 +336,20 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
         break;
       default:
         if(m_bTextInput) {
+
           switch(nKey) {
             case SDLK_BACKSPACE:
-              if(!m_TextInput.empty()) {
-                m_TextInput.erase(m_TextInput.end()-1);
-              }
+	      if(m_TextInput != "") {
+		m_TextInput = utf8::utf8_delete(m_TextInput, utf8::utf8_length(m_TextInput));
+	      }
               return true;
             default:
-              if(nChar) {
-                char c[2];
-                c[0] = nChar;
-                c[1] = '\0';
-                m_TextInput.append(c);
-              }
+	      if(utf8::utf8_length(i_utf8Char) == 1) { // alt/... and special keys must not be kept
+		m_TextInput = utf8::utf8_concat(m_TextInput, i_utf8Char);
+	      }
               return true;
           }
+
         }
         break;
     }
@@ -346,7 +357,7 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
     return false;
   }
   
-  UIMsgBox *UIWindow::msgBox(std::string Text,UIMsgBoxButton Buttons,bool bTextInput,bool bQuery) {
+  UIMsgBox *UIWindow::msgBox(std::string Text,UIMsgBoxButton Buttons,bool bTextInput,bool bQuery, bool i_verticallyLarge) {
     unsigned int nNumButtons = 0;
     
     if(Buttons & UI_MSGBOX_OK)
@@ -364,7 +375,13 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
     FontGlyph* v_fg = m_curFont->getGlyph(Text);
 
     const unsigned int nButtonSize = 57;
-    int w = (v_fg->realWidth() > nNumButtons*115 ? (v_fg->realWidth()) : nNumButtons*115) + 16 + 100;
+    int w;
+
+    if(i_verticallyLarge) {
+      w = (getPosition().nWidth*4)/5;
+    } else {
+      w = (v_fg->realWidth() > nNumButtons*115 ? (v_fg->realWidth()) : nNumButtons*115) + 16 + 100;
+    }
     int h = v_fg->realHeight() + nButtonSize + 24 + 100;
     
     if(bTextInput) h+=40;    
@@ -446,8 +463,9 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
   ===========================================================================*/
   void UIWindow::putTextS(int x,int y, std::string Text,
 			  int& o_width, int &o_height,
-			  float i_xper, float i_yper) {
-    if(m_curFont == NULL) return;
+			  float i_xper, float i_yper, float i_perCentered) {
+    if(m_curFont == NULL)
+      return;
 
     /* Draw text at location */
     Color c0 = MAKE_COLOR(GET_RED(getTextStyle().c0),
@@ -472,21 +490,21 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
     v_fm->printStringGrad(v_fg,
 			  getAbsPosX()+x + (int)(((float)v_fg->realWidth()) * i_xper),
 			  getAbsPosY()+y + (int)(((float)v_fg->realHeight()) * i_yper),
-			  c0, c1, c2, c3);
+			  c0, c1, c2, c3, i_perCentered);
     o_width  = v_fg->realWidth();
     o_height = v_fg->realHeight();
 }
 
-  void UIWindow::putText(int x,int y,std::string Text, float i_xper, float i_yper) {
+  void UIWindow::putText(int x,int y,std::string Text, float i_xper, float i_yper, float i_perCentered) {
     int v_width, v_height;
     putTextS(x, y, Text,
 	     v_width, v_height,
-	     i_xper, i_yper);
+	     i_xper, i_yper, i_perCentered);
   }
    
   void UIWindow::putImage(int x,int y,int nWidth,int nHeight,Texture *pImage) {
     if(pImage != NULL) {
-      getApp()->getDrawLib()->drawImage(Vector2f(x+getAbsPosX(),y+getAbsPosY()),
+      m_drawLib->drawImage(Vector2f(x+getAbsPosX(),y+getAbsPosY()),
 					Vector2f(x+nWidth+getAbsPosX(),y+nHeight+getAbsPosY()),
 					pImage,MAKE_COLOR(255,255,255,(int)(255*getOpacity()/100)), true);
     }
@@ -505,137 +523,140 @@ void UIMsgBox::makeActiveButton(UIMsgBoxButton i_button) {
   }
   
   void UIWindow::putRect(int x,int y,int nWidth,int nHeight,Color c) {
-    getApp()->getDrawLib()->drawBox(Vector2f(x+getAbsPosX(),y+getAbsPosY()),
+    m_drawLib->drawBox(Vector2f(x+getAbsPosX(),y+getAbsPosY()),
                       Vector2f(x+nWidth+getAbsPosX(),y+nHeight+getAbsPosY()),
                       0,MAKE_COLOR(GET_RED(c),GET_GREEN(c),GET_BLUE(c),(int)(GET_ALPHA(c)*getOpacity()/100)),0);
   }
 
-  void UIWindow::putElem(int x,int y,int nWidth,int nHeight,UIElem Elem,bool bDisabled,bool bActive) {
-    Texture *vTexture = NULL;
+void UIWindow::putElem(int x,int y,int nWidth,int nHeight,UIElem Elem,bool bDisabled,bool bActive) {
+  m_drawLib->setTexture(NULL, BLEND_MODE_NONE);
+  Texture *vTexture = NULL;
     
-    struct _ElemTable {
-      UIElem E;
-      int nX, nY, nWidth, nHeight;
-    };
+  struct _ElemTable {
+    UIElem E;
+    int nX, nY, nWidth, nHeight;
+  };
     
-    static _ElemTable Table[] = {
-      {UI_ELEM_LARGE_BUTTON_UP,0,0,207,57},
-      {UI_ELEM_LARGE_BUTTON_DOWN,0,57,207,57},
-      {UI_ELEM_SMALL_BUTTON_UP,0,114,115,57},
-      {UI_ELEM_SMALL_BUTTON_DOWN,115,114,115,57},
-      {UI_ELEM_CHECKBUTTON_UNCHECKED_UP,222,0,34,34},
-      {UI_ELEM_CHECKBUTTON_UNCHECKED_DOWN,222,34,34,34},
-      {UI_ELEM_CHECKBUTTON_CHECKED_UP,222,68,34,34},
-      {UI_ELEM_CHECKBUTTON_CHECKED_DOWN,222,222,34,34},
-      {UI_ELEM_RADIOBUTTON_UNCHECKED_UP,34,222,34,34},
-      {UI_ELEM_RADIOBUTTON_UNCHECKED_DOWN,68,222,34,34},
-      {UI_ELEM_RADIOBUTTON_CHECKED_UP,102,222,34,34},
-      {UI_ELEM_RADIOBUTTON_CHECKED_DOWN,0,222,34,34},
-      {UI_ELEM_SCROLLBUTTON_DOWN_UP,1,180,20,20},
-      {UI_ELEM_SCROLLBUTTON_DOWN_DOWN,22,180,20,20},
-      {UI_ELEM_SCROLLBUTTON_UP_UP,43,180,20,20},
-      {UI_ELEM_SCROLLBUTTON_UP_DOWN,64,180,20,20},
-      {UI_ELEM_SCROLLBUTTON_RIGHT_UP,85,180,20,20},
-      {UI_ELEM_SCROLLBUTTON_RIGHT_DOWN,106,180,20,20},
-      {UI_ELEM_SCROLLBUTTON_LEFT_UP,127,180,20,20},
-      {UI_ELEM_SCROLLBUTTON_LEFT_DOWN,148,180,20,20},
-      {UI_ELEM_FRAME_TL,169,180,8,8},
-      {UI_ELEM_FRAME_TM,172,180,20,8},
-      {UI_ELEM_FRAME_TR,187,180,8,8},
-      {UI_ELEM_FRAME_ML,169,189,8,8},
-      {UI_ELEM_FRAME_MM,178,184,8,20},
-      {UI_ELEM_FRAME_MR,187,184,8,20},
-      {UI_ELEM_FRAME_BL,169,198,8,8},
-      {UI_ELEM_FRAME_BM,172,198,20,8},
-      {UI_ELEM_FRAME_BR,187,198,8,8},
-      {(UIElem)-1,-1,-1,-1,-1}
-    };
+  static _ElemTable Table[] = {
+    {UI_ELEM_LARGE_BUTTON_UP,0,0,207,57},
+    {UI_ELEM_LARGE_BUTTON_DOWN,0,57,207,57},
+    {UI_ELEM_SMALL_BUTTON_UP,0,114,115,57},
+    {UI_ELEM_SMALL_BUTTON_DOWN,115,114,115,57},
+    {UI_ELEM_CHECKBUTTON_UNCHECKED_UP,222,0,34,34},
+    {UI_ELEM_CHECKBUTTON_UNCHECKED_DOWN,222,34,34,34},
+    {UI_ELEM_CHECKBUTTON_CHECKED_UP,222,68,34,34},
+    {UI_ELEM_CHECKBUTTON_CHECKED_DOWN,222,222,34,34},
+    {UI_ELEM_RADIOBUTTON_UNCHECKED_UP,34,222,34,34},
+    {UI_ELEM_RADIOBUTTON_UNCHECKED_DOWN,68,222,34,34},
+    {UI_ELEM_RADIOBUTTON_CHECKED_UP,102,222,34,34},
+    {UI_ELEM_RADIOBUTTON_CHECKED_DOWN,0,222,34,34},
+    {UI_ELEM_SCROLLBUTTON_DOWN_UP,1,180,20,20},
+    {UI_ELEM_SCROLLBUTTON_DOWN_DOWN,22,180,20,20},
+    {UI_ELEM_SCROLLBUTTON_UP_UP,43,180,20,20},
+    {UI_ELEM_SCROLLBUTTON_UP_DOWN,64,180,20,20},
+    {UI_ELEM_SCROLLBUTTON_RIGHT_UP,85,180,20,20},
+    {UI_ELEM_SCROLLBUTTON_RIGHT_DOWN,106,180,20,20},
+    {UI_ELEM_SCROLLBUTTON_LEFT_UP,127,180,20,20},
+    {UI_ELEM_SCROLLBUTTON_LEFT_DOWN,148,180,20,20},
+    {UI_ELEM_FRAME_TL,169,180,8,8},
+    {UI_ELEM_FRAME_TM,172,180,20,8},
+    {UI_ELEM_FRAME_TR,187,180,8,8},
+    {UI_ELEM_FRAME_ML,169,189,8,8},
+    {UI_ELEM_FRAME_MM,178,184,8,20},
+    {UI_ELEM_FRAME_MR,187,184,8,20},
+    {UI_ELEM_FRAME_BL,169,198,8,8},
+    {UI_ELEM_FRAME_BM,172,198,20,8},
+    {UI_ELEM_FRAME_BR,187,198,8,8},
+    {(UIElem)-1,-1,-1,-1,-1}
+  };
         
-    int nElem = 0;
-    _ElemTable *p = NULL;
-    while(Table[nElem].E != -1) {
-      if(Table[nElem].E == Elem) {
-        p = &Table[nElem];
-        break;
-      }
-      nElem++;
-    }
-    
-    if(p==NULL)
-      return;
-    
-    int w = nWidth;
-    int h = nHeight;
-
-    if(w < 0)
-      w = p->nWidth;
-    if(h < 0)
-      h = p->nHeight;
-    
-    float fX1 = ((float)p->nX) / 256.0f;
-    float fY1 = ((float)p->nY) / 256.0f;
-    float fX2 = ((float)p->nX+p->nWidth) / 256.0f;
-    float fY2 = ((float)p->nY+p->nHeight) / 256.0f;
-    
-    Color c1,c2,c3,c4;
-    c1=c2=c3=c4=MAKE_COLOR(255,255,255,(int)(255*getOpacity()/100));
-    
-    int cx = getAbsPosX() + x;
-    int cy = getAbsPosY() + y;
-    vTexture =NULL;
-    /* Nice. Now we know what to draw */    
-    if(bDisabled) {
-      vTexture = UITexture::getMiscDisabledTexture();
-    } else {
-      vTexture = UITexture::getMiscTexture();
-    }
-    getApp()->getDrawLib()->setTexture(vTexture,BLEND_MODE_A);
-    getApp()->getDrawLib()->startDraw(DRAW_MODE_POLYGON);
-
-    getApp()->getDrawLib()->setColor(c1);
-    getApp()->getDrawLib()->glTexCoord(fX1,fY1);
-    getApp()->getDrawLib()->glVertexSP(cx,cy);
-
-    getApp()->getDrawLib()->setColor(c2);
-    getApp()->getDrawLib()->glTexCoord(fX2,fY1);        
-    getApp()->getDrawLib()->glVertexSP(cx+w,cy);
-
-    getApp()->getDrawLib()->setColor(c3);
-    getApp()->getDrawLib()->glTexCoord(fX2,fY2);        
-    getApp()->getDrawLib()->glVertexSP(cx+w,cy+h);
-
-    getApp()->getDrawLib()->setColor(c4);
-    getApp()->getDrawLib()->glTexCoord(fX1,fY2);
-    getApp()->getDrawLib()->glVertexSP(cx,cy+h);
-
-    getApp()->getDrawLib()->endDraw();
-    
-    /* Active? If so we want a nice blinking overlay */
-    if(bActive && bDisabled == false) {
-      float s = 160 + 76*sin(getApp()->getXMTime()*10);
-      int n = (int)s;
-      if(n<0) n=0;
-      if(n>255) n=255; /* just to be sure, i'm lazy */    
-      c1=c2=c3=c4=MAKE_COLOR(255,255,255,(int)(n*getOpacity()/100));
-
-      vTexture = UITexture::getMiscActiveTexture();
-      getApp()->getDrawLib()->setTexture(vTexture,BLEND_MODE_A);
-      getApp()->getDrawLib()->startDraw(DRAW_MODE_POLYGON);
-      getApp()->getDrawLib()->setColor(c1);
-      getApp()->getDrawLib()->glTexCoord(fX1,fY1);
-      getApp()->getDrawLib()->glVertexSP(cx,cy);
-      getApp()->getDrawLib()->setColor(c2);
-      getApp()->getDrawLib()->glTexCoord(fX2,fY1);        
-      getApp()->getDrawLib()->glVertexSP(cx+w,cy);
-      getApp()->getDrawLib()->setColor(c3);
-      getApp()->getDrawLib()->glTexCoord(fX2,fY2);
-      getApp()->getDrawLib()->glVertexSP(cx+w,cy+h);
-      getApp()->getDrawLib()->setColor(c4);
-      getApp()->getDrawLib()->glTexCoord(fX1,fY2);
-      getApp()->getDrawLib()->glVertexSP(cx,cy+h);
-      getApp()->getDrawLib()->endDraw();
+  int nElem = 0;
+  _ElemTable *p = NULL;
+  while(Table[nElem].E != -1) {
+    if(Table[nElem].E == Elem) {
+      p = &Table[nElem];
+      break;
     }
+    nElem++;
   }
+    
+  if(p==NULL)
+    return;
+    
+  int w = nWidth;
+  int h = nHeight;
+
+  if(w < 0)
+    w = p->nWidth;
+  if(h < 0)
+    h = p->nHeight;
+
+  float fX1 = ((float)p->nX) / 256.0f;
+  float fY1 = ((float)p->nY) / 256.0f;
+  float fX2 = ((float)p->nX+p->nWidth) / 256.0f;
+  float fY2 = ((float)p->nY+p->nHeight) / 256.0f;
+
+  Color c1,c2,c3,c4;
+  c1=c2=c3=c4=MAKE_COLOR(255,255,255,(int)(255*getOpacity()/100));
+
+  int cx = getAbsPosX() + x;
+  int cy = getAbsPosY() + y;
+  vTexture =NULL;
+  /* Nice. Now we know what to draw */    
+  if(bDisabled) {
+    vTexture = UITexture::getMiscDisabledTexture();
+  } else {
+    vTexture = UITexture::getMiscTexture();
+  }
+  m_drawLib->setTexture(vTexture, BLEND_MODE_A);
+  m_drawLib->startDraw(DRAW_MODE_POLYGON);
+
+  m_drawLib->setColor(c1);
+  m_drawLib->glTexCoord(fX1,fY1);
+  m_drawLib->glVertexSP(cx,cy);
+
+  m_drawLib->setColor(c2);
+  m_drawLib->glTexCoord(fX2,fY1);        
+  m_drawLib->glVertexSP(cx+w,cy);
+
+  m_drawLib->setColor(c3);
+  m_drawLib->glTexCoord(fX2,fY2);        
+  m_drawLib->glVertexSP(cx+w,cy+h);
+
+  m_drawLib->setColor(c4);
+  m_drawLib->glTexCoord(fX1,fY2);
+  m_drawLib->glVertexSP(cx,cy+h);
+
+  m_drawLib->endDraw();
+    
+  /* Active? If so we want a nice blinking overlay */
+  if(bActive && bDisabled == false) {
+    float s = 160 + 76*sin(getApp()->getXMTime()*10);
+    int n = (int)s;
+    if(n<0)
+      n=0;
+    if(n>255)
+      n=255; /* just to be sure, i'm lazy */    
+    c1=c2=c3=c4=MAKE_COLOR(255,255,255,(int)(n*getOpacity()/100));
+
+    vTexture = UITexture::getMiscActiveTexture();
+    m_drawLib->setTexture(vTexture,BLEND_MODE_A);
+    m_drawLib->startDraw(DRAW_MODE_POLYGON);
+    m_drawLib->setColor(c1);
+    m_drawLib->glTexCoord(fX1,fY1);
+    m_drawLib->glVertexSP(cx,cy);
+    m_drawLib->setColor(c2);
+    m_drawLib->glTexCoord(fX2,fY1);        
+    m_drawLib->glVertexSP(cx+w,cy);
+    m_drawLib->setColor(c3);
+    m_drawLib->glTexCoord(fX2,fY2);
+    m_drawLib->glVertexSP(cx+w,cy+h);
+    m_drawLib->setColor(c4);
+    m_drawLib->glTexCoord(fX1,fY2);
+    m_drawLib->glVertexSP(cx,cy+h);
+    m_drawLib->endDraw();
+  }
+}
   
   /*
 largebuttonup (0,0)  (207x57)
@@ -678,6 +699,7 @@ UIRoot::UIRoot()
 {
   m_pApp = GameApp::instance();
   m_bShowContextMenu = true;
+  m_lastHover = NULL;
   _InitWindow();
 }
 
@@ -697,7 +719,8 @@ UIRoot::UIRoot()
   
   void UIRoot::_RootPaint(int x,int y,UIWindow *pWindow,UIRect *pRect) {
     /* Hidden? */
-    if(pWindow->isHidden()) return;
+    if(pWindow->isHidden())
+      return;
   
     /* Clip rect to this window */
     UIRect WindowRect;
@@ -721,7 +744,7 @@ UIRoot::UIRoot()
         printf("Draw Rect: %d %d %d %d\n",WindowRect.nX,WindowRect.nY,WindowRect.nWidth,WindowRect.nHeight);*/
         _RootPaint(x+pWindow->getPosition().nX,y+pWindow->getPosition().nY,pWindow->getChildren()[i],&WindowRect);      
       }
-      getApp()->getDrawLib()->setClipRect(NULL);
+      m_drawLib->setClipRect(NULL);
     }
   }
   
@@ -731,8 +754,8 @@ UIRoot::UIRoot()
     /* Clip to full screen */
     Screen.nX = 0;
     Screen.nY = 0;
-    Screen.nWidth = getApp()->getDrawLib()->getDispWidth();
-    Screen.nHeight = getApp()->getDrawLib()->getDispHeight();
+    Screen.nWidth = m_drawLib->getDispWidth();
+    Screen.nHeight = m_drawLib->getDispHeight();
       
     /* Draw root's children */
 #ifdef ENABLE_OPENGL
@@ -751,24 +774,24 @@ UIRoot::UIRoot()
       int nContextHelpHeight = 20;
       
       /* Shade out bottom of screen */
-      getApp()->getDrawLib()->setBlendMode(BLEND_MODE_A);
-      getApp()->getDrawLib()->startDraw(DRAW_MODE_POLYGON);
+      m_drawLib->setBlendMode(BLEND_MODE_A);
+      m_drawLib->startDraw(DRAW_MODE_POLYGON);
       //glColor4f(0,0,0,0);//fully transparent??
-      getApp()->getDrawLib()->setColorRGBA(0,0,0,0);
-      getApp()->getDrawLib()->glVertexSP(0,getApp()->getDrawLib()->getDispHeight()-nContextHelpHeight);
-      getApp()->getDrawLib()->glVertexSP(getApp()->getDrawLib()->getDispWidth(),getApp()->getDrawLib()->getDispHeight()-nContextHelpHeight);
+      m_drawLib->setColorRGBA(0,0,0,0);
+      m_drawLib->glVertexSP(0,m_drawLib->getDispHeight()-nContextHelpHeight);
+      m_drawLib->glVertexSP(m_drawLib->getDispWidth(),m_drawLib->getDispHeight()-nContextHelpHeight);
       //glColor4f(0,0,0,0.7);
-      getApp()->getDrawLib()->setColorRGBA(0,0,0,255 * 7 / 100);
-      getApp()->getDrawLib()->glVertexSP(getApp()->getDrawLib()->getDispWidth(),getApp()->getDrawLib()->getDispHeight());
-      getApp()->getDrawLib()->glVertexSP(0,getApp()->getDrawLib()->getDispHeight());
-      getApp()->getDrawLib()->endDraw();
+      m_drawLib->setColorRGBA(0,0,0,255 * 7 / 100);
+      m_drawLib->glVertexSP(m_drawLib->getDispWidth(),m_drawLib->getDispHeight());
+      m_drawLib->glVertexSP(0,m_drawLib->getDispHeight());
+      m_drawLib->endDraw();
         
       if(!m_CurrentContextHelp.empty()) {
         /* Print help string */
-	setFont(getApp()->getDrawLib()->getFontSmall());
+	setFont(m_drawLib->getFontSmall());
 	setTextSolidColor(MAKE_COLOR(255,255,0,255));
-	putText(getApp()->getDrawLib()->getDispWidth()  -5,
-		getApp()->getDrawLib()->getDispHeight() -1,
+	putText(m_drawLib->getDispWidth()  -5,
+		m_drawLib->getDispHeight() -1,
 		m_CurrentContextHelp, -1.0, -1.0);
       }
     }
@@ -806,8 +829,8 @@ UIRoot::UIRoot()
     _RootMouseEvent(this,UI_ROOT_MOUSE_WHEEL_DOWN,x,y);
   } 
   
-  bool UIRoot::keyDown(int nKey, SDLMod mod,int nChar) {
-    if(!_RootKeyEvent(this,UI_ROOT_KEY_DOWN,nKey,mod, nChar)) {
+  bool UIRoot::keyDown(int nKey, SDLMod mod, const std::string& i_utf8Char) {
+    if(!_RootKeyEvent(this,UI_ROOT_KEY_DOWN,nKey,mod, i_utf8Char)) {
       switch(nKey) {
         case SDLK_UP:
           activateUp();
@@ -821,23 +844,29 @@ UIRoot::UIRoot()
         case SDLK_RIGHT:
           activateRight();
           return true;
+        case SDLK_TAB:
+          activateNext();
+          return true;
+        case SDLK_BACKSPACE:
+          activatePrevious();
+          return true;
       }
       return false;
     }
     return true;
   }
   
-  bool UIRoot::keyUp(int nKey, SDLMod mod) {
-    return _RootKeyEvent(this,UI_ROOT_KEY_UP,nKey,mod, 0);   
+  bool UIRoot::keyUp(int nKey, SDLMod mod, const std::string& i_utf8Char) {
+    return _RootKeyEvent(this,UI_ROOT_KEY_UP,nKey,mod, i_utf8Char);
   }
   
-  bool UIRoot::_RootKeyEvent(UIWindow *pWindow,UIRootKeyEvent Event,int nKey, SDLMod mod,int nChar) {
+  bool UIRoot::_RootKeyEvent(UIWindow *pWindow,UIRootKeyEvent Event,int nKey, SDLMod mod, const std::string& i_utf8Char) {
     /* Hidden or disabled? */
     if(pWindow->isHidden() || pWindow->isDisabled()) return false;
 
     /* First try if any children want it */
     for(unsigned int i=0;i<pWindow->getChildren().size();i++) {
-      if(_RootKeyEvent(pWindow->getChildren()[i],Event,nKey,mod,nChar))
+      if(_RootKeyEvent(pWindow->getChildren()[i],Event,nKey,mod, i_utf8Char))
         return true;
     }
 
@@ -846,8 +875,8 @@ UIRoot::UIRoot()
       bool b = false;
     
       switch(Event) {
-        case UI_ROOT_KEY_DOWN: b=pWindow->keyDown(nKey, mod,nChar); break;
-        case UI_ROOT_KEY_UP: b=pWindow->keyUp(nKey, mod); break;        
+      case UI_ROOT_KEY_DOWN: b=pWindow->keyDown(nKey, mod, i_utf8Char); break;
+      case UI_ROOT_KEY_UP: b=pWindow->keyUp(nKey, mod, i_utf8Char); break;        
       }
       
       return b;
@@ -858,6 +887,73 @@ UIRoot::UIRoot()
     return true;
   }
   
+bool UIRoot::joystickAxisMotion(Uint8 i_joyNum, Uint8 i_joyAxis, Sint16 i_joyAxisValue) {
+  if(!_RootJoystickAxisMotionEvent(this, i_joyNum, i_joyAxis, i_joyAxisValue)) {
+    if(i_joyAxis % 2 == 0) { // horizontal
+      if(i_joyAxisValue < -(GUI_JOYSTICK_MINIMUM_DETECTION)) {
+	activateLeft();
+	return true;
+      } else if(i_joyAxisValue > GUI_JOYSTICK_MINIMUM_DETECTION) {
+	activateRight();
+	return true;
+      }
+    } else { // vertical
+      if(i_joyAxisValue < -(GUI_JOYSTICK_MINIMUM_DETECTION)) {
+	activateUp();
+	return true;
+      } else if(i_joyAxisValue > GUI_JOYSTICK_MINIMUM_DETECTION) {
+	activateDown();
+	return true;
+      }
+    }
+  }
+  return false;
+}
+
+bool UIRoot::joystickButtonDown(Uint8 i_joyNum, Uint8 i_joyButton) {
+  return _RootJoystickButtonDownEvent(this, i_joyNum, i_joyButton);
+}
+
+bool UIRoot::_RootJoystickAxisMotionEvent(UIWindow *pWindow, Uint8 i_joyNum, Uint8 i_joyAxis, Sint16 i_joyAxisValue) {
+  /* Hidden or disabled? */
+  if(pWindow->isHidden() || pWindow->isDisabled()) return false;
+
+  /* First try if any children want it */
+  for(unsigned int i=0;i<pWindow->getChildren().size();i++) {
+    if(_RootJoystickAxisMotionEvent(pWindow->getChildren()[i], i_joyNum, i_joyAxis, i_joyAxisValue))
+      return true;
+  }
+
+  /* Try this */
+  if(pWindow != this && pWindow->isActive()) {
+    return pWindow->joystickAxisMotion(i_joyNum, i_joyAxis, i_joyAxisValue);
+  }   
+  else return false;
+    
+  /* Ok */
+  return true;
+}
+
+bool UIRoot::_RootJoystickButtonDownEvent(UIWindow *pWindow, Uint8 i_joyNum, Uint8 i_joyButton) {
+  /* Hidden or disabled? */
+  if(pWindow->isHidden() || pWindow->isDisabled()) return false;
+
+  /* First try if any children want it */
+  for(unsigned int i=0;i<pWindow->getChildren().size();i++) {
+    if(_RootJoystickButtonDownEvent(pWindow->getChildren()[i], i_joyNum, i_joyButton))
+      return true;
+  }
+
+  /* Try this */
+  if(pWindow != this && pWindow->isActive()) {
+    return pWindow->joystickButtonDown(i_joyNum, i_joyButton);
+  }   
+  else return false;
+    
+  /* Ok */
+  return true;
+}
+
   bool UIRoot::_RootMouseEvent(UIWindow *pWindow,UIRootMouseEvent Event,int x,int y) {
     /* Hidden or disabled? */
     if(pWindow->isHidden() || pWindow->isDisabled()) return false;
@@ -879,12 +975,21 @@ UIRoot::UIRoot()
       /* Recurse children */
       for(int i=(int)pWindow->getChildren().size()-1; i>=0; i--) {
         if(pWindow->getChildren()[i]->offerMouseEvent()) {
-          if(_RootMouseEvent(pWindow->getChildren()[i],Event,wx,wy)) return true;
-        }
+          if(_RootMouseEvent(pWindow->getChildren()[i],Event,wx,wy)) {
+	    return true;
+	  }
+	}
       }
       
       /* Then it must be this */
       if(pWindow != this) {
+	if(m_lastHover != pWindow) { // change of focus for the children
+	  if(m_lastHover != NULL) {
+	    m_lastHover->mouseOut(x, y);
+	  }
+	  m_lastHover = pWindow;
+	}
+
         switch(Event) {
           case UI_ROOT_MOUSE_LBUTTON_DOWN: pWindow->mouseLDown(wx,wy); break;
           case UI_ROOT_MOUSE_RBUTTON_DOWN: pWindow->mouseRDown(wx,wy); break;
@@ -989,21 +1094,25 @@ UIRoot::UIRoot()
   ===========================================================================*/
   unsigned int UIRoot::_UpdateActivationMap(UIWindow *pWindow, UIRootActCandidate *pMap,
 					    unsigned int nNum, unsigned int nMax) {
-    if(nNum >= nMax)
+    if(nNum >= nMax){
       return nNum;
+    }
   
     /* Find all windows which accepts activations, and list them in a nice structure */
-    if(!pWindow->isDisabled() && pWindow->offerActivation()) {
-      pMap[nNum].x = pWindow->getAbsPosX() + pWindow->getPosition().nWidth / 2;
-      pMap[nNum].y = pWindow->getAbsPosY() + pWindow->getPosition().nHeight / 2;
+    if(!pWindow->isDisabled() && pWindow->offerActivation() && pWindow->canGetFocus()) {
+      pMap[nNum].x = pWindow->getAbsPosX(); 
+      pMap[nNum].nWidth= pWindow->getPosition().nWidth;
+      pMap[nNum].y = pWindow->getAbsPosY();
+      pMap[nNum].nHeight = pWindow->getPosition().nHeight;
       pMap[nNum].pWindow = pWindow;
       nNum++;
     }
     
     /* Recurse */
     for(unsigned int i=0;i<pWindow->getChildren().size();i++) {
-      if(!pWindow->isHidden() && !pWindow->isDisabled())
+      if(!pWindow->isHidden() && !pWindow->isDisabled()){
         nNum = _UpdateActivationMap(pWindow->getChildren()[i],pMap,nNum,nMax);
+      }
     }
     
     /* Return number of candidates */
@@ -1011,90 +1120,181 @@ UIRoot::UIRoot()
   }
 
   int UIRoot::_GetActiveIdx(UIRootActCandidate *pMap, unsigned int nNum) {
-    for(unsigned int i=0; i<nNum; i++)
-      if(pMap[i].pWindow->isActive())
+    for(unsigned int i=0; i<nNum; i++){
+      if(pMap[i].pWindow->isActive()){
         return i;
+      }
+    }
     return -1;
   }
 
-  void UIRoot::_ActivateByVector(int dx, int dy) {
-    /* Update the activation map */
-    UIRootActCandidate Map[128];
-    unsigned int nNum = _UpdateActivationMap(this, Map, 0, 128);
+void UIRoot::_ActivateByVector(int dx, int dy) {
+  /* Update the activation map */
+  UIRootActCandidate Map[128];
+  unsigned int nNum = _UpdateActivationMap(this, Map, 0, 128);
     
-    if(nNum > 0) {
-      /* Find the currently active window */
-      int nActive = _GetActiveIdx(Map, nNum);
+  if(nNum > 0) {
+    /* Find the currently active window */
+    int nActive = _GetActiveIdx(Map, nNum);
 
-      /* No active? */
-      if(nActive < 0) {
-        /* Simply activate the first */        
-        deactivate(this);
-        if(dx<0 || dy<0)
-          Map[0].pWindow->setActive(true);
-        else if(dx>0 || dy>0)
-          Map[nNum-1].pWindow->setActive(true);
-        return;
+    /* No active? */
+    if(nActive < 0) {
+      /* Simply activate the first */        
+      deactivate(this);
+      if(dx<0 || dy<0)
+	Map[0].pWindow->setActive(true);
+      else if(dx>0 || dy>0)
+	Map[nNum-1].pWindow->setActive(true);
+      return;
+    }
+    else {
+      /* Okay, determine where to go from here */
+      /* Calculate a "best estimate" rating for each candidate, and select the best */
+      int nBest          = -1;
+	  int hBest    = -1;
+      int vBest    = -1;
+	  int hCurrent = -1;
+      int vCurrent = -1;
+      unsigned int uActive = nActive;
+	  int ax = Map[uActive].x + Map[uActive].nWidth / 2;
+      int ay = Map[uActive].y + Map[uActive].nHeight / 2;
+
+      for(unsigned int i=0; i<nNum; i++) {
+	if(i != uActive) {
+	  /* Calculate rating */
+      int vx = Map[i].x - ax;
+	  int vy = Map[i].y - ay;
+	  int vx2 = Map[i].x + Map[i].nWidth  - ax;
+	  int vy2 = Map[i].y + Map[i].nHeight - ay;
+
+      /* Avoid crazy candidates */
+      int mx= (vx+vx2)/2;
+      int my= (vy+vy2)/2;
+      if( (dx>0 && mx<=0) || (dx<0 && mx>=0) || (dy>0 && my<=0) || (dy<0 && my>=0) ){
+          continue;
+      }
+      
+      vx = Map[i].x - ax;
+	  vy = Map[i].y - ay;
+	  vx2 = Map[i].x + Map[i].nWidth  - ax;
+	  vy2 = Map[i].y + Map[i].nHeight - ay;
+
+      
+      
+      int xx1 = vx, yy1 = vy;
+      /* Distance with corners */
+      if(vx>0 && vy2<0){
+        yy1 = vy2;
+      }
+      else if(vx2<0 && vy>0){
+          xx1 = vx2;
+      }
+      else if(vx2<0 && vy2<0) {
+          xx1 = vx2;
+          yy1= vy2;
+      }
+      /* Distance with edges */
+      else if(vx<=0 && vx2>=0 && vy >= 0){
+        xx1 = 0;
+        yy1 = vy;
+      }
+      else if(vx<=0 && vx2>=0 && vy2 <= 0){
+        xx1 = 0;
+        yy1 = vy2;
+      }
+      else if(vy<=0 && vy2>=0 && vx >= 0){
+        xx1 = vx;
+        yy1 = 0;
+      }
+      else if(vy<=0 && vy2>=0 && vx2 <= 0){
+        xx1 = vx2;
+        yy1 = 0;
+      }
+
+      if(dx != 0 && dy == 0){
+          if(xx1 != 0){
+            hCurrent = abs(xx1);
+            vCurrent = abs(yy1);
+          }
+      }
+
+      if(dx == 0 && dy != 0){
+          if(yy1 != 0){
+            hCurrent = abs(yy1);
+            vCurrent = abs(xx1);
+          }
+      }
+
+            
+	  /* Good? */
+	  if( (vCurrent<=vBest && vBest>=0) || vBest < 0 ) {
+	    if( (vCurrent < vBest) || (hCurrent < hBest) || vBest <0 ) {
+            vBest = vCurrent;
+            hBest = hCurrent;
+            nBest = i;
+        }
+	  }
+	}
+      }
+        
+      /* Did we get something? */
+      if(nBest>=0) {
+	deactivate(this);
+	Map[nBest].pWindow->setActive(true);          
+          
+	m_CurrentContextHelp = Map[nBest].pWindow->getContextHelp();
       }
       else {
-        /* Okay, determine where to go from here */
-        /* Calculate a "best estimate" rating for each candidate, and select the best */
-        int nBest          = -1;
-	int nCurrentRating = -1;
-	unsigned int uActive = nActive;
-        for(unsigned int i=0; i<nNum; i++) {
-          if(i != uActive) {
-            /* Calculate rating */
-            int vx = Map[i].x - Map[uActive].x;
-            int vy = Map[i].y - Map[uActive].y;
-            int nRating = 0;
-            float xx1 = vx, yy1 = vy;
-            float fDist = sqrt(xx1*xx1 + yy1*yy1);
-            if(fDist == 0.0f) continue;
-            xx1 /= fDist; yy1 /= fDist;
-            float xx2 = dx, yy2 = dy;
-           
-            float fAngle = (acos(xx1*xx2 + yy1*yy2) / 3.14159f) * 180.0f;
-            if((vx<0 && dx<0) || (vx>0 && dx>0) || (vy<0 && dy<0) || (vy>0 && dy>0)) {
-              if(fAngle < 45.0f) {
-                nRating = 1500 - (int)(fAngle + fDist*2) + 1000;
-	      } else {
-		nRating = 1500 - (int)(fAngle + fDist*2) + 500;
-	      }
-            } else {
-	      nRating = 1500 - (int)(fAngle + fDist*3) + 100;
-	    }
-            
-            /* Good? */
-            if(nRating > 0 && nRating > nCurrentRating) {
-	      nCurrentRating = nRating;
-              nBest = i;
-	      //printf("best is %i (rating=%i)\n", nBest, nRating);
-            }
-          }
-        }
-        
-        /* Did we get something? */
-        if(nBest>=0) {
-          deactivate(this);
-          Map[nBest].pWindow->setActive(true);          
-          
-          m_CurrentContextHelp = Map[nBest].pWindow->getContextHelp();
-        }
-        else {
-          deactivate(this);
-          if(dx<0 || dy<0) {
-            Map[0].pWindow->setActive(true);
-            m_CurrentContextHelp = Map[0].pWindow->getContextHelp();
-          }
-          else if(dx>0 || dy>0) {
-            Map[nNum-1].pWindow->setActive(true);
-            m_CurrentContextHelp = Map[nNum-1].pWindow->getContextHelp();
-          }
-        }
+	deactivate(this);
+	if(dx<0 || dy<0) {
+	  Map[0].pWindow->setActive(true);
+	  m_CurrentContextHelp = Map[0].pWindow->getContextHelp();
+	}
+	else if(dx>0 || dy>0) {
+	  Map[nNum-1].pWindow->setActive(true);
+	  m_CurrentContextHelp = Map[nNum-1].pWindow->getContextHelp();
+	}
       }
     }
   }
+}
+
+void UIRoot::_ActivateByStep(int step) {
+  /* Update the activation map */
+  UIRootActCandidate Map[128];
+  unsigned int nNum = _UpdateActivationMap(this, Map, 0, 128);
+
+  if(nNum > 0) {
+    /* Find the currently active window */
+    int nActive = _GetActiveIdx(Map, nNum);
+
+    /* No active? */
+    if(nActive < 0) {
+      /* Simply activate the first */        
+      deactivate(this);
+      if(step > 0){
+        Map[0].pWindow->setActive(true);
+      }
+      else if(step < 0) {
+        Map[nNum-1].pWindow->setActive(true);
+      }
+      return;
+    }
+    else {
+        unsigned int nBest = nActive + step;
+        if(nBest < 0 ){
+            nBest += nNum;
+        }
+        if(nBest >= nNum){
+            nBest -= nNum;
+        }
+        deactivate(this);
+        Map[nBest].pWindow->setActive(true);          
+        m_CurrentContextHelp = Map[nBest].pWindow->getContextHelp();
+        return;
+    }
+  }
+}
 
   void UIRoot::activateUp(void) {
     _ActivateByVector(0,-1);
@@ -1112,6 +1312,14 @@ UIRoot::UIRoot()
     _ActivateByVector(1,0);
   }
 
+  void UIRoot::activateNext(void) {
+    _ActivateByStep(1);
+  }
+
+  void UIRoot::activatePrevious(void) {
+    _ActivateByStep(-1);
+  }
+
 void UIRoot::dispatchMouseHover() {
   int nX,nY;
   GameApp::getMousePos(&nX,&nY);
@@ -1140,7 +1348,7 @@ void UIProgressBar::paint()
   // 2.paint progress in red
   putRect(0, 0, width * m_progress / 100, height, MAKE_COLOR(255,0,0,255));
   // 3.write curOp left centered
-  setFont(getApp()->getDrawLib()->getFontSmall());
+  setFont(m_drawLib->getFontSmall());
   setTextSolidColor(MAKE_COLOR(255,255,255,128));
   putText(0, 0, m_curOp, 0.0, 0.0);
 }
