@@ -23,6 +23,16 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #include <string>
 
+enum LogLevel {LOG_ERROR, LOG_WARNING, LOG_INFO, LOG_DEBUG};
+
+			  // using ## gnu extension to allow empty args list
+#define LogError(format, ...)                                        Logger::LogLevelMsg(LOG_ERROR,   format, ## __VA_ARGS__);
+#define LogWarning(format, ...)                                      Logger::LogLevelMsg(LOG_WARNING, format, ## __VA_ARGS__);
+#define LogInfo(format, ...)                                         Logger::LogLevelMsg(LOG_INFO,    format, ## __VA_ARGS__);
+
+// a class using LogDebug must be aware of XMSession
+#define LogDebug(format, ...)   if(XMSession::instance()->debug()) { Logger::LogLevelMsg(LOG_DEBUG,   format, ## __VA_ARGS__); }
+
 class Logger {
   public:
   static void init(const std::string& i_logFile);
@@ -30,7 +40,8 @@ class Logger {
   static bool isInitialized();
 
   static void setVerbose(bool i_value);
-  static void Log(const char *pcFmt, ...);
+  static void LogLevelMsg(LogLevel i_level, const char *pcFmt, ...);
+  static void LogData(void* data, unsigned int len);
 
   private:
   static bool  m_isInitialized;
