@@ -23,12 +23,14 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #include "../VCommon.h"
 #include "VExcept.h"
+#include <stdlib.h> // for rand
+#include <math.h>
 
 /*===========================================================================
   Vectors
   ===========================================================================*/
 #define VECTOR_COMPARE_EPSILON    0.001f
-#define PI 3.14159265
+#define PI 3.14159265f
 #define rad2deg(x) ((x)*57.295779524)
 #define deg2rad(x) ((x)*0.017453292)
 
@@ -309,8 +311,25 @@ int intersectLineCircle2f(const Vector2f &Cp,float Cr,const Vector2f &A0,const V
                           Vector2f &Res1,Vector2f &Res2);
 int intersectLineLine2f(const Vector2f &A0,const Vector2f &A1,const Vector2f &B0,
                         const Vector2f &B1,Vector2f &Res);
-bool circleTouchCircle2f(const Vector2f &Cp1,float Cr1,const Vector2f &Cp2,float Cr2);
-  
+inline bool circleTouchCircle2f(const Vector2f &Cp1, float Cr1,
+				const Vector2f &Cp2, float Cr2)
+{
+  float xSquared = Cp2.x - Cp1.x;
+  xSquared *= xSquared;
+
+  float ySquared = Cp2.y - Cp1.y;
+  ySquared *= ySquared;
+
+  float sumRadiiSquared = Cr1 + Cr2;
+  sumRadiiSquared *= sumRadiiSquared;
+
+  return xSquared + ySquared <= sumRadiiSquared;
+}
+
+enum AABBSide {
+  AABB_TOP, AABB_BOTTOM, AABB_LEFT, AABB_RIGHT
+};
+
 class AABB {
  public:
   AABB();
@@ -319,21 +338,84 @@ class AABB {
   Vector2f& getBMin() {return BMin;}
   Vector2f& getBMax() {return BMax;}
 
-  bool lineTouchAABB2f(const Vector2f &P0, const Vector2f &P1);
-  bool circleTouchAABB2f(const Vector2f &Cp, const float Cr);
-  bool pointTouchAABB2f(const Vector2f &P);
-  bool AABBTouchAABB2f(const Vector2f &BMin1, const Vector2f &BMax1);
-  void addPointToAABB2f(const Vector2f &Point);
-  void addPointToAABB2f(const float x, const float y);
+  bool lineTouchBorder(const Vector2f &P0,const Vector2f &P1, Vector2f& o_touchOnePoint, AABBSide& o_side);
+  bool lineTouchAABB2f(const Vector2f& P0, const Vector2f& P1);
+  bool circleTouchAABB2f(const Vector2f& Cp, const float Cr);
+  bool pointTouchAABB2f(const Vector2f& P);
+  bool AABBTouchAABB2f(const Vector2f& BMin1, const Vector2f& BMax1);
+
+  void addPointToAABB2f(const Vector2f& Point);
+  inline void addPointToAABB2f(const float x, const float y) {
+    if(x < BMin.x) BMin.x = x;
+    if(y < BMin.y) BMin.y = y;
+    if(x > BMax.x) BMax.x = x;
+    if(y > BMax.y) BMax.y = y;    
+  }
+
+  // to avoid recalculate it from all the points
+  inline void translateAABB(const float x, const float y) {
+    BMin.x += x;
+    BMax.x += x;
+    BMin.y += y;
+    BMax.y += y;
+  }
 
  private:
   Vector2f BMin, BMax;
 };
 
-float randomNum(float fMin,float fMax);
-int randomIntNum(int nMin, int nMax);
 
-void intersectLineLine2fCramer(Vector2f& A1, Vector2f& A2, Vector2f& B1, Vector2f& B2, Vector2f& contactPoint);
+class BoundingCircle {
+public:
+  BoundingCircle();
+  void reset();
+
+  inline const Vector2f& getCenter() const {
+    return m_center;
+  }
+  inline float getRadius() const {
+    return m_radius;
+  }
+
+  void init(Vector2f& center, float radius);
+
+  bool lineTouch(const Vector2f &P0, const Vector2f &P1);
+  bool circleTouch(const Vector2f &Cp, const float Cr);
+  bool pointTouch(const Vector2f &point);
+
+  void translate(const float x, const float y);
+
+  void addPointToCircle(const float x, const float y);
+  void calculateBoundingCircle();
+
+  // get a bounding box for the circle
+  AABB& getAABB() {
+    return m_aabb;
+  }
+
+private:
+  Vector2f m_center;
+  float    m_radius;
+
+  float m_xmin, m_xmax, m_ymin, m_ymax;
+
+  AABB  m_aabb;
+
+  void calculateBoundingBox();
+};
+
+
+// random with stdlib rand
+inline float randomNum(float fMin,float fMax)
+{
+  return fMin + (fMax-fMin) * ((float)rand() / (float)RAND_MAX);
+}
+inline int randomIntNum(int nMin, int nMax)
+{
+  return nMin + (int) (((float)nMax-nMin) * rand()/(RAND_MAX+1.0));
+}
+
+void intersectLineLine2fCramer(Vector2f A1, Vector2f A2, Vector2f B1, Vector2f B2, Vector2f& contactPoint);
 void calculatePointOnNormal(Vector2f& A1, Vector2f& B1, float length, bool inside, Vector2f& A2, Vector2f& B2);
 void calculatePointOnVector(Vector2f& A1, Vector2f& A2, float length, Vector2f& newA2);
 
@@ -341,4 +423,3 @@ float interpolation_cubic(float i_a, float i_b, float i_c, float i_d, float t);
 Vector2f interpolation_cubic(Vector2f i_a, Vector2f i_b, Vector2f i_c, Vector2f i_d, float t);
 
 #endif
-
