@@ -35,14 +35,12 @@ class StateCreditsMode : public StateReplaying {
   
   virtual bool render();
   /* input */
-  virtual void keyDown(int nKey, SDLMod mod,int nChar);
-  virtual void keyUp(int nKey,   SDLMod mod);
-  virtual void mouseDown(int nButton);
-  virtual void mouseDoubleClick(int nButton);
-  virtual void mouseUp(int nButton);
+  virtual void xmKey(InputEventType i_type, const XMKey& i_xmkey);
   
   private:
   Credits* m_credits;
+
+  void abort();
 };
 
 #endif
