@@ -31,7 +31,7 @@ class StateDeadJust : public StateScene {
   virtual void enter();
   
   /* input */
-  virtual void keyDown(int nKey, SDLMod mod,int nChar);
+  virtual void xmKey(InputEventType i_type, const XMKey& i_xmkey);
 
   virtual void restartLevel(bool i_reloadLevel = false);
   virtual void nextLevel(bool i_positifOrder = true);
