@@ -33,15 +33,16 @@ public:
 
   void setReplay(std::string replayName);
 
-  void keyDown(int nKey, SDLMod mod,int nChar);
+  virtual void xmKey(InputEventType i_type, const XMKey& i_xmkey);
+  virtual void executeOneCommand(std::string cmd, std::string args);
 
 protected:
   void callAfterThreadFinished(int threadResult);
   virtual void checkEvents() {}
   
 private:
-  std::string m_replayName;
-  bool        m_launchReplaying;
+  std::string  m_replayName;
+  bool         m_launchReplaying;
 };
 
 #endif
