@@ -20,15 +20,15 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #include "StateLevelInfoViewer.h"
 #include "StatePreplayingReplay.h"
-#include "drawlib/DrawLib.h"
-#include "GameText.h"
-#include "XMSession.h"
+#include "../drawlib/DrawLib.h"
+#include "../GameText.h"
+#include "../XMSession.h"
 #include "StateReplaying.h"
 #include "StateMessageBox.h"
-#include "Replay.h"
-#include "Game.h"
-#include "helpers/Log.h"
-#include "helpers/Text.h"
+#include "../Replay.h"
+#include "../Game.h"
+#include "../helpers/Log.h"
+#include "../helpers/Text.h"
 
 /* static members */
 UIRoot*  StateLevelInfoViewer::m_sGUI = NULL;
@@ -41,11 +41,18 @@ StateLevelInfoViewer::StateLevelInfoViewer(const std::string& level,
 {
   m_level = level;
   m_name  = "StateLevelInfoViewer";
+
+  StateManager::instance()->registerAsObserver("REPLAYS_DELETE", this);
+  StateManager::instance()->registerAsObserver("REPLAYS_UPDATED", this);
+  if(XMSession::instance()->debug() == true) {
+    StateManager::instance()->registerAsEmitter("REPLAYS_UPDATED");
+  }
 }
 
 StateLevelInfoViewer::~StateLevelInfoViewer()
 {
-
+  StateManager::instance()->unregisterAsObserver("REPLAYS_DELETE", this);
+  StateManager::instance()->unregisterAsObserver("REPLAYS_UPDATED", this);
 }
 
 
@@ -55,14 +62,14 @@ void StateLevelInfoViewer::enter()
   m_GUI = m_sGUI;
   updateGUI();
 
-  GameApp::instance()->playMusic("menu1");
+  GameApp::instance()->playMenuMusic("menu1");
 
   StateMenu::enter();
 }
 
 void StateLevelInfoViewer::enterAfterPop()
 {
-  GameApp::instance()->playMusic("menu1");
+  GameApp::instance()->playMenuMusic("menu1");
   StateMenu::enterAfterPop();
 }
 
@@ -126,25 +133,24 @@ void StateLevelInfoViewer::checkEvents()
   }
 }
 
-void StateLevelInfoViewer::send(const std::string& i_id, UIMsgBoxButton i_button, const std::string& i_input) {
+void StateLevelInfoViewer::sendFromMessageBox(const std::string& i_id, UIMsgBoxButton i_button, const std::string& i_input) {
   if(i_id == "REPLAYS_DELETE") {
     if(i_button == UI_MSGBOX_YES) {
-      m_commands.push("REPLAYS_DELETE");
+      addCommand("REPLAYS_DELETE");
     }
     return;
+  } else {
+    StateMenu::sendFromMessageBox(i_id, i_button, i_input);
   }
 }
 
-void StateLevelInfoViewer::send(const std::string& i_id, const std::string& i_message) {
-  if(i_message == "REPLAYS_UPDATED") {
-    updateLevelInfoViewerReplays();
-    return;
-  }
-}
-
-void StateLevelInfoViewer::executeOneCommand(std::string cmd) {
+void StateLevelInfoViewer::executeOneCommand(std::string cmd, std::string args)
+{
   UIListEntry *pEntry = NULL;
 
+  LogDebug("cmd [%s [%s]] executed by state [%s].",
+	   cmd.c_str(), args.c_str(), getName().c_str());
+
   if(cmd == "REPLAYS_DELETE") {
     UIList* v_list = reinterpret_cast<UIList*>(m_GUI->getChild("LEVEL_VIEWER_FRAME:LEVEL_VIEWER_TABS:LEVEL_VIEWER_REPLAYS_TAB:LEVEL_VIEWER_REPLAYS_LIST"));
     if(v_list->getSelected() >= 0 && v_list->getSelected() < v_list->getEntries().size()) {
@@ -153,26 +159,31 @@ void StateLevelInfoViewer::executeOneCommand(std::string cmd) {
 	try {
 	  Replay::deleteReplay(pEntry->Text[0]);
 	} catch(Exception &e) {
-	  Logger::Log(e.getMsg().c_str());
+	  LogError(e.getMsg().c_str());
 	}
 	try {
 	  xmDatabase::instance("main")->replays_delete(pEntry->Text[0]);
 	} catch(Exception &e) {
-	  Logger::Log(e.getMsg().c_str());
+	  LogError(e.getMsg().c_str());
 	}
 	StateManager::instance()->sendAsynchronousMessage("REPLAYS_UPDATED");
 	updateLevelInfoViewerReplays();
       }
     }
+  } else if(cmd == "REPLAYS_UPDATED") {
+    updateLevelInfoViewerReplays();
+  } else {
+    GameState::executeOneCommand(cmd, args);
   }
 }
 
-void StateLevelInfoViewer::keyDown(int nKey, SDLMod mod,int nChar)
-{
-  StateMenu::keyDown(nKey, mod, nChar);
-
-  if(nKey == SDLK_ESCAPE){
+void StateLevelInfoViewer::xmKey(InputEventType i_type, const XMKey& i_xmkey) {
+  if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_ESCAPE, KMOD_NONE)) {
     m_requestForEnd = true;
+  } 
+
+  else {
+    StateMenu::xmKey(i_type, i_xmkey);
   }
 }
 
