@@ -19,7 +19,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 =============================================================================*/
 
 #include "StateManager.h"
-#include "helpers/Log.h"
+#include "GameState.h"
 #include "StatePause.h"
 #include "StateFinished.h"
 #include "StateDeadMenu.h"
@@ -31,17 +31,18 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "StateEditWebConfig.h"
 #include "StateLevelPackViewer.h"
 #include "StateRequestKey.h"
-#include "Game.h"
-#include "XMSession.h"
-#include "Sound.h"
-#include "drawlib/DrawLib.h"
-#include "SysMessage.h"
-#include "GameText.h"
-#include "xmscene/Camera.h"
-#include "VideoRecorder.h"
+#include "StateOptions.h"
+#include "StateVote.h"
+#include "../net/NetClient.h"
 
-#define MENU_SHADING_TIME 0.3
-#define MENU_SHADING_VALUE 150
+#include "../GameText.h"
+#include "../XMSession.h"
+#include "../drawlib/DrawLib.h"
+#include "../Game.h"
+#include "../helpers/Log.h"
+#include "../SysMessage.h"
+#include "../xmscene/Camera.h"
+#include "../VideoRecorder.h"
 
 StateManager::StateManager()
 {
@@ -81,9 +82,13 @@ StateManager::~StateManager()
     delete popState();
   }
 
+  deleteToDeleteState();
+
   if(m_videoRecorder != NULL) {
     delete m_videoRecorder;
   }
+
+  m_registeredStates.clear();
 }
 
 void StateManager::pushState(GameState* pNewState)
@@ -116,16 +121,18 @@ GameState* StateManager::popState()
 
 void StateManager::flush() {
   while(m_statesStack.size() > 0 && m_statesStack.back()->requestForEnd()) {
-    //Logger::Log("Flush %s", m_statesStack.back()->getName().c_str());
+    LogDebug("Flush %s", m_statesStack.back()->getName().c_str());
     delete popState();
   }
 
+  deleteToDeleteState();
+
   if(m_statesStack.size() == 0) {
     GameApp::instance()->requestEnd();
   }
 }
 
-GameState* StateManager::replaceState(GameState* pNewState)
+void StateManager::replaceState(GameState* pNewState)
 {
   GameState* pPreviousState = NULL;
 
@@ -141,7 +148,7 @@ GameState* StateManager::replaceState(GameState* pNewState)
   calculateWhichStateIsRendered();
   calculateFps();
 
-  return pPreviousState;
+  m_toDeleteStates.push_back(pPreviousState);
 }
 
 bool StateManager::needUpdateOrRender() {
@@ -178,7 +185,7 @@ void StateManager::update()
 
   while(stateIterator != tmp.end()){
     (*stateIterator)->executeCommands();
-    stateIterator++;
+    ++stateIterator;
   }
 
   // flush states
@@ -194,13 +201,16 @@ void StateManager::update()
     if((*RstateIterator)->updateStatesBehind() == false)
       break;
     
-    RstateIterator++;
+    ++RstateIterator;
   }
 
   if(oneUpdate == true){
     m_updateFpsNbFrame++;
   }
 
+  /* update net */
+  NetClient::instance()->executeNetActions();
+
   /* update fps */
   if(m_lastFpsTime + 1000 < GameApp::getXMTimeInt()) {
     m_currentRenderFps = m_renderFpsNbFrame;
@@ -211,6 +221,38 @@ void StateManager::update()
   }
 }
 
+void StateManager::renderOverAll() {
+
+  // net infos
+  if(NetClient::instance()->isConnected()) {
+    FontManager* v_fm = GameApp::instance()->getDrawLib()->getFontSmall();
+    FontGlyph* v_fg;
+    int vborder = 10;
+    int v_voffset = 0;
+    
+    v_fg = GameApp::instance()->getDrawLib()->getFontSmall()->getGlyph(GAMETEXT_CONNECTED_PLAYERS);
+    v_fm->printString(v_fg,
+		      GameApp::instance()->getDrawLib()->getDispWidth() - v_fg->realWidth() - vborder, vborder,
+		      MAKE_COLOR(240,240,240,255), -1.0, true);
+    v_voffset += v_fg->realHeight();
+    
+    // you
+    v_fg = GameApp::instance()->getDrawLib()->getFontSmall()->getGlyph(XMSession::instance()->profile());
+    v_fm->printString(v_fg,
+		      GameApp::instance()->getDrawLib()->getDispWidth() - v_fg->realWidth() - vborder, vborder+v_voffset,
+		      MAKE_COLOR(200,200,200,255), -1.0, true);     
+    v_voffset += v_fg->realHeight();
+    
+    for(unsigned int i=0; i<NetClient::instance()->otherClients().size(); i++) {
+      v_fg = GameApp::instance()->getDrawLib()->getFontSmall()->getGlyph(NetClient::instance()->otherClients()[i]->name());
+      v_fm->printString(v_fg,
+			GameApp::instance()->getDrawLib()->getDispWidth() - v_fg->realWidth() - vborder, vborder+v_voffset,
+			MAKE_COLOR(200,200,200,255), -1.0, true);     
+      v_voffset += v_fg->realHeight();
+    }
+  }
+}
+
 void StateManager::render()
 {
   std::vector<GameState*>::iterator stateIterator;
@@ -237,9 +279,11 @@ void StateManager::render()
 	(*stateIterator)->render();
       }
 
-      stateIterator++;
+      ++stateIterator;
     }
 
+    renderOverAll();
+
     // FPS
     if(XMSession::instance()->fps()) {
       drawFps();
@@ -258,8 +302,6 @@ void StateManager::render()
       if(m_statesStack.back()->showCursor()) {
 	drawCursor();
       }
-    } else {
-      drawCursor(); // to remove after state managing
     }
 
     drawLib->flushGraphics();
@@ -271,7 +313,7 @@ void StateManager::render()
 	(*stateIterator)->onRenderFlush();
       }
       
-      stateIterator++;
+      ++stateIterator;
     }
   }
 }
@@ -282,11 +324,11 @@ VideoRecorder* StateManager::getVideoRecorder() {
 
 void StateManager::drawFps() {
   char cTemp[256];        
-  sprintf(cTemp, "u(%i) d(%i)", getCurrentUpdateFPS(), getCurrentRenderFPS());
+  snprintf(cTemp, 256, "u(%i) d(%i)", getCurrentUpdateFPS(), getCurrentRenderFPS());
 
   FontManager* v_fm = GameApp::instance()->getDrawLib()->getFontSmall();
   FontGlyph* v_fg = v_fm->getGlyph(cTemp);
-  v_fm->printString(v_fg, 0, 130, MAKE_COLOR(255,255,255,255), true);
+  v_fm->printString(v_fg, 0, 130, MAKE_COLOR(255,255,255,255), -1.0, true);
 }
 
 void StateManager::drawStack() {
@@ -321,10 +363,10 @@ void StateManager::drawStack() {
     drawLib->drawBox(Vector2f(xoff,     yoff - (i * h)), Vector2f(xoff + w/2, yoff - ((i+1) * h)), 1.0, bg_update);
     drawLib->drawBox(Vector2f(xoff+w/2, yoff - (i * h)), Vector2f(xoff + w,   yoff - ((i+1) * h)), 1.0, bg_render);
     v_fg = v_fm->getGlyph(m_statesStack[i]->getName());
-    v_fm->printString(v_fg, (w-v_fg->realWidth())/2 + xoff, yoff - ((i+1) * h - v_fg->realHeight()/2), font_color, true);
+    v_fm->printString(v_fg, (w-v_fg->realWidth())/2 + xoff, yoff - ((i+1) * h - v_fg->realHeight()/2), font_color, 0.0, true);
 
     i++;
-    stateIterator++;
+    ++stateIterator;
   }
 }
 
@@ -346,39 +388,10 @@ void StateManager::drawCursor() {
   }
 }
 
-void StateManager::keyDown(int nKey, SDLMod mod,int nChar)
-{
+void StateManager::xmKey(InputEventType i_type, const XMKey& i_xmkey) {
   if(m_statesStack.size() == 0)
     return;
-  (m_statesStack.back())->keyDown(nKey, mod, nChar);
-}
-
-void StateManager::keyUp(int nKey,   SDLMod mod)
-{
-  if(m_statesStack.size() == 0)
-    return;
-  (m_statesStack.back())->keyUp(nKey, mod);
-}
-
-void StateManager::mouseDown(int nButton)
-{
-  if(m_statesStack.size() == 0)
-    return;
-  (m_statesStack.back())->mouseDown(nButton);
-}
-
-void StateManager::mouseDoubleClick(int nButton)
-{
-  if(m_statesStack.size() == 0)
-    return;
-  (m_statesStack.back())->mouseDoubleClick(nButton);
-}
-
-void StateManager::mouseUp(int nButton)
-{
-  if(m_statesStack.size() == 0)
-    return;
-  (m_statesStack.back())->mouseUp(nButton);
+  (m_statesStack.back())->xmKey(i_type, i_xmkey);
 }
 
 void StateManager::changeFocus(bool i_hasFocus) {
@@ -403,7 +416,7 @@ void StateManager::calculateWhichStateIsRendered()
       hideState = true;
     }
     
-    stateIterator++;
+    ++stateIterator;
   }
 }
 
@@ -434,24 +447,18 @@ void StateManager::calculateFps()
       maxRenderFps = renderFps;
     }
 
-    stateIterator++;
+    ++stateIterator;
   }
 
   m_maxUpdateFps = maxUpdateFps;
   m_maxRenderFps = maxRenderFps;
   m_maxFps = (m_maxUpdateFps > m_maxRenderFps) ? m_maxUpdateFps : m_maxRenderFps;
 
-//  Logger::Log("MaxUpdateFps: %d MaxRenderFps: %d Max: %d TopStateRenderFps: %d",
-//	      m_maxUpdateFps,
-//	      m_maxRenderFps,
-//	      m_maxFps,
-//	      topStateRenderFps);
-
   stateIterator = m_statesStack.begin();
   while(stateIterator != m_statesStack.end()){
     (*stateIterator)->setCurrentRenderFps(topStateRenderFps);
     (*stateIterator)->setMaxFps(m_maxFps);
-    stateIterator++;
+    ++stateIterator;
   }
 
   m_curRenderFps = topStateRenderFps;
@@ -468,6 +475,8 @@ void StateManager::cleanStates() {
   StateMainMenu::clean();
   StatePause::clean();
   StateRequestKey::clean();
+  StateOptions::clean();
+  StateVote::clean();
 }
 
 void StateManager::refreshStaticCaptions() {
@@ -490,6 +499,10 @@ bool StateManager::isTopOfTheStates(GameState* i_state) {
   return m_statesStack[m_statesStack.size()-1] == i_state;
 }
 
+int StateManager::numberOfStates() {
+  return m_statesStack.size();
+}
+
 int StateManager::getCurrentUpdateFPS() {
   return m_currentUpdateFps;
 }
@@ -513,215 +526,157 @@ bool StateManager::doRender()
   return false;
 }
 
-void StateManager::sendSynchronousMessage(std::string cmd)
+void
+StateManager::registerAsObserver(std::string message,
+				 GameState* self)
 {
-  std::vector<GameState*>::iterator stateIterator = m_statesStack.begin();
+  std::map<std::string, std::vector<GameState*> >::iterator itFind;
 
-  while(stateIterator != m_statesStack.end()){
-    (*stateIterator)->executeOneCommand(cmd);
+  itFind = m_registeredStates.find(message);
+  if(itFind != m_registeredStates.end()){
+    if(self != NULL) {
 
-    stateIterator++;
+      // check if the state is already registered
+      if(XMSession::instance()->debug() == true) {
+	std::vector<GameState*>& states = (*itFind).second;
+	std::vector<GameState*>::iterator it = states.begin();
+	std::string name = self->getName();
+
+	while(it != states.end()){
+	  if((*it)->getName() == name){
+	    LogWarning("state [%s] already registered as observer for this message [%s].",
+			name.c_str(), message.c_str());
+	    break;
+	  }
+	  ++it;
+	}
+      }
+
+      (*itFind).second.push_back(self);
+    }
+  } else {
+    m_registeredStates[message] = std::vector<GameState*>();
+    if(self != NULL) {
+      itFind = m_registeredStates.find(message);
+      (*itFind).second.push_back(self);
+    }
   }
 }
 
-void StateManager::sendAsynchronousMessage(std::string cmd)
+void
+StateManager::unregisterAsObserver(std::string message, GameState* self)
 {
-  std::vector<GameState*>::iterator stateIterator = m_statesStack.begin();
+  std::map<std::string, std::vector<GameState*> >::iterator itFind;
 
-  while(stateIterator != m_statesStack.end()){
-    (*stateIterator)->send("STATE_MANAGER", cmd);
+  itFind = m_registeredStates.find(message);
+  if(itFind != m_registeredStates.end()){
+    std::vector<GameState*>& states = (*itFind).second;
+    std::vector<GameState*>::iterator stateIterator = states.begin();
 
-    stateIterator++;
+    while(stateIterator != states.end()){
+      if(self == (*stateIterator)) {
+	states.erase(stateIterator);
+	return;
+      }
+
+      ++stateIterator;
+    }
+  } else {
+    LogWarning("unregisterAsObserver message [%s] state [%], but not registered.",
+		message.c_str(),
+		self->getName().c_str());
   }
 }
 
-GameState::GameState(bool drawStateBehind,
-		     bool updateStatesBehind,
-		     bool i_doShade,
-		     bool i_doShadeAnim)
+void
+StateManager::registerAsEmitter(std::string message)
 {
-  m_isHide             = false;
-  m_drawStateBehind    = drawStateBehind;
-  m_updateStatesBehind = updateStatesBehind;
-  m_requestForEnd      = false;
-
-  // default rendering and update fps
-  m_updateFps          = 50;
-  m_renderFps          = 50;
-  m_curRenderFps       = m_renderFps;
-
-  m_maxFps             = 0;
-
-  m_updatePeriod       = 0;
-  m_updateCounter      = 0;
-
-  // shade
-  m_doShade     = i_doShade;
-  m_doShadeAnim = i_doShadeAnim;
-
-  m_showCursor = true;
+  // TODO::show debug informations
+  return registerAsObserver(message, NULL);
 }
 
-GameState::~GameState() {
-}
-
-bool GameState::doUpdate()
+void
+StateManager::sendSynchronousMessage(std::string message, std::string args)
 {
-  m_updateCounter += 1.0;
+  std::map<std::string, std::vector<GameState*> >::iterator itFind;
 
-  if(m_updateCounter >= m_updatePeriod){
-    m_updateCounter -= m_updatePeriod;
-    return true;
+  itFind = m_registeredStates.find(message);
+  if(itFind != m_registeredStates.end()){
+    std::vector<GameState*>& states = (*itFind).second;
+    std::vector<GameState*>::iterator stateIterator = states.begin();
+
+    if(states.size() == 0){
+      if(XMSession::instance()->debug()) {
+	LogWarning("sendSynchronousMessage message [%s [%s]] sent and there's no state to receive it.",
+		   message.c_str(), args.c_str());
+      }
+    }
+
+    while(stateIterator != states.end()){
+      (*stateIterator)->executeOneCommand(message, args);
+
+      LogDebug("sendSynchronousMessage [%s [%s]] to [%s]",
+	       message.c_str(), args.c_str(), (*stateIterator)->getName().c_str());
+
+      ++stateIterator;
+    }
+  } else {
+    if(XMSession::instance()->debug()) {
+      LogWarning("sendSynchronousMessage message [%s [%s]] sent and there's no state to receive it.",
+		 message.c_str(), args.c_str());
+    }
   }
+}
+
+void
+StateManager::sendAsynchronousMessage(std::string message, std::string args)
+{
+  std::map<std::string, std::vector<GameState*> >::iterator itFind;
+
+  itFind = m_registeredStates.find(message);
+  if(itFind != m_registeredStates.end()){
+    std::vector<GameState*>& states = (*itFind).second;
+    std::vector<GameState*>::iterator stateIterator = states.begin();
+
+    if(states.size() == 0){
+      if(XMSession::instance()->debug()) {
+	LogWarning("sendAsynchronousMessage message [%s [%s]] sent and there's no state to receive it.",
+		   message.c_str(), args.c_str());
+      }
+    }
+
+    while(stateIterator != states.end()){
+      (*stateIterator)->send(message, args);
+
+      LogDebug("sendAsynchronousMessage [%s [%s]] to [%s]",
+	       message.c_str(), args.c_str(),
+	       (*stateIterator)->getName().c_str());
+
+      ++stateIterator;
+    }
+  } else {
+    if(XMSession::instance()->debug()) {
+      LogWarning("sendAsynchronousMessage message [% [%s]s] sent and there's no state to receive it.",
+		 message.c_str(),
+		 args.c_str());
+    }
+  }
+}
+
+void StateManager::deleteToDeleteState()
+{
+  while(m_toDeleteStates.size() != 0){
+    delete m_toDeleteStates.back();
+    m_toDeleteStates.pop_back();
+  }
+}
+
+bool StateManager::isThereASuchState(const std::string& i_name) {
+  for(unsigned int i=0; i<m_statesStack.size(); i++) {
+    if(m_statesStack[i]->getName() == i_name) {
+      return true;
+    }
+  }
+
   return false;
 }
-
-void GameState::enter() {
-  m_nShadeTime = GameApp::getXMTime();
-}
-
-bool GameState::render() {
-  // shade
-  if(XMSession::instance()->ugly() == false && m_doShade) {
-    float v_currentTime = GameApp::getXMTime();
-    int   v_nShade;
-    
-    if(v_currentTime - m_nShadeTime < MENU_SHADING_TIME && m_doShadeAnim) {
-      v_nShade = (int ) ((v_currentTime - m_nShadeTime) * (MENU_SHADING_VALUE / MENU_SHADING_TIME));
-    } else {
-      v_nShade = MENU_SHADING_VALUE;
-    }
-
-    DrawLib* drawLib = GameApp::instance()->getDrawLib();
-    drawLib->drawBox(Vector2f(0,0),
-		     Vector2f(drawLib->getDispWidth(),
-			      drawLib->getDispHeight()),
-		     0, MAKE_COLOR(0,0,0, v_nShade));
-  }
-
-  return renderOverShadow();
-}
-
-bool GameState::renderOverShadow() {
-  // do nothing by default
-  return true;
-}
-
-std::string GameState::getId() const {
-  return m_id;
-}
-
-void GameState::setId(const std::string& i_id) {
-  m_id = i_id;
-}
-
-void GameState::send(const std::string& i_id, UIMsgBoxButton i_button, const std::string& i_input) {
-  /* by default, do nothing */
-  Logger::Log("** Warning ** : StateMessageBoxReceiver::send() received, but nothing done !");
-}
-
-void GameState::send(const std::string& i_id, const std::string& i_message)
-{
-}
-
-void GameState::executeCommands()
-{
-  while(m_commands.empty() == false){
-    executeOneCommand(m_commands.front());
-    m_commands.pop();
-  }
-
-}
-
-void GameState::executeOneCommand(std::string cmd)
-{
-  // default one do nothing.
-}
-
-void GameState::keyDown(int nKey, SDLMod mod,int nChar) {
-  GameApp* gameApp = GameApp::instance();
-
-  if(nKey == SDLK_F12) {
-    gameApp->gameScreenshot();
-    return;        
-  }
-
-  if(nKey == SDLK_F8) {
-    gameApp->enableWWW(XMSession::instance()->www() == false);
-    StateManager::instance()->sendAsynchronousMessage("CHANGE_WWW_ACCESS");
-    return;        
-  }
-
-  if(nKey == SDLK_F7) {
-    gameApp->enableFps(XMSession::instance()->fps() == false);
-    return;        
-  }
-
-  if(nKey == SDLK_F9) {
-    gameApp->switchUglyMode(XMSession::instance()->ugly() == false);
-    if(XMSession::instance()->ugly()) {
-      SysMessage::instance()->displayText(SYS_MSG_UGLY_MODE_ENABLED);
-    } else {
-      SysMessage::instance()->displayText(SYS_MSG_UGLY_MODE_DISABLED);
-    }
-    return;        
-  }
-
-  if(nKey == SDLK_RETURN && (mod & KMOD_ALT) != 0) {
-    gameApp->getDrawLib()->toogleFullscreen();
-    XMSession::instance()->setWindowed(XMSession::instance()->windowed() == false);
-    return;
-  }
-
-  if(nKey == SDLK_F10) {
-    gameApp->switchTestThemeMode(XMSession::instance()->testTheme() == false);
-    if(XMSession::instance()->testTheme()) {
-      SysMessage::instance()->displayText(SYS_MSG_THEME_MODE_ENABLED);
-    } else {
-      SysMessage::instance()->displayText(SYS_MSG_THEME_MODE_DISABLED);
-    }
-    return;        
-  }
-
-  if(nKey == SDLK_F11) {
-    gameApp->switchUglyOverMode(XMSession::instance()->uglyOver() == false);
-    if(XMSession::instance()->uglyOver()) {
-      SysMessage::instance()->displayText(SYS_MSG_UGLY_OVER_MODE_ENABLED);
-    } else {
-      SysMessage::instance()->displayText(SYS_MSG_UGLY_OVER_MODE_DISABLED);
-    }
-    return;        
-  }
-
-  /* activate/desactivate interpolation */
-  if(nKey == SDLK_i && (mod & KMOD_CTRL) != 0) {
-    XMSession::instance()->setEnableReplayInterpolation(!XMSession::instance()->enableReplayInterpolation());
-    if(XMSession::instance()->enableReplayInterpolation()) {
-      SysMessage::instance()->displayText(SYS_MSG_INTERPOLATION_ENABLED);
-    } else {
-      SysMessage::instance()->displayText(SYS_MSG_INTERPOLATION_DISABLED);
-    }
-    StateManager::instance()->sendAsynchronousMessage("INTERPOLATION_CHANGED");
-
-    return;
-  }
-
-  if(nKey == SDLK_m && (mod & KMOD_CTRL) != 0) {
-    XMSession::instance()->setMirrorMode(XMSession::instance()->mirrorMode() == false);
-    InputHandler::instance()->setMirrored(XMSession::instance()->mirrorMode());
-    StateManager::instance()->sendAsynchronousMessage("MIRRORMODE_CHANGED");
-  }
-
-  if(nKey == SDLK_s && (mod & KMOD_CTRL) != 0) {
-    GameApp::instance()->playMusic(""); // tell the game the last music played is "" otherwise, it doesn't know when music is disable then enable back
-    XMSession::instance()->setEnableAudio(! XMSession::instance()->enableAudio());
-    Sound::setActiv(XMSession::instance()->enableAudio());
-    if(XMSession::instance()->enableAudio()) {
-      SysMessage::instance()->displayText(SYS_MSG_AUDIO_ENABLED);
-    } else {
-      SysMessage::instance()->displayText(SYS_MSG_AUDIO_DISABLED);
-    }
-    StateManager::instance()->sendAsynchronousMessage("ENABLEAUDIO_CHANGED");
-  }
-
-}
