@@ -20,21 +20,26 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #include "StatePlaying.h"
 #include "StatePause.h"
-#include "Game.h"
-#include "XMSession.h"
-#include "helpers/Log.h"
-#include "GameText.h"
+#include "../Game.h"
+#include "../XMSession.h"
+#include "../helpers/Log.h"
+#include "../GameText.h"
 #include "StateMessageBox.h"
 #include "StateFinished.h"
 #include "StateDeadJust.h"
 #include "StateDeadMenu.h"
-#include "Sound.h"
-#include "xmscene/Camera.h"
-#include "xmscene/Bike.h"
-#include "CameraAnimation.h"
-#include "Universe.h"
-#include "Trainer.h"
-#include "SysMessage.h"
+#include "../Sound.h"
+#include "../xmscene/Camera.h"
+#include "../xmscene/Bike.h"
+#include "../CameraAnimation.h"
+#include "../Universe.h"
+#include "../Trainer.h"
+#include "../SysMessage.h"
+#include "../Renderer.h"
+#include "../xmscene/BikeController.h"
+#include "../LuaLibGame.h"
+
+#define MINIMUM_VELOCITY_TO_GET_MAXIMUM_DEATH_SOUND 70.0
 
 StatePlaying::StatePlaying(Universe* i_universe):
   StateScene(i_universe)
@@ -43,12 +48,24 @@ StatePlaying::StatePlaying(Universe* i_universe):
   m_gameIsFinished = false;
   m_displayStats   = false;
 
+	for(unsigned int i=0; i<INPUT_NB_PLAYERS; i++){
+		m_changeDirKeyAlreadyPress[i] = false;
+	}
+
   /* prepare stats */
   makeStatsStr();
+
+  StateManager::instance()->registerAsObserver("OPTIONS_UPDATED", this);
+
+  if(XMSession::instance()->debug() == true) {
+    StateManager::instance()->registerAsEmitter("STATS_UPDATED");
+    StateManager::instance()->registerAsEmitter("LEVELS_UPDATED");
+  }
 }
 
 StatePlaying::~StatePlaying()
 {
+  StateManager::instance()->unregisterAsObserver("OPTIONS_UPDATED", this);
 }
 
 void StatePlaying::enter()
@@ -57,15 +74,7 @@ void StatePlaying::enter()
 
   m_gameIsFinished = false;
 
-  if(XMSession::instance()->hidePlayingInformation() == false) {
-    GameRenderer::instance()->setShowEngineCounter(XMSession::instance()->showEngineCounter());
-    GameRenderer::instance()->setShowMinimap(XMSession::instance()->showMinimap());
-    GameRenderer::instance()->setShowTimePanel(true);
-  } else {
-    GameRenderer::instance()->setShowEngineCounter(false);
-    GameRenderer::instance()->setShowMinimap(false);
-    GameRenderer::instance()->setShowTimePanel(false);
-  }
+  updateWithOptions();
 
   std::string v_level_name;
   try {
@@ -77,10 +86,10 @@ void StatePlaying::enter()
     }
   }
   catch(Exception &e) {
-    Logger::Log("** Warning ** : level '%s' cannot be loaded", v_level_name.c_str());
+    LogWarning("level '%s' cannot be loaded", v_level_name.c_str());
 
     char cBuf[256];
-    sprintf(cBuf,GAMETEXT_LEVELCANNOTBELOADED, v_level_name.c_str());
+    snprintf(cBuf, 256, GAMETEXT_LEVELCANNOTBELOADED, v_level_name.c_str());
 
     StateMessageBox* v_msgboxState = new StateMessageBox(this, cBuf, UI_MSGBOX_OK);
     v_msgboxState->setId("ERROR");
@@ -89,15 +98,10 @@ void StatePlaying::enter()
 
   setScoresTimes();
 
-  if(m_universe != NULL) {
-    if(m_universe->getScenes().size() > 0) {
-      // play music of the first world
-      GameApp::instance()->playMusic(m_universe->getScenes()[0]->getLevelSrc()->Music());
-    }
-  }
+  playLevelMusic();
 
   // read keys for more reactivity
-  InputHandler::instance()->dealWithActivedKeys(m_universe);
+  dealWithActivedKeys(m_universe);
 
   // reset trainer mode use
   Trainer::instance()->resetTrainerUse();
@@ -119,7 +123,8 @@ void StatePlaying::leave()
 	  if(m_universe->getScenes()[0]->Players()[0]->isDead()     == false &&
 	     m_universe->getScenes()[0]->Players()[0]->isFinished() == false) {
 	    
-	    xmDatabase::instance("main")->stats_abortedLevel(XMSession::instance()->profile(),
+	    xmDatabase::instance("main")->stats_abortedLevel(XMSession::instance()->sitekey(),
+							     XMSession::instance()->profile(),
 							     m_universe->getScenes()[0]->getLevelSrc()->Id(),
 							     m_universe->getScenes()[0]->getTime());
 	  }
@@ -131,12 +136,43 @@ void StatePlaying::leave()
 
 void StatePlaying::enterAfterPop()
 {
+  StateScene::enterAfterPop();
+
   // recheck keys
-  InputHandler::instance()->dealWithActivedKeys(m_universe);
+  dealWithActivedKeys(m_universe);
   m_fLastPhysTime = GameApp::getXMTime();
   m_displayStats = false;
 }
 
+void StatePlaying::updateWithOptions() {
+  if(XMSession::instance()->hidePlayingInformation() == false) {
+    GameRenderer::instance()->setShowEngineCounter(XMSession::instance()->showEngineCounter());
+    GameRenderer::instance()->setShowMinimap(XMSession::instance()->showMinimap());
+    GameRenderer::instance()->setShowTimePanel(true);
+  } else {
+    GameRenderer::instance()->setShowEngineCounter(false);
+    GameRenderer::instance()->setShowMinimap(false);
+    GameRenderer::instance()->setShowTimePanel(false);
+  }
+
+  GameRenderer::instance()->setShowGhostsText(true);
+
+  for(unsigned int i=0; i<m_universe->getScenes().size(); i++) {
+    m_universe->getScenes()[i]->setDeathAnim(XMSession::instance()->enableDeadAnimation());
+    m_universe->getScenes()[i]->setShowGhostTimeDiff(XMSession::instance()->showGhostTimeDifference());
+  }
+}
+
+void StatePlaying::executeOneCommand(std::string cmd, std::string args) {
+  if(cmd == "OPTIONS_UPDATED") {
+    updateWithOptions();
+  }
+
+  else {
+    StateScene::executeOneCommand(cmd, args);
+  }
+}
+
 bool StatePlaying::update()
 {
   if(StateScene::update() == false)
@@ -183,9 +219,8 @@ bool StatePlaying::update()
   return true;
 }
 
-void StatePlaying::keyDown(int nKey, SDLMod mod,int nChar)
-{
-  if(nKey == SDLK_ESCAPE){
+void StatePlaying::xmKey(InputEventType i_type, const XMKey& i_xmkey) {
+  if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_ESCAPE, KMOD_NONE)) {
     if(isLockedScene() == false) {
       /* Escape pauses */
       m_displayStats = true;
@@ -193,8 +228,8 @@ void StatePlaying::keyDown(int nKey, SDLMod mod,int nChar)
     }
   }
 
-#if defined(ENABLE_ZOOMING)
-  else if(nKey == SDLK_KP0 && (mod & KMOD_LCTRL) == KMOD_LCTRL){
+#if defined(ENABLE_DEV)
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_KP0, KMOD_LCTRL)) {
     if(m_universe != NULL) {
       for(unsigned int j=0; j<m_universe->getScenes().size(); j++) {
 	for(unsigned int i=0; i<m_universe->getScenes()[j]->Players().size(); i++) {
@@ -206,7 +241,8 @@ void StatePlaying::keyDown(int nKey, SDLMod mod,int nChar)
       }
     }
   }
-  else if(nKey == SDLK_KP0 && (mod & (KMOD_CTRL|KMOD_SHIFT|KMOD_ALT|KMOD_META)) == 0){        //TRAINER
+
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_KP0, KMOD_NONE)) {
     if(m_universe != NULL) {
       for(unsigned int j=0; j<m_universe->getScenes().size(); j++) {
         Trainer::instance()->storePosition( m_universe->getScenes()[j]->getLevelSrc()->Id(),
@@ -218,7 +254,9 @@ void StatePlaying::keyDown(int nKey, SDLMod mod,int nChar)
       }
     }
   }
-  else if(nKey == SDLK_BACKSPACE){       //TRAINER
+
+  // TRAINER
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_BACKSPACE, KMOD_NONE)) {
     if(m_universe != NULL) {
       for(unsigned int j=0; j<m_universe->getScenes().size(); j++) {
         if( Trainer::instance()->isRestorePositionAvailable( m_universe->getScenes()[j]->getLevelSrc()->Id() ) ) {
@@ -234,7 +272,8 @@ void StatePlaying::keyDown(int nKey, SDLMod mod,int nChar)
       }
     }
   }
-  else if(nKey == SDLK_KP_MINUS){        //TRAINER
+
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_MINUS, KMOD_NONE)) {
     if(m_universe != NULL) {
       for(unsigned int j=0; j<m_universe->getScenes().size(); j++) {
         if( Trainer::instance()->isRestorePositionAvailable( m_universe->getScenes()[j]->getLevelSrc()->Id() ) ) {
@@ -250,7 +289,8 @@ void StatePlaying::keyDown(int nKey, SDLMod mod,int nChar)
       }
     }
   }
-  else if(nKey == SDLK_KP_PLUS){        //TRAINER
+
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_PLUS, KMOD_NONE)) {
     if(m_universe != NULL) {
       for(unsigned int j=0; j<m_universe->getScenes().size(); j++) {
         if( Trainer::instance()->isRestorePositionAvailable( m_universe->getScenes()[j]->getLevelSrc()->Id() ) ) {
@@ -269,38 +309,15 @@ void StatePlaying::keyDown(int nKey, SDLMod mod,int nChar)
 #endif
 
   else {
-    // to avoid people changing direction during the autozoom
-    if(m_autoZoom == false){
-      /* Notify the controller */
-      InputHandler::instance()->handleInput(m_universe, INPUT_KEY_DOWN, nKey, mod);
+    if(i_type == INPUT_DOWN) {
+      if(m_autoZoom == false){
+	// to avoid people changing direction during the autozoom
+	handleInput(m_universe, INPUT_DOWN, i_xmkey);
+      }
+    } else {
+      handleInput(m_universe, INPUT_UP, i_xmkey);
     }
-  }
-
-  StateScene::keyDown(nKey, mod, nChar);
-}
-
-void StatePlaying::keyUp(int nKey, SDLMod mod) {
-  InputHandler::instance()->handleInput(m_universe, INPUT_KEY_UP, nKey, mod);
-  StateScene::keyUp(nKey, mod);
-}
-
-void StatePlaying::mouseDown(int nButton)
-{
-  InputHandler::instance()->handleInput(m_universe, INPUT_KEY_DOWN, nButton, KMOD_NONE);
-
-  StateScene::mouseDown(nButton);
-}
-
-void StatePlaying::mouseUp(int nButton)
-{
-  InputHandler::instance()->handleInput(m_universe, INPUT_KEY_UP, nButton, KMOD_NONE);
-
-  StateScene::mouseUp(nButton);
-}
-
-void StatePlaying::send(const std::string& i_id, UIMsgBoxButton i_button, const std::string& i_input) {
-  if(i_id == "ERROR") {
-    m_commands.push("ERROR");
+    StateScene::xmKey(i_type, i_xmkey);
   }
 }
 
@@ -314,7 +331,7 @@ void StatePlaying::onOneFinish() {
     }
   }
   
-  /* update profiles */
+  /* update profile and stats */
   if(m_universe != NULL) {
     if(m_universe->getScenes().size() == 1) {
       if(m_universe->getScenes()[0]->Players().size() == 1) {
@@ -324,30 +341,23 @@ void StatePlaying::onOneFinish() {
 	  v_finish_time  = m_universe->getScenes()[0]->Players()[0]->finishTime();
 	}
         // Updating the stats if the Trainer has not been used
-        if(Trainer::instance()->trainerHasBeenUsed() == false){ // then, even time played is not saved
-            xmDatabase::instance("main")->profiles_addFinishTime(XMSession::instance()->profile(),
-                                         m_universe->getScenes()[0]->getLevelSrc()->Id(),
-                                         TimeStamp,
-                                         v_finish_time);
+        if(Trainer::instance()->trainerHasBeenUsed() == false){
+            xmDatabase::instance("main")->profiles_addFinishTime(XMSession::instance()->sitekey(),
+								 XMSession::instance()->profile(),
+								 m_universe->getScenes()[0]->getLevelSrc()->Id(),
+								 TimeStamp,
+								 v_finish_time);
         }
-        StateManager::instance()->sendAsynchronousMessage("STATS_UPDATED");
-      }
-    }
-  }	  
-  
-  /* Update stats */
-  /* update stats only in one player mode */
-  if(m_universe != NULL) {
-    if(m_universe->getScenes().size() == 1) {
-      if(m_universe->getScenes()[0]->Players().size() == 1) {
-	xmDatabase::instance("main")->stats_levelCompleted(XMSession::instance()->profile(),
+	xmDatabase::instance("main")->stats_levelCompleted(XMSession::instance()->sitekey(),
+							   XMSession::instance()->profile(),
 							   m_universe->getScenes()[0]->getLevelSrc()->Id(),
 							   m_universe->getScenes()[0]->Players()[0]->finishTime());
 	StateManager::instance()->sendAsynchronousMessage("LEVELS_UPDATED");
-	StateManager::instance()->sendAsynchronousMessage("STATS_UPDATED");
+        StateManager::instance()->sendAsynchronousMessage("STATS_UPDATED");
       }
     }
   }
+  
   StateManager::instance()->pushState(new StateFinished(m_universe, this));
 }
 
@@ -362,7 +372,8 @@ void StatePlaying::onAllDead() {
   if(m_universe != NULL) {
     if(m_universe->getScenes().size() == 1) {
       if(m_universe->getScenes()[0]->Players().size() == 1) {
-	xmDatabase::instance("main")->stats_died(XMSession::instance()->profile(),
+	xmDatabase::instance("main")->stats_died(XMSession::instance()->sitekey(),
+						 XMSession::instance()->profile(),
 						 m_universe->getScenes()[0]->getLevelSrc()->Id(),
 						 m_universe->getScenes()[0]->getTime());
 	StateManager::instance()->sendAsynchronousMessage("STATS_UPDATED");
@@ -372,7 +383,24 @@ void StatePlaying::onAllDead() {
   
   /* Play the DIE!!! sound */
   try {
-    Sound::playSampleByName(Theme::instance()->getSound("Headcrash")->FilePath(), 0.3);
+    float v_deathVolume;
+    float v_maxVelocity = 0.0;
+
+    if(m_universe != NULL) {
+      for(unsigned int i=0; i<m_universe->getScenes().size(); i++) {
+	for(unsigned int j=0; j<m_universe->getScenes()[i]->Players().size(); j++) {
+	  if(m_universe->getScenes()[i]->Players()[j]->getBikeLinearVel() > v_maxVelocity) {
+	    v_maxVelocity = m_universe->getScenes()[i]->Players()[j]->getBikeLinearVel();
+	  }
+	}
+      }
+    }
+    // make deathVolume dependant of the velocity of the fastest of the players
+    v_deathVolume = v_maxVelocity / MINIMUM_VELOCITY_TO_GET_MAXIMUM_DEATH_SOUND;
+    if(v_deathVolume > 1.0) {
+      v_deathVolume = 1.0;
+    }
+    Sound::playSampleByName(Theme::instance()->getSound("Headcrash")->FilePath(), v_deathVolume);
   } catch(Exception &e) {
   }
   
@@ -389,7 +417,8 @@ void StatePlaying::abortPlaying() {
       if(m_universe->getScenes()[0]->Players().size() == 1) {
 	if(m_universe->getScenes()[0]->Players()[0]->isDead()     == false &&
 	   m_universe->getScenes()[0]->Players()[0]->isFinished() == false) {
-	  xmDatabase::instance("main")->stats_abortedLevel(XMSession::instance()->profile(),
+	  xmDatabase::instance("main")->stats_abortedLevel(XMSession::instance()->sitekey(),
+							   XMSession::instance()->profile(),
 							   m_universe->getScenes()[0]->getLevelSrc()->Id(),
 							   m_universe->getScenes()[0]->getTime());
 	  StateManager::instance()->sendAsynchronousMessage("STATS_UPDATED");
@@ -402,41 +431,7 @@ void StatePlaying::abortPlaying() {
 }
 
 void StatePlaying::nextLevel(bool i_positifOrder) {
-  GameApp*  pGame  = GameApp::instance();
-  std::string v_nextLevel;
-  std::string v_currentLevel;
-  
-  // take the level id of the first world
-  if(m_universe != NULL) {
-    if(m_universe->getScenes().size() > 0) {
-      v_currentLevel = m_universe->getScenes()[0]->getLevelSrc()->Id();
-    }
-  }
-
-  if(i_positifOrder) {
-    v_nextLevel = pGame->determineNextLevel(v_currentLevel);
-  } else {
-    v_nextLevel = pGame->determinePreviousLevel(v_currentLevel);
-  }
-
-  /* update stats */
-  if(v_nextLevel != "") {
-    if(m_universe != NULL) {
-      if(m_universe->getScenes().size() == 1) {
-	if(m_universe->getScenes()[0]->Players().size() == 1) {
-	  if(m_universe->getScenes()[0]->Players()[0]->isDead()     == false &&
-	     m_universe->getScenes()[0]->Players()[0]->isFinished() == false) {
-	    xmDatabase::instance("main")->stats_abortedLevel(XMSession::instance()->profile(),
-							     v_currentLevel,
-							     m_universe->getScenes()[0]->getTime());
-	    StateManager::instance()->sendAsynchronousMessage("STATS_UPDATED");
-	  }
-	}
-      }
-    }
-  }
-
-  nextLevelToPlay(i_positifOrder);
+  playingNextLevel(i_positifOrder);
 }
 
 void StatePlaying::restartLevel(bool i_reloadLevel) {
@@ -446,7 +441,8 @@ void StatePlaying::restartLevel(bool i_reloadLevel) {
       if(m_universe->getScenes()[0]->Players().size() == 1) {
 	if(m_universe->getScenes()[0]->Players()[0]->isDead()     == false &&
 	   m_universe->getScenes()[0]->Players()[0]->isFinished() == false) {
-	  xmDatabase::instance("main")->stats_levelRestarted(XMSession::instance()->profile(),
+	  xmDatabase::instance("main")->stats_levelRestarted(XMSession::instance()->sitekey(),
+							     XMSession::instance()->profile(),
 							     m_universe->getScenes()[0]->getLevelSrc()->Id(),
 							     m_universe->getScenes()[0]->getTime());
 	  StateManager::instance()->sendAsynchronousMessage("STATS_UPDATED");
@@ -465,3 +461,176 @@ bool StatePlaying::renderOverShadow() {
 
   return true;
 }
+
+void StatePlaying::handleInput(Universe* i_universe, InputEventType Type, const XMKey& i_xmkey) {
+	unsigned int p, pW;
+	Biker *v_biker;
+	
+	switch(Type) {
+	case INPUT_DOWN:
+		p = 0; // player number p
+		pW = 0; // number of players in the previous worlds
+		for(unsigned int j=0; j<i_universe->getScenes().size(); j++) {
+			for(unsigned int i=0; i<i_universe->getScenes()[j]->Players().size(); i++) {
+				v_biker = i_universe->getScenes()[j]->Players()[i];
+				
+				// if else is not valid while axis up can be a signal for two sides
+				if(InputHandler::instance()->getDRIVE(p) == i_xmkey) {
+					/* Start driving */
+					if(i_xmkey.isAnalogic()) {
+						v_biker->getControler()->setThrottle(fabs(i_xmkey.getAnalogicValue()));
+					} else {
+						v_biker->getControler()->setThrottle(1.0f);
+					}
+				}
+				
+				if(InputHandler::instance()->getBRAKE(p) == i_xmkey) {
+					/* Brake */
+					v_biker->getControler()->setBreak(1.0f);
+				}
+				
+				if((InputHandler::instance()->getFLIPLEFT(p)    == i_xmkey && XMSession::instance()->mirrorMode() == false) ||
+					 (InputHandler::instance()->getFLIPRIGHT(p) == i_xmkey && XMSession::instance()->mirrorMode())) {
+					/* Pull back */
+					if(i_xmkey.isAnalogic()) {
+						v_biker->getControler()->setPull(fabs(i_xmkey.getAnalogicValue()));
+					} else {
+						v_biker->getControler()->setPull(1.0f);
+					}
+				}
+				
+				if((InputHandler::instance()->getFLIPRIGHT(p) == i_xmkey && XMSession::instance()->mirrorMode() == false) ||
+					 (InputHandler::instance()->getFLIPLEFT(p)    == i_xmkey && XMSession::instance()->mirrorMode())) {
+					/* Push forward */
+					if(i_xmkey.isAnalogic()) {
+						v_biker->getControler()->setPull(-fabs(i_xmkey.getAnalogicValue()));
+					} else {
+						v_biker->getControler()->setPull(-1.0f);
+					}
+				}
+				
+				if(InputHandler::instance()->getCHANGEDIR(p) == i_xmkey) {
+					/* Change dir */
+					if(m_changeDirKeyAlreadyPress[p] == false){
+						v_biker->getControler()->setChangeDir(true);
+						m_changeDirKeyAlreadyPress[p] = true;
+					}
+				}
+				p++;
+			}
+			pW+= i_universe->getScenes()[j]->Players().size();
+		}
+		
+		break;
+	case INPUT_UP:
+		p = 0; // player number p
+		pW = 0; // number of players in the previous worlds
+		for(unsigned int j=0; j<i_universe->getScenes().size(); j++) {
+			for(unsigned int i=0; i<i_universe->getScenes()[j]->Players().size(); i++) {
+				v_biker = i_universe->getScenes()[j]->Players()[i];
+				
+				// if else is not valid while axis up can be a signal for two sides
+				if(InputHandler::instance()->getDRIVE(p) == i_xmkey) {
+					/* Stop driving */
+					v_biker->getControler()->setThrottle(0.0f);
+				}
+				
+				if(InputHandler::instance()->getBRAKE(p) == i_xmkey) {
+					/* Don't brake */
+					v_biker->getControler()->setBreak(0.0f);
+				}
+				
+				if((InputHandler::instance()->getFLIPLEFT(p)    == i_xmkey && XMSession::instance()->mirrorMode() == false) ||
+					 (InputHandler::instance()->getFLIPRIGHT(p) == i_xmkey && XMSession::instance()->mirrorMode())) {
+					/* Pull back */
+					v_biker->getControler()->setPull(0.0f);
+				}
+				
+				if((InputHandler::instance()->getFLIPRIGHT(p) == i_xmkey && XMSession::instance()->mirrorMode() == false) ||
+					 (InputHandler::instance()->getFLIPLEFT(p)    == i_xmkey && XMSession::instance()->mirrorMode())) {
+					/* Push forward */
+					v_biker->getControler()->setPull(0.0f);
+				}
+
+				if(InputHandler::instance()->getCHANGEDIR(p) == i_xmkey) {
+					m_changeDirKeyAlreadyPress[p] = false;
+				}
+				p++;
+			}
+			pW+= i_universe->getScenes()[j]->Players().size();
+		}
+		break;
+	}
+	
+	/* Have the script hooked this key? */
+	if(Type == INPUT_DOWN) {
+		for(int i=0; i<InputHandler::instance()->getNumScriptKeyHooks(); i++) {
+			if(InputHandler::instance()->getScriptKeyHooks(i).nKey == i_xmkey) {
+				/* Invoke script */
+				InputHandler::instance()->getScriptKeyHooks(i).pGame->getLuaLibGame()->scriptCallVoid(InputHandler::instance()->getScriptKeyHooks(i).FuncName);
+			}
+			for(int j=0; j<INPUT_NB_PLAYERS; j++) {
+				if(InputHandler::instance()->getScriptActionKeys(j, i) == i_xmkey) {
+					InputHandler::instance()->getScriptKeyHooks(i).pGame->getLuaLibGame()->scriptCallVoid(InputHandler::instance()->getScriptKeyHooks(i).FuncName);
+				}
+			}	
+		}
+	}
+}
+
+void StatePlaying::dealWithActivedKeys(Universe* i_universe) {
+	Uint8 *v_keystate  = SDL_GetKeyState(NULL);
+	Uint8 v_mousestate = SDL_GetMouseState(NULL, NULL);
+	unsigned int p, pW;
+	Biker *v_biker;
+	
+	p = 0; // player number p
+	pW = 0; // number of players in the previous worlds
+	for(unsigned int j=0; j<i_universe->getScenes().size(); j++) {
+		for(unsigned int i=0; i<i_universe->getScenes()[j]->Players().size(); i++) {
+			v_biker = i_universe->getScenes()[j]->Players()[i];
+			
+			if(InputHandler::instance()->getDRIVE(p).isPressed(v_keystate, v_mousestate)) {
+				/* Start driving */
+				v_biker->getControler()->setThrottle(1.0f);
+			} else {
+				v_biker->getControler()->setThrottle(0.0f);
+			}
+			
+			if(InputHandler::instance()->getBRAKE(p).isPressed(v_keystate, v_mousestate)) {
+				/* Brake */
+				v_biker->getControler()->setBreak(1.0f);
+			} else {
+				v_biker->getControler()->setBreak(0.0f);
+			}
+
+			// pull
+			if((InputHandler::instance()->getFLIPLEFT(p).isPressed(v_keystate, v_mousestate) && XMSession::instance()->mirrorMode() == false) ||
+				 (InputHandler::instance()->getFLIPRIGHT(p).isPressed(v_keystate, v_mousestate) && XMSession::instance()->mirrorMode())) {
+				/* Pull back */
+				v_biker->getControler()->setPull(1.0f);
+			} else {
+				// push // must be in pull else block to not set pull to 0
+				if((InputHandler::instance()->getFLIPRIGHT(p).isPressed(v_keystate, v_mousestate) && XMSession::instance()->mirrorMode() == false) ||
+					 (InputHandler::instance()->getFLIPLEFT(p).isPressed(v_keystate, v_mousestate)    && XMSession::instance()->mirrorMode())) {
+					/* Push forward */
+					v_biker->getControler()->setPull(-1.0f);
+				} else {
+					v_biker->getControler()->setPull(0.0f);
+				}
+			}
+			
+			if(InputHandler::instance()->getCHANGEDIR(p).isPressed(v_keystate, v_mousestate)) {
+				/* Change dir */
+				if(m_changeDirKeyAlreadyPress[p] == false){
+					v_biker->getControler()->setChangeDir(true);
+					m_changeDirKeyAlreadyPress[p] = true;
+				}
+			} else {
+				m_changeDirKeyAlreadyPress[p] = false;
+			}
+			p++;
+		}
+		pW+= i_universe->getScenes()[j]->Players().size();
+	}
+}
