@@ -34,8 +34,10 @@ class StateReplaying : public StateScene {
   virtual void leave();
   
   virtual bool update();
+  virtual void nextLevel(bool i_positifOrder = true);
+
   /* input */
-  virtual void keyDown(int nKey, SDLMod mod,int nChar);
+  virtual void xmKey(InputEventType i_type, const XMKey& i_xmkey);
 
   virtual void restartLevel(bool i_reloadLevel = false);
 
