@@ -19,63 +19,127 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 =============================================================================*/
 
 #include "StateScene.h"
-#include "PhysSettings.h"
-#include "xmscene/Camera.h"
-#include "XMSession.h"
-#include "xmscene/Entity.h"
+#include "../PhysSettings.h"
+#include "../xmscene/Camera.h"
+#include "../XMSession.h"
+#include "../xmscene/Entity.h"
+#include "../xmscene/Scene.h"
 #include "StateMessageBox.h"
-#include "drawlib/DrawLib.h"
+#include "../drawlib/DrawLib.h"
 #include "StatePreplayingGame.h"
-#include "helpers/Log.h"
-#include "helpers/Text.h"
-#include "CameraAnimation.h"
-#include "Renderer.h"
-#include "Universe.h"
-#include "VideoRecorder.h"
-#include "GameText.h"
-#include "Game.h"
+#include "../helpers/Log.h"
+#include "../helpers/Text.h"
+#include "../CameraAnimation.h"
+#include "../Renderer.h"
+#include "../Universe.h"
+#include "../VideoRecorder.h"
+#include "../GameText.h"
+#include "../Game.h"
+#include "StateMainMenu.h"
+#include "../net/NetClient.h"
+#include "../net/NetActions.h"
 
 #define INPLAY_ANIMATION_TIME 1.0
 #define INPLAY_ANIMATION_SPEED 10
 #define PRESTART_ANIMATION_MARGIN_SIZE 5
 
 /* control the particle generation by ask the particle renders to limit themself if there are too much particles on the screen */
-#define NB_PARTICLES_TO_RENDER_LIMITATION 130
+#define NB_PARTICLES_TO_RENDER_LIMITATION 512
 
-StateScene::StateScene(bool i_doShade, bool i_doShadeAnim)
-: GameState(false, false, i_doShade, i_doShadeAnim) {
-    m_fLastPhysTime = -1.0;
-    // while playing, we want 100 fps for the physic
-    m_updateFps     = 100;
-    m_showCursor = false;
-    m_cameraAnim = NULL;
-    m_universe   = NULL;
+#define STATS_LEVELS_NOTES_SIZE 15
 
-    m_benchmarkNbFrame   = 0;
-    m_benchmarkStartTime = GameApp::getXMTime();
-}
+void StateScene::init() {
+  Sprite *v_sprite;
 
-StateScene::StateScene(Universe* i_universe, bool i_doShade, bool i_doShadeAnim)
-  : GameState(false, false, i_doShade, i_doShadeAnim)
-{
   m_fLastPhysTime = -1.0;
   // while playing, we want 100 fps for the physic
   m_updateFps     = 100;
   m_showCursor = false;
   m_cameraAnim = NULL;
-  m_universe   = i_universe;
+  m_universe   = NULL;
 
   m_benchmarkNbFrame   = 0;
-  m_benchmarkStartTime = GameApp::getXMTime();  
+  m_benchmarkStartTime = GameApp::getXMTime();
+
+  /* stats */
+  m_difficulty = -1.0;
+  m_quality    = -1.0;
+    
+  m_uncheckedTex = m_qualityTex = m_difficultyTex = NULL;
+  v_sprite = Theme::instance()->getSprite(SPRITE_TYPE_UI, "qsChoiceUnchecked");
+  if(v_sprite != NULL) {
+    m_uncheckedTex = v_sprite->getTexture();
+  }
+  
+  v_sprite = Theme::instance()->getSprite(SPRITE_TYPE_UI, "qsChoiceQuality");
+  if(v_sprite != NULL) {
+    m_qualityTex = v_sprite->getTexture();
+  }
+  
+  v_sprite = Theme::instance()->getSprite(SPRITE_TYPE_UI, "qsChoiceDifficulty");
+  if(v_sprite != NULL) {
+    m_difficultyTex = v_sprite->getTexture();
+  }
+
+  // message registering
+  initMessageRegistering();
+}
+
+StateScene::StateScene(bool i_doShade, bool i_doShadeAnim)
+: GameState(false, false, i_doShade, i_doShadeAnim)
+{
+  init();
+}
+
+StateScene::StateScene(Universe* i_universe, bool i_doShade, bool i_doShadeAnim)
+  : GameState(false, false, i_doShade, i_doShadeAnim)
+{
+  init();
+  m_universe   = i_universe;
+
+  if(m_universe != NULL) {
+    if(m_universe->getScenes().size() != 0) {
+      if(NetClient::instance()->isConnected()) {
+	NA_playingLevel na(m_universe->getScenes()[0]->getLevelSrc()->Id());
+	NetClient::instance()->send(&na, 0);
+      }
+    }
+  }
 }
 
 StateScene::~StateScene()
 {
+  StateManager::instance()->unregisterAsObserver("ERROR", this);
+  StateManager::instance()->unregisterAsObserver("FINISH", this);
+  StateManager::instance()->unregisterAsObserver("RESTART", this);
+  StateManager::instance()->unregisterAsObserver("NEXTLEVEL", this);
+  StateManager::instance()->unregisterAsObserver("PREVIOUSLEVEL", this);
+  StateManager::instance()->unregisterAsObserver("ABORT", this);
+  StateManager::instance()->unregisterAsObserver("INTERPOLATION_CHANGED", this);
+  StateManager::instance()->unregisterAsObserver("MIRRORMODE_CHANGED", this);
+
   if(m_cameraAnim != NULL) {
     delete m_cameraAnim;
   }
 }
 
+void StateScene::initMessageRegistering()
+{
+  StateManager::instance()->registerAsObserver("ERROR", this);
+  StateManager::instance()->registerAsObserver("FINISH", this);
+  StateManager::instance()->registerAsObserver("RESTART", this);
+  StateManager::instance()->registerAsObserver("NEXTLEVEL", this);
+  StateManager::instance()->registerAsObserver("PREVIOUSLEVEL", this);
+  StateManager::instance()->registerAsObserver("ABORT", this);
+  StateManager::instance()->registerAsObserver("INTERPOLATION_CHANGED", this);
+  StateManager::instance()->registerAsObserver("MIRRORMODE_CHANGED", this);
+
+  if(XMSession::instance()->debug() == true) {
+    StateManager::instance()->registerAsEmitter("FAVORITES_UPDATED");
+    StateManager::instance()->registerAsEmitter("BLACKLISTEDLEVELS_UPDATED");
+  }
+}
+
 void StateScene::enter()
 {
   GameState::enter();
@@ -90,43 +154,50 @@ void StateScene::enter()
   m_fLastPhysTime = GameApp::getXMTime();
 }
 
+void StateScene::enterAfterPop()
+{
+  GameState::enterAfterPop();
+}
+
 bool StateScene::update()
 {
   if(doUpdate() == false){
     return false;
   }
-  
-  InputHandler::instance()->updateUniverseInput(m_universe); // update input for the universe
 
-  int nPhysSteps = 0;
-
-  if(isLockedScene() == false) {  
-    // don't update if that's not required
-    // don't do this infinitely, maximum miss 10 frames, then give up
-		// in videoRecording mode, don't try to do more to allow to record at a good framerate
-    while (( m_fLastPhysTime + (PHYS_STEP_SIZE)/100.0 <= GameApp::getXMTime()) && nPhysSteps < 10 && (XMSession::instance()->enableVideoRecording() == false || nPhysSteps == 0)) {
+  try {
+    int nPhysSteps = 0;
+    
+    if(isLockedScene() == false) {  
+      // don't update if that's not required
+      // don't do this infinitely, maximum miss 10 frames, then give up
+      // in videoRecording mode, don't try to do more to allow to record at a good framerate
+      while (( m_fLastPhysTime + (PHYS_STEP_SIZE)/100.0 <= GameApp::getXMTime()) && nPhysSteps < 10 && (XMSession::instance()->enableVideoRecording() == false || nPhysSteps == 0)) {
+	if(m_universe != NULL) {
+	  for(unsigned int i=0; i<m_universe->getScenes().size(); i++) {
+	    m_universe->getScenes()[i]->updateLevel(PHYS_STEP_SIZE, m_universe->getCurrentReplay());
+	  }
+	}
+	m_fLastPhysTime += PHYS_STEP_SIZE/100.0;
+	nPhysSteps++;
+      }
+      
+      // if the delay is too long, reinitialize
+      if(m_fLastPhysTime + PHYS_STEP_SIZE/100.0 < GameApp::getXMTime()) {
+	m_fLastPhysTime = GameApp::getXMTime();
+      }
+      
+      // update camera scrolling
       if(m_universe != NULL) {
-	for(unsigned int i=0; i<m_universe->getScenes().size(); i++) {
-	  m_universe->getScenes()[i]->updateLevel(PHYS_STEP_SIZE, m_universe->getCurrentReplay());
-	}
-      }
-      m_fLastPhysTime += PHYS_STEP_SIZE/100.0;
-      nPhysSteps++;
-    }
-
-    // if the delay is too long, reinitialize
-    if(m_fLastPhysTime + PHYS_STEP_SIZE/100.0 < GameApp::getXMTime()) {
-      m_fLastPhysTime = GameApp::getXMTime();
-    }
-
-    // update camera scrolling
-    if(m_universe != NULL) {
-      for(unsigned int j=0; j<m_universe->getScenes().size(); j++) {
-	for(unsigned int i=0; i<m_universe->getScenes()[j]->Cameras().size(); i++) {
-	  m_universe->getScenes()[j]->Cameras()[i]->setScroll(true, m_universe->getScenes()[j]->getGravity());
+	for(unsigned int j=0; j<m_universe->getScenes().size(); j++) {
+	  for(unsigned int i=0; i<m_universe->getScenes()[j]->Cameras().size(); i++) {
+	    m_universe->getScenes()[j]->Cameras()[i]->setScroll(true, m_universe->getScenes()[j]->getGravity());
+	  }
 	}
       }
     }
+  } catch(Exception &e) {
+    StateManager::instance()->replaceState(new StateMessageBox(NULL, splitText(e.getMsg(), 50), UI_MSGBOX_OK));  
   }
 
   runAutoZoom();
@@ -192,30 +263,34 @@ void StateScene::onRenderFlush() {
   }
 }
 
-void StateScene::keyUp(int nKey, SDLMod mod) {
-  switch(nKey) {
-  case SDLK_TAB:
-    if(m_cameraAnim != NULL) {
-      if(autoZoom() && m_cameraAnim->allowNextStep()) {
-	m_cameraAnim->goNextStep();
+void StateScene::xmKey(InputEventType i_type, const XMKey& i_xmkey) {
+  GameApp* pGame = GameApp::instance();  
+
+  if(i_xmkey == XMKey(SDLK_TAB, KMOD_NONE)) {
+    if(i_type == INPUT_UP) {
+      if(m_cameraAnim != NULL) {
+	if(autoZoom() && m_cameraAnim->allowNextStep()) {
+	  m_cameraAnim->goNextStep();
+	}
+      }
+    } else {
+      if(autoZoom() == false) {
+	setAutoZoom(true);
       }
     }
-    break;
   }
-}
 
-void StateScene::keyDown(int nKey, SDLMod mod,int nChar)
-{
-  GameApp*  pGame = GameApp::instance();
-
-  if(nKey == SDLK_RETURN && (mod & (KMOD_CTRL|KMOD_SHIFT|KMOD_ALT|KMOD_META)) == 0){
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_RETURN, KMOD_NONE)) {
     restartLevel();
-  } else if(nKey == SDLK_F2){
+  }
+
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_F2, KMOD_NONE)) {
     if(m_universe != NULL) {
       m_universe->switchFollowCamera();
     }
   }
-  else if(nKey == SDLK_F3){
+
+  else if(i_type == INPUT_DOWN && i_xmkey == InputHandler::instance()->getSwitchFavorite()) {
     if(m_universe != NULL) {
       if(m_universe->getScenes().size() > 0) { // just add the first world
 	pGame->switchLevelToFavorite(m_universe->getScenes()[0]->getLevelSrc()->Id(), true);
@@ -223,7 +298,8 @@ void StateScene::keyDown(int nKey, SDLMod mod,int nChar)
       }
     }
   }
-  else if(nKey == SDLK_b && (mod & KMOD_CTRL) != 0){
+
+  else if(i_type == INPUT_DOWN && i_xmkey == InputHandler::instance()->getSwitchBlacklist()) {
     if(m_universe != NULL) {
       if(m_universe->getScenes().size() > 0) { // just blacklist the first world
 	pGame->switchLevelToBlacklist(m_universe->getScenes()[0]->getLevelSrc()->Id(), true);
@@ -231,13 +307,17 @@ void StateScene::keyDown(int nKey, SDLMod mod,int nChar)
       }
     }
   }
-  else if(nKey == SDLK_PAGEUP){
+
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_PAGEUP, KMOD_NONE)) {
     nextLevel();
   }
 
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_PAGEDOWN, KMOD_NONE)) {
+    nextLevel(false);
+  }
 
-#if defined(ENABLE_ZOOMING)
-  else if(nKey == SDLK_KP7){
+#if defined(ENABLE_DEV)
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_KP7, KMOD_NONE)) {
     /* Zoom in */
     if(m_universe != NULL) {
       for(unsigned int j=0; j<m_universe->getScenes().size(); j++) {
@@ -248,7 +328,8 @@ void StateScene::keyDown(int nKey, SDLMod mod,int nChar)
       }
     }
   }
-  else if(nKey == SDLK_KP9){
+
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_KP9, KMOD_NONE)) {
     /* Zoom out */
     if(m_universe != NULL) {
       for(unsigned int j=0; j<m_universe->getScenes().size(); j++) {
@@ -259,7 +340,8 @@ void StateScene::keyDown(int nKey, SDLMod mod,int nChar)
       }
     }
   }
-  else if(nKey == SDLK_HOME){
+
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_HOME, KMOD_NONE)) {
     if(m_universe != NULL) {
       for(unsigned int j=0; j<m_universe->getScenes().size(); j++) {
 	for(unsigned int i=0; i<m_universe->getScenes()[j]->Cameras().size(); i++) {
@@ -269,7 +351,8 @@ void StateScene::keyDown(int nKey, SDLMod mod,int nChar)
       }
     }
   }
-  else if(nKey == SDLK_KP6){
+
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_KP6, KMOD_NONE)) {
     if(m_universe != NULL) {
       for(unsigned int j=0; j<m_universe->getScenes().size(); j++) {
 	for(unsigned int i=0; i<m_universe->getScenes()[j]->Cameras().size(); i++) {
@@ -279,7 +362,8 @@ void StateScene::keyDown(int nKey, SDLMod mod,int nChar)
       }
     }
   }
-  else if(nKey == SDLK_KP4){
+
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_KP4, KMOD_NONE)) {
     if(m_universe != NULL) {
       for(unsigned int j=0; j<m_universe->getScenes().size(); j++) {
 	for(unsigned int i=0; i<m_universe->getScenes()[j]->Cameras().size(); i++) {
@@ -289,7 +373,8 @@ void StateScene::keyDown(int nKey, SDLMod mod,int nChar)
       }
     }
   }
-  else if(nKey == SDLK_KP8){
+
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_KP8, KMOD_NONE)) {
     if(m_universe != NULL) {
       for(unsigned int j=0; j<m_universe->getScenes().size(); j++) {
 	for(unsigned int i=0; i<m_universe->getScenes()[j]->Cameras().size(); i++) {
@@ -299,7 +384,8 @@ void StateScene::keyDown(int nKey, SDLMod mod,int nChar)
       }
     }
   }
-  else if(nKey == SDLK_KP2){
+
+  else if(i_type == INPUT_DOWN && i_xmkey == XMKey(SDLK_KP2, KMOD_NONE)) {
     if(m_universe != NULL) {
       for(unsigned int j=0; j<m_universe->getScenes().size(); j++) {
 	for(unsigned int i=0; i<m_universe->getScenes()[j]->Cameras().size(); i++) {
@@ -311,25 +397,19 @@ void StateScene::keyDown(int nKey, SDLMod mod,int nChar)
   }
 #endif
 
-  else if(nKey == SDLK_PAGEDOWN){
-    nextLevel(false);
-  } else  if(nKey == SDLK_TAB){
-    if(autoZoom() == false) {
-      setAutoZoom(true);
-    }
-  } else{
-    GameState::keyDown(nKey, mod, nChar);
+  else {
+    GameState::xmKey(i_type, i_xmkey);
   }
 }
 
-void StateScene::send(const std::string& i_id, UIMsgBoxButton i_button, const std::string& i_input) {
+void StateScene::sendFromMessageBox(const std::string& i_id, UIMsgBoxButton i_button, const std::string& i_input) {
   if(i_id == "ERROR") {
-    m_commands.push("ERROR");
+    addCommand("ERROR");
   }
-}
 
-void StateScene::send(const std::string& i_id, const std::string& i_message) {
-  m_commands.push(i_message);
+  else {
+    GameState::sendFromMessageBox(i_id, i_button, i_input);
+  }
 }
 
 void StateScene::setScoresTimes() {
@@ -391,6 +471,16 @@ void StateScene::abortPlaying() {
 }
 
 void StateScene::closePlaying() {
+  if(NetClient::instance()->isConnected()) {
+    // stop playing
+    NA_playingLevel na("");
+    NetClient::instance()->send(&na, 0);
+  }
+
+  if(NetClient::instance()->isConnected()) {
+    NetClient::instance()->endPlay();
+  }
+
   if(m_universe != NULL) {
     delete m_universe;
     m_universe = NULL;
@@ -450,44 +540,60 @@ void StateScene::runAutoZoom() {
   }
 }
 
-void StateScene::executeOneCommand(std::string cmd)
+void StateScene::executeOneCommand(std::string cmd, std::string args)
 {
-  Logger::Log("StateScene::executeOneCommand::%s", cmd.c_str());
+  LogDebug("cmd [%s [%s]] executed by state [%s].",
+	   cmd.c_str(), args.c_str(), getName().c_str());
 
   if(cmd == "ERROR") {
     closePlaying();
-    m_requestForEnd = true;
-    return;
+
+    // there is no other state before
+    if(StateManager::instance()->numberOfStates() == 1) {
+      // run the mainmenu state
+      StateManager::instance()->replaceState(new StateMainMenu());
+    } else {
+      m_requestForEnd = true;
+    }
   }
 
-  if(cmd == "FINISH") {
+  else if(cmd == "FINISH") {
     closePlaying();
-    m_requestForEnd = true;
-    return;
+
+    // there is no other state before
+    if(StateManager::instance()->numberOfStates() == 1) {
+      // run the mainmenu state
+      StateManager::instance()->replaceState(new StateMainMenu());
+    } else {
+      m_requestForEnd = true;
+    }
   }
 
-  if(cmd == "RESTART") {
+  else if(cmd == "RESTART") {
     restartLevel();
-    return;
   }
 
-  if(cmd == "NEXTLEVEL") {
+  else if(cmd == "NEXTLEVEL") {
     nextLevel();
-    return;
   }
 
-  if(cmd == "PREVIOUSLEVEL") {
+  else if(cmd == "PREVIOUSLEVEL") {
     nextLevel(false);
-    return;
   }
 
-  if(cmd == "ABORT") {
+  else if(cmd == "ABORT") {
     abortPlaying();
-    m_requestForEnd = true;
-    return;
+
+    // there is no other state before
+    if(StateManager::instance()->numberOfStates() == 1) {
+      // run the mainmenu state
+      StateManager::instance()->replaceState(new StateMainMenu());
+    } else {
+      m_requestForEnd = true;
+    }
   }
 
-  if(cmd == "INTERPOLATION_CHANGED") {
+  else if(cmd == "INTERPOLATION_CHANGED") {
     if(m_universe != NULL) {
       for(unsigned int j=0; j<m_universe->getScenes().size(); j++) {
 	for(unsigned int i=0; i<m_universe->getScenes()[j]->Players().size(); i++) {
@@ -497,7 +603,7 @@ void StateScene::executeOneCommand(std::string cmd)
     }
   }
 
-  if(cmd == "MIRRORMODE_CHANGED") {
+  else if(cmd == "MIRRORMODE_CHANGED") {
     if(m_universe != NULL) {
       for(unsigned int j=0; j<m_universe->getScenes().size(); j++) {
 	for(unsigned int i=0; i<m_universe->getScenes()[j]->Cameras().size(); i++) {
@@ -507,6 +613,9 @@ void StateScene::executeOneCommand(std::string cmd)
     }
   }
 
+  else {
+    GameState::executeOneCommand(cmd, args);
+  }
 }
 
 void StateScene::displayStats() {
@@ -525,7 +634,80 @@ void StateScene::displayStats() {
   v_fm->printString(v_fg,
 		    GameApp::instance()->getDrawLib()->getDispWidth() - v_fg->realWidth()   - vborder,
 		    GameApp::instance()->getDrawLib()->getDispHeight() - v_fg->realHeight() - vborder,
-		    MAKE_COLOR(220,255,255,255), true);
+		    MAKE_COLOR(220,255,255,255), -1.0, true);
+
+  // quality
+  int v_quality_yoffset = 5;
+
+  if(m_quality >= 0.0) {
+    v_fg = GameApp::instance()->getDrawLib()->getFontSmall()->getGlyph(GAMETEXT_QUALITY);
+    v_fm->printString(v_fg,
+		      A.x - vborder*2,
+		      A.y - vborder*2 - v_fg->realHeight() - STATS_LEVELS_NOTES_SIZE -v_quality_yoffset,
+		      MAKE_COLOR(220,255,255,255), -1.0, true);
+    
+    if(XMSession::instance()->ugly()) {
+      for(int i=0; i<(int)(m_quality); i++) {
+	GameApp::instance()->getDrawLib()->drawCircle(Vector2f(A.x - vborder*2 + STATS_LEVELS_NOTES_SIZE/2 + (STATS_LEVELS_NOTES_SIZE*i),
+							       A.y - vborder*2 - STATS_LEVELS_NOTES_SIZE/2 -v_quality_yoffset),
+						      STATS_LEVELS_NOTES_SIZE/2,
+						      1.0, 0, MAKE_COLOR(255, 0, 0, 255));
+      }
+    } else {
+      for(int i=0; i<5; i++) {
+	if(i<(int)(m_quality)) {
+	  GameApp::instance()->getDrawLib()->drawImage(Vector2f(A.x - vborder*2 + (STATS_LEVELS_NOTES_SIZE*i),
+								A.y - vborder*2 - STATS_LEVELS_NOTES_SIZE -v_quality_yoffset),
+						       Vector2f(A.x - vborder*2 + STATS_LEVELS_NOTES_SIZE + (STATS_LEVELS_NOTES_SIZE*i),
+								A.y - vborder*2 -v_quality_yoffset),
+						       m_qualityTex, 0xFFFFFFFF, true);
+	} else {
+	  GameApp::instance()->getDrawLib()->drawImage(Vector2f(A.x - vborder*2 + (STATS_LEVELS_NOTES_SIZE*i),
+								A.y - vborder*2 - STATS_LEVELS_NOTES_SIZE -v_quality_yoffset),
+						       Vector2f(A.x - vborder*2 + STATS_LEVELS_NOTES_SIZE + (STATS_LEVELS_NOTES_SIZE*i),
+								A.y - vborder*2 -v_quality_yoffset),
+						       m_uncheckedTex, 0xFFFFFFFF, true);
+	}
+      }
+    }
+  }
+
+  // difficulty
+  int v_difficulty_yoffset = v_fg->realHeight() + STATS_LEVELS_NOTES_SIZE + v_quality_yoffset;
+
+  if(m_difficulty >= 0.0) {
+    v_fg = GameApp::instance()->getDrawLib()->getFontSmall()->getGlyph(GAMETEXT_DIFFICULTY);
+    v_fm->printString(v_fg,
+		      A.x - vborder*2,
+		      A.y - vborder*2 - v_fg->realHeight() - STATS_LEVELS_NOTES_SIZE -v_difficulty_yoffset,
+		      MAKE_COLOR(220,255,255,255), -1.0, true);
+    
+    if(XMSession::instance()->ugly()) {
+      for(int i=0; i<(int)(m_difficulty); i++) {
+	GameApp::instance()->getDrawLib()->drawCircle(Vector2f(A.x - vborder*2 + STATS_LEVELS_NOTES_SIZE/2 + (STATS_LEVELS_NOTES_SIZE*i),
+							       A.y - vborder*2 - STATS_LEVELS_NOTES_SIZE/2 -v_difficulty_yoffset),
+						      STATS_LEVELS_NOTES_SIZE/2,
+						      1.0, 0, MAKE_COLOR(255, 0, 0, 255));
+      }
+    } else {
+      for(int i=0; i<5; i++) {
+	if(i<(int)(m_difficulty)) {
+	  GameApp::instance()->getDrawLib()->drawImage(Vector2f(A.x - vborder*2 + (STATS_LEVELS_NOTES_SIZE*i),
+								A.y - vborder*2 - STATS_LEVELS_NOTES_SIZE -v_difficulty_yoffset),
+						       Vector2f(A.x - vborder*2 + STATS_LEVELS_NOTES_SIZE + (STATS_LEVELS_NOTES_SIZE*i),
+							      A.y - vborder*2 -v_difficulty_yoffset),
+						       m_difficultyTex, 0xFFFFFFFF, true);
+	} else {
+	  GameApp::instance()->getDrawLib()->drawImage(Vector2f(A.x - vborder*2 + (STATS_LEVELS_NOTES_SIZE*i),
+								A.y - vborder*2 - STATS_LEVELS_NOTES_SIZE -v_difficulty_yoffset),
+						       Vector2f(A.x - vborder*2 + STATS_LEVELS_NOTES_SIZE + (STATS_LEVELS_NOTES_SIZE*i),
+								A.y - vborder*2 -v_difficulty_yoffset),
+						       m_uncheckedTex, 0xFFFFFFFF, true);
+	}
+      }
+    }
+  }
+
 }
 
 void StateScene::makeStatsStr() {
@@ -540,10 +722,10 @@ void StateScene::makeStatsStr() {
       int v_nbPlayed, v_nbDied, v_nbCompleted, v_nbRestart, v_playedTime;
       std::string v_idLevel = m_universe->getScenes()[0]->getLevelSrc()->Id();
       
-      v_result = v_pDb->readDB("SELECT nbPlayed, nbDied, nbCompleted, nbRestarted, playedTime "
+      v_result = v_pDb->readDB("SELECT IFNULL(SUM(nbPlayed),0), IFNULL(SUM(nbDied),0), IFNULL(SUM(nbCompleted),0), IFNULL(SUM(nbRestarted),0), IFNULL(SUM(playedTime),0) "
 			       "FROM stats_profiles_levels "
 			       "WHERE id_profile=\"" + xmDatabase::protectString(XMSession::instance()->profile()) + "\" "
-			       "AND id_level=\""     + xmDatabase::protectString(v_idLevel) + "\";",
+			       "AND id_level=\""     + xmDatabase::protectString(v_idLevel) + "\" GROUP BY id_profile;",
 			       nrow);
       if(nrow != 1) {
 	/* not statistics */
@@ -570,6 +752,22 @@ void StateScene::makeStatsStr() {
 	   formatTime(v_playedTime).c_str());
       m_statsStr = c_tmp;
       v_pDb->read_DB_free(v_result);
+
+
+      // quality and difficulty
+      v_result = v_pDb->readDB("SELECT difficulty, quality "
+			       "FROM weblevels "
+			       "WHERE id_level=\"" + xmDatabase::protectString(v_idLevel) + "\";",
+			       nrow);
+      if(nrow != 1) {
+	/* not statistics */
+	m_difficulty = -1.0;
+	m_quality    = -1.0;
+      } else {
+	m_difficulty = atof(v_pDb->getResult(v_result, 2, 0, 0));
+	m_quality    = atof(v_pDb->getResult(v_result, 2, 0, 1));
+      }
+      v_pDb->read_DB_free(v_result);
     }
   }
 }
@@ -582,6 +780,45 @@ void StateScene::nextLevel(bool i_positifOrder) {
   /* do nothing, it's depends of the scene ; often empty for animation steps */
 }
 
+void StateScene::playingNextLevel(bool i_positifOrder) {
+  GameApp*  pGame  = GameApp::instance();
+  std::string v_nextLevel;
+  std::string v_currentLevel;
+  
+  // take the level id of the first world
+  if(m_universe != NULL) {
+    if(m_universe->getScenes().size() > 0) {
+      v_currentLevel = m_universe->getScenes()[0]->getLevelSrc()->Id();
+    }
+  }
+
+  if(i_positifOrder) {
+    v_nextLevel = pGame->determineNextLevel(v_currentLevel);
+  } else {
+    v_nextLevel = pGame->determinePreviousLevel(v_currentLevel);
+  }
+
+  /* update stats */
+  if(v_nextLevel != "") {
+    if(m_universe != NULL) {
+      if(m_universe->getScenes().size() == 1) {
+	if(m_universe->getScenes()[0]->Players().size() == 1) {
+	  if(m_universe->getScenes()[0]->Players()[0]->isDead()     == false &&
+	     m_universe->getScenes()[0]->Players()[0]->isFinished() == false) {
+	    xmDatabase::instance("main")->stats_abortedLevel(XMSession::instance()->sitekey(),
+							     XMSession::instance()->profile(),
+							     v_currentLevel,
+							     m_universe->getScenes()[0]->getTime());
+	    StateManager::instance()->sendAsynchronousMessage("STATS_UPDATED");
+	  }
+	}
+      }
+    }
+  }
+
+  nextLevelToPlay(i_positifOrder);
+}
+
 void StateScene::restartLevelToPlay(bool i_reloadLevel) {
   std::string v_level;
   
@@ -631,3 +868,16 @@ void StateScene::nextLevelToPlay(bool i_positifOrder) {
     StateManager::instance()->replaceState(new StatePreplayingGame(v_nextLevel, v_currentLevel == v_nextLevel));
   }
 }
+
+void StateScene::playLevelMusic() {
+  if(m_universe != NULL) {
+    if(m_universe->getScenes().size() > 0) {
+      // play music of the first world
+      if(m_universe->getScenes()[0]->getLevelSrc()->Music() == "" && XMSession::instance()->musicOnAllLevels()) {
+	GameApp::instance()->playGameMusic(Theme::instance()->getHashMusic(m_universe->getScenes()[0]->getLevelSrc()->Id()));
+      } else {
+	GameApp::instance()->playGameMusic(m_universe->getScenes()[0]->getLevelSrc()->Music());
+      } 
+    }
+  }
+}
