@@ -29,21 +29,16 @@ public:
 		     bool updateStatesBehind = false);
   virtual ~StateUpgradeLevels();
 
-  virtual void send(const std::string& i_id,
-		    UIMsgBoxButton i_button,
-		    const std::string& i_input);
-  virtual void executeOneCommand(std::string cmd);
+  virtual void sendFromMessageBox(const std::string& i_id,
+				  UIMsgBoxButton i_button,
+				  const std::string& i_input);
+  virtual void executeOneCommand(std::string cmd, std::string args);
 
   void callAfterThreadFinished(int threadResult);
-  void keyDown(int nKey, SDLMod mod,int nChar);
-
-  void setCurrentUpdatedLevel(std::string levelName);
+  virtual void xmKey(InputEventType i_type, const XMKey& i_xmkey);
 
 protected:
   virtual void checkEvents() {}
-
-private:
-  std::string m_curUpdLevelName;
 };
 
 #endif
