@@ -41,11 +41,12 @@ distribution.
 // or order will break putstring.	
 TiXmlBase::Entity TiXmlBase::entity[ NUM_ENTITY ] = 
 {
-	{ "&amp;",  5, '&' },
-	{ "&lt;",   4, '<' },
-	{ "&gt;",   4, '>' },
-	{ "&quot;", 6, '\"' },
-	{ "&apos;", 6, '\'' }
+	{ "&amp;",   5, '&' },
+	{ "&lt;",    4, '<' },
+	{ "&gt;",    4, '>' },
+	{ "&quot;",  6, '\"' },
+	{ "&apos;",  6, '\'' },
+	{ "&rsquo;", 7, '`' }
 };
 
 // Bunch of unicode info at:
