@@ -27,24 +27,20 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "../Game.h"
 #include "BikeParameters.h"
 #include "BikeAnchors.h"
-#include "Sound.h"
+#include "../Sound.h"
+#include "../helpers/Log.h"
+#include "PhysicsSettings.h"
 
-#define PHYSICAL_ENGINE_REDUCTION 0.05
-
-BikeState::BikeState() {
-  m_bikeParameters = new BikeParameters();
+BikeState::BikeState(PhysicsSettings* i_physicsSettings) {
+  m_bikeParameters = new BikeParameters(i_physicsSettings);
   m_bikeAnchors    = new BikeAnchors();
 
-  reInitializeSpeed();
   reInitializeAnchors();
 
   GameTime = 0;
 }
 
 BikeState& BikeState::operator=(const BikeState& i_copy) {
-  this->m_curBrake = i_copy.m_curBrake;
-  this->m_curEngine = i_copy.m_curEngine;
-
   *(this->m_bikeParameters) = *(i_copy.m_bikeParameters);
   *(this->m_bikeAnchors)    = *(i_copy.m_bikeAnchors);
 
@@ -169,24 +165,6 @@ BikeState::~BikeState() {
   }
 
 
-void BikeState::reInitializeSpeed() {
-  m_curBrake  = 0.0;
-  m_curEngine = 0.0;
-}
-
-float BikeState::CurrentBrake() const {
-  return m_curBrake;
-}
-
-float BikeState::CurrentEngine() const {
-  return m_curEngine;
-}
-
-void BikeState::physicalUpdate() {
-  if(m_curEngine > 0.0)
-    m_curEngine -= m_bikeParameters->MaxEngine() * PHYSICAL_ENGINE_REDUCTION; 
-}
-
 void BikeState::reInitializeAnchors() {
   m_bikeAnchors->update(m_bikeParameters);
 }
@@ -199,14 +177,15 @@ BikeParameters* BikeState::Parameters() {
   return m_bikeParameters;
 }
 
-BikeState* Biker::getState() {
-  return &m_bikeState;
-}
-
-Biker::Biker(Theme *i_theme, BikerTheme* i_bikerTheme,
+Biker::Biker(PhysicsSettings* i_physicsSettings,
+	     Theme *i_theme, BikerTheme* i_bikerTheme,
 	     const TColor& i_colorFilter,
 	     const TColor& i_uglyColorFilter) {
+
   m_EngineSound = new EngineSoundSimulator();
+  m_physicsSettings = i_physicsSettings;
+  m_bikeState   = new BikeState(m_physicsSettings);
+  m_localNetId = -1;
 
   /* sound engine */
   try {
@@ -238,13 +217,19 @@ Biker::Biker(Theme *i_theme, BikerTheme* i_bikerTheme,
   m_colorFilter = i_colorFilter;
   m_uglyColorFilter = i_uglyColorFilter;
   m_doInterpolation = true;
-	m_bodyDetach    = false;
-	m_wheelDetach   =  false;
+  m_bodyDetach      = false;
+  m_wheelDetach     =  false;
+  m_changeDirPer      = 1.0;
 }
 
 Biker::~Biker() { 
 	cleanCollisionPoints();
   delete m_EngineSound;
+  delete m_bikeState;
+}
+
+void Biker::setLocalNetId(int i_value) {
+  m_localNetId = i_value;
 }
 
 const TColor& Biker::getColorFilter() const {
@@ -263,14 +248,29 @@ int Biker::deadTime() const {
   return m_deadTime;
 }
 
+int Biker::localNetId() const {
+  return m_localNetId;
+}
 
 float Biker::getBikeEngineRPM() {
-  return m_bikeState.fBikeEngineRPM;
+  return m_bikeState->fBikeEngineRPM;
+}
+
+PhysicsSettings* Biker::getPhysicsSettings() {
+  return m_physicsSettings;
 }
 
 void Biker::updateToTime(int i_time, int i_timeStep,
 			     CollisionSystem *v_collisionSystem, Vector2f i_gravity,
-			     MotoGame *i_motogame) {
+			     Scene *i_motogame) {
+  /* update direction - even if the biker is dead of finished */
+  if(m_changeDirPer < 1.0) {
+    m_changeDirPer += ((float)i_timeStep)*3.0 / 100.0;
+    if(m_changeDirPer > 1.0) {
+      m_changeDirPer = 1.0;
+    }
+  }
+
   if(isFinished() || isDead()) return;
 
   /* sound */
@@ -278,6 +278,7 @@ void Biker::updateToTime(int i_time, int i_timeStep,
   if(m_playSound) {
     m_EngineSound->update(i_time);
   }
+
 }
 
 bool Biker::isTouching(const Entity* i_entity) const {
@@ -531,7 +532,7 @@ void BikeState::interpolateGameState(std::vector<BikeState*> &i_ghostBikeStates,
 }
 
   void BikeState::convertStateFromReplay(SerializedBikeState *pReplayState,
-          BikeState *pBikeS) 
+					 BikeState *pBikeS, PhysicsSettings* i_physicsSettings) 
   {
     bool bUpdateRider=true,bUpdateAltRider=true;
     
@@ -548,7 +549,7 @@ void BikeState::interpolateGameState(std::vector<BikeState*> &i_ghostBikeStates,
     _16BitsToMatrix(pReplayState->nRearWheelRot,pBikeS->fRearWheelRot);
     
     /* Update engine stuff */
-    pBikeS->fBikeEngineRPM = ENGINE_MIN_RPM + (ENGINE_MAX_RPM - ENGINE_MIN_RPM) * ((float)pReplayState->cBikeEngineRPM) / 255.0f;
+    pBikeS->fBikeEngineRPM = i_physicsSettings->EngineRpmMin() + (i_physicsSettings->EngineRpmMax()-i_physicsSettings->EngineRpmMin()) * ((float)pReplayState->cBikeEngineRPM) / 255.0f;
     
     pBikeS->SwingAnchorP.x = pBikeS->Anchors()->AR.x*pBikeS->fFrameRot[0] + pBikeS->Anchors()->AR.y*pBikeS->fFrameRot[1] + pBikeS->CenterP.x;
     pBikeS->SwingAnchorP.y = pBikeS->Anchors()->AR.x*pBikeS->fFrameRot[2] + pBikeS->Anchors()->AR.y*pBikeS->fFrameRot[3] + pBikeS->CenterP.y;
@@ -610,22 +611,32 @@ void BikeState::interpolateGameState(std::vector<BikeState*> &i_ghostBikeStates,
     if(bUpdateRider) {        
       /* Calculate head position */
       V = (pBikeS->ShoulderP - pBikeS->LowerBodyP);
-      V.normalize();
-      pBikeS->HeadP = pBikeS->ShoulderP + V*pBikeS->Parameters()->fNeckLength;
+
+      try {
+	V.normalize();
+	pBikeS->HeadP = pBikeS->ShoulderP + V*pBikeS->Parameters()->fNeckLength;
+      } catch(Exception &e) {
+	pBikeS->HeadP = pBikeS->ShoulderP;
+      }
     }
     
     if(bUpdateAltRider) {
       /* Calculate head position (Alt.) */
       V = (pBikeS->Shoulder2P - pBikeS->LowerBody2P);
-      V.normalize();
-      pBikeS->Head2P = pBikeS->Shoulder2P + V*pBikeS->Parameters()->fNeckLength;
+
+      try {
+	V.normalize();
+	pBikeS->Head2P = pBikeS->Shoulder2P + V*pBikeS->Parameters()->fNeckLength;
+      } catch(Exception &e) {
+	pBikeS->Head2P = pBikeS->Shoulder2P;
+      }
     }
     
     /* Internally we'd like to know the abs. relaxed position of the wheels */
     pBikeS->RFrontWheelP.x = pBikeS->Anchors()->Fp.x*pBikeS->fFrameRot[0] + pBikeS->Anchors()->Fp.y*pBikeS->fFrameRot[1] + pBikeS->CenterP.x;
     pBikeS->RFrontWheelP.y = pBikeS->Anchors()->Fp.x*pBikeS->fFrameRot[2] + pBikeS->Anchors()->Fp.y*pBikeS->fFrameRot[3] + pBikeS->CenterP.y;
     pBikeS->RRearWheelP.x = pBikeS->Anchors()->Rp.x*pBikeS->fFrameRot[0] + pBikeS->Anchors()->Rp.y*pBikeS->fFrameRot[1] + pBikeS->CenterP.x;
-    pBikeS->RRearWheelP.y = pBikeS->Anchors()->Rp.x*pBikeS->fFrameRot[2] + pBikeS->Anchors()->Rp.y*pBikeS->fFrameRot[3] + pBikeS->CenterP.y;     
+    pBikeS->RRearWheelP.y = pBikeS->Anchors()->Rp.x*pBikeS->fFrameRot[2] + pBikeS->Anchors()->Rp.y*pBikeS->fFrameRot[3] + pBikeS->CenterP.y;
 
     // time
     pBikeS->GameTime = GameApp::floatToTime(pReplayState->fGameTime);
@@ -641,3 +652,15 @@ void Biker::cleanCollisionPoints() {
 std::vector<Vector2f> &Biker::CollisionPoints() {
 	return m_collisionPoints;
 }
+
+float Biker::getRearWheelVelocity() {
+  return 0.0;
+}
+
+float Biker::getFrontWheelVelocity() {
+  return 0.0;
+}
+
+float Biker::changeDirPer() const {
+  return m_changeDirPer;
+}
