@@ -22,9 +22,10 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #define __BIKE_H__
 
 #include <string>
+#include <vector>
 #include "BasicSceneStructs.h"
-#include "helpers/Color.h"
-#include "helpers/VMath.h"
+#include "../helpers/Color.h"
+#include "../helpers/VMath.h"
 
 class Level;
 class Entity;
@@ -32,7 +33,7 @@ class Zone;
 class OnBikerHooks;
 class Replay;
 class CollisionSystem;
-class MotoGame;
+class Scene;
 class Replay;
 class BikerTheme;
 class Theme;
@@ -40,11 +41,11 @@ class BikeAnchors;
 class BikeParameters;
 class BikeController;
 class EngineSoundSimulator;
+class PhysicsSettings;
 
 class BikeState {
   public:
     DriveDir Dir;         /* Driving left or right? */
-  
     float fBikeEngineRPM;
   
     Vector2f RearWheelP;  /* Rear wheel position */
@@ -103,23 +104,18 @@ class BikeState {
     int GameTime; // time of the state, for replays;
     
     /* Bonusinfo */    
-  BikeState();
-  ~BikeState();
+  BikeState(PhysicsSettings* i_physicsSettings);
+  virtual ~BikeState();
   
   BikeState& operator=(const BikeState& i_copy);
 
-  void reInitializeSpeed();
-  void physicalUpdate();
   void reInitializeAnchors();
 
-  float CurrentBrake() const;
-  float CurrentEngine() const;
-    
   BikeAnchors* Anchors();
   BikeParameters* Parameters();
 
   static void interpolateGameState(std::vector<BikeState*> &i_ghostBikeStates, BikeState *p,float t);
-  static void convertStateFromReplay(SerializedBikeState *pReplayState,BikeState *pBikeS);
+  static void convertStateFromReplay(SerializedBikeState *pReplayState,BikeState *pBikeS, PhysicsSettings* i_physicsSettings);
 
   static signed char _MapCoordTo8Bits(float fRef,float fMaxDiff,float fCoord);
   static float _Map8BitsToCoord(float fRef,float fMaxDiff,signed char c);
@@ -128,7 +124,6 @@ class BikeState {
 
   private:
   /* Driving */
-  float m_curBrake, m_curEngine;    
   BikeParameters* m_bikeParameters;
   BikeAnchors*    m_bikeAnchors;
 
@@ -138,22 +133,32 @@ class BikeState {
 
 class Biker {
  public:
-  Biker(Theme *i_theme, BikerTheme* i_bikerTheme,
+  Biker(PhysicsSettings* i_physicsSettings,
+	Theme *i_theme, BikerTheme* i_bikerTheme,
 	const TColor& i_colorFilter,
 	const TColor& i_uglyColorFilter);
   virtual ~Biker();
-  virtual BikeState* getState();
+  inline BikeState* getState()
+  {
+    return m_bikeState;
+  }
   virtual bool getRenderBikeFront() = 0; /* display the bikefront ? (for detach) */
   virtual float getBikeEngineSpeed() = 0; /* engine speed */
   virtual float getBikeLinearVel() = 0; /* bike linear velocity */
+  virtual double getAngle() = 0; /* biker angle */
+  virtual std::string getVeryQuickDescription() const = 0;
   virtual std::string getQuickDescription() const = 0;
   virtual std::string getDescription() const = 0;
   virtual float getBikeEngineRPM();
   virtual void  updateToTime(int i_time, int i_timeStep,
 			     CollisionSystem *i_collisionSystem, Vector2f i_gravity,
-			     MotoGame *i_motogame);
+			     Scene *i_motogame);
   void setPlaySound(bool i_value);
 
+  PhysicsSettings* getPhysicsSettings();
+  int localNetId() const; // id of the biker for the network part (must be 0, 1, 2 or 3)
+  void setLocalNetId(int i_value);
+
   void setFinished(bool i_value, int i_finishTime);
   void setDead(bool i_value, int i_deadTime);
   bool isFinished() const;
@@ -165,6 +170,8 @@ class Biker {
   virtual bool  isWheelSpinning();
   virtual Vector2f getWheelSpinPoint();
   virtual Vector2f getWheelSpinDir();
+  virtual float getRearWheelVelocity();
+  virtual float getFrontWheelVelocity();
   virtual void initToPosition(Vector2f i_position, DriveDir i_direction, Vector2f i_gravity);
   virtual void resetAutoDisabler(); /* a player can have a disabler when nothing append */
 
@@ -191,12 +198,14 @@ class Biker {
 
   void setInterpolation(bool bValue);
 
-	std::vector<Vector2f> &CollisionPoints();
+  std::vector<Vector2f> &CollisionPoints();
+
+  float changeDirPer() const;
 
  protected:
   BikerTheme* m_bikerTheme;
   bool m_playSound;
-  BikeState m_bikeState;
+  BikeState* m_bikeState;
   EngineSoundSimulator* m_EngineSound;
   bool m_finished;
   int m_finishTime;
@@ -208,6 +217,8 @@ class Biker {
   std::vector<Entity *> m_entitiesTouching;
   std::vector<Zone *>   m_zonesTouching;
 
+  PhysicsSettings* m_physicsSettings;
+
   /* Wheels spinning dirt up... muzakka! :D */
   bool m_bWheelSpin;                  /* Do it captain */
   Vector2f m_WheelSpinPoint,m_WheelSpinDir; /* Where and how much */
@@ -216,9 +227,14 @@ class Biker {
   TColor m_uglyColorFilter;
   bool m_doInterpolation;
 
-	/* collision points */
-	void cleanCollisionPoints();
-	std::vector<Vector2f> m_collisionPoints;
+  /* collision points */
+  void cleanCollisionPoints();
+  std::vector<Vector2f> m_collisionPoints;
+
+  /* changedir anim */
+  float m_changeDirPer; // between 0.0 and 1.0, give the % of the change dir done (only for graphisms)
+
+  int m_localNetId;
 };
 
 class OnBikerHooks {
