@@ -19,21 +19,53 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 =============================================================================*/
 
 #include "BikeGhost.h"
-#include "Replay.h"
-#include "GameText.h"
-#include "helpers/Text.h"
-#include "GameEvents.h"
+#include "../Replay.h"
+#include "../GameText.h"
+#include "../helpers/Text.h"
+#include "../GameEvents.h"
 #include "Level.h"
 
 #define INTERPOLATION_MAXIMUM_TIME  300
 #define INTERPOLATION_MAXIMUM_SPACE 5.0
 
-Ghost::Ghost(std::string i_replayFile, bool i_isActiv,
+Ghost::Ghost(PhysicsSettings* i_physicsSettings,
 	     Theme *i_theme, BikerTheme* i_bikerTheme,
 	     const TColor& i_colorFilter,
-	     const TColor& i_uglyColorFilter) : Biker(i_theme, i_bikerTheme,
-						      i_colorFilter,
-						      i_uglyColorFilter) {
+	     const TColor& i_uglyColorFilter)
+: Biker(i_physicsSettings, i_theme, i_bikerTheme, i_colorFilter, i_uglyColorFilter) {
+  m_diffToPlayer = 0.0;
+  m_reference = false;
+}
+
+Ghost::~Ghost() {
+}
+
+float Ghost::diffToPlayer() const {
+  return m_diffToPlayer;
+}
+
+bool Ghost::getRenderBikeFront() {
+  return true;
+}
+
+void Ghost::setReference(bool i_value) {
+  m_reference = i_value;
+}
+
+bool Ghost::isReference() const {
+  return m_reference;
+}
+
+void Ghost::setInfo(const std::string& i_info) {
+  m_info = i_info;
+}
+
+FileGhost::FileGhost(std::string i_replayFile, PhysicsSettings* i_physicsSettings,
+	     bool i_isActiv,
+	     Theme *i_theme, BikerTheme* i_bikerTheme,
+	     const TColor& i_colorFilter,
+	     const TColor& i_uglyColorFilter)
+  : Ghost (i_physicsSettings, i_theme, i_bikerTheme, i_colorFilter, i_uglyColorFilter) {
   std::string v_levelId;
   std::string v_playerName;
 
@@ -43,31 +75,28 @@ Ghost::Ghost(std::string i_replayFile, bool i_isActiv,
   // 4 states for cubical interpolation
 
   // 50% of states are before the time T, 50% are after
-  m_ghostBikeStates.push_back(new BikeState());
-  m_ghostBikeStates.push_back(new BikeState());
-  m_ghostBikeStates.push_back(new BikeState());
-  m_ghostBikeStates.push_back(new BikeState());
+  m_ghostBikeStates.push_back(new BikeState(m_physicsSettings));
+  m_ghostBikeStates.push_back(new BikeState(m_physicsSettings));
+  m_ghostBikeStates.push_back(new BikeState(m_physicsSettings));
+  m_ghostBikeStates.push_back(new BikeState(m_physicsSettings));
 
   for(unsigned int i=0; i<m_ghostBikeStates.size(); i++) {
-    m_replay->peekState(m_ghostBikeStates[i]);
+    m_replay->peekState(m_ghostBikeStates[i], m_physicsSettings);
   }
-  m_bikeState = *(m_ghostBikeStates[0]);
+  *m_bikeState = *(m_ghostBikeStates[0]); // copy
 
-  m_diffToPlayer = 0.0;
-
-  m_info = "";
   m_isActiv = i_isActiv;
   m_linearVelocity = 0.0;
   m_teleportationOccured = false;
 }
 
-Ghost::~Ghost() {
+FileGhost::~FileGhost() {
   for(unsigned int i=0; i<m_ghostBikeStates.size(); i++) {
     delete m_ghostBikeStates[i];
   }
 }
 
-void Ghost::execReplayEvents(int i_time, MotoGame *i_motogame) {
+void FileGhost::execReplayEvents(int i_time, Scene *i_motogame) {
   std::vector<RecordedGameEvent *> *v_replayEvents;
   v_replayEvents = m_replay->getEvents();
   
@@ -98,11 +127,7 @@ void Ghost::execReplayEvents(int i_time, MotoGame *i_motogame) {
   }
 }
 
-void Ghost::setInfo(std::string i_info) {
-  m_info = i_info;
-}
-
-std::string Ghost::getDescription() const {
+std::string FileGhost::getDescription() const {
   char c_tmp[1024];
 
   snprintf(c_tmp, 1024,
@@ -115,7 +140,11 @@ std::string Ghost::getDescription() const {
     "\n(" + formatTime(m_replay->getFinishTime()) + ")";
 }
 
-std::string Ghost::getQuickDescription() const {
+std::string FileGhost::getVeryQuickDescription() const {
+  return m_replay->getPlayerName();
+}
+
+std::string FileGhost::getQuickDescription() const {
   char c_tmp[1024];
 
   snprintf(c_tmp, 1024,
@@ -125,11 +154,11 @@ std::string Ghost::getQuickDescription() const {
   return std::string(c_tmp);
 }
 
-std::string Ghost::playerName() {
+std::string FileGhost::playerName() {
   return m_replay->getPlayerName();
 }
 
-void Ghost::initLastToTakeEntities(Level* i_level) {
+void FileGhost::initLastToTakeEntities(Level* i_level) {
   std::vector<RecordedGameEvent *> *v_replayEvents;
   v_replayEvents = m_replay->getEvents();
     
@@ -138,7 +167,7 @@ void Ghost::initLastToTakeEntities(Level* i_level) {
 
   /* Start looking for events */
   for(unsigned int i=0; i<v_replayEvents->size(); i++) {
-    MotoGameEvent *v_event = (*v_replayEvents)[i]->Event;
+    SceneEvent *v_event = (*v_replayEvents)[i]->Event;
       
     if(v_event->getType() == GAME_EVENT_ENTITY_DESTROYED) {
       if(i_level->getEntityById(((MGE_EntityDestroyed*)v_event)->EntityId())->IsToTake()) {
@@ -149,11 +178,7 @@ void Ghost::initLastToTakeEntities(Level* i_level) {
   }
 }
 
-float Ghost::diffToPlayer() const {
-  return m_diffToPlayer;
-}
-
-void Ghost::updateDiffToPlayer(std::vector<float> &i_lastToTakeEntities) {
+void FileGhost::updateDiffToPlayer(std::vector<float> &i_lastToTakeEntities) {
   /* no strawberry, no update */
   if(i_lastToTakeEntities.size() == 0) {
     return;
@@ -168,16 +193,16 @@ void Ghost::updateDiffToPlayer(std::vector<float> &i_lastToTakeEntities) {
                  - m_lastToTakeEntities[i_lastToTakeEntities.size()-1];
 }
 
-void Ghost::initToPosition(Vector2f i_position, DriveDir i_direction, Vector2f i_gravity) {
+void FileGhost::initToPosition(Vector2f i_position, DriveDir i_direction, Vector2f i_gravity) {
   m_teleportationOccured = true;
   m_linearVelocity = 0.0;
-  printf("init to position\n");
 }
 
-void Ghost::updateToTime(int i_time, int i_timeStep,
+void FileGhost::updateToTime(int i_time, int i_timeStep,
 			 CollisionSystem *i_collisionSystem, Vector2f i_gravity,
-			 MotoGame *i_motogame) {
+			 Scene *i_motogame) {
   Biker::updateToTime(i_time, i_timeStep, i_collisionSystem, i_gravity, i_motogame);
+  DriveDir v_previousDir = m_bikeState->Dir;
 
   /* back in the past */
   // m_ghostBikeStates.size()/2-1 : it's the more recent frame in the past
@@ -185,7 +210,7 @@ void Ghost::updateToTime(int i_time, int i_timeStep,
     m_replay->fastrewind(m_ghostBikeStates[m_ghostBikeStates.size()/2-1]->GameTime - i_time, 1);
 
     for(unsigned int i=0; i<m_ghostBikeStates.size(); i++) {
-      m_replay->peekState(m_ghostBikeStates[i]);
+      m_replay->peekState(m_ghostBikeStates[i], m_physicsSettings);
     }
 
     m_finished = false;
@@ -194,15 +219,17 @@ void Ghost::updateToTime(int i_time, int i_timeStep,
 
   /* stop the motor at the end */
   if(m_replay->endOfFile() && m_ghostBikeStates[m_ghostBikeStates.size()-1]->GameTime <= i_time) { // end of file, and interpolation is finished (before last future frame is in the feature)
-    m_bikeState.fBikeEngineRPM = 0.0;
-    m_bikeState = *(m_ghostBikeStates[m_ghostBikeStates.size()-1]); // put the last frame
+    m_bikeState->fBikeEngineRPM = 0.0;
+    m_replay->peekState(m_ghostBikeStates[m_ghostBikeStates.size()-1], m_physicsSettings); // take the last frame
+    *m_bikeState = *(m_ghostBikeStates[m_ghostBikeStates.size()-1]); // copy last frame
     m_linearVelocity = 0.0;
 
     if(m_replay->didFinish()) {
       m_finished   = true;
       m_finishTime = m_replay->getFinishTime();
     } else {
-      m_dead = true;
+      m_dead     = true;
+      m_deadTime = m_bikeState->GameTime;
     }
   } else {
     BikeState* v_state;
@@ -228,15 +255,15 @@ void Ghost::updateToTime(int i_time, int i_timeStep,
 	  *(m_ghostBikeStates[m_ghostBikeStates.size()-1]) = *(m_ghostBikeStates[m_ghostBikeStates.size()-1-1]);
 	} else {
 	  // read the replay
-	  m_replay->loadState(m_ghostBikeStates[m_ghostBikeStates.size()-1]);
-	  m_replay->loadState(&m_bikeState);
+	  m_replay->loadState(m_ghostBikeStates[m_ghostBikeStates.size()-1], m_physicsSettings);
+	  m_replay->loadState(m_bikeState, m_physicsSettings);
 	}
 
       } while(m_ghostBikeStates[m_ghostBikeStates.size()/2]->GameTime < i_time &&
 	      m_ghostBikeStates[m_ghostBikeStates.size()/2]->GameTime > m_ghostBikeStates[m_ghostBikeStates.size()/2-1]->GameTime);
 
 
-      m_bikeState = *(m_ghostBikeStates[m_ghostBikeStates.size()/2-1]);
+      *m_bikeState = *(m_ghostBikeStates[m_ghostBikeStates.size()/2-1]); // copy
 
       v_beforePos  = m_ghostBikeStates[m_ghostBikeStates.size()/2-1]->CenterP;
       v_beforeTime = m_ghostBikeStates[m_ghostBikeStates.size()/2-1]->GameTime;
@@ -271,34 +298,81 @@ void Ghost::updateToTime(int i_time, int i_timeStep,
 	  v_interpolation_value = (i_time - m_ghostBikeStates[m_ghostBikeStates.size()/2-1]->GameTime) 
 	    /((float)(m_ghostBikeStates[m_ghostBikeStates.size()/2]->GameTime - m_ghostBikeStates[m_ghostBikeStates.size()/2-1]->GameTime));
 
-	  BikeState::interpolateGameState(m_ghostBikeStates, &m_bikeState, v_interpolation_value);
+	  BikeState::interpolateGameState(m_ghostBikeStates, m_bikeState, v_interpolation_value);
 	}
       }
     }
   }
 
+  /* update change position */
+  if(v_previousDir != m_bikeState->Dir) {
+    m_changeDirPer = 0.0;
+  }
+
   if(m_isActiv) {
     m_teleportationOccured = false;
     execReplayEvents(i_time, i_motogame);
   }
 }
 
-int Ghost::getFinishTime() {
+int FileGhost::getFinishTime() {
   return m_replay->getFinishTime();
 }
 
-std::string Ghost::levelId() {
+std::string FileGhost::levelId() {
    return m_replay->getLevelId();
 }
 
-bool Ghost::getRenderBikeFront() {
-  return true;
-}
-
-float Ghost::getBikeLinearVel() {
+float FileGhost::getBikeLinearVel() {
   return m_linearVelocity;
 }
 
-float Ghost::getBikeEngineSpeed() {
+float FileGhost::getBikeEngineSpeed() {
   return 0.0; /* unable to know it */
 }
+
+double FileGhost::getAngle() {
+  return 0.0; /* to do */
+}
+
+
+NetGhost::NetGhost(PhysicsSettings* i_physicsSettings,
+		   Theme *i_theme, BikerTheme* i_bikerTheme,
+		   const TColor& i_colorFilter,
+		   const TColor& i_uglyColorFilter) 
+  : Ghost(i_physicsSettings, i_theme, i_bikerTheme, i_colorFilter, i_uglyColorFilter) {
+}
+
+NetGhost::~NetGhost() {
+}
+
+void NetGhost::updateDiffToPlayer(std::vector<float> &i_lastToTakeEntities) {
+}
+
+std::string NetGhost::getVeryQuickDescription() const {
+  return m_info;
+}
+
+std::string NetGhost::getQuickDescription() const {
+  char c_tmp[256];
+  snprintf(c_tmp, 256, GAMETEXT_WEBGHOSTOF, m_info.c_str()); 
+  return std::string(c_tmp);
+}
+
+std::string NetGhost::getDescription() const {
+  char c_tmp[256];
+  snprintf(c_tmp, 256, GAMETEXT_WEBGHOSTOF, m_info.c_str()); 
+  return std::string(c_tmp);
+}
+
+float NetGhost::getBikeEngineSpeed() {
+  return 0.0; /* unable to know it */
+}
+
+float NetGhost::getBikeLinearVel() {
+  return 0.0; /* to do */
+}
+
+double NetGhost::getAngle() {
+  return 0.0; /* to do */
+}
