@@ -23,8 +23,9 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #include "Bike.h"
 #include "BikeGhost.h"
-#include "SomersaultCounter.h"
+#include "../SomersaultCounter.h"
 #include "BikeController.h"
+#include "../include/xm_ode.h"
 
 class ExternalForce {
 public:
@@ -40,24 +41,26 @@ private:
 };
 
 
-class ReplayBiker : public Ghost {
+class ReplayBiker : public FileGhost {
  public:
-  ReplayBiker(std::string i_replayFile, Theme *i_theme, BikerTheme* i_bikerTheme);
+  ReplayBiker(std::string i_replayFile, PhysicsSettings* i_physicsSettings,
+	      Theme *i_theme, BikerTheme* i_bikerTheme);
   std::string getQuickDescription() const;
+  std::string getVeryQuickDescription() const;
  private:
 };
 
 class PlayerBiker : public Biker {
  public:
-  PlayerBiker(Vector2f i_position, DriveDir i_direction, Vector2f i_gravity, Theme *i_theme, BikerTheme* i_bikerTheme, const TColor& i_filterColor, const TColor& i_filterUglyColor);
-  ~PlayerBiker();
+  PlayerBiker(PhysicsSettings* i_physicsSettings, Vector2f i_position, DriveDir i_direction, Vector2f i_gravity, Theme *i_theme, BikerTheme* i_bikerTheme, const TColor& i_filterColor, const TColor& i_filterUglyColor);
+  virtual ~PlayerBiker();
 
   void updateToTime(int i_time, int i_timeStep,
-		    CollisionSystem *i_collisionSystem, Vector2f i_gravity,
-		    MotoGame *i_motogame);
+		    CollisionSystem *i_collisionSystem, Vector2f i_gravity, Scene *i_motogame);
   void initToPosition(Vector2f i_position, DriveDir i_direction, Vector2f i_gravity);
   BikeController* getControler();
 
+  std::string getVeryQuickDescription() const;
   std::string getQuickDescription() const;
   std::string getDescription() const;
   void setBodyDetach(bool state);
@@ -71,6 +74,10 @@ class PlayerBiker : public Biker {
   bool isSqueeking();
   float howMuchSqueek();
 
+  virtual float getRearWheelVelocity();
+  virtual float getFrontWheelVelocity();
+  virtual double getAngle();
+
  private:
   SomersaultCounter m_somersaultCounter;
   BikeController m_BikeC;
