@@ -20,15 +20,20 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #include <algorithm>
 #include "Block.h"
-#include "Collision.h"
-#include "PhysSettings.h"
-#include "helpers/Log.h"
-#include "VFileIO.h"
-#include "VXml.h"
-#include "BSP.h"
+#include "../Collision.h"
+#include "../PhysSettings.h"
+#include "../helpers/Log.h"
+#include "../VFileIO.h"
+#include "../VXml.h"
+#include "../BSP.h"
+#include "../chipmunk/chipmunk.h"
+#include "ChipmunkWorld.h"
+#include "PhysicsSettings.h"
 
 #define XM_DEFAULT_BLOCK_TEXTURE "default"
-#define XM_DEFAULT_PHYS_BLOCK_GRIP 20
+#define XM_DEFAULT_PHYS_BLOCK_MASS 30.0
+#define XM_DEFAULT_PHYS_BLOCK_FRICTION 0.5
+#define XM_DEFAULT_PHYS_BLOCK_ELASTICITY 0.0
 
 /* Vertex */
 ConvexBlockVertex::ConvexBlockVertex(const Vector2f& i_position, const Vector2f& i_texturePosition) {
@@ -75,8 +80,12 @@ Block::Block(std::string i_id) {
   m_textureScale     = 1.0;
   m_background       = false;
   m_dynamic          = false;
+  m_physics          = false;
   m_isLayer          = false;
-  m_grip             = XM_DEFAULT_PHYS_BLOCK_GRIP;
+  m_gripPer20        = 20.0;
+  m_mass             = XM_DEFAULT_PHYS_BLOCK_MASS;
+  m_friction         = XM_DEFAULT_PHYS_BLOCK_FRICTION;
+  m_elasticity       = XM_DEFAULT_PHYS_BLOCK_ELASTICITY;
   m_dynamicPosition  = m_initialPosition;
   m_dynamicRotation  = m_initialRotation;
   m_dynamicRotationCenter = Vector2f(0.0, 0.0);
@@ -87,6 +96,11 @@ Block::Block(std::string i_id) {
   m_layer            = -1;
   m_edgeDrawMethod   = angle;
   m_edgeAngle        = DEFAULT_EDGE_ANGLE;
+  mBody              = NULL;
+  m_shape            = NULL;
+  m_collisionElement = NULL;
+  m_collisionMethod  = None;
+  m_collisionRadius  = 0.0f;
 }
 
 Block::~Block() {
@@ -121,17 +135,20 @@ Vector2f Block::DynamicPositionCenter() const {
   return m_dynamicPositionCenter;
 }
 
-float Block::DynamicRotation() const {
-  return m_dynamicRotation;
-}
-
-void Block::updateCollisionLines() {
+void Block::updateCollisionLines(bool setDirty, bool forceUpdate) {
   bool manageCollisions = (m_collisionLines.size() != 0);
   /* Ignore background blocks */
   if(isDynamic() == false
      || manageCollisions == false)
     return;
 
+  if(forceUpdate == false && isBackground() == true)
+    return;
+
+  if(setDirty == true){
+    m_isBBoxDirty = true;
+  }
+
   /* Build rotation matrix for block */
   float fR[4]; 
   fR[0] =  cosf(DynamicRotation());
@@ -140,12 +157,11 @@ void Block::updateCollisionLines() {
   fR[3] =  cosf(DynamicRotation());
   unsigned int z = 0;
 
-  m_isBBoxDirty = true;
-
-  for(unsigned int j=0; j<Vertices().size(); j++) {            
+  for(unsigned int j=0; j<Vertices().size(); j++) {
+    // FIXME::aren't each vertice transform twice ??
     unsigned int jnext = j==Vertices().size()-1? 0 : j+1;
-    BlockVertex *pVertex1 = Vertices()[j];          
-    BlockVertex *pVertex2 = Vertices()[jnext];          
+    BlockVertex *pVertex1 = Vertices()[j];
+    BlockVertex *pVertex2 = Vertices()[jnext];
 
 
     /* Transform vertices */
@@ -168,32 +184,129 @@ void Block::updateCollisionLines() {
   }
 }
 
+void Block::translateCollisionLines(float x, float y)
+{
+  bool manageCollisions = (m_collisionLines.size() != 0);
+
+  if(isDynamic() == false
+     || manageCollisions == false
+     || isBackground() == true)
+    return;
+
+  std::vector<Line*>::iterator it = m_collisionLines.begin();
+  while(it != m_collisionLines.end()) {
+    (*it)->x1 += x;
+    (*it)->x2 += x;
+    (*it)->y1 += y;
+    (*it)->y2 += y;
+
+    it++;
+  }
+}
+
 void Block::setDynamicPosition(const Vector2f& i_dynamicPosition) {
+  Vector2f diff = i_dynamicPosition - m_dynamicPosition;
   m_dynamicPosition = i_dynamicPosition;
-  updateCollisionLines();
+
+  m_BCircle.translate(diff.x, diff.y);
+  translateCollisionLines(diff.x, diff.y);
+}
+
+void Block::setPhysicsPosition(float ix, float iy) {
+  mBody->p.x = ix * CHIP_SCALE_RATIO;
+  mBody->p.y = iy * CHIP_SCALE_RATIO;
 }
 
 void Block::setDynamicPositionAccordingToCenter(const Vector2f& i_dynamicPosition) {
-  m_dynamicPosition = i_dynamicPosition - m_dynamicPositionCenter;
-  updateCollisionLines();
+  Vector2f newPos = i_dynamicPosition - m_dynamicPositionCenter;
+  Vector2f diff = newPos - m_dynamicPosition;
+  m_dynamicPosition = newPos;
+
+  m_BCircle.translate(diff.x, diff.y);
+  translateCollisionLines(diff.x, diff.y);
 }
 
-void Block::setDynamicRotation(float i_dynamicRotation) {
+bool Block::setDynamicRotation(float i_dynamicRotation) {
   m_dynamicRotation = i_dynamicRotation;
-  updateCollisionLines();
+
+  // if the center of rotation is the center of the bounding circle,
+  // we don't have to set the bounding box to dirty
+  if(DynamicRotationCenter() + DynamicPosition() == m_BCircle.getCenter()) {
+    updateCollisionLines(false, false);
+    return false;
+  } else {
+    updateCollisionLines(true, true);
+    return true;
+  }
 }
 
-int Block::loadToPlay(CollisionSystem& io_collisionSystem) {
+void Block::setCollisionMethod(CollisionMethod method)
+{
+  m_collisionMethod = method;
+}
+
+void Block::setCollisionRadius(float radius)
+{
+  m_collisionRadius = radius;
+}
+
+void Block::translate(float x, float y)
+{
+  m_dynamicPosition.x += x;
+  m_dynamicPosition.y += y;
+  // dont recalculate the BoundingCircle
+  m_BCircle.translate(x, y);
+  translateCollisionLines(x, y);
+}
+
+void Block::updatePhysics(int timeStep, CollisionSystem* io_collisionSystem) {
+  if(isPhysics()) {
+    // move block according to chipmunk, only if they have moved
+    bool moved = false;
+
+    Vector2f newPos = Vector2f(((mBody->p.x ) / CHIP_SCALE_RATIO) , (mBody->p.y ) / CHIP_SCALE_RATIO);
+    if(DynamicPosition() != newPos) {
+      setDynamicPosition(newPos);
+      moved = true;
+    }
+
+    if(DynamicRotation() != mBody->a) {
+      setDynamicRotation(mBody->a);
+      moved = true;
+    }
+
+    if(moved == true){
+      // inform collision system that there has been a change
+      io_collisionSystem->moveDynBlock(this);
+    }
+    
+    cpBodyResetForces(mBody);
+  }
+}
+
+int Block::loadToPlay(CollisionSystem* io_collisionSystem, ChipmunkWorld* i_chipmunkWorld, PhysicsSettings* i_physicsSettings) {
 
   m_dynamicPosition       = m_initialPosition;
   m_dynamicRotation       = m_initialRotation;
   m_dynamicRotationCenter = Vector2f(0.0, 0.0);
   m_dynamicPositionCenter = Vector2f(0.0, 0.0);
+  float tx = 0;
+  float ty = 0;
+
 
   /* Do the "convexifying" the BSP-way. It might be overkill, but we'll
      probably appreciate it when the input data is very complex. It'll also 
      let us handle crossing edges, and other kinds of weird input. */
   BSP v_BSPTree;
+  cpBody*  myBody  = NULL;
+  cpVect*  myVerts = NULL;
+  
+  if(i_chipmunkWorld != NULL) {
+    if (isPhysics()) {
+      unsigned int size = 2*sizeof(cpVect)*Vertices().size();
+      myVerts = (cpVect*)malloc(size);
+    }
+  }
 
   /* Define edges */
   for(unsigned int i=0; i<Vertices().size(); i++) {
@@ -204,37 +317,83 @@ int Block::loadToPlay(CollisionSystem& io_collisionSystem) {
     /* add static lines */
     if(isBackground() == false && isDynamic() == false && m_layer == -1) {
       /* Add line to collision handler */
-      io_collisionSystem.defineLine(DynamicPosition().x + Vertices()[i]->Position().x,
-				    DynamicPosition().y + Vertices()[i]->Position().y,
-				    DynamicPosition().x + Vertices()[inext]->Position().x,
-				    DynamicPosition().y + Vertices()[inext]->Position().y,
-				    Grip());
-    }      
+      io_collisionSystem->defineLine(DynamicPosition().x + Vertices()[i]->Position().x,
+				     DynamicPosition().y + Vertices()[i]->Position().y,
+				     DynamicPosition().x + Vertices()[inext]->Position().x,
+				     DynamicPosition().y + Vertices()[inext]->Position().y,
+				     GripPer20() * i_physicsSettings->BikeWheelBlockGrip() / 20.0);
+      
+
+      if(i_chipmunkWorld != NULL) {
+	// Create/duplicate terrain for chipmunks objects to use
+	cpVect a = cpv( (DynamicPosition().x + Vertices()[i]->Position().x) * CHIP_SCALE_RATIO,
+			(DynamicPosition().y + Vertices()[i]->Position().y) * CHIP_SCALE_RATIO);
+	cpVect b = cpv( (DynamicPosition().x + Vertices()[inext]->Position().x) * CHIP_SCALE_RATIO,
+			(DynamicPosition().y + Vertices()[inext]->Position().y) * CHIP_SCALE_RATIO);
+
+	cpShape *seg = cpSegmentShapeNew(i_chipmunkWorld->getBody(), a, b, 0.0f);
+	seg->group = 0;
+	seg->u = m_friction;
+	seg->e = m_elasticity;
+	cpSpaceAddStaticShape(i_chipmunkWorld->getSpace(), seg);
+      }
+    }
+
     /* add dynamic lines */
     if(isLayer() == false && isDynamic()) {
       /* Define collision lines */
       Line *v_line = new Line;
       v_line->x1 = v_line->y1 = v_line->x2 = v_line->y2 = 0.0f;
-      v_line->fGrip = m_grip;
+      v_line->fGrip = GripPer20() * i_physicsSettings->BikeWheelBlockGrip() / 20.0;
       m_collisionLines.push_back(v_line);
     }
 
-    /* Add line to BSP generator */
-    v_BSPTree.addLineDefinition(Vertices()[i]->Position(), Vertices()[inext]->Position());
+    if(isPhysics()) {
+      // collect vertice count to find middle
+      tx += Vertices()[i]->Position().x;      
+      ty += Vertices()[i]->Position().y;      
+    } else {
+      v_BSPTree.addLineDefinition(Vertices()[i]->Position(), Vertices()[inext]->Position());
+    }
+  }
+
+  if(isPhysics()) {
+    // calculate midpoint
+    float mdx = tx / Vertices().size();
+    float mdy = ty / Vertices().size();
+
+    // for physics objects we reorient around a center of gravity
+    // determined by the vertices
+    for(unsigned int i=0; i<Vertices().size(); i++) {
+      Vertices()[i]->setPosition(Vector2f(Vertices()[i]->Position().x - mdx, Vertices()[i]->Position().y - mdy));
+    }
+
+    // then BSP-ify them
+    for(unsigned int i=0; i<Vertices().size(); i++) {
+      unsigned int inext = i+1;
+      if(inext == Vertices().size()) inext=0;
+
+      /* Add line to BSP generator */ 
+      v_BSPTree.addLineDefinition(Vertices()[i]->Position(), Vertices()[inext]->Position());
+    }
+
+    // modify the object coords with the midpoint
+    m_dynamicPosition.x += mdx;
+    m_dynamicPosition.y += mdy;
   }
 
   /* define dynamic block in the collision system */
   if(isLayer() == false && isDynamic()) {
-    updateCollisionLines();
-    io_collisionSystem.addDynBlock(this);
+    updateCollisionLines(true, true);
+    m_collisionElement = io_collisionSystem->addDynBlock(this);
   }
 
   if(isDynamic() == false && m_layer == -1){
-    io_collisionSystem.addStaticBlock(this, isLayer());
+    io_collisionSystem->addStaticBlock(this, isLayer());
   }
 
   if(isLayer() == true && m_layer != -1) {
-    io_collisionSystem.addBlockInLayer(this, m_layer);
+    io_collisionSystem->addBlockInLayer(this, m_layer);
   }
 
   /* Compute */
@@ -244,23 +403,92 @@ int Block::loadToPlay(CollisionSystem& io_collisionSystem) {
   for(unsigned int i=0; i<v_BSPPolys.size(); i++) {
     addPoly(v_BSPPolys[i], io_collisionSystem);
   }
-  
-  updateCollisionLines();
+
+  if(i_chipmunkWorld != NULL) {
+    if(isPhysics()) {
+      if(getCollisionMethod() == Circle && getCollisionRadius() != 0.0f) {
+	float radius = getCollisionRadius();
+
+	cpFloat circle_moment = cpMomentForCircle(m_mass, radius, 0.0f, cpvzero);
+	myBody = cpBodyNew(m_mass, circle_moment);
+	myBody->p = cpv((DynamicPosition().x * CHIP_SCALE_RATIO),
+			(DynamicPosition().y * CHIP_SCALE_RATIO));
+	cpSpaceAddBody(i_chipmunkWorld->getSpace(), myBody);
+	mBody = myBody;
+
+	m_shape = cpCircleShapeNew(mBody, radius * CHIP_SCALE_RATIO, cpvzero);
+	m_shape->u = m_friction;
+	m_shape->e = m_elasticity;
+	if(isBackground() == true) {
+	  m_shape->group = 1;
+	}
+
+	cpSpaceAddShape(i_chipmunkWorld->getSpace(), m_shape);
+      } else {
+	// create body vertices for chipmunk constructors
+	for(unsigned int i=0; i<Vertices().size(); i++) {
+	  cpVect ma = cpv(Vertices()[i]->Position().x * CHIP_SCALE_RATIO,
+			  Vertices()[i]->Position().y * CHIP_SCALE_RATIO);
+	  myVerts[i] =  ma;
+	}
+
+	// Create body to attach shapes to
+	cpFloat bMoment = cpMomentForPoly(m_mass,Vertices().size(), myVerts, cpvzero); 
+	free(myVerts);
+
+	// create body 
+	myBody = cpBodyNew(m_mass, bMoment);
+	myBody->p = cpv((DynamicPosition().x * CHIP_SCALE_RATIO),
+			(DynamicPosition().y * CHIP_SCALE_RATIO));
+	cpSpaceAddBody(i_chipmunkWorld->getSpace(), myBody);
+	mBody = myBody;
+
+	// go through calculated BSP polys, adding one or more shape to the
+	for(unsigned int i=0; i<v_BSPPolys.size(); i++) {
+	  unsigned int size = 2*sizeof(cpVect)*v_BSPPolys[i]->Vertices().size();
+	  myVerts = (cpVect*)malloc(size);
+
+	  // translate for chipmunk
+	  for(unsigned int j=0; j<v_BSPPolys[i]->Vertices().size(); j++) {
+	    cpVect ma = cpv(v_BSPPolys[i]->Vertices()[j].x * CHIP_SCALE_RATIO,
+			    v_BSPPolys[i]->Vertices()[j].y * CHIP_SCALE_RATIO);
+	    myVerts[j] =  ma;
+	  }
+
+	  // collision shape
+	  m_shape = cpPolyShapeNew(myBody, v_BSPPolys[i]->Vertices().size(), myVerts, cpvzero);
+	  m_shape->u = m_friction;
+	  m_shape->e = m_elasticity;
+	  if (isBackground()) {
+	    m_shape->group = 1;
+	  }
+
+	  cpSpaceAddShape(i_chipmunkWorld->getSpace(), m_shape);
+
+	  // free the temporary vertices array
+	  free(myVerts);
+	}
+      }
+    }
+  }
+
+  updateCollisionLines(true);
 
   if(v_BSPTree.getNumErrors() > 0) {
-    Logger::Log("Error due to the block %s", Id().c_str());
+    LogError("Error due to the block %s", Id().c_str());
   }
 
   return v_BSPTree.getNumErrors();  
 }
 
-void Block::addPoly(BSPPoly* i_poly, CollisionSystem& io_collisionSystem) {
+void Block::addPoly(BSPPoly* i_poly, CollisionSystem* io_collisionSystem) {
   ConvexBlock *v_block = new ConvexBlock(this);
-  
+  float scale = TextureScale();
+
   for(unsigned int i=0; i<i_poly->Vertices().size(); i++) {
     v_block->addVertex(i_poly->Vertices()[i],
-		       Vector2f((InitialPosition().x + i_poly->Vertices()[i].x) * 0.25,
-                                (InitialPosition().y + i_poly->Vertices()[i].y) * 0.25));
+		       Vector2f((InitialPosition().x + i_poly->Vertices()[i].x) * 0.25 * scale,
+                                (InitialPosition().y + i_poly->Vertices()[i].y) * 0.25 * scale));
   }
   m_convexBlocks.push_back(v_block);
 }
@@ -277,16 +505,8 @@ void Block::unloadToPlay() {
   m_collisionLines.clear();
 }
 
-std::string Block::Id() const {
-  return m_id;
-}
-
-bool Block::isBackground() const {
-  return m_background;
-}
-
-bool Block::isDynamic() const {
-  return m_dynamic;
+bool Block::isPhysics() const {
+  return m_physics;
 }
 
 bool Block::isLayer() const {
@@ -302,12 +522,20 @@ Vector2f Block::InitialPosition() const {
   return m_initialPosition;
 }
 
-float Block::Grip() const {
-  return m_grip;
+float Block::GripPer20() const {
+  return m_gripPer20;
 }
 
-std::string Block::Texture() const {
-  return m_texture;
+float Block::Mass() const {
+  return m_mass;
+}
+
+float Block::Friction() const {
+  return m_friction;
+}
+
+float Block::Elasticity() const {
+  return m_elasticity;
 }
 
 float Block::TextureScale() const {
@@ -334,37 +562,68 @@ void Block::setDynamic(bool i_dynamic) {
   m_dynamic = i_dynamic;
 }
 
+void Block::setPhysics(bool i_physics) {
+  m_physics = i_physics;
+
+  if(m_physics) {
+    // we're.. kinda.. dynamic?
+    //
+    setDynamic(true);
+  }
+}
+
 void Block::setIsLayer(bool i_isLayer) {
   m_isLayer = i_isLayer;
 }
 
-void Block::setGrip(float i_grip) {
-  m_grip = i_grip;
+void Block::setGripPer20(float i_gripPer20) {
+  m_gripPer20 = i_gripPer20;
+}
+
+void Block::setMass(float i_mass) {
+  m_mass = i_mass;
+}
+
+void Block::setFriction(float i_friction) {
+  m_friction = i_friction;
+}
+
+void Block::setElasticity(float i_elasticity) {
+  m_elasticity = i_elasticity;
 }
 
 AABB& Block::getAABB()
 {
-  if(m_isBBoxDirty == true){
-    m_BBox.reset();
-
-    if(isDynamic() == true){
+  if(isDynamic() == true) {
+    // will be dirty only once too.
+    // after, the bounding circle is translated and rotated.
+    if(m_isBBoxDirty == true){
+      m_BCircle.reset();
+      //updateCollisionLines(true);
       for(unsigned int i=0; i<m_collisionLines.size(); i++){
 	Line* pLine = m_collisionLines[i];
 	// add only the first point because the second
 	// point of the line n is the same as the first
 	// point of the line n+1
-	m_BBox.addPointToAABB2f(pLine->x1, pLine->y1);
+	m_BCircle.addPointToCircle(pLine->x1, pLine->y1);
       }
-    } else{
+      m_BCircle.calculateBoundingCircle();
+      m_isBBoxDirty = false;
+    }
+    return m_BCircle.getAABB();
+  } else {
+    // will be dirty only once for a static block
+    if(m_isBBoxDirty == true){
+      m_BBox.reset();
+
       for(unsigned int i=0; i<Vertices().size(); i++) {
 	m_BBox.addPointToAABB2f(DynamicPosition() + Vertices()[i]->Position());
       }
+
+      m_isBBoxDirty = false;
     }
-
-    m_isBBoxDirty = false;
+    return m_BBox;
   }
-
-  return m_BBox;
 }
 
 BlockVertex::BlockVertex(const Vector2f& i_position, const std::string& i_edgeEffect) {
@@ -375,10 +634,11 @@ BlockVertex::BlockVertex(const Vector2f& i_position, const std::string& i_edgeEf
 BlockVertex::~BlockVertex() {
 }
 
-Vector2f BlockVertex::Position() const {
-  return m_position;
+void BlockVertex::setPosition(const Vector2f& i_position) {
+  m_position = i_position;
 }
 
+
 void BlockVertex::setTexturePosition(const Vector2f& i_texturePosition) {
   m_texturePosition = i_texturePosition;
 }
@@ -387,69 +647,17 @@ void BlockVertex::setColor(const TColor &i_color) {
   m_color = i_color;
 }
 
-void Block::saveXml(FileHandle *i_pfh) {
-  FS::writeLineF(i_pfh,"\t<block id=\"%s\">", Id().c_str());
-    
-  std::string v_position;
-  std::ostringstream v_x, v_y;
-  v_x << InitialPosition().x;
-  v_y << InitialPosition().y;
-  
-  v_position = "\t\t<position x=\"" + v_x.str() + "\" y=\"" + v_y.str() + "\"";
-  
-  if(isBackground()) {
-    v_position = v_position + " background=\"true\"";
-  }
-  
-  if(isDynamic()) {
-    v_position = v_position + " dynamic=\"true\"";
+bool Block::isPhysics_readFromXml(XMLDocument* i_xmlSource, TiXmlElement *pElem) {
+  TiXmlElement* pPositionElem   = XML::findElement(*i_xmlSource, pElem, std::string("position"));
+
+  if(pPositionElem == NULL) {
+    return false;
   }
 
-  if(isLayer()) {
-    v_position = v_position + " islayer=\"true\"";
-
-    //    if(m_layer == -1){
-    //      throw exception, a backgroundlevel block must have a layer id
-    //    }
-  }
-  if(m_layer != -1){
-    std::ostringstream layer;
-    layer << m_layer;
-    v_position = v_position + " layerid=\"" + layer.str() + "\"";
-  }
-
-  v_position = v_position + " />";
-
-  FS::writeLineF(i_pfh, (char*) v_position.c_str());
-  
-  if(Grip() != DEFAULT_PHYS_WHEEL_GRIP) {
-    FS::writeLineF(i_pfh,"\t\t<physics grip=\"%f\"/>", Grip());
-  }
-
-  if(getEdgeDrawMethod() != angle && edgeAngle() != DEFAULT_EDGE_ANGLE){
-    std::string v_edgeDraw = edgeToString(getEdgeDrawMethod());
-    std::ostringstream v_angle;
-    v_angle << edgeAngle();
-    std::string v_edge = "\t\t<edges drawmethod=\"" + v_edgeDraw
-      + "\" angle=\"" + v_angle.str() + "\" />";
-    FS::writeLineF(i_pfh, (char*)v_edge.c_str());
-  }
-
-  FS::writeLineF(i_pfh,"\t\t<usetexture id=\"%s\"/>", Texture().c_str());
-  for(unsigned int j=0;j<Vertices().size();j++) {
-    if(Vertices()[j]->EdgeEffect() != "")
-      FS::writeLineF(i_pfh,"\t\t<vertex x=\"%f\" y=\"%f\" edge=\"%s\"/>",
-                           Vertices()[j]->Position().x,
-                           Vertices()[j]->Position().y,
-                           Vertices()[j]->EdgeEffect().c_str());
-    else
-      FS::writeLineF(i_pfh,"\t\t<vertex x=\"%f\" y=\"%f\"/>",
-                           Vertices()[j]->Position().x,
-                           Vertices()[j]->Position().y);
-  }
-  FS::writeLineF(i_pfh,"\t</block>");
+  return XML::getOption(pPositionElem,"physics","false") == "true";
 }
 
+
 Block* Block::readFromXml(XMLDocument* i_xmlSource, TiXmlElement *pElem) {
   
   Block *pBlock = new Block(XML::getOption(pElem, "id"));
@@ -459,6 +667,7 @@ Block* Block::readFromXml(XMLDocument* i_xmlSource, TiXmlElement *pElem) {
   TiXmlElement* pPositionElem   = XML::findElement(*i_xmlSource, pElem, std::string("position"));
   TiXmlElement* pPhysicsElem    = XML::findElement(*i_xmlSource, pElem, std::string("physics"));
   TiXmlElement* pEdgeElem       = XML::findElement(*i_xmlSource, pElem, std::string("edges"));
+  TiXmlElement* pColElem        = XML::findElement(*i_xmlSource, pElem, std::string("collision"));
 
 
   if(pUseTextureElem != NULL) {
@@ -474,16 +683,37 @@ Block* Block::readFromXml(XMLDocument* i_xmlSource, TiXmlElement *pElem) {
 
     pBlock->setBackground(XML::getOption(pPositionElem,"background","false") == "true");
     pBlock->setDynamic(XML::getOption(pPositionElem,"dynamic","false") == "true");
+    /* setDynamic must be done before setPhysics - physics implies dynamic */
+    pBlock->setPhysics(XML::getOption(pPositionElem,"physics","false") == "true");
     pBlock->setIsLayer(XML::getOption(pPositionElem,"islayer","false") == "true");
     pBlock->setLayer(atoi(XML::getOption(pPositionElem,"layerid","-1").c_str()));
  }
 
   if(pPhysicsElem != NULL) {
-    char str[5];
-    snprintf(str, 5, "%f", DEFAULT_PHYS_WHEEL_GRIP);
-    pBlock->setGrip(atof(XML::getOption(pPhysicsElem, "grip", str).c_str()));
+    char str[16];
+
+    pBlock->setGripPer20(atof(XML::getOption(pPhysicsElem, "grip", "20.0").c_str()));
+
+    snprintf(str, 16, "%f", XM_DEFAULT_PHYS_BLOCK_MASS);
+    std::string mass = XML::getOption(pPhysicsElem, "mass", str);
+    if(mass == "INFINITY"){
+      // Chipmunk's INFINITY is too big (make blocks disapear), so put a big value instead
+      float bigValue = 5000000000000000.0;
+      pBlock->setMass(bigValue);
+    }
+    else
+      pBlock->setMass(atof(mass.c_str()));
+
+    snprintf(str, 16, "%f", XM_DEFAULT_PHYS_BLOCK_FRICTION);
+    pBlock->setFriction(atof(XML::getOption(pPhysicsElem, "friction", str).c_str()));
+
+    snprintf(str, 16, "%f", XM_DEFAULT_PHYS_BLOCK_ELASTICITY);
+    pBlock->setElasticity(atof(XML::getOption(pPhysicsElem, "elasticity", str).c_str()));
   } else {
-    pBlock->setGrip(DEFAULT_PHYS_WHEEL_GRIP);
+    pBlock->setGripPer20(20.0);
+    pBlock->setMass(XM_DEFAULT_PHYS_BLOCK_MASS);
+    pBlock->setFriction(XM_DEFAULT_PHYS_BLOCK_FRICTION);
+    pBlock->setElasticity(XM_DEFAULT_PHYS_BLOCK_ELASTICITY);
   }
 
   if(pEdgeElem != NULL) {
@@ -497,6 +727,12 @@ Block* Block::readFromXml(XMLDocument* i_xmlSource, TiXmlElement *pElem) {
     pBlock->setEdgeAngle(DEFAULT_EDGE_ANGLE);
   }
 
+  if(pColElem != NULL) {
+    std::string methodStr = XML::getOption(pColElem, "type", "None");
+    pBlock->setCollisionMethod(pBlock->stringToColMethod(methodStr));
+    pBlock->setCollisionRadius(atof(XML::getOption(pColElem, "radius", "0.0").c_str()));    
+  }
+
   float lastX = 0.0;
   float lastY = 0.0;
   bool  firstVertex = true;
@@ -508,30 +744,6 @@ Block* Block::readFromXml(XMLDocument* i_xmlSource, TiXmlElement *pElem) {
                                                       atof( XML::getOption(pj, "y", "0").c_str())),
                                              XML::getOption(pj, "edge", ""));
 
-      std::string k;
-      Vector2f v_TexturePosition;
-      k = XML::getOption(pj, "tx", "");
-      if(k != "") {
-        v_TexturePosition.x = atof( k.c_str() );
-      } else {
-        v_TexturePosition.x = pVertex->Position().x * pBlock->TextureScale();
-      }
-
-      k = XML::getOption(pj, "ty", "");
-      if(k != "") {
-        v_TexturePosition.y = atof( k.c_str() );
-      } else {
-        v_TexturePosition.y = pVertex->Position().y * pBlock->TextureScale();
-      }
-      pVertex->setTexturePosition(v_TexturePosition);
-      
-      TColor v_color = TColor(atoi(XML::getOption(pj,"r","255").c_str()),
-                              atoi(XML::getOption(pj,"g","255").c_str()),
-                              atoi(XML::getOption(pj,"b","255").c_str()),
-                              atoi(XML::getOption(pj,"a","255").c_str())
-                              );
-      pVertex->setColor(v_color);
-
       if(firstVertex == true){
 	firstVertex = false;	
       }
@@ -557,14 +769,21 @@ void Block::saveBinary(FileHandle *i_pfh) {
       FS::writeString(i_pfh,   Id());
       FS::writeBool(i_pfh,     isBackground());
       FS::writeBool(i_pfh,     isDynamic());
+      FS::writeBool(i_pfh,     isPhysics());
       FS::writeBool(i_pfh,     isLayer());
       FS::writeInt_LE(i_pfh,   getLayer());
       FS::writeString(i_pfh,   Texture());
+      FS::writeFloat_LE(i_pfh, TextureScale());
       FS::writeFloat_LE(i_pfh, InitialPosition().x);
       FS::writeFloat_LE(i_pfh, InitialPosition().y);
-      FS::writeFloat_LE(i_pfh, Grip());
+      FS::writeFloat_LE(i_pfh, GripPer20());
+      FS::writeFloat_LE(i_pfh, Mass());
+      FS::writeFloat_LE(i_pfh, Friction());
+      FS::writeFloat_LE(i_pfh, Elasticity());
       FS::writeInt_LE(i_pfh,   getEdgeDrawMethod());
       FS::writeFloat_LE(i_pfh, edgeAngle());
+      FS::writeInt_LE(i_pfh,   getCollisionMethod());
+      FS::writeFloat_LE(i_pfh, getCollisionRadius());
 
       FS::writeShort_LE(i_pfh, Vertices().size());
         
@@ -580,17 +799,24 @@ Block* Block::readFromBinary(FileHandle *i_pfh) {
 
   pBlock->setBackground(FS::readBool(i_pfh));
   pBlock->setDynamic(FS::readBool(i_pfh));
+  pBlock->setPhysics(FS::readBool(i_pfh));
   pBlock->setIsLayer(FS::readBool(i_pfh));
   pBlock->setLayer(FS::readInt_LE(i_pfh));
   pBlock->setTexture(FS::readString(i_pfh));
+  pBlock->setTextureScale(FS::readFloat_LE(i_pfh));
 
   Vector2f v_Position;
   v_Position.x = FS::readFloat_LE(i_pfh);
   v_Position.y = FS::readFloat_LE(i_pfh);
   pBlock->setInitialPosition(v_Position);
-  pBlock->setGrip(FS::readFloat_LE(i_pfh));
+  pBlock->setGripPer20(FS::readFloat_LE(i_pfh));
+  pBlock->setMass(FS::readFloat_LE(i_pfh));
+  pBlock->setFriction(FS::readFloat_LE(i_pfh));
+  pBlock->setElasticity(FS::readFloat_LE(i_pfh));
   pBlock->setEdgeDrawMethod((EdgeDrawMethod)FS::readInt_LE(i_pfh));
   pBlock->setEdgeAngle(FS::readFloat_LE(i_pfh));
+  pBlock->setCollisionMethod((CollisionMethod)FS::readInt_LE(i_pfh));
+  pBlock->setCollisionRadius(FS::readFloat_LE(i_pfh));
   
   int nNumVertices = FS::readShort_LE(i_pfh);
   pBlock->Vertices().reserve(nNumVertices);
@@ -734,14 +960,14 @@ void Block::calculateEdgePosition_inout(Vector2f i_vA1,  Vector2f i_vB1, Vector2
     AABB aabbB;
     aabbB.addPointToAABB2f(o_b1);
     aabbB.addPointToAABB2f(o_b2);
-    Logger::Log("(%f,%f) (%f,%f) (%f,%f) (%f,%f) inter: (%f,%f)",
+    LogDebug("(%f,%f) (%f,%f) (%f,%f) (%f,%f) inter: (%f,%f)",
 		o_a1.x, o_a1.y, o_a2.x, o_a2.y,
 		o_b1.x, o_b1.y, o_b2.x, o_b2.y,
 		inter.x, inter.y);
-    Logger::Log("aabbA: (%f,%f)(%f,%f)",
+    LogDebug("aabbA: (%f,%f)(%f,%f)",
 		aabbA.getBMin().x, aabbA.getBMin().y,
 		aabbA.getBMax().x, aabbA.getBMax().y);
-    Logger::Log("aabbB: (%f,%f)(%f,%f)",
+    LogDebug("aabbB: (%f,%f)(%f,%f)",
 		aabbB.getBMin().x, aabbB.getBMin().y,
 		aabbB.getBMax().x, aabbB.getBMax().y);
     if(aabbA.pointTouchAABB2f(inter) == true && aabbB.pointTouchAABB2f(inter) == true){
@@ -749,7 +975,7 @@ void Block::calculateEdgePosition_inout(Vector2f i_vA1,  Vector2f i_vB1, Vector2
       o_a2 = o_b2;
       o_b2 = tmp;
       o_swapDone = true;
-      Logger::Log("swap done");
+      LogDebug("swap done");
     }
 #endif
   } else {
@@ -767,6 +993,16 @@ float Block::edgeAngle()
   return m_edgeAngle;
 }
 
+Block::CollisionMethod Block::getCollisionMethod()
+{
+  return m_collisionMethod;
+}
+
+float Block::getCollisionRadius()
+{
+  return m_collisionRadius;
+}
+
 void Block::setEdgeDrawMethod(EdgeDrawMethod method)
 {
   m_edgeDrawMethod = method;
@@ -777,21 +1013,6 @@ void Block::setEdgeAngle(float angle)
   m_edgeAngle = angle;
 }
 
-std::string Block::edgeToString(Block::EdgeDrawMethod method)
-{
-  switch(method){
-  case inside:
-    return std::string("inside");
-    break;
-  case outside:
-    return std::string("outside");
-    break;
-  default:
-    return std::string("angle");
-    break;
-  }
-}
-
 Block::EdgeDrawMethod Block::stringToEdge(std::string method)
 {
   if(method == "inside")
@@ -801,3 +1022,11 @@ Block::EdgeDrawMethod Block::stringToEdge(std::string method)
   else
     return angle;
 }
+
+Block::CollisionMethod Block::stringToColMethod(std::string method)
+{
+  if(method == "Circle")
+    return Circle;
+  else
+    return None;
+}
