@@ -24,14 +24,20 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 class CollisionSystem;
 class Line;
 
-#include "helpers/VMath.h"
-#include "helpers/Color.h"
+#include "../helpers/VMath.h"
+#include "../helpers/Color.h"
+#include <vector>
 
 class FileHandle;
 class XMLDocument;
 class TiXmlElement;
 class BSPPoly;
 class Block;
+class cpBody;
+class cpShape;
+class ChipmunkWorld;
+class PhysicsSettings;
+template<class T> struct ColElement;
 
 #define DEFAULT_EDGE_ANGLE 270.0f
 
@@ -88,13 +94,17 @@ class BlockVertex {
   BlockVertex(const Vector2f& i_position, const std::string& i_edgeEffect = "");
   ~BlockVertex();
 
-  Vector2f Position() const;
+  inline Vector2f& Position() {
+    return m_position;
+  }
   /* edge from this vertex to the following */
   /* called many many many times, so we inline it, and make it return a ref */
   inline std::string& EdgeEffect() {
     return m_edgeEffect;
   }
 
+  void setPosition(const Vector2f& i_position);
+
   void setTexturePosition(const Vector2f& i_texturePosition);
   void setColor(const TColor& i_color);
 
@@ -110,28 +120,38 @@ class Block {
   Block(std::string i_id);
   ~Block();
 
-  std::string Id() const;
-  std::string Texture() const;
+  inline std::string& Id() {
+    return m_id;
+  }
+  inline std::string& Texture() {
+    return m_texture;
+  }
   Vector2f InitialPosition() const;
-  /* called many many many times, so we inline it, and make it return a ref */
   inline Vector2f& DynamicPosition() {
     return m_dynamicPosition;
   }
-  float DynamicRotation() const;
-
-  /* called many many many times, so we inline it, and make it return a ref */
+  inline float DynamicRotation() const {
+    return m_dynamicRotation;
+  }
   inline Vector2f& DynamicRotationCenter() {
     return m_dynamicRotationCenter;
   }
-
   Vector2f DynamicPositionCenter() const;
-  bool isBackground() const;
-  bool isDynamic() const;
+
+  bool isBackground() const {
+    return m_background;
+  }
+  bool isDynamic() const {
+    return m_dynamic;
+  }
+  bool isPhysics() const;
   bool isLayer() const;
-  float Grip() const;
+  float GripPer20() const;
+  float Mass() const;
+  float Friction() const;
+  float Elasticity() const;
   float TextureScale() const;
   std::vector<BlockVertex*>& Vertices();
-  /* called many many many times, so we inline it */
   inline std::vector<ConvexBlock*>& ConvexBlocks() {
     return m_convexBlocks;
   }
@@ -139,13 +159,31 @@ class Block {
   EdgeDrawMethod getEdgeDrawMethod();
   float edgeAngle();
 
+  typedef enum{None, Circle} CollisionMethod;
+  CollisionMethod getCollisionMethod();
+  float getCollisionRadius();
+
+  inline ColElement<Block>* getColElement() {
+    return m_collisionElement;
+  }
+  cpBody* getPhysicBody() {
+    return mBody;
+  }
+  cpShape* getPhysicShape() {
+    return m_shape;
+  }
+
   void setTexture(const std::string& i_texture);
   void setTextureScale(float i_textureScale);
   void setInitialPosition(const Vector2f& i_initialPosition);
   void setBackground(bool i_background);
   void setDynamic(bool i_dynamic);
+  void setPhysics(bool i_physics);
   void setIsLayer(bool i_isLayer);
-  void setGrip(float i_grip);
+  void setGripPer20(float i_gripPer20);
+  void setMass(float i_mass);
+  void setFriction(float i_friction);
+  void setElasticity(float i_elasticity);
   void setCenter(const Vector2f& i_center);
   void setEdgeDrawMethod(EdgeDrawMethod method);
   // in degres, not radian
@@ -155,18 +193,31 @@ class Block {
   void setDynamicPosition(const Vector2f& i_dynamicPosition);
   /* postion where to display - the center */
   void setDynamicPositionAccordingToCenter(const Vector2f& i_dynamicPosition);
-  /* angle position ; consider of the block center */
-  void setDynamicRotation(float i_dynamicRotation);
+  /* angle position ; consider of the block center
+     returns true if bounding circle has changed.
+   */
+  bool setDynamicRotation(float i_dynamicRotation);
 
-  int loadToPlay(CollisionSystem& io_collisionSystem);
+  void setCollisionMethod(CollisionMethod method);
+  void setCollisionRadius(float radius);
+
+  void translate(float x, float y);
+  void setPhysicsPosition(float ix, float iy);
+
+  int loadToPlay(CollisionSystem* io_collisionSystem, ChipmunkWorld* i_chipmunkWorld, PhysicsSettings* i_physicsSettings);
   void unloadToPlay();
 
-  void saveXml(FileHandle *i_pfh);
   void saveBinary(FileHandle *i_pfh);
+  static bool isPhysics_readFromXml(XMLDocument* i_xmlSource, TiXmlElement *pElem);
   static Block* readFromXml(XMLDocument* i_xmlSource, TiXmlElement *pElem);
   static Block* readFromBinary(FileHandle *i_pfh);
   AABB& getAABB();
-  std::vector<Line *>& getCollisionLines() {return m_collisionLines;}
+  BoundingCircle& getBCircle() {
+    return m_BCircle;
+  }
+  std::vector<Line *>& getCollisionLines() {
+    return m_collisionLines;
+  }
 
   int getGeom() {
     return m_geom;
@@ -185,6 +236,7 @@ class Block {
   std::vector<int>& getEdgeGeoms();
   bool edgeGeomExists(std::string texture);
 
+  void updatePhysics(int timeStep, CollisionSystem* io_collisionSystem);
 
   // calculate edge position
   void calculateEdgePosition_angle(Vector2f  i_vA, Vector2f i_vB,
@@ -216,15 +268,31 @@ private:
 
   bool  m_background;
   bool  m_dynamic;
+  bool  m_physics;
   bool  m_isLayer;
   // the background layer of the block.
   // in case of a non m_backgroundLevel, if m_layer is != -1, then it's
   // a static block from the second static blocks layer
   int   m_layer;
-  float m_grip;                         /* GRIP of the block */
+  float m_gripPer20;                         /* GRIP of the block /20 of the physicSettings */
+
+  /* chipmunk physics */
+  float m_mass;       /* mass of the block */
+  float m_friction;   /* friction of the block */
+  float m_elasticity; /* elasticity of the block */
+  cpBody* mBody;
+  cpShape* m_shape;
+
+  // AABB for static blocks
   AABB  m_BBox;
+  // BoundingCircle for dynamic blocks
+  BoundingCircle m_BCircle;
   bool  m_isBBoxDirty;
 
+  ColElement<Block>* m_collisionElement;
+  CollisionMethod    m_collisionMethod;
+  float              m_collisionRadius;
+
   // the geom used to render the block
   int   m_geom;
 
@@ -233,16 +301,18 @@ private:
   Vector2f m_dynamicRotationCenter;
   Vector2f m_dynamicPositionCenter;
   Vector2f m_dynamicPosition; /* Block position */
-  std::vector<Line *> m_collisionLines; /* Line to collide against */
+  std::vector<Line*> m_collisionLines; /* Line to collide against */
 
-  void addPoly(BSPPoly* i_poly, CollisionSystem& io_collisionSystem);
-  void updateCollisionLines();
+  void addPoly(BSPPoly* i_poly, CollisionSystem* io_collisionSystem);
+  // dynamic background blocks only need to compute the collision lines once.
+  void updateCollisionLines(bool setDirty = false, bool forceUpdate = false);
+  void translateCollisionLines(float x, float y);
 
   EdgeDrawMethod m_edgeDrawMethod;
   float m_edgeAngle;
 
-  std::string edgeToString(EdgeDrawMethod method);
-  EdgeDrawMethod stringToEdge(std::string method);
+  EdgeDrawMethod  stringToEdge(std::string method);
+  CollisionMethod stringToColMethod(std::string method);
 };
 
 #endif /* __BLOCK_H__ */
