@@ -19,7 +19,7 @@
   =============================================================================*/
 
 #include "Camera.h"
-#include "Game.h"
+#include "../Game.h"
 #include "Bike.h"
 
 #define ZOOM_DEFAULT 0.24
@@ -28,8 +28,8 @@
 #define CAMERA_OFFSETY_DEFAULT 0.0
 
 // declared for Active Camera Zooming
-#define SPEED_UNTIL_ZOOM_BEGIN 0.24
-#define SPEED_UNTIL_ZOOM_END   0.15
+#define SPEED_UNTIL_ZOOM_BEGIN 24.0
+#define SPEED_UNTIL_ZOOM_END   15.0
 #define CAM_ZOOM_NEAR    0.24
 #define CAM_ZOOM_FAR     0.18
 #define ZOOM_OUT_SPEED  -0.0004
@@ -38,6 +38,10 @@
 #define TRESHOLD_OUT     1.0 * 1000
 #define DEFAULT_ROTATIONANGLE_SPEED 1.0
 
+#ifdef ENABLE_OPENGL
+#include "include/xm_OpenGL.h"
+#endif
+
 Camera::Camera(Vector2i upperleft, Vector2i downright){
   m_fScale         = ZOOM_DEFAULT;
   m_cameraOffsetX  = CAMERA_OFFSETX_DEFAULT;
@@ -46,6 +50,7 @@ Camera::Camera(Vector2i upperleft, Vector2i downright){
   setRenderSurface(upperleft, downright);
   m_mirrored = false;
   m_allowActiveZoom = true;
+  m_cameraDeathOffset = Vector2f(0.0, 0.0);
 
   prepareForNewLevel();
 }
@@ -211,6 +216,14 @@ void Camera::guessDesiredCameraZoom() {
   }
 }
 
+void Camera::setPlayerDead() {
+  if(m_playerToFollow->getState()->Dir == DD_RIGHT) {
+    m_cameraDeathOffset = m_playerToFollow->getState()->CenterP - m_playerToFollow->getState()->KneeP;
+  } else {
+    m_cameraDeathOffset = m_playerToFollow->getState()->CenterP - m_playerToFollow->getState()->Knee2P;
+  }
+}
+
 void Camera::setScroll(bool isSmooth, const Vector2f& gravity) {
   float v_move_camera_max;
   float v_fDesiredHorizontalScrollShift = 0.0;
@@ -240,15 +253,42 @@ void Camera::setScroll(bool isSmooth, const Vector2f& gravity) {
   }
     
   /* Determine scroll */
-//  if(m_playerToFollow->isDead()) {
-//    if(m_playerToFollow->getState()->Dir == DD_RIGHT) {
-//      m_Scroll = -m_playerToFollow->getState()->KneeP;
-//    } else {
-//      m_Scroll = -m_playerToFollow->getState()->Knee2P;
-//    }
-//  } else {
-  m_Scroll = -m_playerToFollow->getState()->CenterP;
-//  }
+  if(m_playerToFollow->isDead()) {
+    if(m_playerToFollow->getState()->Dir == DD_RIGHT) {
+      m_Scroll = -m_playerToFollow->getState()->KneeP - m_cameraDeathOffset;
+    } else {
+      m_Scroll = -m_playerToFollow->getState()->Knee2P - m_cameraDeathOffset;
+    }
+
+    // update X
+    if(m_cameraDeathOffset.x != 0.0 && m_cameraDeathOffset.y != 0.0) {
+      if(fabs(m_cameraDeathOffset.x) > 0.015) {
+	if(m_cameraDeathOffset.x > 0.0) {
+	  m_cameraDeathOffset.x -= 0.01;
+	} else {
+	  m_cameraDeathOffset.x += 0.01;
+	}
+      } else {
+	m_cameraDeathOffset.x = 0.0; 
+      }
+    }
+
+    // update Y
+    if(m_cameraDeathOffset.y != 0.0 && m_cameraDeathOffset.y != 0.0) {
+      if(fabs(m_cameraDeathOffset.x) > 0.015) {
+	if(m_cameraDeathOffset.y > 0.0) {
+	  m_cameraDeathOffset.y -= 0.01;
+	} else {
+	  m_cameraDeathOffset.y += 0.01;
+	}
+      } else {
+	m_cameraDeathOffset.y = 0.0; 
+      }
+    }
+
+  } else {
+    m_Scroll = -m_playerToFollow->getState()->CenterP;
+  }
 
   /* Driving direction? */
   guessDesiredCameraPosition(v_fDesiredHorizontalScrollShift,
@@ -448,3 +488,40 @@ void Camera::allowActiveZoom(bool i_value) {
 
   initZoom();
 }
+
+void Camera::clearGhostsVisibility() {
+  m_ghostVisibility.clear();
+}
+
+void Camera::checkGhostVisibilityExists(unsigned int i) {
+  CameraGhostVisibility cgv;
+
+  for(unsigned int j=m_ghostVisibility.size(); j<=i; j++) {
+    cgv.isIn = false;
+    m_ghostVisibility.push_back(cgv);
+  }
+}
+
+void Camera::setGhostIn(unsigned int i) {
+  checkGhostVisibilityExists(i);
+
+  if(m_ghostVisibility[i].isIn == false) {
+    m_ghostVisibility[i].lastIn = GameApp::getXMTime();
+  }
+  m_ghostVisibility[i].isIn   = true;
+}
+
+void Camera::setGhostOut(unsigned int i) {
+  checkGhostVisibilityExists(i);
+  m_ghostVisibility[i].isIn = false;
+}
+
+float Camera::getGhostLastIn(unsigned int i) {
+  checkGhostVisibilityExists(i);
+  return m_ghostVisibility[i].lastIn;
+}
+
+bool Camera::isGhostIn(unsigned int i) {
+  checkGhostVisibilityExists(i);
+  return m_ghostVisibility[i].isIn;
+}
