@@ -24,6 +24,11 @@
 #include "../VCommon.h"
 #include "Bike.h"
 
+struct CameraGhostVisibility {
+  bool isIn;
+  float lastIn;
+};
+
 class RenderSurface {
 public:
   void update(Vector2i upperleft, Vector2i downright){
@@ -91,6 +96,8 @@ public:
   bool isMirrored();
   void setMirrored(bool i_value);
 
+  void setPlayerDead(); // inform that the player dies
+
   float rotationAngle();
   void setRotationAngle(float i_value);
   void setRotationSpeed(float i_value);
@@ -99,12 +106,22 @@ public:
 
   void allowActiveZoom(bool i_value);
 
+  // assume that a ghost added is never removed and do not change of id (which is the case)
+  void clearGhostsVisibility();
+  void setGhostIn(unsigned int i);
+  void setGhostOut(unsigned int i);
+  bool isGhostIn(unsigned int i);
+  float getGhostLastIn(unsigned int i);
+
 private:
   bool  m_mirrored;
   float m_rotationAngle;
   float m_rotationSpeed;
   float m_desiredRotationAngle;
 
+  std::vector<CameraGhostVisibility> m_ghostVisibility;
+  void checkGhostVisibilityExists(unsigned int i);
+
   float m_fScale;
   // for not zooming in and out all the time
   unsigned int m_timeTresholdIn;
@@ -129,6 +146,7 @@ private:
   RenderSurface m_renderSurf;
 
   float m_lastSpeedTime;
+  Vector2f m_cameraDeathOffset;
 
   void resetActiveZoom();
 };
