@@ -25,20 +25,24 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "Level.h"
-#include "VFileIO.h"
-#include "XMBuild.h"
-#include "VXml.h"
-#include "helpers/Color.h"
-#include "helpers/Log.h"
-#include "db/xmDatabase.h"
+#include "../VFileIO.h"
+#include "../XMBuild.h"
+#include "../VXml.h"
+#include "../helpers/Color.h"
+#include "../helpers/Log.h"
+#include "../db/xmDatabase.h"
 #include "Block.h"
 #include "Entity.h"
 #include "Zone.h"
 #include "SkyApparence.h"
-#include "Collision.h"
+#include "../Collision.h"
 #include "Scene.h"
+#include "../chipmunk/chipmunk.h"
+#include "ChipmunkWorld.h"
+#include "PhysicsSettings.h"
+#include "../Theme.h"
 
-#define CACHE_LEVEL_FORMAT_VERSION 19
+#define CACHE_LEVEL_FORMAT_VERSION 25
 
 Level::Level() {
   m_xmotoTooOld = false;
@@ -51,7 +55,19 @@ Level::Level() {
   m_borderTexture = "";
   m_numberLayer   = 0;
   m_isScripted    = false;
-  m_sky = new SkyApparence();
+  m_isPhysics     = false;
+  m_sky           = new SkyApparence();
+
+  m_rSpriteForStrawberry      = "Strawberry";
+  m_rSpriteForWecker 	      = "Wrecker";
+  m_rSpriteForFlower 	      = "Flower";
+  m_rSpriteForStar            = "Star";
+  m_rSoundForPickUpStrawberry = "PickUpStrawberry";
+
+  m_strawberrySprite = NULL;
+  m_wreckerSprite    = NULL;
+  m_flowerSprite     = NULL;
+  m_starSprite       = NULL;
 }
 
 Level::~Level() {
@@ -143,6 +159,23 @@ bool Level::isScripted() const {
   return m_isScripted;
 }
 
+bool Level::isPhysics() const {
+  return m_isPhysics;
+}
+
+void Level::updatePhysics(int timeStep, CollisionSystem* p_CollisionSystem, ChipmunkWorld* i_chipmunkWorld) {
+  if(i_chipmunkWorld == NULL) {
+    return;
+  }
+
+  cpSpaceStep(i_chipmunkWorld->getSpace(), ((double)timeStep)/100.0);
+
+  // loop through all blocks, looking for chipmunky ones
+  for(unsigned int i=0; i<m_blocks.size(); i++) {
+    m_blocks[i]->updatePhysics(timeStep, p_CollisionSystem);
+  }
+}
+
 Block* Level::getBlockById(const std::string& i_id) {
   for(unsigned int i=0; i<m_blocks.size(); i++) {
     if(m_blocks[i]->Id() == i_id) {
@@ -252,6 +285,11 @@ void Level::killEntity(const std::string& i_entityId) {
   throw Exception("Entity '" + i_entityId + "' can't be killed");
 }
 
+std::vector<Joint*>& Level::Joints()
+{
+  return m_joints;
+}
+
 std::vector<Entity *>& Level::Entities() {
   return m_entities;
 }
@@ -282,185 +320,36 @@ void Level::revertEntityDestroyed(const std::string& i_entityId) {
   throw Exception("Entity '" + i_entityId + "' can't be reverted");
 }
 
-void Level::updateToTime(MotoGame& i_scene) {
-  bool v_b;
+void Level::updateToTime(Scene& i_scene, PhysicsSettings* i_physicsSettings) {
+  int      v_time    = i_scene.getTime();
+  Vector2f v_gravity = i_scene.getGravity();
 
-  for(unsigned int i=0;i<Entities().size();i++) {
-    v_b = Entities()[i]->updateToTime(i_scene.getTime(), i_scene.getGravity());
+  std::vector<Entity*> v_entities = Entities();
+  unsigned int         size       = v_entities.size();
+
+  for(unsigned int i=0; i<size; i++) {
+    v_entities[i]->updateToTime(v_time, v_gravity, i_physicsSettings);
   }
-  for(unsigned int i=0;i<EntitiesExterns().size();i++) {
-    v_b = EntitiesExterns()[i]->updateToTime(i_scene.getTime(), i_scene.getGravity());
+
+  std::vector<Entity*> v_externalEntities = EntitiesExterns();
+  size                                    = v_externalEntities.size();
+
+  for(unsigned int i=0; i<size; i++) {
+    v_externalEntities[i]->updateToTime(v_time, v_gravity, i_physicsSettings);
   }
 }
 
-void Level::saveXML(void) {
-  FileHandle *pfh = FS::openOFile(m_fileName);
-  if(pfh == NULL) {
-    /* Failed! */
-    Logger::Log("** Warning ** : failed to save level '%s'",m_fileName.c_str());
-    return;
-  }
-    
-  FS::writeLineF(pfh,"<?xml version=\"1.0\" encoding=\"utf-8\"?>");
-
-  std::string v_levelTag;
-  v_levelTag = "<level id=\"" + m_id + "\"";
-  if(m_pack != "") {
-    v_levelTag += " levelpack=\"" + m_pack + "\"";
-  }
-  if(m_packNum != "") {
-    v_levelTag += " levelpackNum=\"" + m_packNum + "\"";
-  }
-  if(m_requiredVersion != "") {
-    v_levelTag += " rversion=\"" + m_requiredVersion + "\"";
-  }
-  v_levelTag += ">";
-  FS::writeLineF(pfh, "%s", v_levelTag.c_str());
-  
-  /* INFO */
-  FS::writeLineF(pfh,"\t<info>");
-  FS::writeLineF(pfh,"\t\t<name>%s</name>",m_name.c_str());
-  FS::writeLineF(pfh,"\t\t<description>%s</description>",m_description.c_str());
-  FS::writeLineF(pfh,"\t\t<author>%s</author>",m_author.c_str());
-  FS::writeLineF(pfh,"\t\t<date>%s</date>",m_date.c_str());
-  if(m_sky->Drifted()) {
-    FS::writeLineF(pfh,
-			 "\t\t<sky zoom=\"%f\" offset=\"%f\" color_r=\"%i\" color_g=\"%i\" color_b=\"%i\" color_a=\"%i\" drifted=\"true\" driftZoom=\"%f\" driftColor_r=\"%i\" driftColor_g=\"%i\" driftColor_b=\"%i\" driftColor_a=\"%i\">%s</sky>",
-			 m_sky->Zoom(),
-			 m_sky->Offset(),
-			 m_sky->TextureColor().Red(),
-			 m_sky->TextureColor().Green(),
-			 m_sky->TextureColor().Blue(),
-			 m_sky->TextureColor().Alpha(),
-			 m_sky->DriftZoom(),
-			 m_sky->DriftTextureColor().Red(),
-			 m_sky->DriftTextureColor().Green(),
-			 m_sky->DriftTextureColor().Blue(),
-			 m_sky->DriftTextureColor().Alpha(),
-			 m_sky->Texture().c_str());
-  } else {
-    FS::writeLineF(pfh,
-			 "\t\t<sky zoom=\"%f\" offset=\"%f\" color_r=\"%i\" color_g=\"%i\" color_b=\"%i\" color_a=\"%i\">%s</sky>",
-			 m_sky->Zoom(),
-			 m_sky->Offset(),
-			 m_sky->TextureColor().Red(),
-			 m_sky->TextureColor().Green(),
-			 m_sky->TextureColor().Blue(),
-			 m_sky->TextureColor().Alpha(),
-			 m_sky->Texture().c_str());
-  }
-  FS::writeLineF(pfh,"\t\t<border texture=\"%s\" />",m_borderTexture.c_str());
-  FS::writeLineF(pfh,"\t\t<music name=\"%s\" />", m_music.c_str());
-  FS::writeLineF(pfh,"\t</info>");
-
-  if(m_numberLayer > 0){
-    FS::writeLineF(pfh, "\t\t<layeroffsets>");
-    for(int i=0; i<m_numberLayer; i++){
-      std::string front = "";
-      if(m_isLayerFront[i] == true){
-	front = " frontlayer=\"true\"";
-      }
-
-      FS::writeLineF(pfh, "\t\t\t<layeroffset x=\"%f\" y=\"%f\"%s/>",
-			   m_layerOffsets[i].x, m_layerOffsets[i].y, front.c_str());
-    }
-    FS::writeLineF(pfh, "\t\t</layeroffsets>");
-  }
-  
-  /* replacement sprites */
-  if(m_rSpriteForStrawberry 	 != "" ||
-     m_rSpriteForWecker     	 != "" ||
-     m_rSpriteForFlower     	 != "" ||
-     m_rSpriteForStar       	 != "" ||
-     m_rSoundForPickUpStrawberry != ""
-     ) {
-    FS::writeLineF(pfh,"\t<theme_replacements>");
-    if(m_rSpriteForStrawberry != "") {
-      FS::writeLineF(pfh,"\t\t<sprite_replacement old_name=\"Strawberry\" new_name=\"%s\" />",
-			   m_rSpriteForStrawberry.c_str());
-    }
-    if(m_rSpriteForWecker != "") {
-      FS::writeLineF(pfh,"\t\t<sprite_replacement old_name=\"Wrecker\" new_name=\"%s\" />",
-			   m_rSpriteForWecker.c_str());
-    }
-    if(m_rSpriteForFlower != "") {
-      FS::writeLineF(pfh,"\t\t<sprite_replacement old_name=\"Flower\" new_name=\"%s\" />",
-			   m_rSpriteForFlower.c_str());
-    }
-    if(m_rSpriteForStar != "") {
-      FS::writeLineF(pfh,"\t\t<sprite_replacement old_name=\"Star\" new_name=\"%s\" />",
-			   m_rSpriteForStar.c_str());
-    }
-    if(m_rSoundForPickUpStrawberry != "") {
-      FS::writeLineF(pfh,"\t\t<sound_replacement old_name=\"PickUpStrawberry\" new_name=\"%s\" />",
-			   m_rSoundForPickUpStrawberry.c_str());
-    }
-    FS::writeLineF(pfh,"\t</theme_replacements>");
-  }
-
-  /* MISC */
-  if(m_scriptFileName != "" && m_scriptSource != "") {
-    FS::writeLineF(pfh,"\t<script source=\"%s\">",m_scriptFileName.c_str());
-    FS::writeByte(pfh,'\t'); FS::writeByte(pfh,'\t');
-    
-    /* Write script and transform dangerous characters */
-    for(unsigned int i=0;i<m_scriptSource.length();i++) {
-      int c = m_scriptSource[i];
-      if(c == '<') FS::writeBuf(pfh,"&lt;",4);
-      else if(c == '>') FS::writeBuf(pfh,"&gt;",4);
-      else FS::writeByte(pfh,c);
-    }
-    //FS::writeBuf(pfh,(char *)m_ScriptSource.c_str(),m_ScriptSource.length());
-    FS::writeLineF(pfh,"</script>");
-  }
-  else if(m_scriptFileName != "")
-    FS::writeLineF(pfh,"\t<script source=\"%s\"/>",m_scriptFileName.c_str());
-  else if(m_scriptSource != "") {
-    FS::writeLineF(pfh,"\t<script>");
-    FS::writeByte(pfh,'\t'); FS::writeByte(pfh,'\t');
-    /* Write script and transform dangerous characters */
-    for(unsigned int i=0;i<m_scriptSource.length();i++) {
-      int c = m_scriptSource[i];
-      if(c == '<') FS::writeBuf(pfh,"&lt;",4);
-      else if(c == '>') FS::writeBuf(pfh,"&gt;",4);
-      else FS::writeByte(pfh,c);
-    }
-    //      FS::writeBuf(pfh,(char *)m_ScriptSource.c_str(),m_ScriptSource.length());
-    FS::writeLineF(pfh,"</script>");
-  }
-  
-  FS::writeLineF(pfh,"\t<limits left=\"%f\" right=\"%f\" top=\"%f\" bottom=\"%f\"/>",m_leftLimit,m_rightLimit,m_topLimit,m_bottomLimit);
-  
-  /* BLOCKS */
-  for(unsigned int i=0;i<m_blocks.size();i++) {
-    m_blocks[i]->saveXml(pfh);
-  }
-  
-  /* ENTITIES */
-  for(unsigned int i=0;i<m_entities.size();i++) {
-    m_entities[i]->saveXml(pfh);
-  }
-  
-  /* ZONE */
-  for(unsigned int i=0;i<m_zones.size();i++) {
-    m_zones[i]->saveXml(pfh);
-  }
-  
-  FS::writeLineF(pfh,"\n</level>");           
-  FS::closeFile(pfh);
-}
-
-void Level::loadXML(void) {
+void Level::loadXML() {
   /* Load XML document and fetch tinyxml handle */
   unloadLevelBody();
   if(m_xmlSource == NULL) {
     m_xmlSource = new XMLDocument();
   }
 
-  m_xmlSource->readFromFile(m_fileName, NULL);
+  m_xmlSource->readFromFile(m_fileName, NULL, true);
 
   TiXmlDocument *pDoc = m_xmlSource->getLowLevelAccess();
-  if(pDoc == NULL) throw Exception("failed to load level XML");
+  if(pDoc == NULL) throw Exception("failed to load level XML " + m_fileName);
   
   /* Start the fantastic parsing by fetching the <level> element */
   TiXmlElement *pLevelElem = XML::findElement(*m_xmlSource, NULL,std::string("level"));    
@@ -478,7 +367,7 @@ void Level::loadXML(void) {
     if(compareVersionNumbers(XMBuild::getVersionString(),m_requiredVersion) < 0) {
       /* Our version is too low to load this */
       m_xmotoTooOld = true;
-      Logger::Log("** Warning ** : Level '%s' requires a newer version (%s) to load!",m_id.c_str(),m_requiredVersion.c_str());
+      LogWarning("Level '%s' requires a newer version (%s) to load!",m_id.c_str(),m_requiredVersion.c_str());
     }
   }
   
@@ -500,6 +389,7 @@ void Level::loadXML(void) {
   m_numberLayer = 0;
   
   m_isScripted = false;
+  m_isPhysics  = false;
 
   m_sky->reInit();
 
@@ -605,7 +495,10 @@ void Level::loadXML(void) {
       /* Music */
       TiXmlElement *pMusicElem = XML::findElement(*m_xmlSource, NULL, std::string("music"));
       if(pMusicElem != NULL) {
-	m_music = XML::getOption(pMusicElem, "name");  
+	m_music = XML::getOption(pMusicElem, "name");
+	if(m_music == "None") {
+	  m_music = "";
+	}
       }
     }
 
@@ -625,34 +518,24 @@ void Level::loadXML(void) {
 	m_layerOffsets.push_back(offset);
 	m_isLayerFront.push_back(XML::getOption(pElem, "frontlayer", "false") == "true");
       }
-//      for(int i=0; i<m_numberLayer; i++){
-//	Logger::Log("Level::loadXML offsets layer %d: %f, %f isfront: %s",
-//		    i, m_layerOffsets[i].x, m_layerOffsets[i].y, m_isLayerFront[i]?"true":"false");
-//      }
     }
 
-    /* replacement theme */
-    m_rSpriteForStrawberry 	= "";
-    m_rSpriteForWecker 	   	= "";
-    m_rSpriteForFlower 	   	= "";
-    m_rSpriteForStar   	   	= "";
-    m_rSoundForPickUpStrawberry = "";
-
     TiXmlElement *pThemeReplaceElem = XML::findElement(*m_xmlSource, pLevelElem,std::string("theme_replacements"));
     if(pThemeReplaceElem != NULL) {
       /* Get replacements  for sprites */
-      for(TiXmlElement *pElem = pThemeReplaceElem->FirstChildElement("sprite_replacement"); pElem!=NULL;
+      for(TiXmlElement *pElem = pThemeReplaceElem->FirstChildElement("sprite_replacement");
+	  pElem!=NULL;
 	  pElem=pElem->NextSiblingElement("sprite_replacement")) {
 	std::string v_old_name = XML::getOption(pElem, "old_name");
 	std::string v_new_name = XML::getOption(pElem, "new_name");
 	/* for efficacity and before other are not required, only change main ones */
-	if        (v_old_name == "Strawberry") {
+	if        (v_old_name == "Strawberry" && v_new_name != "") {
 	  m_rSpriteForStrawberry = v_new_name;
-	} else	if(v_old_name == "Wrecker") {
+	} else	if(v_old_name == "Wrecker"    && v_new_name != "") {
 	  m_rSpriteForWecker     = v_new_name;
-	} else 	if(v_old_name == "Flower") {
+	} else 	if(v_old_name == "Flower"     && v_new_name != "") {
 	  m_rSpriteForFlower     = v_new_name;
-	} else 	if(v_old_name == "Star") {
+	} else 	if(v_old_name == "Star"       && v_new_name != "") {
 	  m_rSpriteForStar       = v_new_name;
 	}
       }
@@ -694,30 +577,48 @@ void Level::loadXML(void) {
       m_rightLimit = atof( XML::getOption(pLimitsElem,"right","50").c_str() );
     }
 
-    /* Get entities */
-    for(TiXmlElement *pElem = pLevelElem->FirstChildElement("entity"); pElem!=NULL; 
-        pElem=pElem->NextSiblingElement("entity")) {
-      m_entities.push_back(Entity::readFromXml(pElem));
-    }    
-
-    try {
-      m_playerStart = getStartEntity()->InitialPosition();
-    } catch(Exception &e) {
-      Logger::Log("Warning : no player start entity for level %s", Id().c_str());
-      m_playerStart = Vector2f(0.0, 0.0);
-    }    
-
     /* Get zones */
     for(TiXmlElement *pElem = pLevelElem->FirstChildElement("zone"); pElem!=NULL;
         pElem=pElem->NextSiblingElement("zone")) {
       m_zones.push_back(Zone::readFromXml(pElem));
     }
     
+    /* determine whether the levels is physics or not */
+    //for(TiXmlElement *pElem = pLevelElem->FirstChildElement("block"); pElem!=NULL;
+    //    pElem=pElem->NextSiblingElement("block")) {
+    //  if(Block::isPhysics_readFromXml(m_xmlSource, pElem)) {
+    //	m_isPhysics = true;
+    //  }
+    //}
+
     /* Get blocks */
     for(TiXmlElement *pElem = pLevelElem->FirstChildElement("block"); pElem!=NULL;
         pElem=pElem->NextSiblingElement("block")) {
-      m_blocks.push_back(Block::readFromXml(m_xmlSource, pElem));
-    }  
+      Block* v_block = Block::readFromXml(m_xmlSource, pElem);
+      if(v_block->isPhysics()) {
+	m_isPhysics = true;
+      }
+      m_blocks.push_back(v_block);
+    }
+
+    /* Get entities */
+    for(TiXmlElement *pElem = pLevelElem->FirstChildElement("entity"); pElem!=NULL; 
+        pElem=pElem->NextSiblingElement("entity")) {
+      Entity* v_entity = Entity::readFromXml(pElem);
+      if(v_entity->Speciality() == ET_JOINT) {
+	Joint* v_joint = (Joint*)v_entity;
+	v_joint->readFromXml(pElem);
+	m_joints.push_back(v_joint);
+      } else
+	m_entities.push_back(v_entity);
+    }    
+
+    try {
+      m_playerStart = getStartEntity()->InitialPosition();
+    } catch(Exception &e) {
+      LogWarning("no player start entity for level %s", Id().c_str());
+      m_playerStart = Vector2f(0.0, 0.0);
+    }
   }
 
   /* if blocks with layerid but level with no layer, throw an exception */
@@ -725,7 +626,7 @@ void Level::loadXML(void) {
     int blockLayer = m_blocks[i]->getLayer();
     if(blockLayer != -1){
       if((getNumberLayer() == 0) || (blockLayer >= getNumberLayer())){
-	Logger::Log("** Warning ** : Level '%s' Block '%s' as a layer, but the level has no layer", m_id.c_str(), m_blocks[i]->Id().c_str());
+	LogWarning("Level '%s' Block '%s' as a layer, but the level has no layer", m_id.c_str(), m_blocks[i]->Id().c_str());
 	throw Exception("the block has a layer but the level is without layers");
       }
     }
@@ -755,9 +656,9 @@ bool Level::loadReducedFromFile() {
   try {
     cached = importBinaryHeaderFromFile(cacheFileName, m_checkSum);
   } catch (Exception &e) {
-    Logger::Log("** Warning **: Exception while loading binary level, will load "
-	      "XML instead for '%s' (%s)", FileName().c_str(),
-	      e.getMsg().c_str());
+    LogWarning("Exception while loading binary level, will load "
+	       "XML instead for '%s' (%s)", FileName().c_str(),
+	       e.getMsg().c_str());
   }
   
   // If we couldn't get it from the cache, then load from (slow) XML
@@ -803,7 +704,7 @@ void Level::exportBinary(const std::string &FileName, const std::string& pSum) {
   /* Export binary... */
   FileHandle *pfh = FS::openOFile(FileName);
   if(pfh == NULL) {
-    Logger::Log("** Warning ** : Failed to export binary: %s",FileName.c_str());
+    LogWarning("Failed to export binary: %s",FileName.c_str());
   }
   else {
     exportBinaryHeader(pfh);
@@ -857,7 +758,13 @@ void Level::exportBinary(const std::string &FileName, const std::string& pSum) {
     FS::writeInt_LE(pfh,m_entities.size());
     for(unsigned int i=0;i<m_entities.size();i++) {
       m_entities[i]->saveBinary(pfh);
-    }  
+    }
+
+    // write joints
+    FS::writeInt_LE(pfh, m_joints.size());
+    for(unsigned int i=0; i<m_joints.size(); i++) {
+      m_joints[i]->saveBinary(pfh);
+    }
       
     /* Write zones */
     FS::writeInt_LE(pfh,m_zones.size());
@@ -879,6 +786,7 @@ void Level::loadFullyFromFile() {
     loadXML();
     exportBinary(getNameInCache(), m_checkSum);
   }
+  loadRemplacementSprites();
 }
 
 void Level::importBinaryHeader(FileHandle *pfh) {
@@ -904,6 +812,7 @@ void Level::importBinaryHeader(FileHandle *pfh) {
   m_date   	= FS::readString(pfh);
   m_music       = FS::readString(pfh);
   m_isScripted  = FS::readBool(pfh);
+  m_isPhysics   = FS::readBool(pfh);
 }
 
 void Level::importHeader(const std::string& i_id,
@@ -915,7 +824,8 @@ void Level::importHeader(const std::string& i_id,
 			 const std::string& i_author,
 			 const std::string& i_date,
 			 const std::string& i_music,
-			 bool i_isScripted
+			 bool i_isScripted,
+			 bool i_isPhysics
 			 ) {
   unloadLevelBody();
 
@@ -933,6 +843,7 @@ void Level::importHeader(const std::string& i_id,
   m_date   	= i_date;
   m_music       = i_music;
   m_isScripted  = i_isScripted;
+  m_isPhysics   = i_isPhysics;
 }
 
   /*===========================================================================
@@ -940,7 +851,7 @@ void Level::importHeader(const std::string& i_id,
   ===========================================================================*/
 bool Level::importBinaryHeaderFromFile(const std::string &FileName, const std::string& pSum) {
   /* Import binary */
-  FileHandle *pfh = FS::openIFile(FileName);
+  FileHandle *pfh = FS::openIFile(FileName, true);
   if(pfh == NULL) {
     return false;
   }
@@ -948,7 +859,7 @@ bool Level::importBinaryHeaderFromFile(const std::string &FileName, const std::s
   try {
     importBinaryHeader(pfh);
     if(m_checkSum != pSum) {
-      Logger::Log("** Warning ** : CRC check failed, can't import: %s",FileName.c_str());
+      LogWarning("CRC check failed, can't import: %s",FileName.c_str());
       FS::closeFile(pfh);
       return false;
     }
@@ -980,6 +891,7 @@ void Level::exportBinaryHeader(FileHandle *pfh) {
   FS::writeString(pfh	    , m_date);
   FS::writeString(pfh	    , m_music);
   FS::writeBool(  pfh	    , m_isScripted);
+  FS::writeBool(  pfh	    , m_isPhysics);
 }
 
 bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
@@ -991,7 +903,7 @@ bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
   m_xmotoTooOld = false;
   
   /* Import binary */
-  FileHandle *pfh = FS::openIFile(FileName);
+  FileHandle *pfh = FS::openIFile(FileName, true);
   if(pfh == NULL) {
     return false;
   }
@@ -1004,7 +916,7 @@ bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
       /* Right */
       std::string md5sum = FS::readString(pfh);
       if(md5sum != pSum) {
-        Logger::Log("** Warning ** : CRC check failed, can't import: %s",FileName.c_str());
+        LogWarning("CRC check failed, can't import: %s",FileName.c_str());
         bRet = false;
       }
       else {
@@ -1018,6 +930,7 @@ bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
         m_date = FS::readString(pfh);
         m_music = FS::readString(pfh);
 	m_isScripted = FS::readBool(pfh);
+	m_isPhysics  = FS::readBool(pfh);
 
 	/* sky */
         m_sky->setTexture(FS::readString(pfh));
@@ -1057,6 +970,15 @@ bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
 	m_rSpriteForStar       	    = FS::readString(pfh);
 	m_rSoundForPickUpStrawberry = FS::readString(pfh);
 
+	if(m_rSpriteForStrawberry == "")
+	  m_rSpriteForStrawberry = "Strawberry";
+	if(m_rSpriteForFlower == "")
+	  m_rSpriteForFlower = "Flower";
+	if(m_rSpriteForWecker == "")
+	  m_rSpriteForWecker = "Wrecker";
+	if(m_rSpriteForStar == "")
+	  m_rSpriteForStar = "Star";
+
 	/* layers */
 	m_numberLayer = FS::readInt_LE(pfh);
 	for(int i=0; i<m_numberLayer; i++){
@@ -1092,16 +1014,26 @@ bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
         int nNumEntities = FS::readInt_LE(pfh);
         m_entities.reserve(nNumEntities);
         for(int i=0;i<nNumEntities;i++) {
-          m_entities.push_back(Entity::readFromBinary(pfh));
+	  Entity* v_entity = Entity::readFromBinary(pfh);
+	  m_entities.push_back(v_entity);
         }
 
 	try {
 	  m_playerStart = getStartEntity()->InitialPosition();
 	} catch(Exception &e) {
-	  Logger::Log("Warning : no player start entity for level %s", Id().c_str());
+	  LogWarning("no player start entity for level %s", Id().c_str());
 	  m_playerStart = Vector2f(0.0, 0.0);
 	}
 
+        int nNumJoints = FS::readInt_LE(pfh);
+        m_joints.reserve(nNumJoints);
+        for(int i=0; i<nNumJoints; i++) {
+	  Entity* v_entity = Entity::readFromBinary(pfh);
+	  Joint* v_joint = (Joint*)v_entity;
+	  v_joint->readFromBinary(pfh);
+	  m_joints.push_back(v_joint);
+        }
+
         /* Read zones */
         int nNumZones = FS::readInt_LE(pfh);
         m_zones.reserve(nNumZones);
@@ -1111,7 +1043,7 @@ bool Level::importBinary(const std::string &FileName, const std::string& pSum) {
       }
     }
     else {
-      Logger::Log("** Warning ** : Invalid binary format (%d), can't import: %s",nFormat,FileName.c_str());
+      LogWarning("Invalid binary format (%d), can't import: %s",nFormat,FileName.c_str());
       bRet = false;
     }
              
@@ -1153,10 +1085,6 @@ int Level::compareLevel(const Level& i_lvl1, const Level& i_lvl2) {
   return 0;
 }
 
-int Level::compareLevelRandom(const Level& i_lvl1, const Level& i_lvl2) {
-  return (randomNum(0, 1) < 0.5 ? 1 : -1);
-}
-
 int Level::compareLevelSamePack(const Level& i_lvl1, const Level& i_lvl2) {
   std::string n1, n2;
   
@@ -1207,13 +1135,13 @@ int Level::compareVersionNumbers(const std::string &v1,const std::string &v2) {
   return 0;    
 }
 
-int Level::loadToPlay() {
+int Level::loadToPlay(ChipmunkWorld* i_chipmunkWorld, PhysicsSettings* i_physicsSettings) {
   int v_nbErrors = 0;
 
   /* preparing blocks */
   for(unsigned int i=0; i<m_blocks.size(); i++) {
     try {
-      v_nbErrors += m_blocks[i]->loadToPlay(*m_pCollisionSystem);
+      v_nbErrors += m_blocks[i]->loadToPlay(m_pCollisionSystem, i_chipmunkWorld, i_physicsSettings);
     } catch(Exception &e) {
       throw Exception("Fail to load block '" + m_blocks[i]->Id() + "' :\n" + e.getMsg());
     }
@@ -1230,6 +1158,11 @@ int Level::loadToPlay() {
       m_nbEntitiesToTake++;
     }
   }
+
+  // create joints
+  for(unsigned int i=0; i<m_joints.size(); i++) {
+    m_joints[i]->loadToPlay(this, i_chipmunkWorld);
+  }
   
   return v_nbErrors;
 }
@@ -1257,6 +1190,10 @@ void Level::unloadToPlay() {
   m_entitiesExterns.clear();
 
   m_nbEntitiesToTake = 0;
+
+  for(unsigned int i=0; i<m_joints.size(); i++) {
+    m_joints[i]->unloadToPlay();
+  }
 }
 
 void Level::addLimits() {
@@ -1360,36 +1297,49 @@ void Level::rebuildCache() {
 }
 
 std::string Level::SpriteForStrawberry() const {
-  if(m_rSpriteForStrawberry != "") {
-    return m_rSpriteForStrawberry;
-  }
-  return "Strawberry";
+  return m_rSpriteForStrawberry;
 }
 
 std::string Level::SpriteForWecker() const {
-  if(m_rSpriteForWecker != "") {
-    return m_rSpriteForWecker;
-  }
-  return "Wrecker";
+  return m_rSpriteForWecker;
 }
 
 std::string Level::SpriteForFlower() const {
-  if(m_rSpriteForFlower != "") {
-    return m_rSpriteForFlower;
-  }
-  return "Flower";
+  return m_rSpriteForFlower;
 }
 
 std::string Level::SpriteForStar() const {
-  if(m_rSpriteForStar != "") {
-    return m_rSpriteForStar;
-  }
-  return "Star";
+  return m_rSpriteForStar;
 }
 
 std::string Level::SoundForPickUpStrawberry() const {
-  if(m_rSoundForPickUpStrawberry != "") {
-    return m_rSoundForPickUpStrawberry;
-  }
-  return "PickUpStrawberry";
+  return m_rSoundForPickUpStrawberry;
+}
+
+Sprite* Level::strawberrySprite()
+{
+  return m_strawberrySprite;
+}
+
+Sprite* Level::wreckerSprite()
+{
+  return m_wreckerSprite;
+}
+
+Sprite* Level::flowerSprite()
+{
+  return m_flowerSprite;
+}
+
+Sprite* Level::starSprite()
+{
+  return m_starSprite;
+}
+
+void Level::loadRemplacementSprites()
+{
+  m_strawberrySprite = Theme::instance()->getSprite(SPRITE_TYPE_ANIMATION, SpriteForStrawberry());
+  m_wreckerSprite    = Theme::instance()->getSprite(SPRITE_TYPE_ANIMATION, SpriteForWecker());
+  m_flowerSprite     = Theme::instance()->getSprite(SPRITE_TYPE_ANIMATION, SpriteForFlower());
+  m_starSprite       = Theme::instance()->getSprite(SPRITE_TYPE_ANIMATION, SpriteForStar());
 }
