@@ -21,26 +21,30 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #ifndef __MOTOGAME_H__
 #define __MOTOGAME_H__
 
-#include "helpers/VMath.h"
-#include "Collision.h"
-#include "helpers/Color.h"
+#include "../helpers/VMath.h"
+#include "../Collision.h"
+#include "../helpers/Color.h"
 #include "BasicSceneStructs.h"
 #include "Bike.h"
+#include <queue>
 
 #define MOTOGAME_DEFAULT_GAME_MESSAGE_DURATION 500
-#define REPLAY_SPEED_INCREMENT 0.25
+#define REPLAY_SPEED_INCREMENT                 0.25
+#define PHYS_STEP_SIZE                         1
 
 class Level;
 class BikeState;
+class FileGhost;
 class Ghost;
+class NetGhost;
 class PlayerBiker;
 class ReplayBiker;
-class MotoGameOnBikerHooks;
+class SceneOnBikerHooks;
 class LuaLibGame;
 class xmDatabase;
 class Camera;
 class Entity;
-class MotoGameEvent;
+class SceneEvent;
 class Replay;
 class GameRenderer;
 class CollisionSystem;
@@ -53,7 +57,8 @@ class SDynamicObject;
 class Theme;
 class BikerTheme;
 class Biker;
-
+class ChipmunkWorld;
+class PhysicsSettings;
 
 /*===========================================================================
   Serialized bike state
@@ -98,18 +103,18 @@ struct GameMessage {
   Game object
   ===========================================================================*/
 
-class MotoGameHooks {
+class SceneHooks {
 public:
-  virtual ~MotoGameHooks() {};
+  virtual ~SceneHooks() {};
   virtual void OnTakeEntity() = 0;
 };
 
-class MotoGame {
+class Scene {
 public:          
-  MotoGame();
-  ~MotoGame();
+  Scene();
+  ~Scene();
 
-  void setHooks(MotoGameHooks *i_motoGameHooks);
+  void setHooks(SceneHooks *i_motoGameHooks);
 
   /* update of the structure */
   void loadLevel(xmDatabase *i_db, const std::string& i_id_level);
@@ -133,13 +138,13 @@ public:
   std::vector<GameMessage *> &getGameMessage(void) {return m_GameMessages;}
 
   /* serialization */
-  static void getSerializedBikeState(BikeState *i_bikeState, int i_time, SerializedBikeState *pState);
+  static void getSerializedBikeState(BikeState *i_bikeState, int i_time, SerializedBikeState *pState, PhysicsSettings* i_physicsSettings);
   static void unserializeGameEvents(DBuffer *Buffer, std::vector<RecordedGameEvent *> *v_ReplayEvents, bool bDisplayInformation = false);
 
   /* events */
-  void createGameEvent(MotoGameEvent *p_event);
-  void destroyGameEvent(MotoGameEvent *p_event);
-  MotoGameEvent* getNextGameEvent();
+  void createGameEvent(SceneEvent *p_event);
+  void destroyGameEvent(SceneEvent *p_event);
+  SceneEvent* getNextGameEvent();
   int getNumPendingGameEvents();
   void cleanEventsQueue();
   void executeEvents(Replay *p_replay);
@@ -163,13 +168,18 @@ public:
   /* action for events */
   void SetEntityPos(std::string pEntityID, float pX, float pY);
   void SetEntityPos(Entity* pEntity,  float pX, float pY);
+  void translateEntity(std::string pEntityID, float x, float y);
+  void translateEntity(Entity* pEntity, float x, float y);
   void PlaceInGameArrow(float pX, float pY, float pAngle);
   void PlaceScreenArrow(float pX, float pY, float pAngle);
   void HideArrow();
   void MoveBlock(std::string pBlockID, float pX, float pY);
+  void MoveBlock(Block* pBlock, float pX, float pY);
   void SetBlockPos(std::string pBlockID, float pX, float pY);
+  void SetPhysicsBlockPos(std::string pBlockID, float pX, float pY);
   void SetBlockCenter(std::string pBlockID, float pX, float pY);
   void SetBlockRotation(std::string pBlockID, float pAngle);
+  void SetBlockRotation(Block* pBlock, float pAngle);
   void SetEntityDrawAngle(std::string pEntityID, float pAngle);
 
   void CameraZoom(float pZoom);
@@ -198,21 +208,28 @@ public:
   ReplayBiker* addReplayFromFile(std::string i_ghostFile,
 				 Theme *i_theme, BikerTheme* i_bikerTheme,
 				 bool i_enableEngineSound);
-  Ghost* addGhostFromFile(std::string i_ghostFile, std::string i_info,
-			  Theme *i_theme, BikerTheme* i_bikerTheme,
-			  const TColor& i_filterColor,
-			  const TColor& i_filterUglyColor);
-  PlayerBiker* addPlayerBiker(Vector2f i_position, DriveDir i_direction,
+  FileGhost* addGhostFromFile(std::string i_ghostFile, const std::string& i_info, bool i_isReference,
+			      Theme *i_theme, BikerTheme* i_bikerTheme,
+			      const TColor& i_filterColor,
+			      const TColor& i_filterUglyColor);
+  PlayerBiker* addPlayerBiker(int i_localNetId, Vector2f i_position, DriveDir i_direction,
 			      Theme *i_theme, BikerTheme* i_bikerTheme,
 			      const TColor& i_filterColor,
 			      const TColor& i_filterUglyColor,
 			      bool i_enableEngineSound);
 
+  NetGhost* addNetGhost(const std::string& i_info, Theme *i_theme,
+			BikerTheme* i_bikerTheme,
+			const TColor& i_filterColor,
+			const TColor& i_filterUglyColor);
+
   std::vector<Ghost *> &Ghosts();
   std::vector<Biker*> &Players();
 
   bool doesPlayEvents() const;
 
+  PhysicsSettings* getPhysicsSettings();
+
   void fastforward(int i_time);
   void fastrewind(int i_time);
   void pause();
@@ -222,9 +239,9 @@ public:
   void setSpeed(float f);
   bool isPaused();
 
-  void handleEvent(MotoGameEvent *pEvent);
+  void handleEvent(SceneEvent *pEvent);
 
-  void setInfos(std::string i_infos);
+  void setInfos(const std::string& i_infos);
   std::string getInfos() const;
 
   LuaLibGame* getLuaLibGame();
@@ -244,14 +261,15 @@ public:
 private:
 
   /* Data */
-  std::queue<MotoGameEvent*> m_GameEventQueue;
+  std::queue<SceneEvent*> m_GameEventQueue;
   int m_time;
   float m_floattantTimeStepDiff; // to play slowly replay
-  int m_lastStateSerializationTime; 
+  int m_lastStateSerializationTime;
+  int m_lastStateUploadTime;
   float m_speed_factor; /* nb hundreadths to increment each time ; is a float so that manage slow */
   bool  m_is_paused;
 
-  MotoGameHooks *m_motoGameHooks;
+  SceneHooks *m_motoGameHooks;
   std::string m_infos;
   bool m_bDeathAnimEnabled;
   Vector2f m_PhysGravity; /* gravity */
@@ -267,20 +285,27 @@ private:
   std::vector<Biker*> m_players;
 
   bool m_showGhostTimeDiff;
+  void onRewinding();
 
   std::vector<Ghost*> m_ghosts;
   std::vector<float> m_myLastStrawberries;
 
-  GameRenderer *m_renderer;
+  GameRenderer* m_renderer;
+  ChipmunkWorld* m_chipmunkWorld;
 
   std::vector<SDynamicObject*> m_SDynamicObjects;
 
   int m_nLastEventSeq;
 
+  PhysicsSettings* m_physicsSettings;
+
   /* for EveryHundreath function */
   int m_lastCallToEveryHundreath;
   bool m_playEvents;
 
+  // some part of the game can be update only half on the time
+  bool m_halfUpdate;
+
   std::vector<Camera*> m_cameras;
   unsigned int m_currentCamera;
 
@@ -301,20 +326,20 @@ private:
   void cleanScriptDynamicObjects();
   void nextStateScriptDynamicObjects(int i_nbCents);
 
-  void _SerializeGameEventQueue(DBuffer* Buffer,MotoGameEvent *pEvent);      
+  void _SerializeGameEventQueue(DBuffer* Buffer,SceneEvent *pEvent);      
   void _UpdateDynamicCollisionLines(void);
 };
 
-class MotoGameOnBikerHooks : public OnBikerHooks {
+class SceneOnBikerHooks : public OnBikerHooks {
 public:
-  MotoGameOnBikerHooks(MotoGame* i_motoGame, int i_playerNumber);
-  virtual ~MotoGameOnBikerHooks();
+  SceneOnBikerHooks(Scene* i_motoGame, int i_playerNumber);
+  virtual ~SceneOnBikerHooks();
   void onSomersaultDone(bool i_counterclock);
   void onWheelTouches(int i_wheel, bool i_touch);
   void onHeadTouches();
 
 private:
-  MotoGame* m_motoGame;
+  Scene* m_motoGame;
   int m_playerNumber;
 };
 
